/*
 * a1-i.o.c
 * Copyright (C) 2011-2017, 2019, 2020, Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <stddef.h>

#include <a1-inter.h>
#include <a1-types.h>
#include <e4.h>
#include <lxcast.h>

#define I_MODE(t)			(*((X1f4_E4_C_MODE *) (t)))
#define I_USER(t)			(*((X1f4_E4_C_USER *) (t)))

#define l_USER(e, output) \
    {									      \
	X1f4_E4_C_USER *l;						      \
									      \
	l = (void *) (output);						      \
	*l = (e);							      \
    }

extern const void *const x1f4_a1_walk_link;

int
_x1f4_a1_draw_call(void *context, void *output, void **input, int infer,
		   const char *name, unsigned size)
{
    int status;

    do {
	X1f4_E4_C_MODE head, lock, tail;
	struct a1_instant_type *instant_data;
	struct x1f4_dxcast_type *dxcast_data;
	const struct x1f4_function_type *function_data, *function_text;
	struct x1f4_linetext_type *linetext_data;
	unsigned class, count;
	void **track, **used;

	if (output == x1f4_a1_walk_link) {
	    track = input[1];
	} else {
	    track = input;
	}

	used = track[-1];

	instant_data = used[X1f4_E4_USED_META];

	dxcast_data = I_USER(track[0]);

	if (X1f4_E4_CALL < dxcast_data->type) {
	} else {
	    status = _x1f4_a1_stat_call
		(instant_data->shuffle_data, name, size);
	    if (1) {
		break;
	    }
	}

	linetext_data = I_USER(&dxcast_data->data);
	if (linetext_data->function.function) {
	} else {
	    status = _x1f4_a1_stat_zero
		(instant_data->shuffle_data, name, size);
	    if (1) {
		break;
	    }
	}

	lock = I_MODE(track[1]);

	function_text = used[X1f4_E4_RENDITION];

	class = function_text->count - 4 + 1;
	if (lock < -(int) class || !(lock < (int) class)) {
	    status = _x1f4_a1_stat_land
		(instant_data->shuffle_data, lock, class);
	    if (1) {
		break;
	    }
	}

	function_data = &instant_data->fail_node->linetext.function;

	if (function_data->flags
	    & (X1f4_E4_SIDE_LIST | X1f4_E4_SLIP_LIST)) {
	    function_data = instant_data->s.function_data;
	}

	head = I_MODE(track[2]);
	tail = I_MODE(track[3]);

	count = function_data->count;
	count += 1 ^ infer;
	if (head < -(int) count || !(head < (int) count)) {
	    status = _x1f4_a1_stat_rend
		(instant_data->shuffle_data, head, function_data->count);
	    if (1) {
		break;
	    }
	}
	if (tail < -(int) count || !(tail < (int) count)) {
	    status = _x1f4_a1_stat_rend
		(instant_data->shuffle_data, tail, function_data->count);
	    if (1) {
		break;
	    }
	}

	if (head < 0) {
	    head += count;
	}
	if (tail < 0) {
	    tail += count;
	}

	if (infer) {
	    tail++;
	} else {
	    if (tail < head) {
		count = count - 1 - head + tail;
	    } else {
		count = tail - head;
	    }
	    if (count) {
	    } else {
		void *dxcast;

		status = instant_data->v.link
		    (instant_data->v.data, &dxcast,
		     sizeof(struct x1f4_dxcast_type));
		if (status) {
		    status = _x1f4_a1_stat_link(instant_data->shuffle_data);
		} else {
		    struct x1f4_dxcast_type *dxcast_data;

		    dxcast_data = dxcast;

		    dxcast_data->call = 0;
		    dxcast_data->type = X1f4_E4_VOID;

		    if (output == x1f4_a1_walk_link) {
			output = input[0];

			input[2] = NULL;
		    }

		    l_USER(dxcast, output);
		}
		if (1) {
		    break;
		}
	    }
	}

	if (output == x1f4_a1_walk_link) {
	    status = _x1f4_a1_lift_call
		(instant_data, output, input, linetext_data, head, tail, infer,
		 function_text, 4, lock);
	} else {
	    status = _x1f4_a1_fold_call
		(instant_data, output, input, linetext_data, head, tail, infer,
		 function_text, 4, lock);
	}
    } while (0);

    return status;
}
