/*
 * a1-l.u.c
 * Copyright (C) 2011-2017, 2019, Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <stddef.h>

#include <a1-inter.h>
#include <a1-types.h>
#include <e4.h>
#include <lxcast.h>

#define I_MODE(t)			(*((X1f4_E4_C_MODE *) (t)))
#define I_USER(t)			(*((X1f4_E4_C_USER *) (t)))

extern const void *const x1f4_a1_walk_link;

static int beta_call(void *, void *, void **);

static const int c_____c_m[] = {
/* *INDENT-OFF* */
    X1f4_E4_CASE,
    X1f4_E4_MODE
/* *INDENT-ON* */
};
const struct x1f4_function_type _x1f4_a1_l_u = {
/* *INDENT-OFF* */
    "ucall",
    beta_call,
    X1f4_E4_CASE,
    c_____c_m,
    2,
    X1f4_E4_KEEP_CALL | X1f4_E4_LINK_PASS | X1f4_E4_SIDE_LIST
    | X1f4_E4_WALK_LINK,
    5
/* *INDENT-ON* */
};

static int
beta_call(void *context, void *output, void **input)
{
    int status;
    struct a1_instant_type *instant_data;
    struct x1f4_dxcast_type *dxcast_data;
    void **track, **used;

    if (output == x1f4_a1_walk_link) {
	track = input[1];
    } else {
	track = input;
    }

    used = track[-1];

    instant_data = used[X1f4_E4_USED_META];

    dxcast_data = I_USER(track[0]);

    if (X1f4_E4_CALL < dxcast_data->type) {
	struct x1f4_linetext_type *linetext_data;

	linetext_data = I_USER(&dxcast_data->data);
	if (linetext_data->function.function) {
	    X1f4_E4_C_MODE lock;
	    const struct x1f4_function_type *function_text;
	    unsigned class;

	    lock = I_MODE(track[1]);

	    function_text = used[X1f4_E4_RENDITION];

	    class = function_text->count - 2 + 1;
	    if (lock < -(int) class || !(lock < (int) class)) {
		status = _x1f4_a1_stat_land
		    (instant_data->shuffle_data, lock, class);
	    } else {
		const struct x1f4_function_type *function_data;
		unsigned count;

		function_data = &instant_data->fail_node->linetext.function;

		if (function_data->flags
		    & (X1f4_E4_SIDE_LIST | X1f4_E4_SLIP_LIST)) {
		    function_data = instant_data->s.function_data;
		}

		count = function_data->count;
		if (count) {
		    if (output == x1f4_a1_walk_link) {
			status = _x1f4_a1_lift_call
			    (instant_data, output, input, linetext_data, 0,
			     count, 1, function_text, 2, lock);
		    } else {
			status = _x1f4_a1_fold_call
			    (instant_data, output, input, linetext_data, 0,
			     count, 1, function_text, 2, lock);
		    }
		} else {
		    status = 0;

		    if (output == x1f4_a1_walk_link) {
			input[2] = NULL;
		    }
		}
	    }
	} else {
	    status =
		_x1f4_a1_stat_zero(instant_data->shuffle_data, "ucall", 5);
	}
    } else {
	status = _x1f4_a1_stat_call(instant_data->shuffle_data, "ucall", 5);
    }

    return status;
}
