/*
 * a1.q.c
 * Copyright (C) 2008-2019, Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

/*
 * NOTES, #1
 *
 * dispatched call completion failed, there's nothing left to do for the
 * function that made the call.  let it fail.  the error may still be trapped
 * by a function further down on the call stack.
 */

/*
 * NOTES, #2
 *
 * arguments are to be processed outside call observation.  it is done like
 * that by the non sequenced a1 interpreter.  the outer temporaries allocation
 * scope is needed by arguments processing.
 */

#include <stddef.h>
#include <string.h>

#include <a1-copy.h>
#include <a1-defs.h>
#include <a1-inter.h>
#include <a1-types.h>
#include <c1.h>
#include <c1i.h>
#include <e4.h>
#include <exerrors.h>
#include <lxcast.h>

#define __CHECK_NAME__			0

#define false(e)			0

#define copy_link(copy, call) \
    copy_miss((char *) used + (integral_q) (copy)->datatext.link,	      \
	      (copy)->variable.type, (call))

typedef union link_type {
    X1f4_E4_C_MODE mode;
    X1f4_E4_C_REAL real;
    X1f4_E4_C_TEXT text;
    X1f4_E4_C_USER user;
} link_type;

extern const void *const x1f4_a1_walk_link;

static int case_back(struct a1_trans19_type *, struct x1f4_c1node_type *);
static int case_miss(struct a1_trans19_type *, struct x1f4_c1node_type *);
static int case_node(struct a1_trans19_type *, struct x1f4_c1node_type *);
static int case_slip(struct a1_trans19_type *, struct x1f4_c1node_type *, int);
static int copy_node(struct a1_trans19_type *, struct x1f4_c1node_type *,
		     int *);
static int ever_node(struct a1_trans19_type *, struct x1f4_c1node_type *,
		     int *);
static int fast_lock(struct a1_trans19_type *, struct a1_lock_type *, void *,
		     void *);
static int fast_node(struct a1_trans19_type *, struct x1f4_transfer_type *);
static int fast_port(struct a1_trans19_type *, struct a1_instant_type *,
		     struct a1_lock_type *, struct a1_fast_type **, int,
		     struct x1f4_dispatch_type **);
static int fine_last(struct a1_trans19_type *, struct x1f4_transfer_type *,
		     int *);
static int fine_link(struct a1_trans19_type *, struct x1f4_transfer_type *,
		     int *, int (*)(void *, void *, void **), unsigned);
static int head_node(struct a1_trans19_type *, struct x1f4_c1node_type *);
static int lead_back(struct a1_trans19_type *, struct x1f4_c1node_type *);
static int lead_miss(struct a1_trans19_type *, struct x1f4_c1node_type *);
static int lead_node(struct a1_trans19_type *, struct x1f4_c1node_type *);
static int link_aime(struct a1_trans19_type *, struct a1_node_type *,
		     struct x1f4_transfer_type *);
static int lock_node(struct a1_trans19_type *, struct x1f4_transfer_type *);
static int long_aime(struct a1_trans19_type *, struct a1_node_type *,
		     struct x1f4_transfer_type *);
static int long_lock(struct a1_trans19_type *, void *, void *);
static int long_port(struct a1_trans19_type *, struct a1_instant_type *, int,
		     void *, struct x1f4_dispatch_type **);
static int lose_node(struct a1_trans19_type *, struct x1f4_c1node_type *);
static int miss_aime(struct a1_trans19_type *, struct a1_node_type *);
static int miss_line(struct a1_trans19_type *, struct a1_link_type *,
		     unsigned);
static int near_node(struct a1_trans19_type *, struct a1_node_type *,
		     struct a1_lock_type *);
static int pick_lead(struct a1_trans19_type *, int);
static int pick_node(struct a1_trans19_type *, int);
static int pick_slip(struct a1_trans19_type *, int);
static int post_lead(struct a1_trans19_type *);
static int side_call(struct a1_trans19_type *,
		     const struct x1f4_function_type *);
static int slip_node(struct a1_trans19_type *);
static int tail_node(struct a1_trans19_type *, struct x1f4_c1node_type *);
static int text_line(struct a1_trans19_type *, struct a1_node_type *,
		     struct a1_link_type *);
static int type_line(struct a1_trans19_type *, struct a1_node_type *,
		     struct a1_link_type *, void **);

static int
case_back(struct a1_trans19_type *trans19_data,
	  struct x1f4_c1node_type *c1node_data)
{
    int status;

    if (0) {
    } else {
	int slipping = 0;

	status = copy_node(trans19_data, c1node_data, &slipping);
	if (status) {
	} else {
	    if (slipping) {
	    } else {
		status = tail_node(trans19_data, c1node_data + 1);
	    }
	}
    }

    return status;
}


static int
case_miss(struct a1_trans19_type *trans19_data,
	  struct x1f4_c1node_type *c1node_data)
{
    int status;

    status = head_node(trans19_data, c1node_data + 1);
    if (status) {
    } else {
	int slipping = 0;

	status = copy_node(trans19_data, c1node_data, &slipping);
	if (status) {
	} else {
	    if (slipping) {
	    } else {
		status = tail_node(trans19_data, c1node_data + 1);
	    }
	}
    }

    return status;
}


static int
case_node(struct a1_trans19_type *trans19_data,
	  struct x1f4_c1node_type *c1node_data)
{
    int status;

    if (c1node_data->state & X1f4_C1_1st_STATE) {
	status = case_miss(trans19_data, c1node_data);
    } else {
	int deck;

	status = x1f4_cast_expression
	    ((c1node_data + 1)->data.data, &deck, trans19_data->i.miss,
	     trans19_data->i.used);
	if (status) {
	} else {
	    if (deck < 0) {
		status = _x1f4_a1_stat_case(trans19_data, -deck);
	    } else {
		case_slip(trans19_data, c1node_data, deck);
	    }
	}
    }

    return status;
}


static int
case_slip(struct a1_trans19_type *trans19_data,
	  struct x1f4_c1node_type *c1node_data, int deck)
{
    if (deck) {
	trans19_data->e.slip = c1node_data->c1node_data;
    } else {
	c1node_data++;
	trans19_data->e.slip = c1node_data->c1node_data->c1node_data;
    }

    return 0;
}


static int
copy_node(struct a1_trans19_type *trans19_data,
	  struct x1f4_c1node_type *c1node_data, int *sideline)
{
    int status;
    void *expression, *slip;

    expression = (c1node_data + 1)->data.data;

    slip = trans19_data->e.port;

    while (1) {
	int slipping;
	struct x1f4_transfer_type transfer;
	union link_type link;

	status = x1f4_span_expression
	    (expression, slip, 0, &slipping, &link, &transfer,
	     trans19_data->i.used);
	if (status) {
	    break;
	} else {
	    if (slipping) {
		status = fine_last(trans19_data, &transfer, sideline);
		if (*sideline) {
		    break;
		} else {
		    if (status) {
			break;
		    }
		}
	    } else {
		int type;
		void *data;

		type = x1f4_type_expression(expression);

		if (type == X1f4_E4_CASE) {
		    struct x1f4_dxcast_type *dxcast_data;

		    dxcast_data = *(X1f4_E4_C_USER *) &link;
		    type = dxcast_data->type;
		    data = &dxcast_data->data;
		} else {
		    data = &link;
		}

		if (type == X1f4_E4_MODE) {
		    type = *(X1f4_E4_C_MODE *) data;
		} else if (type == X1f4_E4_BILL) {
		    type = *(X1f4_E4_C_BILL *) data;
		} else if (type == X1f4_E4_REAL) {
		    type = *(X1f4_E4_C_REAL *) data ? 1 : 0;
		} else {
		    status = _x1f4_a1_stat_case(trans19_data, type);
		    if (1) {
			break;
		    }
		}

		if (1) {
		    case_slip(trans19_data, c1node_data, type);
		    if (1) {
			*sideline = 0;
			if (1) {
			    break;
			}
		    }
		}
	    }
	}
    }

    return status;
}


static int
ever_node(struct a1_trans19_type *trans19_data,
	  struct x1f4_c1node_type *c1node_data, int *sideline)
{
    int status;
    void *expression, *slip;

    expression = (c1node_data + 0)->data.data;

    slip = trans19_data->e.port;

    while (1) {
	int slipping;
	struct x1f4_transfer_type transfer;
	union link_type link;

	status = x1f4_span_expression
	    (expression, slip, 0, &slipping, &link, &transfer,
	     trans19_data->i.used);
	if (status) {
	    break;
	} else {
	    if (slipping) {
		status = fine_last(trans19_data, &transfer, sideline);
		if (*sideline) {
		    break;
		} else {
		    if (status) {
			break;
		    }
		}
	    } else {
		if (1) {
		    struct x1f4_c1node_type *c1node_slip;

		    c1node_slip = c1node_data->c1node_data;

		    if (c1node_slip) {
			if (c1node_data == trans19_data->e.deck) {
			    status = x1f4_last_expression
				(c1node_slip->data.data, trans19_data->i.miss,
				 trans19_data->i.used);
			    if (status) {
			    } else {
				c1node_slip = c1node_slip->c1node_data;
			    }
			}
		    }

		    trans19_data->e.slip = c1node_slip;
		    if (1) {
			*sideline = 0;
			if (1) {
			    break;
			}
		    }
		}
	    }
	}
    }

    return status;
}


static int
fast_lock(struct a1_trans19_type *trans19_data, struct a1_lock_type *lock_data,
	  void *dispatch, void *dispatched_output)
{
    int status;
    struct a1_instant_type *instant_data;
    void *fast;

    instant_data = &trans19_data->r.instant;

    status = instant_data->m.link
	(instant_data->m.data, &fast, sizeof(struct a1_fast_type));
    if (status) {
	status = _x1f4_a1_stat_link(trans19_data->r.instant.shuffle_data);
    } else {
	struct a1_fast_type *fast_data, *fast_tail;

	fast_data = fast;

	fast_data->fast_data = NULL;
	fast_data->dispatch_data = dispatch;
	fast_data->dispatched_output = dispatched_output;

	fast_tail = lock_data->fast_tail;

	lock_data->fast_tail = fast_data;

	if (fast_tail) {
	    fast_tail->fast_data = fast_data;
	} else {
	    lock_data->fast_head = fast_data;
	}
    }

    return status;
}


static int
fast_node(struct a1_trans19_type *trans19_data,
	  struct x1f4_transfer_type *transfer_data)
{
    int status;

    do {
	struct a1_pipe_type *pipe_data;
	struct a1_instant_type *instant_data;

	instant_data = &trans19_data->r.instant;

	pipe_data = instant_data->shuffle_data->pipe_long;
	if (pipe_data) {
	    const struct x1f4_function_type *function_data;

	    function_data = transfer_data->function_data;

	    do {
		status = pipe_data->pass.fast
		    (pipe_data->pass.text, function_data);
		if (status) {
		    status = X1f4_EX_CANNOT_CONTINUE;
		    break;
		}

		pipe_data = pipe_data->pipe_long;
	    } while (pipe_data);
	    if (pipe_data) {
		pipe_data = pipe_data->back_pipe;
		if (pipe_data) {
		    _x1f4_a1_back_node(function_data, pipe_data);
		}

		break;
	    }
	}

	status = 0;
    } while (0);

    return status;
}


static int
fast_port(struct a1_trans19_type *trans19_data,
	  struct a1_instant_type *instant_data, struct a1_lock_type *lock_data,
	  struct a1_fast_type **fast, int delete,
	  struct x1f4_dispatch_type **dispatch)
{
    int status;
    struct a1_fast_type *fast_head;
    struct x1f4_dispatch_type *dispatch_data;
    void *output;

    dispatch_data = *dispatch;

    fast_head = *fast;

    output = lock_data->dispatched_output;

    while (1) {
	delete = dispatch_data->call
	    (dispatch_data->back, delete, output, &dispatch_data);
	if (delete) {
	    if (delete == X1f4_EX_CAN_CONTINUE) {
		dispatch_data = NULL;
	    } else {
		status = delete;
		break;
	    }
	}
	{
	    if (dispatch_data) {
		int slipping = 0;

		delete = fine_last
		    (trans19_data, &dispatch_data->transfer, &slipping);
		if (delete) {
		    if (delete == X1f4_EX_CAN_CONTINUE) {
			/*
			 * no aime function failed (the aime function executors
			 * will not return CAN CONTINUE for failure)
			 */
		    } else {
			status = delete;
			break;
		    }
		} else {
		    if (slipping) {
			status = 0;

			if (trans19_data->e.sample) {
			} else {
			    struct a1_lock_type *lock_text;

			    lock_text = trans19_data->e.lock;

			    if (lock_text->dispatch_data) {
				status = fast_lock
				    (trans19_data, lock_text, dispatch_data,
				     output);
			    } else {
				lock_text->dispatch_data = dispatch_data;
				lock_text->dispatched_output = output;
			    }
			    if (fast_head) {
				struct a1_fast_type *tail_fast;

				tail_fast = lock_text->fast_tail;

				lock_text->fast_tail = lock_data->fast_tail;

				if (tail_fast) {
				    tail_fast->fast_data = fast_head;
				} else {
				    lock_text->fast_head = fast_head;
				}

				fast_head = NULL;
			    }
			}

			break;
		    }
		}
	    } else {
		if (fast_head) {
		    struct a1_fast_type *fast_data;

		    fast_data = fast_head;

		    fast_head = fast_head->fast_data;

		    dispatch_data = fast_data->dispatch_data;
		    output = fast_data->dispatched_output;

		    status = instant_data->m.free
			(instant_data->m.data, fast_data);
		    if (status) {
			status = _x1f4_a1_stat_free
			    (trans19_data->r.instant.shuffle_data);
			if (1) {
			    break;
			}
		    }
		} else {
		    status = delete;
		    if (1) {
			break;
		    }
		}
	    }
	}
    }

    *fast = fast_head;

    *dispatch = dispatch_data;

    return status;
}


/*
 * NOTES
 *
 * the function does execute recursively.  If the WALK_LINK function(s)
 * recurse(s) for too many times, the stack will be overrun.  The condition can
 * (!?) be cured, and yet the functions are application introduced, and curing
 * application code is beyond scope here.
 */
static int
fine_last(struct a1_trans19_type *trans19_data,
	  struct x1f4_transfer_type *transfer_data, int *sideline)
{
    int status;
    const struct x1f4_function_type *function_data;
    unsigned flags;

    function_data = transfer_data->function_data;

    flags = function_data->flags;

    if (flags & X1f4_E4_NODE_LINK) {
	status = lock_node(trans19_data, transfer_data);
	if (status) {
	} else {
	    *sideline = 1;
	}
    } else {
	int (*function)(void *, void *, void **);

	function = function_data->function;

	if (function) {
	    if (flags & (X1f4_E4_LEAD_CALL | X1f4_E4_WALK_LINK)) {
		if (flags & X1f4_E4_WALK_LINK) {
		    status = fine_link
			(trans19_data, transfer_data, sideline, function,
			 flags);
		} else {
		    trans19_data->e.sample = 1;
		    trans19_data->x.transfer = *transfer_data;
		    *sideline = 1;
		    if (1) {
			status = 0;
		    }
		}
	    } else {
		void **input, *output;

		input = transfer_data->input;
		output = transfer_data->output;

		input[-1] = trans19_data->i.used;

		status = function(transfer_data->context, output, input);
	    }
	} else {
	    /*
	     * curious RENDITION set (_span_expression would have set it to the
	     * unset function pointer)
	     */
	    *((void **) trans19_data->i.used + X1f4_E4_RENDITION) =
		&trans19_data->r.instant.fail_node->linetext.function;

	    status = _x1f4_a1_stat_miss(trans19_data->r.instant.shuffle_data);
	}
    }

    return status;
}


static int
fine_link(struct a1_trans19_type *trans19_data,
	  struct x1f4_transfer_type *transfer_data, int *sideline,
	  int (*function)(void *, void *, void **), unsigned flags)
{
    int status;
    void *mid[3];

    mid[0] = transfer_data->output;
    mid[1] = transfer_data->input;

    transfer_data->input[-1] = trans19_data->i.used;

    status = function(transfer_data->context, (void *) x1f4_a1_walk_link, mid);
    if (status) {
    } else {
	struct x1f4_dispatch_type *dispatch_data;

	dispatch_data = mid[2];
	while (dispatch_data) {
	    status = fine_last
		(trans19_data, &dispatch_data->transfer, sideline);
	    if (*sideline) {
		if (trans19_data->e.sample) {
		    status = long_lock
			(trans19_data, dispatch_data, transfer_data->output);
		} else {
		    struct a1_lock_type *lock_data;

		    lock_data = trans19_data->e.lock;

		    if (lock_data->dispatch_data) {
			status = fast_lock
			    (trans19_data, lock_data, dispatch_data,
			     transfer_data->output);
		    } else {
			lock_data->dispatch_data = dispatch_data;
			lock_data->dispatched_output = transfer_data->output;
		    }
		}

		break;
	    } else {
		if (status) {
		    if (status == X1f4_EX_CAN_CONTINUE) {
		    } else {
			break;
		    }
		}
		{
		    status = dispatch_data->call
			(dispatch_data->back, status, transfer_data->output,
			 &dispatch_data);
		    if (status) {
			if (status == X1f4_EX_CAN_CONTINUE) {
			    dispatch_data = NULL;
			} else {
			    break;
			}
		    }
		}
	    }
	}
    }

    return status;
}


static int
head_node(struct a1_trans19_type *trans19_data,
	  struct x1f4_c1node_type *c1node_data)
{
    return x1f4_land_expression
	(c1node_data->data.data, trans19_data->i.miss, &trans19_data->e.port);
}


static int
lead_back(struct a1_trans19_type *trans19_data,
	  struct x1f4_c1node_type *c1node_data)
{
    int status;

    if (0) {
    } else {
	int slipping = 0;

	status = ever_node(trans19_data, c1node_data, &slipping);
	if (status) {
	} else {
	    if (slipping) {
	    } else {
		status = tail_node(trans19_data, c1node_data + 0);
	    }
	}
    }

    return status;
}


static int
lead_miss(struct a1_trans19_type *trans19_data,
	  struct x1f4_c1node_type *c1node_data)
{
    int status;

    status = head_node(trans19_data, c1node_data + 0);
    if (status) {
    } else {
	int slipping = 0;

	status = ever_node(trans19_data, c1node_data, &slipping);
	if (status) {
	} else {
	    if (slipping) {
	    } else {
		status = tail_node(trans19_data, c1node_data + 0);
	    }
	}
    }

    return status;
}


static int
lead_node(struct a1_trans19_type *trans19_data,
	  struct x1f4_c1node_type *c1node_data)
{
    int status;

    if (c1node_data->state & X1f4_C1_1st_STATE) {
	status = lead_miss(trans19_data, c1node_data);
    } else {
	status = x1f4_last_expression
	    (c1node_data->data.data, trans19_data->i.miss,
	     trans19_data->i.used);
	if (status) {
	} else {
	    if (c1node_data == trans19_data->e.deck) {
		c1node_data = c1node_data->c1node_data;
		status = x1f4_last_expression
		    (c1node_data->data.data, trans19_data->i.miss,
		     trans19_data->i.used);
		if (status) {
		} else {
		    trans19_data->e.slip = c1node_data->c1node_data;
		}
	    } else {
		trans19_data->e.slip = c1node_data->c1node_data;
	    }
	}
    }

    return status;
}


static int
link_aime(struct a1_trans19_type *trans19_data, struct a1_node_type *node_data,
	  struct x1f4_transfer_type *transfer_data)
{
    int status;

    status = long_aime(trans19_data, node_data, transfer_data);
    if (status) {
    } else {
	status = fast_node(trans19_data, transfer_data);
	if (status) {
	    miss_aime(trans19_data, node_data);
	}
    }

    return status;
}


static int
lock_node(struct a1_trans19_type *trans19_data,
	  struct x1f4_transfer_type *transfer_data)
{
    int status;
    struct a1_instant_type *instant_data;
    void *lock;

    instant_data = &trans19_data->r.instant;

    status = instant_data->m.link
	(instant_data->m.data, &lock, sizeof(struct a1_lock_type));
    if (status) {
	status = _x1f4_a1_stat_link(trans19_data->r.instant.shuffle_data);
    } else {
	struct a1_lock_type *lock_data;
	struct a1_node_type *node_data;
	void *link;

	lock_data = lock;

	node_data = transfer_data->context;

#if __CHECK_NAME__
	if (strcmp(((struct x1f4_function_type *)
		    ((void **)
		     transfer_data->input[-1])[X1f4_E4_RENDITION])->name,
		   node_data->linetext.function.name)) {
	    if (strcmp(((struct x1f4_function_type *)
			((void **)
			 transfer_data->input[-1])[X1f4_E4_RENDITION])->name,
		       "beta")) {
		instant_data->m.free(instant_data->m.data, lock);
		return 1;
	    }
	}
#endif				/* __CHECK_NAME__ */

	status = _x1f4_a1_line_node
	    (instant_data, node_data, sizeof(void *), &link);
	if (status) {
	    instant_data->m.free(instant_data->m.data, lock);
	} else {
	    void **used;

	    used = link;

	    *used = trans19_data->i.link;

	    used++;

	    trans19_data->i.used = used;

	    lock_data->lead = instant_data->s.lead;
	    lock_data->function_lead = instant_data->s.function_data;

	    instant_data->s.function_data =
		((void **) transfer_data->input[-1])[X1f4_E4_RENDITION];

	    /*
	     * provisory: all RENDITION pointers set
	     * should have been instant_data->s.function_data
	     * should have been able to keep old USED vector
	     */
	    used[X1f4_E4_RENDITION] = &node_data->linetext.function;

	    status = link_aime(trans19_data, node_data, transfer_data);

	    if (status) {
		trans19_data->i.used =
		    (char *) trans19_data->i.link + sizeof(void *);

		instant_data->s.lead = lock_data->lead;
		instant_data->s.function_data = lock_data->function_lead;

		instant_data->m.free(instant_data->m.data, link);
		instant_data->m.free(instant_data->m.data, lock);
	    } else {
		struct x1f4_c1node_type *c1node_data;
		void *aime;

		aime = node_data->aime;

		x1f4_node_program(aime, &c1node_data);

		lock_data->fail_node = instant_data->fail_node;

		lock_data->c1node_deck = trans19_data->e.deck;

		lock_data->lock_data = trans19_data->e.lock;

		lock_data->head = trans19_data->e.head;
		lock_data->tail = trans19_data->e.tail;

		lock_data->slip = trans19_data->e.slip;

		lock_data->port = trans19_data->e.port;

		instant_data->fail_node = node_data;

		trans19_data->e.head = NULL;
		trans19_data->e.tail = NULL;

		trans19_data->e.port = NULL;

		trans19_data->e.lock = lock_data;

		trans19_data->e.slip = c1node_data;

		trans19_data->i.link = link;

		trans19_data->i.miss = (char *) used + node_data->post;

		lock_data->fast_head = NULL;
		lock_data->fast_tail = NULL;

		lock_data->node_data = node_data;

		lock_data->output = transfer_data->output;

		lock_data->dispatch_data = NULL;

		lock_data->function_data = transfer_data->function_data;

		/*
		 * keep alien functions from self destruction
		 */
		node_data->leadnode.line.call++;

		x1f4_land_program(aime, &c1node_data);
		if (c1node_data) {
		    if (c1node_data->c1node_data) {
		    } else {
			c1node_data = NULL;
		    }
		}

		trans19_data->e.deck = c1node_data;
	    }
	}
    }

    return status;
}


static int
long_aime(struct a1_trans19_type *trans19_data, struct a1_node_type *node_data,
	  struct x1f4_transfer_type *transfer_data)
{
    int status = 0;

    do {
	struct a1_link_type *link_data;
	struct a1_instant_type *instant_data;
	void *line = NULL;

	instant_data = &trans19_data->r.instant;

	link_data = node_data->link_data;

	if (link_data) {
	    status = type_line
		(trans19_data, node_data, link_data, transfer_data->input);
	    if (status) {
		break;
	    }
	}

	if (transfer_data->function_data->function == _x1f4_a1_deck_aime) {
	    const struct x1f4_function_type *function_data;
	    unsigned prefix, offset;

	    prefix = node_data->linetext.function.count;

	    function_data = instant_data->s.function_data;

	    offset = function_data->count - prefix;
	    if (node_data->linetext.function.flags & X1f4_E4_SLIP_LIST) {
		status = _x1f4_a1_lead_slip
		    (instant_data, transfer_data->input, node_data->link_data,
		     function_data, &node_data->linetext.function,
		     trans19_data->i.used);
		if (status) {
		    text_line(trans19_data, node_data, node_data->link_data);
		    break;
		} else {
		}
	    }
	    if (offset) {
		status = _x1f4_a1_lead_side
		    (instant_data, offset, transfer_data->input, prefix, &line,
		     function_data);
		if (status) {
		    if (node_data->linetext.function.flags
			& X1f4_E4_SLIP_LIST) {
			_x1f4_a1_slip_slip
			    (instant_data, node_data->link_data, -1,
			     function_data, &node_data->linetext.function,
			     trans19_data->i.used);
			text_line
			    (trans19_data, node_data, node_data->link_data);
		    }

		    break;
		}
	    }
	}

	instant_data->s.lead = line;
    } while (0);

    return status;
}


static int
long_lock(struct a1_trans19_type *trans19_data, void *dispatch,
	  void *dispatched_output)
{
    int status;
    struct a1_instant_type *instant_data;
    void *fast;

    instant_data = &trans19_data->r.instant;

    status = instant_data->m.link
	(instant_data->m.data, &fast, sizeof(struct a1_fast_type));
    if (status) {
	status = _x1f4_a1_stat_link(trans19_data->r.instant.shuffle_data);
    } else {
	struct a1_fast_type *fast_data, *fast_tail;

	fast_data = fast;

	fast_data->fast_data = NULL;
	fast_data->dispatch_data = dispatch;
	fast_data->dispatched_output = dispatched_output;

	fast_tail = trans19_data->e.tail;

	trans19_data->e.tail = fast_data;

	if (fast_tail) {
	    fast_tail->fast_data = fast_data;
	} else {
	    trans19_data->e.head = fast_data;
	}
    }

    return status;
}


static int
long_port(struct a1_trans19_type *trans19_data,
	  struct a1_instant_type *instant_data, int delete, void *output,
	  struct x1f4_dispatch_type **dispatch)
{
    int status;
    struct x1f4_dispatch_type *dispatch_data;

    dispatch_data = *dispatch;

    while (1) {
	delete = dispatch_data->call
	    (dispatch_data->back, delete, output, &dispatch_data);
	if (delete) {
	    dispatch_data = NULL;
	    if (delete == X1f4_EX_CAN_CONTINUE) {
	    } else {
		status = delete;
		break;
	    }
	}
	{
	    if (dispatch_data) {
		int slipping = 0;

		delete = fine_last
		    (trans19_data, &dispatch_data->transfer, &slipping);
		if (delete) {
		    if (delete == X1f4_EX_CAN_CONTINUE) {
			/*
			 * no aime function failed (the aime function executors
			 * will not return CAN CONTINUE for failure)
			 */
		    } else {
			dispatch_data = NULL;
			status = delete;
			break;
		    }
		} else {
		    if (slipping) {
			status = 0;

			if (trans19_data->e.sample) {
			} else {
			    struct a1_lock_type *lock_text;

			    lock_text = trans19_data->e.lock;

			    if (lock_text->dispatch_data) {
				status = fast_lock
				    (trans19_data, lock_text, dispatch_data,
				     output);
			    } else {
				lock_text->dispatch_data = dispatch_data;
				lock_text->dispatched_output = output;
			    }
			}

			break;
		    }
		}
	    } else {
		if (0) {
		} else {
		    status = delete;
		    if (1) {
			break;
		    }
		}
	    }
	}
    }

    *dispatch = dispatch_data;

    return status;
}


static int
lose_node(struct a1_trans19_type *trans19_data,
	  struct x1f4_c1node_type *c1node_data)
{
    int status;

    if (c1node_data->data.data) {
	status = lead_back(trans19_data, c1node_data);
    } else {
	status = case_back(trans19_data, c1node_data);
    }

    return status;
}


static int
miss_aime(struct a1_trans19_type *trans19_data, struct a1_node_type *node_data)
{
    int status;

    do {
	struct a1_instant_type *instant_data;

	instant_data = &trans19_data->r.instant;

	if (1) {
	    void *text;

	    text = instant_data->s.lead;
	    if (text) {
		const struct x1f4_function_type *function_lead;
		unsigned prefix;

		prefix = node_data->linetext.function.count;
		function_lead = instant_data->s.function_data;
		status = _x1f4_a1_slip_side
		    (instant_data, prefix, function_lead->count - prefix, text,
		     function_lead);
		if (status) {
		    break;
		}
	    } else {
		status = 0;
	    }
	    if (node_data->linetext.function.flags & X1f4_E4_SLIP_LIST) {
		int excess;

		excess = _x1f4_a1_slip_slip
		    (instant_data, node_data->link_data, -1,
		     instant_data->s.function_data,
		     &node_data->linetext.function, trans19_data->i.used);
		if (excess) {
		    if (status) {
		    } else {
			status = excess;
		    }

		    break;
		}
	    }
	}

	if (1) {
	    struct a1_link_type *link_data;

	    link_data = node_data->link_data;
	    if (link_data) {
		status = text_line(trans19_data, node_data, link_data);
		if (status) {
		    break;
		}
	    }
	}
    } while (0);

    return status;
}


static int
miss_line(struct a1_trans19_type *trans19_data, struct a1_link_type *link_data,
	  unsigned count)
{
    int status = 0;

    if (1) {
	void *instant, *used;

	instant = &trans19_data->r.instant;

	used = trans19_data->i.used;

	for (; count; count--) {
	    int (*miss)(void *, struct a1_link_type *, void *);

	    link_data--;

	    miss = link_data->lose;
	    if (miss) {
		status = miss(instant, link_data, used);
		if (status) {
		    if (1) {
			break;
		    }
		}
	    }
	}
    }

    return status;
}


static int
near_node(struct a1_trans19_type *trans19_data, struct a1_node_type *node_data,
	  struct a1_lock_type *lock_data)
{
    const struct x1f4_function_type *function_data;

    function_data = lock_data->function_data;
    if (function_data->type == X1f4_E4_VOID) {
    } else {
	void *expression;

	x1f4_pick_program(node_data->aime, &expression);
	if (1) {
	    x1f4_fast_expression
		(expression, trans19_data->i.miss, lock_data->output);
	}
    }

    return 0;
}


static int
pick_lead(struct a1_trans19_type *trans19_data, int delete)
{
    int status;

    do {
	struct a1_fast_type *fast_data;
	struct x1f4_dispatch_type *dispatch_data;
	struct a1_instant_type *instant_data;
	void *output;

	instant_data = &trans19_data->r.instant;

	fast_data = trans19_data->e.head;

	dispatch_data = fast_data->dispatch_data;
	output = fast_data->dispatched_output;

	status = long_port
	    (trans19_data, instant_data, delete, output, &dispatch_data);
	delete = status;
	if (status == X1f4_EX_CAN_CONTINUE) {
	    status = 0;
	}
	if (status) {
	    /*
	     * See NOTES, #1
	     */
	    break;
	} else {
	    if (dispatch_data) {
		break;
	    } else {
		trans19_data->e.head = fast_data->fast_data;
		if (trans19_data->e.head) {
		} else {
		    trans19_data->e.tail = NULL;
		}

		status = instant_data->m.free
		    (instant_data->m.data, fast_data);
		if (status) {
		    status = _x1f4_a1_stat_free
			(trans19_data->r.instant.shuffle_data);
		    if (1) {
			break;
		    }
		}

		if (trans19_data->e.head) {
		} else {
		    if (delete) {
			status = delete;
		    } else {
			status = lose_node(trans19_data, trans19_data->e.slip);
		    }

		    break;
		}
	    }
	}
    } while (1);

    return status;
}


static int
pick_node(struct a1_trans19_type *trans19_data, int delete)
{
    int excess, status;
    struct a1_lock_type *lock_data;
    struct a1_node_type *node_data;
    struct a1_instant_type *instant_data;
    struct x1f4_c1node_type *c1node_data;

    lock_data = trans19_data->e.lock;

    node_data = lock_data->node_data;

    if (delete) {
    } else {
	near_node(trans19_data, node_data, lock_data);
    }

    c1node_data = lock_data->slip;

    instant_data = &trans19_data->r.instant;

    trans19_data->e.deck = lock_data->c1node_deck;

    status = _x1f4_a1_miss_fast(instant_data, trans19_data->e.head);

    trans19_data->e.head = lock_data->head;
    trans19_data->e.tail = lock_data->tail;

    trans19_data->e.lock = lock_data->lock_data;

    trans19_data->e.port = lock_data->port;

    trans19_data->e.slip = c1node_data;

    excess = side_call(trans19_data, lock_data->function_data);
    if (excess) {
	if (status) {
	} else {
	    status = excess;
	}
    }

    excess = miss_aime(trans19_data, node_data);
    if (excess) {
	if (status) {
	} else {
	    status = excess;
	}
    }

    if (0) {
    } else {
	struct a1_fast_type *fast_head;
	void **link, *used;

	instant_data->fail_node = lock_data->fail_node;

	instant_data->s.function_data = lock_data->function_lead;
	instant_data->s.lead = lock_data->lead;

	fast_head = lock_data->fast_head;

	link = trans19_data->i.link;

	used = *link;

	trans19_data->i.link = used;
	trans19_data->i.used = (char *) used + sizeof(void *);

	trans19_data->i.miss =
	    (char *) trans19_data->i.used + instant_data->fail_node->post;

	if (1) {
	    if (status) {
	    } else {
		do {
		    struct x1f4_dispatch_type *dispatch_data;

		    dispatch_data = lock_data->dispatch_data;
		    if (dispatch_data) {
			status = fast_port
			    (trans19_data, instant_data, lock_data, &fast_head,
			     delete, &dispatch_data);
			if (1) {
			    if (status) {
				/*
				 * See NOTES, #1
				 */
				break;
			    } else {
				if (dispatch_data) {
				    break;
				} else {
				    delete = 0;
				}
			    }
			}
		    } else {
			if (delete) {
			    status = delete;
			    if (1) {
				break;
			    }
			}
		    }

		    status = lose_node(trans19_data, c1node_data);
		} while (0);
	    }
	}

	if (fast_head) {
	    excess = _x1f4_a1_miss_fast(instant_data, fast_head);
	    if (excess) {
		if (status) {
		} else {
		    status = excess;
		}
	    }
	}

	excess = instant_data->m.free(instant_data->m.data, link);
	if (excess) {
	    if (status) {
	    } else {
		status =
		    _x1f4_a1_stat_free(trans19_data->r.instant.shuffle_data);
	    }
	}

	if (node_data->grid_data == instant_data->shuffle_data->grid_data) {
	    node_data->leadnode.line.call--;
	} else {
	    void *linetext;

	    linetext = &node_data->linetext;
	    excess = _x1f4_a1_slip_news(NULL, &linetext);
	    if (excess) {
		if (status) {
		} else {
		    status = excess;
		}
	    }
	}

	excess = instant_data->m.free(instant_data->m.data, lock_data);
	if (excess) {
	    if (status) {
	    } else {
		status =
		    _x1f4_a1_stat_free(trans19_data->r.instant.shuffle_data);
	    }
	}
    }

    return status;
}


static int
pick_slip(struct a1_trans19_type *trans19_data, int status)
{
    while (status == X1f4_EX_CAN_CONTINUE) {
	if (trans19_data->e.port) {
	    int excess;

	    excess = _x1f4_a1_miss_port(trans19_data);
	    if (excess) {
		if (false(excess == X1f4_EX_CAN_CONTINUE)) {
		} else {
		    status = excess;
		    if (1) {
			break;
		    }
		}
	    } else {
		trans19_data->e.port = NULL;
	    }
	}

	if (trans19_data->e.lock) {
	    struct a1_lock_type *lock_data;

	    lock_data = trans19_data->e.lock;
	    status = x1f4_mind_program
		(lock_data->node_data->aime, trans19_data->i.used);
	    if (status) {
		if (status == X1f4_EX_CAN_CONTINUE) {
		} else {
		    break;
		}
	    }

	    status = pick_node(trans19_data, X1f4_EX_CAN_CONTINUE);
	} else {
	    break;
	}
    }

    return status;
}


static int
post_lead(struct a1_trans19_type *trans19_data)
{
    int status;
    struct x1f4_transfer_type *transfer_data;

    trans19_data->e.sample = 0;

    transfer_data = &trans19_data->x.transfer;
    status = transfer_data->function_data->function
	(transfer_data->context, transfer_data->output, transfer_data->input);
    if (trans19_data->e.head) {
	status = pick_lead(trans19_data, status);
    } else {
	if (status) {
	} else {
	    status = lose_node(trans19_data, trans19_data->e.slip);
	}
    }

    return status;
}


static int
side_call(struct a1_trans19_type *trans19_data,
	  const struct x1f4_function_type *function_data)
{
    int status;

    do {
	struct a1_pipe_type *pipe_data;

	pipe_data = trans19_data->r.instant.shuffle_data->back_pipe;

	if (pipe_data) {
	    do {
		status = pipe_data->pass.lose
		    (pipe_data->pass.text, function_data);
		if (status) {
		    status = X1f4_EX_CANNOT_CONTINUE;
		    break;
		}

		pipe_data = pipe_data->back_pipe;
	    } while (pipe_data);
	    if (pipe_data) {
		pipe_data = pipe_data->back_pipe;
		if (pipe_data) {
		    _x1f4_a1_back_node(function_data, pipe_data);
		}

		break;
	    }
	}

	status = 0;
    } while (0);

    return status;
}


static int
slip_node(struct a1_trans19_type *trans19_data)
{
    int status;
    struct x1f4_c1node_type *c1node_data;

    c1node_data = trans19_data->e.slip;
    if (c1node_data->data.data) {
	status = lead_node(trans19_data, c1node_data);
    } else {
	if ((c1node_data + 1)->data.data) {
	    status = case_node(trans19_data, c1node_data);
	} else {
	    c1node_data++;
	    trans19_data->e.slip = c1node_data->c1node_data;
	    status = 0;
	}
    }

    return status;
}


static int
tail_node(struct a1_trans19_type *trans19_data,
	  struct x1f4_c1node_type *c1node_data)
{
    int status;

    status = x1f4_tail_expression
	(c1node_data->data.data, &trans19_data->e.port);
    if (status) {
    } else {
	trans19_data->e.port = NULL;
    }

    return status;
}


static int
text_line(struct a1_trans19_type *trans19_data, struct a1_node_type *node_data,
	  struct a1_link_type *link_data)
{
    int status = 0;

    if (1) {
	struct a1_shuffle_type *shuffle_data;
	unsigned count;
	void *used;

	shuffle_data = trans19_data->r.instant.shuffle_data;

	used = trans19_data->i.used;

	count = node_data->linetext.function.count;
	for (; count; count--) {
	    int (*miss)(void *, struct a1_link_type *, void *);

	    miss = link_data->lose;
	    if (miss) {
		status = miss(shuffle_data, link_data, used);
		if (status) {
		    if (1) {
			break;
		    }
		}
	    }

	    link_data++;
	}
    }

    return status;
}


static int
type_line(struct a1_trans19_type *trans19_data, struct a1_node_type *node_data,
	  struct a1_link_type *link_data, void **input)
{
    int status = 0;

    if (1) {
	unsigned count;
	void *subtext[4], *used;

	subtext[1] = trans19_data->r.instant.shuffle_data;
	subtext[2] = NULL;
	subtext[3] = NULL;

	used = trans19_data->i.used;

	count = node_data->linetext.function.count;
	for (; count; count--) {
	    int (*line)(void **, void **, struct a1_link_type *, void *);

	    line = link_data->line;
	    if (line) {
		status = line(subtext, input, link_data, used);
		if (status) {
		    miss_line
			(trans19_data, link_data,
			 node_data->linetext.function.count - count);
		    if (1) {
			break;
		    }
		}
	    } else {
		copy_link(link_data, *input);
	    }

	    input++;

	    link_data++;
	}
    }

    return status;
}


int
x1f4_slip_shuffle(struct a1_trans19_type *trans19_data)
{
    int status;

    if (trans19_data->e.sample) {
	status = post_lead(trans19_data);
    } else {
	if (trans19_data->e.slip) {
	    status = slip_node(trans19_data);
	} else {
	    status = pick_node(trans19_data, 0);
	}
    }
    if (status) {
	status = pick_slip(trans19_data, status);
    }

    return status;
}


int
x1f4_step_shuffle(struct a1_trans19_type *trans19_data)
{
    int status;

    if (!trans19_data->e.lock && !trans19_data->e.slip) {
	status = X1f4_EX_COMPLETED;
    } else {
	if (trans19_data->e.sample) {
	    status = post_lead(trans19_data);
	} else {
	    if (trans19_data->e.slip) {
		status = slip_node(trans19_data);
	    } else {
		status = pick_node(trans19_data, 0);
	    }
	}
	if (status) {
	    status = pick_slip(trans19_data, status);
	}
	if (status) {
	    status = -1;
	} else {
	    status = trans19_data->e.sample ? X1f4_EX_LEAD_CALL : 0;
	}
    }

    return status;
}
