/*
 * a12nd.4.c
 * Copyright (C) 2008-2011, 2017-2019, Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <stddef.h>

#include <a1-defs.h>
#include <ctype.0.h>
#include <e4.h>

#define A_TYPE_X(c_type_x, c) \
    (!C_TYPE_X((c_type_x), (c)))

extern const unsigned _x1f4_c_type_0[], _x1f4_c_type_1[];

static void way_far(const char **);
static void way_new(const char **);
static void way_red(const char **);

static const unsigned c_type_4[] = {
/*
 *                   . .     .
 *
 * .
 *
 *
 *
 *
 *
 *
 *
 *
 *
 *
 *
 *
 *
 */
/* *INDENT-OFF* */
    0x00002600, 0x00000001, 0x00000000, 0x00000000,
    0x00000000, 0x00000000, 0x00000000, 0x00000000
/* *INDENT-ON* */
};

static void
way_far(const char **deck)
{
    const char *e;
    unsigned c;

    e = *deck;

    while (1) {
	c = *(const unsigned char *) e;
	if (C_TYPE_X(c_type_4, c)) {
	    e++;
	} else {
	    if (c ^ 35) {
		break;
	    } else {
		do {
		    ++e;
		    c = *(const unsigned char *) e;
		} while (c ^ 10);
	    }
	}
    }

    *deck = e;
}


static void
way_new(const char **deck)
{
    const char *e;
    unsigned c;

    e = *deck;

    c = *(const unsigned char *) e;
    while (C_TYPE_X(c_type_4, c)) {
	e++;
	c = *(const unsigned char *) e;
    }

    *deck = e;
}


static void
way_red(const char **deck)
{
    const char *e;
    unsigned c;

    e = *deck;

    c = *(const unsigned char *) e;
    while (C_TYPE_X(_x1f4_c_type_0, c)) {
	e++;
	c = *(const unsigned char *) e;
    }

    *deck = e;
}


/*
 * allowing:
 *
 * name name "("
 * fname "(" ")" "{"
 * sname "(" ","
 * fname "(" "&"
 * fname "(" "..."
 * fname "(" "void" ")"
 * name ("(" "*")* name "("
 *     -- troubling, if `*' is allowed as unary operator, etc
 * sname "(" name name
 * fname "(" name ("(" "*")* name ")"
 *
 * with -fname- an undefined function name
 * with -sname- an undefined function name, other than "for"
 */
int
x1f4_gawp_shuffle(const char *shuffle, unsigned flags, const void *context,
		  int (*look)(const char *, unsigned, const void *,
			      const struct x1f4_function_type **))
{
    const char *name;
    int excess;
    unsigned c, size;
    void (*slip)(const char **);

    if (flags & SKIP_SCOMMENT) {
	slip = way_far;
    } else {
	slip = way_new;
    }

    slip(&shuffle);
    c = *(const unsigned char *) shuffle;
    if (A_TYPE_X(_x1f4_c_type_1, c)) {
	name = NULL;
	size = 0;
    } else {
	name = shuffle;
	way_red(&shuffle);
	size = shuffle - name;
	slip(&shuffle);
    }
    do {
	c = *(const unsigned char *) shuffle;
	if (A_TYPE_X(_x1f4_c_type_1, c)) {
	    const struct x1f4_function_type *function_data;
	    int match = 40;

	    if (c != 40) {
		excess = 0;
		if (1) {
		    break;
		}
	    }

	    shuffle++;
	    slip(&shuffle);
	    c = *(const unsigned char *) shuffle;
	    if (c != 42) {
		if (name && look) {
		    excess = look(name, size, context, &function_data);
		    if (excess) {
		    } else {
			break;
		    }
		}

		if (A_TYPE_X(_x1f4_c_type_1, c) || !size) {
		    if (size) {
			if (c == 41) {
			    shuffle++;
			    slip(&shuffle);
			    c = *(const unsigned char *) shuffle;
			    if (c == '{') {
				excess = 1;
				if (1) {
				    break;
				}
			    }
			} else {
			    if (c == ',') {
				if (size != 3 || name[0] != 'f'
				    || name[1] != 'o' || name[2] != 'r') {
				    excess = 1;
				    if (1) {
					break;
				    }
				}
			    } else {
				if (c == '&'
				    || (c == '.' && shuffle[1] == '.'
					&& shuffle[2] == '.')) {
				    excess = 1;
				    if (1) {
					break;
				    }
				} else {
				}
			    }
			}
		    }

		    excess = 0;
		    if (1) {
			break;
		    }
		} else {
		    const char *type;
		    unsigned rest;

		    type = shuffle;
		    shuffle++;
		    way_red(&shuffle);
		    rest = shuffle - type;
		    slip(&shuffle);
		    c = *(const unsigned char *) shuffle;
		    if (A_TYPE_X(_x1f4_c_type_1, c)) {
			if (rest == 4) {
			    if (type[0] == 'v' && type[1] == 'o'
				&& type[2] == 'i' && type[3] == 'd'
				&& c == 41) {
				excess = 1;
				if (1) {
				    break;
				}
			    }
			}

			match = 41;
		    } else {
			excess = size != 3 || name[0] != 'f' || name[1] != 'o'
			    || name[2] != 'r';
			if (1) {
			    break;
			}
		    }
		}
	    } else {
		if (size) {
		    shuffle--;
		    c = 40;
		} else {
		    excess = 0;
		    if (1) {
			break;
		    }
		}
	    }

	    while (1) {
		if (c != 40) {
		    excess = 0;
		    if (1) {
			break;
		    }
		} else {
		    shuffle++;
		    slip(&shuffle);
		    c = *(const unsigned char *) shuffle;
		    if (c != 42) {
			excess = 0;
			if (1) {
			    break;
			}
		    } else {
			shuffle++;
			slip(&shuffle);
			c = *(const unsigned char *) shuffle;
			if (C_TYPE_X(_x1f4_c_type_1, c)) {
			    shuffle++;
			    way_red(&shuffle);
			    slip(&shuffle);
			    c = *(const unsigned char *) shuffle;
			    if (c != match) {
				excess = 0;
			    } else {
				excess = 1;
			    }
			    if (1) {
				break;
			    }
			}
		    }
		}
	    }
	} else {
	    shuffle++;
	    way_red(&shuffle);
	    slip(&shuffle);
	    c = *(const unsigned char *) shuffle;
	    if (c != 40) {
		excess = 0;
	    } else {
		excess = 1;
	    }
	}
    } while (0);

    return excess;
}

