/*
 * bqpx.t.c
 * Copyright (C) 2009-2012, 2018, Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <stddef.h>
#include <string.h>

#include <bqfx-bits.h>
#include <bqfx-defs.h>
#include <bqfx-inter.h>
#include <bqfx-lines.h>
#include <bqfx-names.h>
#include <bqfx-types.h>
#include <bqpx-types.h>

#define __HALF_TRANSFER__		1

#define false(e)			0

#undef Q
#define Q(e)				((integral_q) (e))

#define bqfset(bqfset)			((struct bqfset_type *) (bqfset))

static int east(void *, unsigned);
static int edge(void *, unsigned);

static int
east(void *node, unsigned call)
{
    integral_q news, slip;

    news = *KD(node);

    slip = news & ~Lx55555555;

    {
	if (false(call ^ (integral_q_bits - 1))) {
	} else {
	    unsigned deck, ever;

	    bq_q_xls(deck, slip);
	    if (1) {
		deck ^= integral_q_last;
		if (deck ^ (integral_q_bits - 1)) {
		    integral_q *data, *text;

		    *KD(node) = news & ~(Q(1) << deck);

		    PD(node)[integral_q_bits - 1] =
			PD(node)[integral_q_bits - 3];

		    call--;
		    data = KD(node) + call;
		    call--;
		    text = (integral_q *) PD(node) + call;

		    ever = (integral_q_bits - 3 - deck) >> 1;
		    for (; ever; ever--) {
			*data = *(data - 2);
			data -= 2;
			*text = *(text - 2);
			text -= 2;
		    }
		    {
			*data = *(data - 1);
			*text = *(text - 1);
		    }
		} else {
		    *KD(node) = news & ~(Q(1) << deck);

		    PD(node)[integral_q_bits - 1] =
			PD(node)[integral_q_bits - 2];
		}
	    }
	}
    }

    return 0;
}


static int
edge(void *node, unsigned call)
{
    integral_q news, slip;

    news = *KD(node);

    slip = news & ~Lx55555555;

    {
	if (1) {
	    call++;

	    if (call & 1) {
		*KD(node) = news & ~(Q(1) << call);
	    } else {
		if (news & (Q(1) << (call - 1))) {
		    *KD(node) = news & ~(Q(1) << (call - 1));
		    KD(node)[call] = KD(node)[call - 1];
		    call--;
		    PD(node)[call] = PD(node)[call - 1];
		} else {
		    if (news & (Q(1) << (call + 1))) {
			*KD(node) = news & ~(Q(1) << (call + 1));
			if (2) {
			    KD(node)[call] = KD(node)[call + 1];
			    PD(node)[call - 1] = PD(node)[call];
			}
		    } else {
			integral_q back;
			unsigned deck, ever;

			back = slip & (~Q(0) << call);
			if (back) {
			    bq_q_xfs(ever, back);
#if __HEAD_LEFT__
			    slip &= ((Q(1) << ever) - (Q(1) << call))
				>> (ever - call);
			    if (slip) {
				bq_q_xls(deck, slip);
				deck ^= integral_q_last;
				if (1) {
				    integral_q *data, *text;

				    *KD(node) = news & ~(Q(1) << deck);
				    ever = (call - deck) >> 1;
				    data = KD(node) + call;
				    call--;
				    text = (integral_q *) PD(node) + call;
				    for (; ever; ever--) {
					*data = *(data - 2);
					data -= 2;
					*text = *(text - 2);
					text -= 2;
				    }
				    {
					*data = *(data - 1);
					*text = *(text - 1);
				    }
				}
			    } else {
				if (2) {
				    integral_q *data, *text;

				    *KD(node) = news & ~(Q(1) << ever);
				    deck = (ever - call) >> 1;
				    call--;
				    text = (integral_q *) PD(node) + call;
				    call++;
				    data = KD(node) + call;
				    for (; deck; deck--) {
					*text = *(text + 2);
					text += 2;
					*data = *(data + 2);
					data += 2;
				    }
				    {
					*text = *(text + 1);
					*data = *(data + 1);
				    }
				}
			    }
#else
			    bq_q_xls(deck, slip & ((Q(1) << call) - 1));
			    if (deck ^ integral_q_bits) {
				deck ^= integral_q_last;
				if (call - deck < ever - call) {
				    *KD(node) = news & ~(Q(1) << deck);
				    ever = (call - deck) >> 1;
				    for (; ever; ever--) {
					KD(node)[call] = KD(node)[call - 2];
					call--;
					PD(node)[call] = PD(node)[call - 2];
					call--;
				    }
				    {
					KD(node)[call] = KD(node)[call - 1];
					call--;
					PD(node)[call] = PD(node)[call - 1];
				    }
				} else {
				    *KD(node) = news & ~(Q(1) << ever);
				    deck = (ever - call) >> 1;
				    call--;
				    for (; deck; deck--) {
					PD(node)[call] = PD(node)[call + 2];
					call++;
					KD(node)[call] = KD(node)[call + 2];
					call++;
				    }
				    {
					PD(node)[call] = PD(node)[call + 1];
					call++;
					KD(node)[call] = KD(node)[call + 1];
				    }
				}
			    } else {
				if (2) {
				    *KD(node) = news & ~(Q(1) << ever);
				    deck = (ever - call) >> 1;
				    call--;
				    for (; deck; deck--) {
					PD(node)[call] = PD(node)[call + 2];
					call++;
					KD(node)[call] = KD(node)[call + 2];
					call++;
				    }
				    {
					PD(node)[call] = PD(node)[call + 1];
					call++;
					KD(node)[call] = KD(node)[call + 1];
				    }
				}
			    }
#endif				/* __HEAD_LEFT__ */
			} else {
			    bq_q_xls(deck, slip & ((Q(1) << call) - 1));
			    if (1) {
				deck ^= integral_q_last;
				if (1) {
				    integral_q *data, *text;

				    *KD(node) = news & ~(Q(1) << deck);
				    ever = (call - deck) >> 1;
				    data = KD(node) + call;
				    call--;
				    text = (integral_q *) PD(node) + call;
				    for (; ever; ever--) {
					*data = *(data - 2);
					data -= 2;
					*text = *(text - 2);
					text -= 2;
				    }
				    {
					*data = *(data - 1);
					*text = *(text - 1);
				    }
				}
			    }
			}
		    }
		}
	    }
	}
    }

    return 0;
}


int
_libx1f4l2_rail_bqpset(void *bqfset, void *node, integral_q *file,
		       unsigned call, unsigned copy, integral_q pipe,
		       integral_q last, integral_q next, integral_q *club,
		       integral_q *head, integral_q *past, integral_q *tier)
{
    int status;

    do {
	integral_q *fast;

	if (tier) {
	    if (club) {
	    } else {
		head = past;

		_libx1f4l2_four_bqpset
		    (bqfset, last, &club, pipe, &node, &call, &file, &past,
		     &tier, next, &copy);
		if (club) {
		} else {
		    status = 0;
		    if (1) {
			break;
		    }
		}
	    }
	} else {
	    if (1) {
		past = head;

		_libx1f4l2_nine_bqpset
		    (bqfset, last, &club, pipe, &node, &call, &head, &file,
		     &tier, next, &copy);
		if (tier) {
		} else {
		    status = 0;
		    if (1) {
			break;
		    }
		}
	    }
	}

	if (copy ^ half_integral_q_bits) {
	    unsigned deck;

	    fast = file;

	    if (copy) {
	    } else {
		copy = 2;

		*head = *(file + 2);
	    }

	    fast += copy;

	    if (copy < half_integral_q_bits) {
		deck = (half_integral_q_bits - copy) >> 1;

# if __UNROLL_TRANSFER__
#  error
# else
		for (; deck; deck--) {
		    *fast = *(fast + 2);
		    fast += 2;
		}
# endif				/* __UNROLL_TRANSFER__ */
	    } else {
		deck = (copy - half_integral_q_bits) >> 1;

# if __UNROLL_TRANSFER__
#  error
# else
		for (; deck; deck--) {
		    *fast = *(fast - 2);
		    fast -= 2;
		}
# endif				/* __UNROLL_TRANSFER__ */
	    }
	}

	{
	    integral_q *side;
	    unsigned deck;

	    fast = club;

	    *fast = Lxdddddddd ^ Q(1) << integral_q_last;

	    if (0) {
	    } else {
		fast += 4;

		side = fast - 1;

		*side = *fast;
		fast += 2;
		side++;
	    }

	    deck = (integral_q_bits - 6) / 6;
	    for (; deck; deck--) {
		*side = *fast;
		fast += 2;
		side += 2;
		*side = *fast;
		fast += 2;
		side++;
		*side = *fast;
		fast += 2;
		side++;
	    }

	    if (0) {
	    } else {
		if (half_integral_q_bits % 3 ^ 2) {
		    *side = *fast;

		    fast = file;

		    fast += 2;
		    side += 2;
		    *side = *head;
		    side++;
		    *side = *fast;
		    side++;
		} else {
		    *side = *fast;
		    fast += 2;
		    side += 2;
		    *side = *fast;

		    fast = file;

		    side++;
		    *side = *head;
		    side++;
		}
	    }

	    deck = integral_q_bits / 4 - integral_q_bits / 6 - 2;
	    for (; deck; deck--) {
		fast += 2;
		*side = *fast;
		side += 2;
		fast += 2;
		*side = *fast;
		side++;
		fast += 2;
		*side = *fast;
		side++;
	    }

	    {
		fast += 2;
		*side = *fast;
		side += 2;
		fast += 2;
		*side = *fast;
	    }

	    fast = tier;

	    fast += integral_q_bits;

	    side = fast;

	    deck = integral_q_bits / 6;
	    for (; deck; deck--) {
		side--;
		fast -= 2;
		*side = *fast;
		side--;
		fast -= 2;
		*side = *fast;
		side -= 2;
		fast -= 2;
		*side = *fast;
	    }

	    if (0) {
	    } else {
		if (half_integral_q_bits % 3 ^ 2) {
		    fast = file + integral_q_bits - 2;

		    side--;
		    *side = *past;
		    side--;
		    *side = *fast;
		    side -= 2;
		    fast -= 2;
		    *side = *fast;
		} else {
		    side--;
		    fast -= 2;
		    *side = *fast;

		    fast = file + integral_q_bits - 2;

		    side--;
		    *side = *past;
		    side -= 2;
		    *side = *fast;
		}
	    }

	    deck = integral_q_bits / 4 - integral_q_bits / 6 - 2;
	    for (; deck; deck--) {
		side--;
		fast -= 2;
		*side = *fast;
		side--;
		fast -= 2;
		*side = *fast;
		side -= 2;
		fast -= 2;
		*side = *fast;
	    }

	    {
		side--;
		fast -= 2;
		*side = *fast;
		side--;
		fast -= 2;
		*side = *fast;
	    }

	    *(side - 2) = Lxdddddddd;
	}

	if (call ^ integral_q_last) {
	    int excess;

	    *head = *(fast - 4);

	    status = edge(node, call);

	    excess = bqfset(bqfset)->link_m.free
		(bqfset(bqfset)->link_m.data, file);
	    if (excess) {
		status = FREE_ERROR;
	    }
	} else {
	    int excess;

	    *past = *(fast - 4);

	    status = east(node, call);

	    excess = bqfset(bqfset)->link_m.free
		(bqfset(bqfset)->link_m.data, file);
	    if (excess) {
		status = FREE_ERROR;
	    }
	}
    } while (0);

    return status;
}
