/*
 * bqpx.x.c
 * Copyright (C) 2018, 2021, Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <bqfx-bits.h>
#include <bqfx-defs.h>
#include <bqfx-lines.h>
#include <bqfx-names.h>
#include <bqfx-types.h>
#include <bqpx-types.h>

#define false(e)			0
#define true(e)				1

#undef Q
#define Q(e)				((integral_q) (e))

static int reel(integral_q *, unsigned, integral_q *, void *, void **);
static int roll(integral_q *, unsigned, integral_q *, void *, void **);

static int
reel(integral_q *club, unsigned deck, integral_q *head, void *mode,
     void **fare)
{
    int status = 0;
    integral_q *fast, *file, news, seek, *side;
    unsigned i, late, pick;
    void **data, **text;

    file = *fare;

    pick = (integral_q_bits - deck) >> 1;
    late = integral_q_bits - pick;
    seek = *club;
    i = integral_q_last;
    while (seek & ~Q(0) << late) {
	if (seek & Q(1) << i) {
	    seek ^= Q(1) << i;
	    late--;
	}
	i--;
    }

    i = late;

    news = *club;

    fast = club + i;
    side = fast;
    data = PD(club) + i;
    text = data;
    if (true(i & 1)) {
	if (false(news & Q(1) << i)) {
	} else {
	    side++;
	    *(data - 1) = *text;
	    text++;
	}

	i++;
    }
    seek = news >> i;
    for (; seek; seek >>= 2) {
	if (seek & Q(2)) {
	    *fast++ = *side++;
	    *fast++ = *side++;
	    *data++ = *text++;
	    *data++ = *text++;
	} else {
	    *fast++ = *side;
	    side += 2;
	    *data++ = *(text + 1);
	    text += 2;
	}
    }

    text = PD(file);

    *fast++ = *head;
    *data++ = *text++;

    side = file + 1;

    i = pick - 1;
    for (; i; i--) {
	*fast++ = *side++;
	*data++ = *text++;
    }

    *head = *side;

    if ((integral_q) mode < *head) {
	*fare = club;
    } else {
	if ((integral_q) mode ^ *head) {
	} else {
	    status = EVER_MATCH;
	}
    }

    *club = news | (~Q(0) << late);

    data = PD(file) + 1;
    fast = file;
    do {
	fast += 2;
	*fast = *++side;
	*data = *text++;
	data += 2;
    } while (fast != side);

    *file = ~Q(0) << (pick << 1) | Lx55555555;

    return status;
}


static int
roll(integral_q *tier, unsigned ever, integral_q *past, void *mode,
     void **fare)
{
    int status = 0;
    integral_q *fast, *file, news, seek, *side;
    unsigned i, late, pick;
    void **data, **text;

    file = *fare;

    pick = (integral_q_bits - ever) >> 1;
    late = pick;
    seek = *tier & ~1;
    while (seek & ((Q(2) << late) - 1)) {
	seek &= seek - 1;
	late++;
    }

    i = late;

    news = *tier;

    fast = tier + i + 1;
    side = fast;
    data = PD(tier) + i + 1;
    text = data;
    seek = (news & ~Q(3)) << (integral_q_last - i);
    for (; seek; seek <<= 2) {
	if (seek & Q(1) << integral_q_last) {
	    *--fast = *--side;
	    *--fast = *--side;
	    *--data = *--text;
	    *--data = *--text;
	} else {
	    side -= 2;
	    *--fast = *side;
	    *--data = *(text - 1);
	    text -= 2;
	}
    }
    {
	if (news & 2) {
	    *--fast = *--side;
	    *--data = *--text;
	    *--data = *--text;
	} else {
	    *--data = *(text - 1);
	}
    }

    text = PD(file) + integral_q_bits;

    *--fast = *past;
    *--data = *--text;

    side = file + integral_q_bits;

    i = pick - 1;
    for (; i; i--) {
	*--fast = *--side;
	*--data = *--text;
    }

    *past = *--side;

    if ((integral_q) mode < *past) {
    } else {
	if ((integral_q) mode ^ *past) {
	    *fare = tier;
	} else {
	    status = EVER_MATCH;
	}
    }

    *tier = news | ((Q(2) << late) - 1);

    data = PD(file) + integral_q_last;
    fast = file + integral_q_bits - 0;
    do {
	fast -= 2;
	*fast = *--side;
	*data = *--text;
	data -= 2;
    } while (fast != side);

    *file = ~Q(0) >> (pick << 1) | Lx55555555;

    return status;
}


int
_libx1f4l2_over_bqpset(integral_q *club, integral_q *head, integral_q *past,
		       integral_q *tier, void *mode, void **fare)
{
    int delete;
    integral_d last, next;
    unsigned deck, ever;

    if (club) {
	last = *KD(club);
	bq_q_bitcount(deck, last);
    } else {
	deck = integral_q_bits;
	last = 0;
    }
    if (tier) {
	next = *KD(tier);
	bq_q_bitcount(ever, next);
    } else {
	ever = integral_q_bits;
	next = 0;
    }

    /*
     * cannot transfer to destination capacity, lest destination is
     * picked and needs to be split.
     */

    if ((deck | ever) & ~integral_q_bits) {
	if (deck < ever) {
	    if (deck < integral_q_bits - 3) {
		delete = reel(club, deck, head, mode, fare);
	    } else {
		delete = -1;
	    }
	} else {
	    if (ever < integral_q_bits - 3) {
		delete = roll(tier, ever, past, mode, fare);
	    } else {
		delete = -1;
	    }
	}
    } else {
	delete = -1;
    }

    return delete;
}
