/*
 * c1.7.c
 * Copyright (C) 2006-2021, Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

/*
 * TODO
 *
 *.remove unused variables (_x1f4_c1_note_miss would be used to display some
 * information)
 * -- variables removed, in two places.
 *.there seem to be a lot of useless checks for zero byte around, remove maybe
 *.see whether do { ...; break; ... } while (0) may be improved.  If code is
 * inserted for the _while (0)_ or unlikely for the _do_, the code may be
 * missed.
 */
/*
 * FEATURES
 *
 *.second _c1_node_type_ record in double allocations is left with its _state_
 * field unset
 *.when copying the variable tree the memory manager of the source (tree) is
 * used.  the parsing memory manager should be used instead.
 *.the (near_tree) look and replace or insert sequence should be performed in a
 * single mx/s5 operation - same for a1
 */

#include <config.h>

#include <c1-config.h>

#include <stddef.h>
#include <stdlib.h>
#include <string.h>

#include <akeylook-l.h>
#include <c1-defs.h>
#include <c1-inter.h>
#include <c1-parse.h>
#include <c1-types.h>
#include <cardinal.p-d.h>
#include <ckeyhead.h>
#include <ctype.0.h>
#include <e4-m.0.h>
#include <e4.h>
#include <e4fine.h>
#include <e4jack.h>
#include <mxdeck.h>
#include <progress.h>
#include <trans.h>

#define __MIXED_TYPES__			1

#define __DEBUG_MISS__			0
#define __DEBUG_RACK__			0

#if 0
# define __D_DEBUG__			0
#elif __DEBUG_MISS__
# define __D_DEBUG__			1
#elif __DEBUG_RACK__
# define __D_DEBUG__			1
#else
# define __D_DEBUG__			0
#endif				/* 0 */

#define CASE_BILL			1
#define CASE_MODE			2
#define CASE_REAL			3
#define CASE_TEXT			4

#define CASE_VOID			5

#define HALF_CASE			6

#define CASE_LOOP			6
#define CASE_POST			7
#define CASE_RACK			8
#define CASE_TEST			9
#define CASE_WARD			10

#define CASE_TURN			11

#define BACK_CASE			12
#define HEAD_CASE			13

#define TYPE_CASE			14

#define LAST_CASE			15

#define ELSE_CASE			16
#define FOLD_CASE			17
#define ROLL_CASE			18

#define true(e)				1

#define byte(miss)			((unsigned char *) (miss))

#define PROGRAM(program) \
    ((c1_program_type *) program)

#define PROGRESS_MODE(progress, ___m) \
    (((progress)->bits & (___m)) == (___m))

#if __D_DEBUG__
# define dDebug_head(type, miss) \
    dDebug_3("/*--%s%.*s\n", (type), 60 - (miss), \
	     "--------------------------------------------------------------" \
	     "--")
#endif				/* __D_DEBUG__ */

#if __D_DEBUG__
# define dDebug_tail() \
    dDebug_0("  " "------------------------------------*/\n")
#endif				/* __D_DEBUG__ */

#if __D_DEBUG__
# include <stdio.h>
#endif				/* __D_DEBUG__ */

#if __D_DEBUG__
# define dDebug_0(format) \
    fprintf(stderr, format)
# define dDebug_1(format, a) \
    fprintf(stderr, format, a)
# define dDebug_2(format, a, b) \
    fprintf(stderr, format, a, b)
# define dDebug_3(format, a, b, c) \
    fprintf(stderr, format, a, b, c)
#endif				/* __D_DEBUG__ */

#define E_FREE(mcontext, mdata) \
    (PROGRAM(mcontext)->w.free(PROGRAM(mcontext)->w.data, (mdata)))
#define E_LINK(mcontext, mdata, size) \
    (PROGRAM(mcontext)->w.link						      \
	(PROGRAM(mcontext)->w.data, (void *) (mdata), (size)))
#define E_MODE(mcontext, mdata, size) \
    (PROGRAM(mcontext)->w.mode						      \
	(PROGRAM(mcontext)->w.data, (void *) (mdata), (size)))

#define M_FREE(mcontext, mdata) \
    (mcontext)->m.free((mcontext)->m.data, (mdata))
#define M_LINK(mcontext, mdata, size) \
    (mcontext)->m.link((mcontext)->m.data, (void *) (mdata), (size))
#define M_MODE(mcontext, mdata, size) \
    (mcontext)->m.mode((mcontext)->m.data, (void *) (mdata), (size))

#define line_copy(parser_data) \
    (((parser_data)->program - (parser_data)->thirteen) & LINE_COPY)
#define rule_copy(parser_data, ___l) \
    (((___l) - (parser_data)->thirteen) & LINE_COPY)

typedef struct half_type {
    struct trans_type *trans_data;
    void *variable_slip;
} half_type;

extern void _x1f4_e4_bcxx_hack();

extern const struct x1f4_akeytree_type _x1f4_c1_text_root[];

extern const struct x1f4_datatype_type _x1f4_af_last_type, _x1f4_c1_last_type;

extern const unsigned _x1f4_c_type_0[], _x1f4_c_type_1[];

static int copy_ceil(struct parser_type *, struct c1_slip_type *,
		     struct c1_slip_type *);
static int copy_miss(struct parser_type *);
static int copy_nets(struct parser_type *);
static int copy_post(struct parser_type *);
static int copy_raCk(struct parser_type *, unsigned *);
static int copy_rack(struct parser_type *, unsigned *);
static int copy_slip(struct parser_type *, unsigned *);
static int copy_ways(struct parser_type *);
static int ever_line(unsigned *, struct parser_type *);
static int ever_type(const char **, const void **,
		     const struct c1_type_type *);
static int fail_miss(struct parser_type *);
static int fail_slip(struct parser_type *, struct c1_program_type *);
static int fare_aiLe(struct parser_type *);
static int fare_aiMe(struct parser_type *, int);
static int fare_aile(struct parser_type *);
static int fare_aime(struct parser_type *, int, const char *);
static int fare_back(struct parser_type *);
static int fare_bill(struct parser_type *);
static int fare_case(struct parser_type *, int);
static int fare_cell(struct parser_type *);
static int fare_data(struct parser_type *, const char *, unsigned, int);
static int fare_fast(struct parser_type *, struct c1_node_type *);
static int fare_fold(struct parser_type *, int *, void **);
static int fare_half(struct parser_type *, struct c1_hook_type *, const char *);
static int fare_heAd(struct parser_type *);
static int fare_head(struct parser_type *);
static int fare_laSt(struct parser_type *);
static int fare_last(struct parser_type *);
static int fare_loop(struct parser_type *);
static int fare_mode(struct parser_type *);
static int fare_node(struct parser_type *, void *);
static int fare_post(struct parser_type *);
static int fare_rack(struct parser_type *);
static int fare_real(struct parser_type *);
static int fare_side(struct parser_type *);
static int fare_test(struct parser_type *);
static int fare_text(struct parser_type *);
static int fare_turn(struct parser_type *);
static int fare_type(struct parser_type *, int, int);
static int fare_void(struct parser_type *);
static int fare_ward(struct parser_type *);
static int hold_slip(struct parser_type *, const char *, unsigned,
		     const char *, unsigned, int *);
static int hold_such(struct parser_type *, const char *, unsigned, int);
static int iran_back(struct parser_type *);
static int iran_byte(struct parser_type *);
static int iran_data(struct parser_type *);
static int iran_fine(struct parser_type *);
static int iran_node(struct parser_type *, struct c1_node_type **, unsigned);
static int iran_turn(struct parser_type *);
static int keep_line(struct parser_type *);
static int land_part(struct parser_type *, const struct x1f4_progress_type *,
		     const char *, unsigned, const char *, unsigned,
		     const char *, unsigned, const char *, int);
static int land_pass(struct parser_type *, const struct x1f4_progress_type *,
		     const char *, unsigned, const char *, unsigned,
		     const char *, unsigned, const char *, int, int,
		     const char *);
static int lock_slip(struct parser_type *, void *, const char *, unsigned, int,
		     int *);
static int long_back(struct parser_type *);
static int lose_slip(struct parser_type *, struct c1_slip_type *);
static int mean_data(struct parser_type *);
static int mean_zero(struct parser_type *, unsigned,
		     const struct c1_context_type *);
static int near_data(struct parser_type *);
static int near_tree(void *, const void *, const char *, unsigned);
static int part_data(struct parser_type *);
static int post_clip(struct parser_type *, struct c1_hook_type *);
static int post_data(struct parser_type *);
static int post_ever(struct parser_type *, struct c1_hook_type *);
static int post_hook(struct parser_type *, struct c1_hook_type *);
static int post_loop(struct parser_type *, struct c1_hook_type *);
static int post_none(struct parser_type *, struct c1_hook_type *);
static int post_post(struct parser_type *, struct c1_hook_type *);
static int post_test(struct parser_type *, struct c1_hook_type *);
static int post_ward(struct parser_type *, struct c1_hook_type *);
static int rend_data(struct parser_type *, void *, int, char **, unsigned *,
		     int);
static int trim_slip(struct c1_slip_type *, struct c1_slip_type *);
static int type_pass(struct parser_type *, const struct x1f4_fragment_type *,
		     const char *, unsigned, const char *, unsigned,
		     const char *, unsigned, int, char **);

static void copy_fix(struct parser_type *);
static void copy_mae(struct c1_node_type *, int (*)(void *, void *), void *);
static void copy_set(struct c1_miss_type *, unsigned, int (*)(void *, void *),
		     void *);
static void fare_day(struct parser_type *, struct c1_slip_type *);
static void fare_set(struct parser_type *);
static void near_red(struct parser_type *);
static void post_fix(struct parser_type *, unsigned);
static void wind_far(const char **);
static void wind_new(const char **);
static void wind_red(const char **);

static const char end_class[] = "      ({;((";
static int (*const set_class[])(struct parser_type *) = {
/* *INDENT-OFF* */
    NULL,
    fare_bill,
    fare_mode,
    fare_real,
    fare_text,
    fare_void,
    fare_loop,
    fare_post,
    fare_rack,
    fare_test,
    fare_ward,
    fare_turn,
    fare_back,
    fare_heAd,
    fare_cell,
    fare_laSt
/* *INDENT-ON* */
};
#if __SHORT_NAMES__
static const struct x1f4_ckeytree_type line_4[] = {
/* *INDENT-OFF* */
    {	"",		{	CASE_MODE	},	NULL	},
    {	"eger",		{	CASE_MODE	},	NULL	},
    {	NULL,		{	0		},	NULL	}
/* *INDENT-ON* */
}, line_3[] = {
/* *INDENT-OFF* */
    {	"",		{	CASE_BILL	},	NULL	},
    {	"inal",		{	CASE_BILL	},	NULL	},
    {	NULL,		{	0		},	NULL	}
/* *INDENT-ON* */
};
#endif				/* __SHORT_NAMES__ */
static const struct x1f4_ckeytree_type line_2[] = {
/* *INDENT-OFF* */
    {	"f",		{	CASE_TEST	},	NULL	},
#if __SHORT_NAMES__
    {	"nt",		{	0		},	line_4	},
#else
    {	"nteger",	{	CASE_MODE	},	NULL	},
#endif				/* __SHORT_NAMES__ */
    {	NULL,		{	0		},	NULL	}
/* *INDENT-ON* */
}, line_1[] = {
/* *INDENT-OFF* */
    {	"al",		{	CASE_REAL	},	NULL	},
    {	"turn",		{	CASE_TURN	},	NULL	},
    {	NULL,		{	0		},	NULL	}
/* *INDENT-ON* */
}, line_0[] = {
/* *INDENT-OFF* */
    {	"break",	{	CASE_RACK	},	NULL	},
#if __SHORT_NAMES__
    {	"card",		{	0		},	line_3	},
#else
    {	"cardinal",	{	CASE_BILL	},	NULL	},
#endif				/* __SHORT_NAMES__ */
    {	"do",		{	CASE_POST	},	NULL	},
    {	"for",		{	CASE_WARD	},	NULL	},
    {	"i",		{	0		},	line_2	},
    {	"re",		{	0		},	line_1	},
    {	"text",		{	CASE_TEXT	},	NULL	},
    {	"void",		{	CASE_VOID	},	NULL	},
    {	"while",	{	CASE_LOOP	},	NULL	},
    {	NULL,		{	0		},	NULL	}
/* *INDENT-ON* */
}, line_root[] = {
/* *INDENT-OFF* */
    {	"",		{	0		},	line_0	},
    {	NULL,		{	0		},	NULL	}
/* *INDENT-ON* */
}, lock_0[] = {
/* *INDENT-OFF* */
    {	"break",	{	CASE_RACK	},	NULL	},
    {	"do",		{	CASE_POST	},	NULL	},
    {	"for",		{	CASE_WARD	},	NULL	},
    {	"if",		{	CASE_TEST	},	NULL	},
    {	"return",	{	CASE_TURN	},	NULL	},
    {	"while",	{	CASE_LOOP	},	NULL	},
    {	NULL,		{	0		},	NULL	}
/* *INDENT-ON* */
}, lock_root[] = {
/* *INDENT-OFF* */
    {	"",		{	0		},	lock_0	},
    {	NULL,		{	0		},	NULL	}
/* *INDENT-ON* */
};
static const unsigned c_type_4[] = {
/*
 *                   . .     .
 *
 * .
 *
 *
 *
 *
 *
 *
 *
 *
 *
 *
 *
 *
 *
 */
/* *INDENT-OFF* */
    0x00002600, 0x00000001, 0x00000000, 0x00000000,
    0x00000000, 0x00000000, 0x00000000, 0x00000000
/* *INDENT-ON* */
};

static int
copy_ceil(struct parser_type *parser_data, struct c1_slip_type *last_slip,
	  struct c1_slip_type *slip_data)
{
    int status = 0;
    struct c1_miss_type **last = NULL, *last_miss = NULL, *miss_data;
    unsigned count, cover = 0, miss;

    count = slip_data->count;
    if (last_slip) {
	count -= last_slip->count;
    }

    miss_data = slip_data->miss_data;
    for (; count; count--) {
	if (((struct x1f4_variable_type *) miss_data->data)->flags
	    & X1f4_E4_ANCILLARY) {
	    struct c1_miss_type *free_miss;

	    free_miss = miss_data;
	    miss_data = miss_data->last_miss;
	    M_FREE(parser_data, free_miss);
	    if (last_miss) {
		last_miss->last_miss = miss_data;
	    } else {
		slip_data->miss_data = miss_data;
		if (last) {
		} else {
		    struct c1_miss_type *long_miss;

		    long_miss = parser_data->miss_data;
		    if (free_miss == long_miss) {
			last = &parser_data->miss_data;
		    } else {
			struct c1_miss_type *post_miss;

			post_miss = long_miss->last_miss;
			while (post_miss != free_miss) {
			    long_miss = post_miss;
			    post_miss = long_miss->last_miss;
			}
			last = &long_miss->last_miss;
		    }
		}

		*last = miss_data;
	    }

	    cover++;
	} else {
	    last_miss = miss_data;
	    miss_data = miss_data->last_miss;
	}
    }

    if (cover) {
	slip_data->count -= cover;
	parser_data->count -= cover;

	trim_slip(slip_data, last_slip);
    }

    do {
	struct c1_miss_type *miss_data, *miss_slip;
	unsigned i;

	miss = slip_data->count;

	if (!last_slip) {
	    if (miss) {
	    } else {
		break;
	    }

	    if (parser_data->class & 1) {
		status = _x1f4_c1_rule_lift(parser_data, miss, slip_data);
		if (status) {
		    break;
		}
	    }
	    if (parser_data->class & 2) {
		status = _x1f4_c1_rule_miss(parser_data, miss, slip_data, 0);
		if (status) {
		    break;
		}
	    }
	} else {
	    miss -= last_slip->count;

	    if (miss) {
	    } else {
		break;
	    }

	    if (parser_data->class & 1) {
		status = _x1f4_c1_rule_lift(parser_data, miss, slip_data);
		if (status) {
		    break;
		}
	    }
	    if (parser_data->class & 2) {
		status = _x1f4_c1_rule_miss(parser_data, miss, slip_data, 0);
		if (status) {
		    break;
		}
	    }
	}

	miss_data = slip_data->miss_data;

	i = miss;
	i--;
	for (; i; i--) {
	    miss_data = miss_data->last_miss;
	}
	miss_slip = miss_data->last_miss;
	miss_data->last_miss = parser_data->miss_text;

	parser_data->miss_data = miss_slip;
	parser_data->miss_text = slip_data->miss_data;

	parser_data->count -= miss;

	if (1) {
	    if (0) {
	    } else {
		parser_data->trail = parser_data->count;
	    }
	}
    } while (0);

    return status;
}


static int
copy_miss(struct parser_type *parser_data)
{
    int status = 0;
    struct c1_slip_type *last_slip;

    last_slip = parser_data->slip_data;
    if (!last_slip) {
#if 1
	post_fix(parser_data, parser_data->count);
#else
	if (parser_data->class & 2) {
	    status = _x1f4_c1_rule_miss
		(parser_data, parser_data->count, );
	}
#endif				/* 1 */
    } else {
	unsigned count, trail;

	trail = parser_data->trail;
	count = last_slip->count;
	if (trail < count) {
	    trail = count;
	}
	count = parser_data->count - trail;
#if 1
	post_fix(parser_data, count);
#else
	if (parser_data->class & 2) {
	    status = _x1f4_c1_rule_miss
		(parser_data, count, );
	    if (status) {
	    } else {
		parser_data->trail = parser_data->count;
	    }
	} else {
	    if (1) {
		parser_data->trail = parser_data->count;
	    }
	}
#endif				/* 1 */
    }

    return status;
}


static int
copy_nets(struct parser_type *parser_data)
{
    int status;
    struct c1_slip_type *last_slip;

    last_slip = parser_data->slip_data;
    if (!last_slip) {
	status = 0;
    } else {
	if (last_slip->level != parser_data->level) {
	    status = 0;
	} else {
	    struct c1_slip_type *slip_data;

	    slip_data = last_slip->last_slip;

	    parser_data->slip_data = slip_data;

	    x1f4_fini_mxdeck(&last_slip->data);

	    status = copy_ceil(parser_data, slip_data, last_slip);

	    _x1f4_c1_land_hook(parser_data, last_slip);

	    lose_slip(parser_data, last_slip);
	}
    }

    return status;
}


static int
copy_post(struct parser_type *parser_data)
{
    void *v1st, *v2nd;

    v1st = parser_data->v1st;
    v2nd = parser_data->v2nd;

    if (v1st) {
	M_FREE(parser_data, v1st);
    }
    if (v2nd) {
	M_FREE(parser_data, v2nd);
    }

    return 0;
}


static int
copy_raCk(struct parser_type *parser_data, unsigned *shift)
{
    int status;

    if (parser_data->class & 2) {
	status = copy_rack(parser_data, shift);
    } else {
	status = copy_slip(parser_data, shift);
    }

    return status;
}


static int
copy_rack(struct parser_type *parser_data, unsigned *shift)
{
    int status = 0;
    struct c1_hook_type *hook_data;

#if __DEBUG_RACK__
    dDebug_head("break", 5);
    dDebug_1("    level: %2u\n", parser_data->level);
    if (1) {
	struct c1_hook_type *hook_data;
	struct c1_slip_type *slip_data;
	unsigned level = -1;

	hook_data = parser_data->hook_data;
	while (hook_data) {
	    dDebug_1("    level: %2u ", hook_data->level);
	    if (hook_data->class == CASE_LOOP) {
		dDebug_0("LOOP");
	    } else if (hook_data->class == CASE_POST) {
		dDebug_0("POST");
	    } else if (hook_data->class == CASE_TEST) {
		dDebug_0("TEST");
	    } else if (hook_data->class == CASE_WARD) {
		dDebug_0("WARD");
	    } else if (hook_data->class == ELSE_CASE) {
		dDebug_0("ELSE");
	    } else if (hook_data->class == FOLD_CASE) {
		dDebug_0("FOLD");
	    } else if (hook_data->class == ROLL_CASE) {
		dDebug_0("ROLL");
	    } else {
		dDebug_0("?");
	    }
	    dDebug_0("\n");
	    if (level == -1) {
		if (0) {
		} else if (hook_data->class == CASE_LOOP) {
		    level = hook_data->level;
		} else if (hook_data->class == CASE_POST) {
		    level = hook_data->level;
		} else if (hook_data->class == CASE_WARD) {
		    level = hook_data->level;
		}
	    }

	    hook_data = hook_data->hook_data;
	}

	slip_data = parser_data->slip_data;
	while (slip_data) {
	    unsigned count;

	    dDebug_1("    level: %2u", slip_data->level);
	    count = slip_data->count;
	    if (slip_data->last_slip) {
		count -= slip_data->last_slip->count;
	    }
	    dDebug_1(" %2u nodes", count);
	    dDebug_0("\n");
	    if (count) {
		if (level < slip_data->level) {
		    struct c1_miss_type *miss_data;

		    miss_data = slip_data->miss_data;
		    while (count) {
			count--;
			dDebug_0("\t");
			dDebug_0(((struct x1f4_variable_type *)
				  miss_data->data)->name);
			dDebug_0("\n");
			miss_data = miss_data->last_miss;
		    }
		}
	    }

	    slip_data = slip_data->last_slip;
	}
    }
    dDebug_tail();
#endif				/* __DEBUG_RACK__ */

    hook_data = parser_data->hook_data;
    while (true(hook_data)) {
	int class;

	class = hook_data->class;
	if (class == CASE_LOOP || class == CASE_POST || class == CASE_WARD) {
	    struct c1_slip_type *slip_data;
	    unsigned level;

	    level = hook_data->level;

	    *shift = level;

	    slip_data = parser_data->slip_data;
	    while (slip_data) {
		if (level < slip_data->level) {
		    unsigned count;

		    count = slip_data->count;
		    if (slip_data->last_slip) {
			count -= slip_data->last_slip->count;
		    }

		    status = _x1f4_c1_rule_miss
			(parser_data, count, slip_data, 1);
		    if (status) {
			break;
		    } else {
			slip_data = slip_data->last_slip;
		    }
		} else {
		    break;
		}
	    }

	    break;
	} else {
	    hook_data = hook_data->hook_data;
	}
    }

    return status;
}


static int
copy_slip(struct parser_type *parser_data, unsigned *shift)
{
    struct c1_hook_type *hook_data;

    hook_data = parser_data->hook_data;
    while (1) {
	if (hook_data->class == CASE_LOOP
	    || hook_data->class == CASE_POST) {
	    *shift = hook_data->level;

	    break;
	} else {
	    hook_data = hook_data->hook_data;
	}
    }

    return 0;
}


static int
copy_ways(struct parser_type *parser_data)
{
    int status;
    void *variable_tree;

    variable_tree = parser_data->variable_tree;
    if (!variable_tree) {
	status = 0;
    } else {
	x1f4_fini_mxdeck(&variable_tree);

	parser_data->variable_tree = NULL;

	status = copy_miss(parser_data);
    }

    return status;
}


static int
ever_line(unsigned *line, struct parser_type *parser_data)
{
    const char *e;
    int c, status;

    e = parser_data->program;

    status = 0;

    parser_data->slip(&e);

    c = *byte(e);
    if (c) {
	if (c == 0x7b) {
	    *line = HEAD_CASE;
	} else {
	    if (c == 0x7d) {
		*line = BACK_CASE;
	    } else {
		const char *f;

		f = e;
		if (x1f4_head_ctkey(&e, parser_data->head_line, line)) {
		    const struct c1_type_type *side_data, *type_data;

		    /*
		     * TODO
		     *
		     * fix this horror, casing context
		     * (declaration/instruction) via _head_line_
		     */
		    side_data = parser_data->side_data;
		    type_data = parser_data->type_data;
		    if ((side_data || type_data)
			&& parser_data->head_line == line_root) {
			int delete, side;
			const void *cell;

			if (type_data) {
			    delete = ever_type(&e, &cell, type_data);
			    if (delete) {
				if (side_data) {
				    delete = ever_type(&e, &cell, side_data);
				    if (delete) {
				    } else {
					side = 1;
				    }
				}
			    } else {
				side = 0;
			    }
			} else {
			    delete = ever_type(&e, &cell, side_data);
			    if (delete) {
			    } else {
				side = 1;
			    }
			}
			if (delete) {
			    *line = LAST_CASE;
			} else {
			    c = *byte(e);
			    if (c) {
				if (C_TYPE_X(parser_data->e_type_4, c)) {
				    *line = TYPE_CASE;
				    parser_data->side = side;
				    parser_data->cell_type = cell;
				} else {
				    do {
					if (c == '(') {
					    const char *g;

					    g = e + 1;
					    parser_data->slip(&g);
					    if (*g == '*') {
						*line = TYPE_CASE;
						parser_data->side = side;
						parser_data->cell_type = cell;
						break;
					    }
					}

					e = f;
					*line = LAST_CASE;
				    } while (0);
				}
			    } else {
				status = _x1f4_c1_080707_error_2
				    (parser_data, e);
			    }
			}
		    } else {
			*line = LAST_CASE;
		    }
		} else {
		    c = *byte(e);
		    if (c) {
			do {
			    if (C_TYPE_X(parser_data->e_type_4, c)) {
				parser_data->slip(&e);
				c = *byte(e);
				if (C_TYPE_X(_x1f4_c_type_1, c)) {
				    /*
				     * check for (non pointer) void case?
				     */

				    break;
				}
			    }
			    {
				unsigned this;

				this = *line;
				if (this < HALF_CASE) {
				    if (c == '(') {
					const char *g;

					g = e + 1;
					parser_data->slip(&g);
					if (*g == '*') {
					    break;
					}
				    }

				    /*
				     * FEATURES
				     *
				     * this one allows functions with intrinsic
				     * type names, but just as well all sort of
				     * stupid things.
				     */
				    e = f;
				    *line = LAST_CASE;
				} else {
				    if (this == CASE_TURN) {
				    } else {
					if (c != end_class[this]) {
					    /*
					     * FEATURES
					     *
					     * many stupid things are
					     * allowed by this one, hopefully
					     * not too many.
					     */
					    e = f;
					    *line = LAST_CASE;
					}
				    }
				}
			    }
			} while (0);
		    } else {
			status = _x1f4_c1_080707_error_2(parser_data, e);
		    }
		}
	    }
	}
    } else {
	*line = 0;
    }

    parser_data->program = e;

    return status;
}


static int
ever_type(const char **deck, const void **cell,
	  const struct c1_type_type *type_data)
{
    const char *e, *name;
    int false = 1;

    e = *deck;

    name = type_data->name;
    while (name) {
	const char *miss;
	unsigned size;

	miss = e;

	size = type_data->size;
	while (size) {
	    if (*miss != *name) {
		break;
	    } else {
		miss++;
		name++;
		size--;
	    }
	}
	if (size) {
	} else {
	    if (C_TYPE_X(_x1f4_c_type_0, *miss)) {
	    } else {
		false = 0;

		*deck = miss;
		*cell = type_data;

		break;
	    }
	}

	type_data++;

	name = type_data->name;
    }

    return false;
}


static int
fail_miss(struct parser_type *parser_data)
{
    int status;
    struct c1_slip_type *slip_data;

    slip_data = parser_data->slip_data;
    if (!slip_data) {
	status = 0;
    } else {
	do {
	    struct c1_slip_type *slip_free;

	    slip_free = slip_data;
	    slip_data = slip_data->last_slip;

	    {
		void *data;

		data = (void *) slip_free->data;
		if (data != parser_data->variable_deck) {
		    x1f4_fini_mxdeck(&data);
		}
	    }

	    status = copy_ceil(parser_data, slip_data, slip_free);

	    _x1f4_c1_land_hook(parser_data, slip_free);

	    lose_slip(parser_data, slip_free);

	    if (!slip_data) {
		parser_data->slip_data = NULL;
		break;
	    } else {
		if (status) {
		    parser_data->slip_data = slip_data;
		    break;
		}
	    }
	} while (1);
    }

    return status;
}


static int
fail_slip(struct parser_type *parser_data,
	  struct c1_program_type *program_data)
{
    int status;

    do {
	if (parser_data->flags & LEAD_DATATEXT) {
	    int type;
	    const struct c1_context_type *context_data;

	    type = parser_data->shift;

	    context_data = parser_data->context_data;

	    if (type < X1f4_E4_LAST) {
		if (type == X1f4_E4_TEXT) {
		    program_data->u.data = context_data->near_f.data;
		    program_data->u.near = _x1f4_c1_post_text;
		    program_data->v.c.link = context_data->near_f.link;

		    break;
		}
	    } else {
		const struct c1_type_type *type_data;

		if (X1f4_E4_CALL < type) {
		    program_data->u.data = context_data->near_f.data;
		    program_data->u.near = _x1f4_c1_post_call;
		    program_data->v.f.pick = context_data->near_f.pick;

		    break;
		}

		type_data = parser_data->type_data;
		if (type_data) {
		    const char *name;

		    name = type_data->name;
		    while (name) {
			if (type_data->type == type) {
			    break;
			}

			type_data++;

			name = type_data->name;
		    }

		    if (name) {
			if (type_data->lead) {
			    program_data->u.data = context_data->near_f.data;
			    program_data->u.near = _x1f4_c1_post_lead;
			    program_data->v.f.pick = context_data->near_f.pick;
			    program_data->v.f.text = type_data;

			    break;
			}
		    }
		}
	    }

	    if (parser_data->flags & TURNCAST_LINK) {
		if (type == X1f4_E4_CASE) {
		    const struct c1_context_type *context_data;

		    context_data = parser_data->context_data;

		    program_data->u.data = context_data->near_f.data;
		    program_data->u.near = _x1f4_c1_post_cast;
		    program_data->v.f.pick = context_data->near_f.pick;
		    program_data->v.f.text = context_data->link_g.text;

		    break;
		}
	    }
	}

	program_data->u.near = NULL;
    } while (0);

    if (program_data->u.near) {
	status = _x1f4_c1_fit_trans(parser_data);
    } else {
	status = 0;
    }

    return status;
}


static int
fare_aiLe(struct parser_type *parser_data)
{
    int status;

    status = keep_line(parser_data);
    if (status) {
    } else {
	status = fare_aile(parser_data);
    }

    return status;
}


static int
fare_aiMe(struct parser_type *parser_data, int class)
{
    int status;

    status = keep_line(parser_data);
    if (status) {
    } else {
	status = fare_aime(parser_data, class, NULL);
    }

    return status;
}


static int
fare_aile(struct parser_type *parser_data)
{
    int status;

    {
	do {
	    {
		{
		    struct c1_hook_type *hook_data;

		    status = M_LINK
			(parser_data, &hook_data, sizeof(struct c1_hook_type));
		    if (status) {
			status = ALLOC_ERROR;

			break;
		    } else {
			hook_data->hook_data = parser_data->hook_data;

			parser_data->hook_data = hook_data;

			hook_data->class = CASE_POST;
			hook_data->level = parser_data->level;
			hook_data->node_data = parser_data->tail_node;
		    }
		}
	    }

	    parser_data->level++;

	    parser_data->rack++;

	    parser_data->head_line = line_root;

	    parser_data->program = (const char *) parser_data->program + 1;
	} while (0);
    }

    return status;
}


static int
fare_aime(struct parser_type *parser_data, int class, const char *g)
{
    const char *e;
    int status;
    struct x1f4_e4_type *e4_data;
    void *expression;

    e = parser_data->program;

    e4_data = &parser_data->e4;

    e4_data->terminator = 0;

    status = x1f4_inIt_expression
	(&expression, &e, X1f4_E4_SLIPTEST | parser_data->e4_flags,
	 e4_data);

    if (g) {
    } else {
	g = e;
    }

    if (status) {
	if (status == X1f4_E4_ALLOC_ERROR) {
	    status = ALLOC_ERROR;
	} else {
	    status = _x1f4_c1_080707_error_0(parser_data, g);
	}
    } else {
	do {
	    /*
	     * optimizing may change type
	     */
	    if (parser_data->flags & LINE_OPTIMIZE) {
		fare_node(parser_data, expression);
	    }

	    {
		int type;

		type = x1f4_type_expression(expression);
		if (type == X1f4_E4_BILL || type == X1f4_E4_CASE
		    || type == X1f4_E4_MODE || type == X1f4_E4_REAL) {
		} else {
		    if (type != X1f4_E4_VOID) {
			status = _x1f4_c1_091025_error_0(parser_data, g, type);
		    } else {
			status = _x1f4_c1_080707_error_1(parser_data, g);
		    }

		    if (expression) {
			x1f4_fini_expression(&expression);
		    }

		    break;
		}
	    }

	    if (true(*e == 0x29)) {
		e++;
	    } else {
		parser_data->slip(&e);

		if (*e == 0x29) {
		    e++;
		} else {
		    status = _x1f4_c1_080707_error_2(parser_data, e);
		    if (true(expression)) {
			x1f4_fini_expression(&expression);
		    }

		    break;
		}
	    }
	    {
		parser_data->slip(&e);

		if (*e == 0x7b) {
		    e++;
		} else {
		    status = _x1f4_c1_080707_error_3(parser_data, e);
		    if (true(expression)) {
			x1f4_fini_expression(&expression);
		    }

		    break;
		}
	    }

	    {
		struct c1_node_type *node_data;

		status = iran_node(parser_data, &node_data, 1);
		if (status) {
		    if (true(expression)) {
			x1f4_fini_expression(&expression);
		    }

		    break;
		} else {
		    struct c1_hook_type *hook_data;

		    status = M_LINK
			(parser_data, &hook_data, sizeof(struct c1_hook_type));
		    if (status) {
			status = ALLOC_ERROR;
			if (true(expression)) {
			    x1f4_fini_expression(&expression);
			}

			break;
		    } else {
			hook_data->hook_data = parser_data->hook_data;

			parser_data->hook_data = hook_data;

			hook_data->class = class;
			hook_data->level = parser_data->level;
			hook_data->node_data = node_data;

			node_data[0].state = line_copy(parser_data);

			node_data[1].data.data = expression;
		    }
		}
	    }

	    parser_data->level++;

	    parser_data->head_line = line_root;
	} while (0);
    }

    parser_data->program = e;

    return status;
}


static int
fare_back(struct parser_type *parser_data)
{
    int status;

    if (!parser_data->level) {
	status = long_back(parser_data);
    } else {
	do {
	    /*
	     * the _rule_miss processing should occur before hooks processing.
	     */

	    status = copy_nets(parser_data);
	    if (status) {
		break;
	    }

	    parser_data->level--;

	    status = copy_ways(parser_data);
	    if (status) {
		break;
	    }

	    {
		struct c1_hook_type *hook_data;

		hook_data = parser_data->hook_data;
		if (hook_data) {
		    if (hook_data->level != parser_data->level) {
			parser_data->head_line = lock_root;
		    } else {
			status = post_hook(parser_data, hook_data);
			if (status) {
			    break;
			}
		    }
		} else {
		    parser_data->head_line = lock_root;
		}
	    }

	    parser_data->program = (const char *) parser_data->program + 1;
	} while (0);
    }

    return status;
}


static int
fare_bill(struct parser_type *parser_data)
{
    return fare_type(parser_data, 0, X1f4_E4_BILL);
}


static int
fare_case(struct parser_type *parser_data, int class)
{
    const char *e;
    int status;

    e = parser_data->program;

    parser_data->slip(&e);

    parser_data->program = e;

    if (*e != 0x28) {
	status = _x1f4_c1_080707_error_5(parser_data, e);
    } else {
	parser_data->program = e + 1;

	status = fare_aiMe(parser_data, class);
    }

    return status;
}


static int
fare_cell(struct parser_type *parser_data)
{
    return fare_type
	(parser_data, parser_data->side, parser_data->cell_type->type);
}


static int
fare_data(struct parser_type *parser_data, const char *name, unsigned length,
	  int type)
{
    int status;
    struct c1_miss_type *miss_data;

    status = M_LINK
	(parser_data, &miss_data,
	 sizeof(struct c1_miss_type) + sizeof(struct x1f4_variable_type)
	 + length + 1);
    if (status) {
	status = ALLOC_ERROR;
    } else {
	struct x1f4_variable_type *variable_data;

	variable_data = (void *) (miss_data + 1);
	do {
	    char *data;
	    void *variable_tree;

	    data = (char *) (variable_data + 1);
	    memcpy(data, name, length);
	    data[length] = 0;

	    variable_tree = parser_data->variable_tree;

	    if (variable_tree) {
	    } else {
		struct x1f4_mxdeck_type mxdeck;

		mxdeck.trans = &parser_data->m;

		status = x1f4_init_mxdeck
		    (&variable_tree, X1f4_MXDECK_TRANS_MASK, &mxdeck);
		if (status) {
		    status = ALLOC_ERROR;
		    if (1) {
			break;
		    }
		}
	    }

	    status = x1f4_post_mxpath(variable_tree, data, length, miss_data);
	    if (status) {
		if (parser_data->variable_tree) {
		} else {
		    x1f4_fini_mxdeck(&variable_tree);
		}

		M_FREE(parser_data, miss_data);

		if (status != X1f4_MXDECK_EVER_MATCH) {
		    status = ALLOC_ERROR;
		} else {
		    status = _x1f4_c1_080707_error_6
			(parser_data, name, length);
		}
	    } else {
		const struct c1_type_type *cell_type = NULL;
		void *link;

		variable_data->length = length;
		variable_data->name = data;
		variable_data->type = type;

		variable_data->flags = X1f4_E4_ANCILLARY | X1f4_E4_UNIVERSAL;

		link = parser_data->base;

		miss_data->link = link;

		parser_data->base =
		    (char *) link + sizeof(struct c1_lead_type);

		if (type == X1f4_E4_TEXT) {
		    cell_type = &parser_data->program_data->t.text;
		} else {
		    if (type < X1f4_E4_LAST) {
		    } else {
			if (type < X1f4_E4_CALL) {
			    cell_type = parser_data->cell_type;
			} else {
			    cell_type = &_x1f4_af_last_type;
			}
		    }
		}

		miss_data->dash = parser_data->level;

		miss_data->data = variable_data;
		miss_data->type = cell_type;

		parser_data->variable_tree = variable_tree;

		parser_data->count++;

		miss_data->last_miss = parser_data->miss_data;
		parser_data->miss_data = miss_data;
	    }
	} while (0);
    }

    return status;
}


static int
fare_fast(struct parser_type *parser_data, struct c1_node_type *node_data)
{
    const char *e;
    int flags, status, type;
    struct x1f4_e4_type *e4_data;
    void *expression;

    e = parser_data->program;

    e4_data = &parser_data->e4;

    e4_data->terminator = 0x3b;

    flags = parser_data->e4_flags;

    type = parser_data->shift;
    if (type != X1f4_E4_SLIP) {
	e4_data->sequence_set.type = type;
	flags |= X1f4_E4_SEQUENCE;
    }

    status = x1f4_iNIt_expression(&expression, &e, flags, e4_data);
    if (status) {
	if (status == X1f4_E4_ALLOC_ERROR) {
	    status = ALLOC_ERROR;
	} else {
	    status = _x1f4_c1_130608_error_0(parser_data, e, status);
	}
    } else {
	do {
	    if (true(*e == 0x3b)) {
		e++;
	    } else {
		parser_data->slip(&e);

		if (*e == 0x3b) {
		    e++;
		} else {
		    status = _x1f4_c1_080707_error_2(parser_data, e);
		    if (expression) {
			x1f4_fini_expression(&expression);
		    }

		    break;
		}
	    }

	    if (expression) {
		if (node_data) {
		    x1f4_fini_expression(&node_data->data.data);

		    /*
		     * some stuff below set for the second time to same values
		     * (-line-, -state-, etc)
		     */
		} else {
		    status = iran_node(parser_data, &node_data, 0);
		    if (status) {
			if (true(expression)) {
			    x1f4_fini_expression(&expression);
			}

			break;
		    }
		}
		{
		    struct c1_program_type *program_data;

		    program_data = parser_data->program_data;

		    program_data->line = node_data;

		    node_data[0].state = line_copy(parser_data);

		    node_data[0].data.data = expression;

		    if (parser_data->flags & LINE_OPTIMIZE) {
			fare_node(parser_data, expression);
		    }

		    if (type == X1f4_E4_SLIP) {
			parser_data->shift = x1f4_type_expression(expression);
		    }

		    status = fail_slip(parser_data, program_data);
		}
	    }
	} while (0);
    }

    parser_data->program = e;

    return status;
}


static int
fare_fold(struct parser_type *parser_data, int *type, void **expression)
{
    const char *e;
    int status;
    struct x1f4_e4_type *e4_data;

    e = parser_data->program;

    e4_data = &parser_data->e4;

    e4_data->terminator = 0x3b;

    e4_data->completion = X1f4_E4_HALF_COMPLETION;

    status = x1f4_inIt_expression
	(expression, &e, X1f4_E4_SLIPTEST | parser_data->e4_flags, e4_data);
    if (status) {
	if (status == X1f4_E4_ALLOC_ERROR) {
	    status = ALLOC_ERROR;
	} else {
	    status = _x1f4_c1_080707_error_0(parser_data, e);
	}
    } else {
	do {
	    *type = x1f4_type_expression(*expression);

	    if (true(*e == 0x29 || *e == 0x3b)) {
	    } else {
	    }

	    if (parser_data->flags & LINE_OPTIMIZE) {
		fare_node(parser_data, *expression);
	    }
	} while (0);
    }

    parser_data->program = e;

    e4_data->completion = X1f4_E4_BACK_COMPLETION;

    return status;
}


static int
fare_half(struct parser_type *parser_data, struct c1_hook_type *hook_data,
	  const char *g)
{
    const char *e;
    int status;
    struct x1f4_e4_type *e4_data;
    void *expression;

    e = parser_data->program;

    e4_data = &parser_data->e4;

    e4_data->terminator = 0;

    status = x1f4_inIt_expression
	(&expression, &e, X1f4_E4_SLIPTEST | parser_data->e4_flags,
	 e4_data);

    if (g) {
    } else {
	g = e;
    }

    if (status) {
	if (status == X1f4_E4_ALLOC_ERROR) {
	    status = ALLOC_ERROR;
	} else {
	    status = _x1f4_c1_080707_error_0(parser_data, g);
	}
    } else {
	do {
	    /*
	     * optimizing may change type
	     */
	    if (parser_data->flags & LINE_OPTIMIZE) {
		fare_node(parser_data, expression);
	    }

	    {
		int type;

		type = x1f4_type_expression(expression);
		if (type == X1f4_E4_BILL || type == X1f4_E4_CASE
		    || type == X1f4_E4_MODE || type == X1f4_E4_REAL) {
		} else {
		    if (type != X1f4_E4_VOID) {
			status = _x1f4_c1_091025_error_0(parser_data, g, type);
		    } else {
			status = _x1f4_c1_080707_error_1(parser_data, g);
		    }

		    if (expression) {
			x1f4_fini_expression(&expression);
		    }

		    break;
		}
	    }

	    if (true(*e == 0x29)) {
		e++;
	    } else {
		parser_data->slip(&e);

		if (*e == 0x29) {
		    e++;
		} else {
		    status = _x1f4_c1_080707_error_2(parser_data, g);
		    if (true(expression)) {
			x1f4_fini_expression(&expression);
		    }

		    break;
		}
	    }
	    {
		parser_data->slip(&e);

		if (*e == 0x3b) {
		} else {
		    status = _x1f4_c1_080707_error_7(parser_data, g);
		    if (true(expression)) {
			x1f4_fini_expression(&expression);
		    }

		    break;
		}
	    }

	    {
		struct c1_node_type *node_data;

		status = iran_node(parser_data, &node_data, 1);
		if (status) {
		    if (true(expression)) {
			x1f4_fini_expression(&expression);
		    }

		    break;
		} else {
		    struct c1_node_type *post_node;

		    status = iran_node(parser_data, &post_node, 1);
		    if (status) {
			if (true(expression)) {
			    x1f4_fini_expression(&expression);
			}

			break;
		    } else {
			if (hook_data->class == CASE_WARD) {
			    node_data[0].state =
				rule_copy(parser_data, hook_data->line);
			} else {
			    node_data[0].state = line_copy(parser_data);
			}

			node_data[1].data.data = expression;

			node_data[1].node_data = post_node;

			post_node[0].state = 0;

			{
			    struct c1_node_type *half_node;

			    half_node = hook_data->node_data;
			    if (!half_node) {
				post_node[1].node_data =
				    parser_data->head_node;
			    } else {
				post_node[1].node_data = half_node->node_data;
			    }
			}

			parser_data->hook_data = hook_data->hook_data;

			M_FREE(parser_data, hook_data);
		    }
		}
	    }

	    {
		if (parser_data->head_rack) {
		    near_red(parser_data);
		}
	    }

	    parser_data->rack--;

	    parser_data->head_line = lock_root;
	} while (0);
    }

    parser_data->program = e;

    return status;
}


static int
fare_heAd(struct parser_type *parser_data)
{
    int status;

    status = keep_line(parser_data);
    if (status) {
    } else {
	status = fare_head(parser_data);
    }

    return status;
}


static int
fare_head(struct parser_type *parser_data)
{
    int status;

    {
	status = 0;

	{
	    parser_data->level++;

	    parser_data->head_line = line_root;

	    parser_data->program = (const char *) parser_data->program + 1;
	}
    }

    return status;
}


static int
fare_laSt(struct parser_type *parser_data)
{
    const char *e;
    int status;

    e = parser_data->program;
    if (*e == 0x3b) {
	parser_data->program = e + 1;

	status = 0;
    } else {
	status = keep_line(parser_data);
	if (status) {
	} else {
	    status = fare_last(parser_data);
	}
    }

    return status;
}


static int
fare_last(struct parser_type *parser_data)
{
    const char *e;
    int status;
    struct x1f4_e4_type *e4_data;
    void *expression;

    e = parser_data->program;

    e4_data = &parser_data->e4;

    e4_data->terminator = 0x3b;

    status = x1f4_iNIt_expression
	(&expression, &e, parser_data->e4_flags, e4_data);
    if (status) {
	if (status == X1f4_E4_ALLOC_ERROR) {
	    status = ALLOC_ERROR;
	} else {
	    status = _x1f4_c1_080707_error_0(parser_data, e);
	}
    } else {
	do {
	    if (true(*e == 0x3b)) {
		e++;
	    } else {
		parser_data->slip(&e);

		if (*e == 0x3b) {
		    e++;
		} else {
		    status = _x1f4_c1_080707_error_2(parser_data, e);
		    if (expression) {
			x1f4_fini_expression(&expression);
		    }

		    break;
		}
	    }

	    if (expression) {
		struct c1_node_type *node_data;

		status = iran_node(parser_data, &node_data, 0);
		if (status) {
		    if (true(expression)) {
			x1f4_fini_expression(&expression);
		    }

		    break;
		} else {
		    node_data[0].state = line_copy(parser_data);

		    node_data[0].data.data = expression;

		    if (parser_data->flags & LINE_OPTIMIZE) {
			fare_node(parser_data, expression);
		    }
		}
	    }
	} while (0);
    }

    parser_data->program = e;

    return status;
}


static int
fare_loop(struct parser_type *parser_data)
{
    int status;

    status = fare_case(parser_data, CASE_LOOP);
    if (status) {
    } else {
	parser_data->rack++;
    }

    return status;
}


static int
fare_mode(struct parser_type *parser_data)
{
    return fare_type(parser_data, 0, X1f4_E4_MODE);
}


static int
fare_node(struct parser_type *parser_data, void *expression)
{
    int status;
    unsigned r1st, r2nd, s1st, s2nd;

    r1st = parser_data->r1st;
    r2nd = parser_data->r2nd;
    s1st = parser_data->s1st;
    s2nd = parser_data->s2nd;

    do {
	void *v1st, *v2nd;

	v1st = parser_data->v1st;
	v2nd = parser_data->v2nd;

	if (r1st < s1st) {
	    status = M_MODE(parser_data, &v1st, s1st);
	    if (status) {
		break;
	    } else {
		parser_data->r1st = s1st;
		parser_data->v1st = v1st;
	    }
	}

	if (r2nd < s2nd) {
	    status = M_MODE(parser_data, &v2nd, s2nd);
	    if (status) {
		break;
	    } else {
		parser_data->r2nd = s2nd;
		parser_data->v2nd = v2nd;
	    }
	}

	status = x1f4_miss_expression(expression, v1st, v2nd);
	if (status) {
	} else {
	    status = x1f4_trip_expression
		(expression, parser_data->trap, parser_data->linetrap);
	}
    } while (0);

    return status;
}


static int
fare_post(struct parser_type *parser_data)
{
    const char *e;
    int status;

    e = parser_data->program;

    parser_data->slip(&e);

    parser_data->program = e;

    if (*e != 0x7b) {
	status = _x1f4_c1_080707_error_8(parser_data, e);
    } else {
	e++;
	status = fare_aiLe(parser_data);
    }

    return status;
}


static int
fare_rack(struct parser_type *parser_data)
{
    const char *e;
    int status;

    e = parser_data->program;

    parser_data->slip(&e);

    if (*e != 0x3b) {
	status = _x1f4_c1_080707_error_9(parser_data, e);
    } else {
	if (!parser_data->rack) {
	    status = _x1f4_c1_080707_error_a(parser_data, e);
	} else {
	    unsigned shift;

	    status = copy_raCk(parser_data, &shift);
	    if (status) {
	    } else {
		struct c1_hook_type *hook_data;

		status = M_LINK(parser_data, &hook_data,
				sizeof(struct c1_hook_type));
		if (status) {
		    status = ALLOC_ERROR;
		} else {
		    struct c1_node_type *node_data;

		    status = iran_node(parser_data, &node_data, 1);
		    if (status) {
			M_FREE(parser_data, hook_data);
		    } else {
			{
			    struct c1_hook_type *tail_rack;

			    tail_rack = parser_data->tail_rack;
			    if (tail_rack) {
				tail_rack->hook_data = hook_data;
			    } else {
				parser_data->head_rack = hook_data;
			    }

			    parser_data->tail_rack = hook_data;
			}

			node_data[0].state = 0;

			hook_data->node_data = node_data;

			hook_data->level = shift;

			e++;
		    }
		}
	    }
	}
    }

    parser_data->program = e;

    return status;
}


static int
fare_real(struct parser_type *parser_data)
{
    return fare_type(parser_data, 0, X1f4_E4_REAL);
}


static int
fare_side(struct parser_type *parser_data)
{
    const char *e;
    int status;

    e = parser_data->program;
    if (*e == 0x3b) {
	status = _x1f4_c1_090828_error_1(parser_data, e);
    } else {
	status = keep_line(parser_data);
	if (status) {
	} else {
	    status = fare_fast(parser_data, NULL);
	}
    }

    return status;
}


static int
fare_test(struct parser_type *parser_data)
{
    return fare_case(parser_data, CASE_TEST);
}


static int
fare_text(struct parser_type *parser_data)
{
    return fare_type(parser_data, 0, X1f4_E4_TEXT);
}


static int
fare_turn(struct parser_type *parser_data)
{
    int status;

    if (parser_data->level) {
	status = _x1f4_c1_090828_error_0(parser_data, parser_data->program);
    } else {
	if (parser_data->track == TURN_COMPLETION) {
	    status = fare_side(parser_data);
	} else {
	    status = _x1f4_c1_090828_error_0
		(parser_data, parser_data->program);
	}
    }

    return status;
}


static int
fare_type(struct parser_type *parser_data, int side, int type)
{
    const char *e;
#if __MIXED_TYPES__
    int q = 0;
#endif				/* __MIXED_TYPES__ */
    int status;

    e = parser_data->program;

    while (1) {
	const char *f;
	int c, jack;
	unsigned size;

	parser_data->slip(&e);

	c = *byte(e);

	f = e;
	if (c == '(') {
	    struct c1_back_type *back_data;
	    struct x1f4_ejrecord_type *ejrecord_data;
	    unsigned bits;

	    back_data = parser_data->back_data;
	    if (back_data) {
		ejrecord_data = &back_data->data.higherof.ejrecord;
	    } else {
		ejrecord_data = NULL;
	    }

	    status = x1f4_pick_e4jack
		(type, &jack, &bits, &size, &f, &e, 1, 0, parser_data->e4fine,
		 parser_data->slip, parser_data->type_data,
		 parser_data->side_data, ejrecord_data);
	    if (status) {
		if (status == X1f4_E4JACK_ALLOC_ERROR) {
		    status = ALLOC_ERROR;
		} else {
		    status = _x1f4_c1_111221_error_0(parser_data, f, status);
		}

		break;
	    } else {
#if 0
		parser_data->cell_type = NULL;
#endif				/* 0 */
	    }
	} else {
	    if (side) {
#if __MIXED_TYPES__
		if (q) {
		    q = -1;
		} else {
#endif				/* __MIXED_TYPES__ */
		    status = _x1f4_c1_110725_error_0(parser_data, f, type);
		    break;
#if __MIXED_TYPES__
		}
#endif				/* __MIXED_TYPES__ */
	    }
	    {
		wind_red(&e);
		jack = type;
		size = e - f;
	    }
	}
	if (size) {
	    status = x1f4_look_atkey_l(f, size, _x1f4_c1_text_root);
	    if (status) {
		const struct c1_type_type *type_data;

		type_data = parser_data->side_data;
		if (type_data) {
		    const void *cell;

		    if (ever_type(&f, &cell, type_data)) {
		    } else {
#if __MIXED_TYPES__
			if (q) {
			    q = 0;
			    side = 1;
			    parser_data->cell_type = cell;
			    type = parser_data->cell_type->type;

			    continue;
			}
#endif				/* __MIXED_TYPES__ */

			status = _x1f4_c1_080707_error_b
			    (parser_data, f - size, size);
			break;
		    }
		}
		type_data = parser_data->type_data;
		if (type_data) {
		    const void *cell;

		    if (ever_type(&f, &cell, type_data)) {
		    } else {
#if __MIXED_TYPES__
			if (q) {
			    q = 0;
			    side = 0;
			    parser_data->cell_type = cell;
			    type = parser_data->cell_type->type;

			    continue;
			}
#endif				/* __MIXED_TYPES__ */

			status = _x1f4_c1_080707_error_b
			    (parser_data, f - size, size);
			break;
		    }
		}

#if __MIXED_TYPES__
		if (~q) {
		} else {
		    status = _x1f4_c1_110725_error_0(parser_data, f, type);
		    break;
		}
#endif				/* __MIXED_TYPES__ */

		status = fare_data(parser_data, f, size, jack);
		if (status) {
		    break;
		} else {
		    parser_data->slip(&e);

		    c = *byte(e);
		    if (c == 0x2c) {
#if __MIXED_TYPES__
			q = 1;
#endif				/* __MIXED_TYPES__ */
			e++;
		    } else {
			if (c == 0x3b) {
			    e++;
			    break;
			} else {
			    status = _x1f4_c1_080707_error_c(parser_data, e);
			    break;
			}
		    }
		}
	    } else {
#if __MIXED_TYPES__
		if (q) {
		    if (x1f4_lock_e4jack(f, size, &type)) {
		    } else {
			q = 0;
			side = type == X1f4_E4_VOID;

			continue;
		    }
		}
#endif				/* __MIXED_TYPES__ */

		status = _x1f4_c1_080707_error_b(parser_data, f, size);
		break;
	    }
	} else {
	    status = _x1f4_c1_120115_error_0(parser_data, f);
	    break;
	}
    }

    parser_data->program = e;

    return status;
}


static int
fare_void(struct parser_type *parser_data)
{
    return fare_type(parser_data, 1, X1f4_E4_VOID);
}


static int
fare_ward(struct parser_type *parser_data)
{
    const char *e;
    int status;
    void *expression = NULL;

    e = parser_data->program;

    do {
	const char *b;
	char *a, *c, lend[8], rend[8];
	const char *g;
	int back = FOLD_CASE, m = X1f4_E4_VOID, n = X1f4_E4_VOID, over, p = 0,
	    q, r, sign = 0;
	struct c1_slip_type *slip_data;
	const struct x1f4_progress_type *progress_data;
	const struct x1f4_variable_type *variable_data;
	unsigned mode = 0, u, v, w;
	void *deck;

	parser_data->slip(&e);

	if (*e != 0x28) {
	    status = _x1f4_c1_080707_error_5(parser_data, e);
	    break;
	}

	status = keep_line(parser_data);
	if (status) {
	    break;
	}

	if (parser_data->progress_data) {
	} else {
	    status = _x1f4_c1_170722_error_0(parser_data, e);
	    break;
	}

	e++;

	parser_data->slip(&e);

	if (*e == '+') {
	    e++;

	    sign = -1;

	    parser_data->slip(&e);
	} else {
	    if (*e == '-') {
		e++;
		if (C_TYPE_X(_x1f4_c_type_1, *e)) {
		    const char *f;
		    unsigned z;

		    f = e;
		    wind_red(&e);
		    z = e - f;
		    if (z ^ 2) {
			status = _x1f4_c1_190511_error_0(parser_data, f, z);
			break;
		    } else {
			switch (f[0] << 8 | f[1]) {
			case 'e' << 8 | 'q':
			    sign = X1f4_PROGRESS_EQCLAUSE;
			    break;
			case 'g' << 8 | 'e':
			    sign = X1f4_PROGRESS_GECLAUSE;
			    break;
			case 'g' << 8 | 't':
			    sign = X1f4_PROGRESS_GTCLAUSE;
			    break;
			case 'l' << 8 | 'e':
			    sign = X1f4_PROGRESS_LECLAUSE;
			    break;
			case 'l' << 8 | 't':
			    sign = X1f4_PROGRESS_LTCLAUSE;
			}

			if (sign) {
			} else {
			    status =
				_x1f4_c1_190511_error_0(parser_data, f, 2);
			    break;
			}

			mode |= sign;

			parser_data->slip(&e);
		    }
		} else {
		    status = _x1f4_c1_190511_error_0(parser_data, e, 0);
		    break;
		}
	    }
	}

	a = (char *) e;

	wind_red(&e);

	u = e - a;

	parser_data->slip(&e);

	if (!sign && C_TYPE_X(_x1f4_c_type_1, *e)) {
	    char *o;
	    int l;

	    l = u;
	    o = a;

	    a = (char *) e;

	    wind_red(&e);

	    u = e - a;
	    if (u != 2) {
		m = ~X1f4_E4_VOID;
	    } else {
		if (memcmp(a, "in", 2) && memcmp(a, "of", 2)) {
		    m = ~X1f4_E4_VOID;
		} else {
		}
	    }

	    if (m == X1f4_E4_VOID) {
		e = a;

		a = o;
		u = l;
	    } else {
		parser_data->slip(&e);

		status = hold_slip(parser_data, o, l, a, u, &m);
		if (status) {
		    break;
		} else {
		    back = ROLL_CASE;
		}
	    }
	}

	slip_data = parser_data->slip_data;
	if (slip_data) {
	    deck = slip_data->data;
	} else {
	    deck = (void *) parser_data->variable_deck;
	}

	if (*e == ',') {
	    if (sign && !u) {
		status = _x1f4_c1_170722_error_1(parser_data, e);
		break;
	    }

	    e++;

	    parser_data->slip(&e);

	    b = e;

	    wind_red(&e);

	    v = e - b;

	    parser_data->slip(&e);

	    if (sign != -1 && C_TYPE_X(_x1f4_c_type_1, *e)) {
		const char *o;
		int l;

		l = v;
		o = b;

		b = e;

		wind_red(&e);

		v = e - b;
		if (v != 2) {
		    n = ~X1f4_E4_VOID;
		} else {
		    if (memcmp(b, "in", 2) && memcmp(b, "of", 2)) {
			n = ~X1f4_E4_VOID;
		    } else {
		    }
		}

		if (n == X1f4_E4_VOID && m != X1f4_E4_VOID && deck) {
		    const void *miss;

		    if (x1f4_find_mxpath(deck, o, l, &miss)) {
		    } else {
			e = b;

			b = o;
			v = l;

			n = m;
			o = NULL;
		    }
		}

		if (n == X1f4_E4_VOID) {
		    e = b;

		    b = o;
		    v = l;
		} else {
		    parser_data->slip(&e);

		    if (o) {
			status = hold_slip(parser_data, o, l, b, v, &n);
		    } else {
			status = hold_such(parser_data, b, v, n);
		    }
		    if (status) {
			break;
		    } else {
			back = ROLL_CASE;
		    }
		}
	    } else {
		if (v) {
		} else {
		    status = _x1f4_c1_170722_error_2(parser_data, e);
		    break;
		}
	    }
	} else {
	    if (u) {
	    } else {
		status = _x1f4_c1_170722_error_1(parser_data, e);
		break;
	    }

	    b = NULL;
	    v = 0;
	}

	if (*e != 'i') {
	    if (*e != 'o') {
		status = _x1f4_c1_170722_error_3(parser_data, e);
		break;
	    }
	    e++;
	    if (*e != 'f') {
		status = _x1f4_c1_170722_error_3(parser_data, e);
		break;
	    }

	    mode |= X1f4_PROGRESS_BACKWARD;
	} else {
	    e++;
	    if (*e != 'n') {
		status = _x1f4_c1_170722_error_3(parser_data, e);
		break;
	    }
	}
	e++;

	if (C_TYPE_X(_x1f4_c_type_0, *e)) {
	    status = _x1f4_c1_170722_error_3(parser_data, e);
	    break;
	}

	parser_data->program = g = e;

	r = 0;

	status = fare_fold(parser_data, &r, &expression);
	if (status) {
	    break;
	}

	e = parser_data->program;

	over = *e == 0x3b;

	e++;

	parser_data->slip(&e);

	if (over) {
	} else {
	    if (*e == 0x7b) {
		e++;
	    } else {
		status = _x1f4_c1_170722_error_4(parser_data, e);
		break;
	    }
	}

	if (deck) {
	} else {
	    status = _x1f4_c1_170722_error_5(parser_data, a, u);
	    break;
	}

	if (u) {
	    status = lock_slip(parser_data, deck, a, u, m, &p);
	    if (status) {
		break;
	    }
	} else {
	    p = X1f4_E4_VOID;
	}

	if (v) {
	    status = lock_slip(parser_data, deck, b, v, n, &q);
	    if (status) {
		break;
	    }

	    progress_data = parser_data->progress_data;
	    while (progress_data->count) {
		if (progress_data->count == 2) {
		    int arg2;

		    arg2 = progress_data->arg2;
		    if (r == progress_data->type
			&& (p == X1f4_E4_VOID || p == progress_data->arg1)
			&& (q == arg2 || arg2 == X1f4_E4_SLIP)
			&& PROGRESS_MODE(progress_data, mode)) {
			break;
		    }
		}

		progress_data++;
	    }

	    if (progress_data->count) {
		if (p == X1f4_E4_VOID) {
		    p = progress_data->arg1;

		    if (X1f4_E4_LAST < p) {
			int false = 1;
			const struct c1_type_type *type_data;

			type_data = parser_data->type_data;
			while (type_data->name) {
			    if (type_data->type == p) {
				false = 0;
				break;
			    }
			    type_data++;
			}
			if (false) {
			    status = _x1f4_c1_180501_error_0(parser_data, b);
			    if (status) {
				break;
			    }
			}

			parser_data->cell_type = type_data;
		    }

		    /*
		     * -lend- used outside this scope
		     */
		    a = lend;

		    status = rend_data(parser_data, deck, 'A', &a, &u, p);
		    if (status) {
			break;
		    } else {
			back = ROLL_CASE;
		    }
		}
	    } else {
		status = _x1f4_c1_170722_error_7
		    (parser_data, e, mode, p, q, r);
		if (status) {
		    break;
		}
	    }
	} else {
	    progress_data = parser_data->progress_data;
	    while (progress_data->count) {
		if (progress_data->count == 1) {
		    if (r == progress_data->type
			&& p == progress_data->arg1
			&& PROGRESS_MODE(progress_data, mode)) {
			break;
		    }
		}

		progress_data++;
	    }

	    if (progress_data->count) {
	    } else {
		status = _x1f4_c1_170722_error_6(parser_data, e, mode, p, r);
		if (status) {
		    break;
		}
	    }
	}

	if (x1f4_pick_expression(expression, &variable_data)) {
	    int false = 1;
	    const struct c1_type_type *type_data;

	    type_data = parser_data->type_data;
	    while (type_data->name) {
		if (type_data->type == r) {
		    false = 0;
		    break;
		}
		type_data++;
	    }
	    if (false) {
		status = _x1f4_c1_170731_error_0(parser_data, g, r);
		if (status) {
		    break;
		}
	    }

	    parser_data->cell_type = type_data;

	    /*
	     * -rend- used outside this scope
	     */
	    c = rend;

	    status = rend_data(parser_data, deck, 'C', &c, &w, r);
	    if (status) {
		break;
	    }

	    parser_data->level++;

	    status = keep_line(parser_data);
	    if (status) {
		break;
	    }

	    status = _x1f4_c1_rule_fold(parser_data, expression, g, c, w);
	    if (status) {
		break;
	    }

	    back = ROLL_CASE;
	} else {
	    c = (char *) variable_data->name;
	    w = variable_data->length;

	    if ((w == u && !memcmp(c, a, w)) || (w == v && !memcmp(c, b, w))) {
		/* variable name volatile, freed before reporting */
		status = _x1f4_c1_080707_error_6
		    (parser_data, w == u ? a : b, w);
		if (1) {
		    break;
		}
	    }

	    if (back != FOLD_CASE) {
		parser_data->level++;

		status = keep_line(parser_data);
		if (status) {
		    break;
		}
	    }
	}

	if (sign == -1) {
	    status = land_part
		(parser_data, progress_data, a, u, b, v, c, w, g, back);
	} else {
	    status = land_pass
		(parser_data, progress_data, a, u, b, v, c, w, g, back, sign,
		 e);
	}
	if (status) {
	    break;
	}

	if (over) {
	    parser_data->program = e;

	    status = fare_aime(parser_data, CASE_TEST, NULL);
	    if (status) {
		break;
	    }

	    e = parser_data->program;

	    parser_data->program = " else { ; .";

	    status = fare_back(parser_data);
	    if (status) {
		break;
	    }

	    status = fare_rack(parser_data);
	    if (status) {
		break;
	    }

	    status = fare_back(parser_data);
	    if (status) {
		break;
	    }
	}

	status = 0;
    } while (0);

    if (expression) {
	int excess;

	excess = x1f4_fini_expression(&expression);
	if (excess) {
	    if (status) {
	    } else {
		status = ALLOC_ERROR;
	    }
	}
    }

    parser_data->program = e;

    return status;
}


static int
hold_slip(struct parser_type *parser_data, const char *o, unsigned l,
	  const char *c, unsigned w, int *type)
{
    int status;

    do {
	status = x1f4_look_atkey_l(c, w, _x1f4_c1_text_root);
	if (status) {
	    status = x1f4_seek_e4jack(parser_data->type_data, c, w, type);
	    if (status) {
		status = x1f4_seek_e4jack(parser_data->side_data, c, w, type);
	    } else {
	    }
	} else {
	}
	if (status) {
	} else {
	    status = _x1f4_c1_080707_error_b(parser_data, c, w);
	    break;
	}

	if (x1f4_lock_e4jack(o, l, type)) {
	    if (x1f4_fast_e4jack
		(parser_data->type_data, o, l, type,
		 &parser_data->cell_type)) {
		status = _x1f4_c1_180601_error_0(parser_data, o, l);
		break;
	    } else {
	    }
	} else {
	    if (*type == X1f4_E4_VOID) {
		status = _x1f4_c1_110725_error_0(parser_data, o, *type);
		break;
	    }
	}

	status = fare_data(parser_data, c, w, *type);
    } while (0);

    return status;
}


static int
hold_such(struct parser_type *parser_data, const char *c, unsigned w, int type)
{
    int status;

    do {
	status = x1f4_look_atkey_l(c, w, _x1f4_c1_text_root);
	if (status) {
	    int tips;

	    status = x1f4_seek_e4jack(parser_data->type_data, c, w, &tips);
	    if (status) {
		status = x1f4_seek_e4jack(parser_data->side_data, c, w, &tips);
	    } else {
	    }
	} else {
	}
	if (status) {
	} else {
	    status = _x1f4_c1_080707_error_b(parser_data, c, w);
	    break;
	}

	status = fare_data(parser_data, c, w, type);
    } while (0);

    return status;
}


static int
iran_back(struct parser_type *parser_data)
{
    int status;

    while (1) {
	unsigned line;

	status = ever_line(&line, parser_data);
	if (status) {
	    break;
	}
	if (line == BACK_CASE) {
	    if (parser_data->level) {
	    } else {
		status = 0;
		if (1) {
		    break;
		}
	    }
	} else {
	    if (line) {
	    } else {
		status = _x1f4_c1_080707_error_f(parser_data);
		if (1) {
		    break;
		}
	    }
	}

	status = set_class[line](parser_data);
	if (status) {
	    break;
	}
    }

    return status;
}


static int
iran_byte(struct parser_type *parser_data)
{
    int status;

    while (1) {
	unsigned line;

	status = ever_line(&line, parser_data);
	if (status) {
	    break;
	}
	if (line) {
	} else {
	    if (!parser_data->level) {
		status = 0;
	    } else {
		status = _x1f4_c1_080707_error_f(parser_data);
	    }

	    break;
	}

	status = set_class[line](parser_data);
	if (status) {
	    break;
	}
    }

    return status;
}


static int
iran_data(struct parser_type *parser_data)
{
    int status;

    parser_data->head_line = line_root;

    parser_data->variable_tree = NULL;

    parser_data->head_node = NULL;
    parser_data->tail_node = NULL;

    parser_data->head_rack = NULL;
    parser_data->hook_data = NULL;
    parser_data->hook_slip = NULL;
    parser_data->tail_rack = NULL;

    parser_data->miss_data = NULL;
    parser_data->miss_text = NULL;

    parser_data->level = 0;

    parser_data->rack = 0;

    if (parser_data->track == BACK_COMPLETION) {
	status = iran_back(parser_data);
    } else {
	if (parser_data->track == BYTE_COMPLETION) {
	    status = iran_byte(parser_data);
	} else {
	    status = iran_turn(parser_data);
	}
    }

    {
	void *variable_tree;

	variable_tree = parser_data->variable_tree;
	if (variable_tree) {
	    x1f4_fini_mxdeck(&variable_tree);
	}
    }

    {
	struct c1_hook_type *hook_data;

	hook_data = parser_data->hook_data;
	while (hook_data) {
	    struct c1_hook_type *miss_hook;

	    miss_hook = hook_data;
	    hook_data = hook_data->hook_data;

	    if (miss_hook->class == CASE_WARD) {
		void *data;

		data = miss_hook->data;
		if (data) {
		    M_FREE(parser_data, data);
		}
	    }

	    M_FREE(parser_data, miss_hook);
	}
    }

    {
	struct c1_hook_type *hook_data;

	hook_data = parser_data->head_rack;
	if (hook_data) {
	    parser_data->tail_rack->hook_data = parser_data->hook_slip;
	    parser_data->hook_slip = hook_data;
	}
    }

    if (status) {
    } else {
	/*
	 * looks like this one (since it may introduce new nodes) expects
	 * `hook_slip' sort of valid (or something).  hence it has to occur
	 * before `hook_slip' (final!?) processing...
	 */
	status = fail_miss(parser_data);
    }

    {
	struct c1_hook_type *hook_data;

	hook_data = parser_data->hook_slip;
	if (hook_data) {
	    if (status) {
		do {
		    struct c1_hook_type *miss_hook;

		    miss_hook = hook_data;
		    hook_data = hook_data->hook_data;
		    M_FREE(parser_data, miss_hook);
		} while (hook_data);
	    } else {
		do {
		    struct c1_hook_type *miss_hook;

		    hook_data->node_data[1].node_data = NULL;

		    miss_hook = hook_data;
		    hook_data = hook_data->hook_data;
		    M_FREE(parser_data, miss_hook);
		} while (hook_data);
	    }
	}
    }

    {
	struct c1_node_type *tail_node;

	tail_node = parser_data->tail_node;
	if (tail_node) {
	    tail_node->node_data = NULL;
	    if (status) {
		copy_mae(parser_data->head_node, parser_data->m.free,
			 parser_data->m.data);
	    } else {
		parser_data->program_data->node = parser_data->head_node;
	    }
	}
    }

    {
	struct c1_miss_type *miss_data, *miss_text;

	miss_data = parser_data->miss_data;
	miss_text = parser_data->miss_text;
	while (miss_data) {
	    struct c1_miss_type *miss_slip;

	    miss_slip = miss_data->last_miss;
	    miss_data->last_miss = miss_text;
	    miss_text = miss_data;
	    miss_data = miss_slip;
	}

	parser_data->miss_data = NULL;
	parser_data->miss_text = miss_text;
    }

    if (status) {
	copy_set(parser_data->miss_text, parser_data->flags,
		 parser_data->m.free, parser_data->m.data);
    } else {
	parser_data->program_data->miss = parser_data->miss_text;
    }

    return status;
}


static int
iran_fine(struct parser_type *parser_data)
{
    int status;
    struct x1f4_e4fine_type e4fine;
    void *fine;

    e4fine.resource_set.data = parser_data->m.data;

    e4fine.resource_set.free = parser_data->m.free;
    e4fine.resource_set.link = parser_data->m.link;
    e4fine.resource_set.mode = parser_data->m.mode;

    status = x1f4_init_e4fine(&fine, X1f4_E4FINE_RESOURCE, &e4fine);
    if (status) {
    } else {
	parser_data->e4fine = fine;
	parser_data->program_data->fine = fine;
	parser_data->program_data->high = fine;
	parser_data->e4.transfer_set.fine = fine;
    }

    return status;
}


static int
iran_node(struct parser_type *parser_data, struct c1_node_type **node,
	  unsigned shift)
{
    int status;
    struct c1_node_type *node_data;

    status = M_LINK
	(parser_data, &node_data, sizeof(struct c1_node_type) << shift);
    if (status) {
	status = ALLOC_ERROR;
    } else {
	*node = node_data;

	node_data[0].data.data = NULL;
	node_data[shift].data.data = NULL;

	_x1f4_c1_hook_node(parser_data, node_data);
    }

    return status;
}


static int
iran_turn(struct parser_type *parser_data)
{
    int status;

    while (1) {
	unsigned line;

	status = ever_line(&line, parser_data);
	if (status) {
	    break;
	}
	if (1) {
	    if (line) {
	    } else {
		status = _x1f4_c1_080707_error_f(parser_data);
		if (1) {
		    break;
		}
	    }
	}

	status = set_class[line](parser_data);
	if (status) {
	    if (status == LEAVE_STATE) {
		if (parser_data->program_data->line
		    || parser_data->shift == X1f4_E4_SLIP) {
		    status = 0;
		}
	    }

	    break;
	} else {
	    if (line == CASE_TURN) {
		break;
	    }
	}
    }

    return status;
}


static int
keep_line(struct parser_type *parser_data)
{
    int status;

    do {
	void *variable_tree;

	variable_tree = parser_data->variable_tree;
	if (!variable_tree) {
	    status = 0;
	} else {
	    struct c1_slip_type *slip_data;

	    status = M_LINK
		(parser_data, &slip_data, sizeof(struct c1_slip_type));
	    if (status) {
		status = ALLOC_ERROR;
		break;
	    } else {
		struct c1_slip_type *last_slip;

		slip_data->hook_data = NULL;
		slip_data->roll_data = NULL;

		last_slip = parser_data->slip_data;
		if (!last_slip) {
		    fare_day(parser_data, slip_data);

		    slip_data->data = variable_tree;
		    slip_data->last_slip = NULL;
		    slip_data->level = parser_data->level;

		    slip_data->miss_data = parser_data->miss_data;

		    slip_data->node_data = parser_data->tail_node;

		    parser_data->slip_data = slip_data;

		    parser_data->variable_tree = NULL;
		} else {
		    if (1) {
			void *variable_slip;

			status = x1f4_copy_mxdeck
			    (&variable_slip, (void *) last_slip->data);
			if (status) {
			} else {
			    struct half_type half;

			    half.trans_data = &parser_data->m;
			    half.variable_slip = variable_slip;

			    status = x1f4_span_mxdeck
				(variable_tree, &half, near_tree);

			    variable_slip = half.variable_slip;

			    if (status) {
				x1f4_fini_mxdeck(&variable_slip);
			    } else {
				fare_day(parser_data, slip_data);

				slip_data->data = variable_slip;
				slip_data->last_slip = last_slip;
				slip_data->level = parser_data->level;

				slip_data->miss_data = parser_data->miss_data;

				slip_data->node_data = parser_data->tail_node;

				x1f4_fini_mxdeck(&variable_tree);

				parser_data->slip_data = slip_data;

				parser_data->variable_tree = NULL;
			    }
			}
		    }

		    if (status) {
			status = ALLOC_ERROR;
			M_FREE(parser_data, slip_data);
			break;
		    }
		}
	    }
	}

	parser_data->head_line = lock_root;
    } while (0);

    return status;
}


static int
land_part(struct parser_type *parser_data,
	  const struct x1f4_progress_type *progress_data, const char *a,
	  unsigned u, const char *b, unsigned v, const char *c, unsigned w,
	  const char *g, int test)
{
    int status;

    if (0) {
    } else {
	if (0) {
	} else {
	    status = fare_aile(parser_data);
	    if (status) {
	    } else {
		char *fine;
		struct c1_hook_type *hook_data;

		hook_data = parser_data->hook_data;

		status = type_pass
		    (parser_data, &progress_data->do_clause, a, u, b, v, c, w,
		     1, &fine);
		if (status) {
		} else {
		    hook_data->class = CASE_WARD;
		    hook_data->data = fine;
		    hook_data->line = g;

		    *fine = test == ROLL_CASE ? 1 : 0;
		}
	    }
	}
    }

    return status;
}


static int
land_pass(struct parser_type *parser_data,
	  const struct x1f4_progress_type *progress_data, const char *a,
	  unsigned u, const char *b, unsigned v, const char *c, unsigned w,
	  const char *g, int test, int sign, const char *e)
{
    char *line;
    int status;
    const struct x1f4_fragment_type *fragment_data;

    if (sign) {
	const struct x1f4_position_type *position_data;

	position_data = progress_data->position_data;
	while (!(position_data->bits & sign)) {
	    position_data++;
	}

	fragment_data = &position_data->in_clause;
    } else {
	fragment_data = &progress_data->if_clause;
    }

    status = type_pass
	(parser_data, fragment_data, a, u, b, v, c, w, 0, &line);
    if (status) {
    } else {
	parser_data->program = line;

	parser_data->e4_flags |= X1f4_E4_FLEETING;

	status = fare_aime(parser_data, CASE_TEST, e);

	parser_data->e4_flags &= ~X1f4_E4_FLEETING;

	if (status) {
	} else {
	    struct c1_hook_type *hook_data;

	    hook_data = parser_data->hook_data;

	    hook_data->class = test;

	    parser_data->tail_node->state = rule_copy(parser_data, g);

	    status = fare_aile(parser_data);
	    if (status) {
	    } else {
		char *fine;

		hook_data = parser_data->hook_data;

		status = type_pass
		    (parser_data, &progress_data->do_clause, a, u, b, v, c, w,
		     1, &fine);
		if (status) {
		} else {
		    hook_data->class = CASE_WARD;
		    hook_data->data = fine;
		    hook_data->line = g;

		    *fine = 1;
		}
	    }
	}

	M_FREE(parser_data, line);
    }

    return status;
}


static int
lock_slip(struct parser_type *parser_data, void *deck, const char *c,
	  unsigned w, int o, int *type)
{
    int status = 0;

    if (o == X1f4_E4_VOID) {
	const void *miss;

	if (x1f4_find_mxpath(deck, c, w, &miss)) {
	    struct x1f4_variable_type *variable_data;

	    variable_data = ((struct c1_miss_type *) miss)->data;
	    *type = variable_data->type;

	    if (variable_data->flags & X1f4_E4_READ_ONLY) {
		status = _x1f4_c1_180102_error_0(parser_data, c, w);
	    }
	} else {
	    status = _x1f4_c1_170722_error_5(parser_data, c, w);
	}
    } else {
	*type = o;
    }

    return status;
}


static int
long_back(struct parser_type *parser_data)
{
    int status;

    do {
	struct c1_node_type *node_data;

	if (parser_data->track != TURN_COMPLETION) {
	    goto ubcs;
	}

	node_data = parser_data->tail_node;
	if (node_data && !parser_data->hook_slip) {
	    void *expression;

	    expression = node_data->data.data;
	    if (expression) {
		int turn, type;
		struct c1_node_type *lead_node;

		lead_node = parser_data->head_node;
		while (lead_node != node_data) {
		    if (lead_node->data.data) {
		    } else {
			if ((lead_node + 1)->data.data) {
			    if ((lead_node + 1)->node_data == node_data) {
				node_data = NULL;
				if (1) {
				    break;
				}
			    }
			} else {
			}
		    }

		    lead_node = lead_node->node_data;
		}

		if (node_data) {
		    type = x1f4_type_expression(expression);
		    turn = parser_data->shift;
		    if (type == turn || turn == X1f4_E4_SLIP) {
			struct c1_program_type *program_data;

			program_data = parser_data->program_data;

			parser_data->shift = type;

			program_data->line = node_data;

			status = fail_slip(parser_data, program_data);
			if (status) {
			} else {
			    status = LEAVE_STATE;
			}
		    } else {
			const char *program;

			program = parser_data->program;

			parser_data->program =
			    parser_data->thirteen + node_data->state;

			status = fare_fast(parser_data, node_data);

			if (status) {
			} else {
			    status = LEAVE_STATE;

			    parser_data->program = program;
			}
		    }

		    break;
		} else {
		}
	    } else {
	    }
	} else {
	}

	if (parser_data->shift == X1f4_E4_SLIP) {
	    status = LEAVE_STATE;
	    if (1) {
		break;
	    }
	}

      ubcs:
    	status = _x1f4_c1_080707_error_4(parser_data, parser_data->program);
    } while (0);

    return status;
}


static int
lose_slip(struct parser_type *parser_data, struct c1_slip_type *slip_data)
{
    int (*free)(void *, void *);
    struct c1_hook_type *hook_data;
    struct c1_roll_type *roll_data;
    void *data;

    data = parser_data->m.data;
    free = parser_data->m.free;

    hook_data = slip_data->hook_data;
    while (hook_data) {
	struct c1_hook_type *hook_free;

	hook_free = hook_data;
	hook_data = hook_data->hook_data;

	free(data, hook_free);
    }

    roll_data = slip_data->roll_data;
    while (roll_data) {
	struct c1_roll_type *roll_free;

	roll_free = roll_data;
	roll_data = roll_data->roll_data;

	free(data, roll_free);
    }

    free(data, slip_data);

    return 0;
}


static int
mean_data(struct parser_type *parser_data)
{
    int status;

    parser_data->count = 0;
    parser_data->trail = 0;

    parser_data->slip_data = NULL;

    status = iran_data(parser_data);

    copy_fix(parser_data);

    return status;
}


static int
mean_zero(struct parser_type *parser_data, unsigned flags,
	  const struct c1_context_type *context_data)
{
    if (1) {
	parser_data->e4_flags |= X1f4_E4_RESOURCE;
    }
    if (flags & CODELINK_LINK) {
	parser_data->m = context_data->were_w;
	parser_data->e4.resource_set = context_data->were_w;
    } else {
	parser_data->m = context_data->link_m;
	parser_data->e4.resource_set = context_data->link_m;
    }

    return 0;
}


static int
near_data(struct parser_type *parser_data)
{
    int status;
    struct c1_slip_type *slip_data;

    status = M_LINK(parser_data, &slip_data, sizeof(struct c1_slip_type));
    if (status) {
	status = ALLOC_ERROR;
    } else {
	slip_data->data = (void *) parser_data->variable_deck;
	slip_data->hook_data = NULL;
	slip_data->last_slip = NULL;
	slip_data->roll_data = NULL;
	slip_data->level = 0;

	slip_data->count = 0;

	slip_data->node_data = NULL;

	parser_data->count = 0;
	parser_data->trail = 0;

	parser_data->slip_data = slip_data;

	status = iran_data(parser_data);

	copy_fix(parser_data);
    }

    return status;
}


static int
near_tree(void *half, const void *miss, const char *name, unsigned size)
{
    int status;
    struct half_type *half_data;
    void *variable_slip;
    const void **pick;

    half_data = half;

    variable_slip = half_data->variable_slip;

    if (x1f4_pick_mxpath(variable_slip, name, size, &pick)) {
	*pick = miss;

	status = 0;
    } else {
	status = x1f4_post_mxpath(variable_slip, name, size, miss);
    }

    half_data->variable_slip = variable_slip;

    return status;
}


static int
part_data(struct parser_type *parser_data)
{
    int status;

    do {
	if (1) {
	    void *e4fine;

	    e4fine = parser_data->e4fine;
	    if (e4fine) {
		parser_data->e4.transfer_set.fine = e4fine;
		parser_data->program_data->high = e4fine;
	    } else {
		status = iran_fine(parser_data);
		if (status) {
		    break;
		}
	    }
	}

	status = post_data(parser_data);
    } while (0);

    copy_post(parser_data);

    if (status) {
    } else {
	if (1) {
	    unsigned list, post, used;

	    post = (integral_q) parser_data->base;
	    list = post + parser_data->s1st;
	    used = list + parser_data->s2nd;
	    parser_data->program_data->post = post;
	    parser_data->program_data->used = used;
	    parser_data->base = (void *) (integral_q) used;
	} else {
	}
    }

    return status;
}


static int
post_clip(struct parser_type *parser_data, struct c1_hook_type *hook_data)
{
    int status;
    struct c1_node_type *node_data;

    status = iran_node(parser_data, &node_data, 1);
    if (status) {
    } else {
	struct c1_node_type *hook_node;

	hook_node = hook_data->node_data;

	node_data[0].state = 0;

	hook_node[1].node_data = node_data;

	hook_data->class = ELSE_CASE;
	hook_data->node_data = node_data;

	parser_data->level++;

	parser_data->head_line = line_root;
    }

    return status;
}


static int
post_data(struct parser_type *parser_data)
{
    int status;

    if (parser_data->variable_deck) {
	status = near_data(parser_data);
    } else {
	status = mean_data(parser_data);
    }

    return status;
}


static int
post_ever(struct parser_type *parser_data, struct c1_hook_type *hook_data)
{
    const char *e;
    int status;
    int c;

    e = parser_data->program + 1;

    parser_data->slip(&e);

    c = *byte(e);
    if (c != 0x28) {
	status = _x1f4_c1_080707_error_5(parser_data, e);
    } else {
	parser_data->program = e + 1;

	status = post_clip(parser_data, hook_data);
	if (status) {
	} else {
	    parser_data->level--;

	    status = fare_aime(parser_data, CASE_TEST, NULL);
	    if (status) {
	    } else {
		parser_data->program--;
	    }
	}
    }

    return status;
}


static int
post_hook(struct parser_type *parser_data, struct c1_hook_type *hook_data)
{
    int class, status;

    class = hook_data->class;

    if (class == CASE_LOOP) {
	status = post_loop(parser_data, hook_data);
    } else if (class == CASE_POST) {
	status = post_post(parser_data, hook_data);
    } else if (class == CASE_WARD) {
	status = post_ward(parser_data, hook_data);
    } else if (class == ELSE_CASE) {
	status = post_none(parser_data, hook_data);
    } else if (class == FOLD_CASE) {
	status = post_test(parser_data, hook_data);
    } else if (class == ROLL_CASE) {
	status = post_test(parser_data, hook_data);
	if (status) {
	} else {
	    status = fare_back(parser_data);
	    if (status) {
	    } else {
		parser_data->program--;
	    }
	}
    } else {
	do {
	    const char *e;
	    int c;

	    e = parser_data->program + 1;

	    status = 0;

	    parser_data->slip(&e);

	    c = *byte(e);
	    if (c == 0x65) {
		e++;
		c = *byte(e);
		if (c == 0x6c) {
		    e++;
		    c = *byte(e);
		    if (c == 0x73) {
			e++;
			c = *byte(e);
			if (c == 0x65) {
			    e++;
			    c = *byte(e);
			    if (C_TYPE_X(parser_data->e_type_4, c)) {
				parser_data->slip(&e);
				c = *byte(e);
			    }
			    if (c == 0x7b) {
				parser_data->program = e;

				status = post_clip(parser_data, hook_data);

				break;
			    }
			}
		    } else {
			if (c == 0x69) {
			    e++;
			    c = *byte(e);
			    if (c == 0x66) {
				parser_data->program = e;

				status = post_ever(parser_data, hook_data);

				break;
			    }
			}
		    }
		}
	    }

	    status = post_test(parser_data, hook_data);
	} while (0);
    }

    return status;
}


static int
post_loop(struct parser_type *parser_data, struct c1_hook_type *hook_data)
{
    int status;
    struct c1_node_type *node_data;

    status = iran_node(parser_data, &node_data, 1);
    if (status) {
    } else {
	struct c1_node_type *hook_node;

	hook_node = hook_data->node_data;

	node_data[0].state = 0;

	node_data[1].node_data = hook_node;

	hook_node[1].node_data = node_data;

	parser_data->hook_data = hook_data->hook_data;

	parser_data->head_line = lock_root;

	parser_data->rack--;

	M_FREE(parser_data, hook_data);

	{
	    if (parser_data->head_rack) {
		near_red(parser_data);
	    }
	}
    }

    return status;
}


static int
post_none(struct parser_type *parser_data, struct c1_hook_type *hook_data)
{
    int level;
    struct c1_hook_type *hook_slip;

    hook_slip = parser_data->hook_slip;

    parser_data->hook_slip = hook_data;

    level = parser_data->level;

    do {
	struct c1_hook_type *post_hook;

	post_hook = hook_data->hook_data;
	if (!post_hook) {
	    break;
	} else {
	    if (post_hook->level != level) {
		break;
	    } else {
		hook_data = post_hook;
	    }
	}
    } while (1);

    parser_data->hook_data = hook_data->hook_data;

    hook_data->hook_data = hook_slip;

    parser_data->head_line = lock_root;

    return 0;
}


static int
post_post(struct parser_type *parser_data, struct c1_hook_type *hook_data)
{
    int status;

    do {
	const char *e;
	int c;

	e = parser_data->program + 1;

	parser_data->slip(&e);

	c = *byte(e);
	if (c == 0x77) {
	    e++;
	    c = *byte(e);
	    if (c == 0x68) {
		e++;
		c = *byte(e);
		if (c == 0x69) {
		    e++;
		    c = *byte(e);
		    if (c == 0x6c) {
			e++;
			c = *byte(e);
			if (c == 0x65) {
			    e++;
			    c = *byte(e);
			    if (C_TYPE_X(parser_data->e_type_4, c)) {
				parser_data->slip(&e);
				c = *byte(e);
			    }
			    if (c == 0x28) {
				parser_data->program = e + 1;

				status = fare_half
				    (parser_data, hook_data, NULL);

				break;
			    }
			}
		    }
		}
	    }
	}

	status = _x1f4_c1_080707_error_d(parser_data, e);
    } while (0);

    return status;
}


static int
post_test(struct parser_type *parser_data, struct c1_hook_type *hook_data)
{
    struct c1_hook_type *post_hook;

    (hook_data->node_data + 1)->node_data = parser_data->tail_node;

    post_hook = hook_data->hook_data;

    parser_data->hook_data = post_hook;

    parser_data->head_line = lock_root;

    M_FREE(parser_data, hook_data);

    if (post_hook) {
	int level;

	level = parser_data->level;

	if (post_hook->level == level) {
	    struct c1_hook_type *hook_slip;

	    hook_slip = parser_data->hook_slip;

	    parser_data->hook_slip = post_hook;

	    do {
		hook_data = post_hook->hook_data;
		if (!hook_data) {
		    break;
		} else {
		    if (hook_data->level != level) {
			break;
		    } else {
			post_hook = hook_data;
		    }
		}
	    } while (1);

	    parser_data->hook_data = post_hook->hook_data;

	    post_hook->hook_data = hook_slip;
	}
    }

    return 0;
}


static int
post_ward(struct parser_type *parser_data, struct c1_hook_type *hook_data)
{
    const char *e;
    char *fine;
    int late, status;

    e = parser_data->program;

    fine = hook_data->data;

    hook_data->data = NULL;

    late = *fine;

    parser_data->program = fine + 1;

    parser_data->e4_flags |= X1f4_E4_FLEETING;

    status = fare_half(parser_data, hook_data, e);

    parser_data->e4_flags &= ~X1f4_E4_FLEETING;

    M_FREE(parser_data, fine);

    parser_data->program = e - late;

    return status;
}


static int
rend_data(struct parser_type *parser_data, void *deck, int C, char **turn,
	  unsigned *task, int type)
{
    char *name, *rend;
    unsigned size;
    const void *data;

    rend = *turn;

    rend += 7;

    *rend = 0;

    name = rend - 1;

    *name = C;

    size = 1;

    while (x1f4_find_mxpath(deck, name, size, &data)) {
	unsigned jack;

	jack = rand() * RAND_MAX + rand();
	jack &= 524287;
	name = rend;
	x1f4_xprint_dcardinal(&name, jack);
	name--;
	*name = C;
	size = rend - name;
    }

    *task = size;
    *turn = name;

    return fare_data(parser_data, name, size, type);
}


static int
trim_slip(struct c1_slip_type *slip_data, struct c1_slip_type *last_slip)
{
    struct c1_roll_type *roll_data;
    struct c1_miss_type *miss_data;
    unsigned size;

    miss_data = slip_data->miss_data;
    size = slip_data->count;
    if (last_slip) {
	size -= last_slip->count;
    }

    roll_data = slip_data->roll_data;
    while (roll_data) {
	struct c1_half_type *half_data;

	half_data = roll_data->half_data;

	half_data->miss = miss_data;
	half_data->size = size;

	roll_data = roll_data->roll_data;
    }

    return 0;
}


static int
type_pass(struct parser_type *parser_data,
	  const struct x1f4_fragment_type *fragment_data, const char *a,
	  unsigned u, const char *b, unsigned v, const char *c, unsigned w,
	  int do_clause, char **call)
{
    char *line;
    int status;
    unsigned late, post, size;

    post = size = fragment_data->size;

    late = do_clause ? 1 : 0;

    post += fragment_data->fit1 * u;
    post -= fragment_data->fit1 * 2;
    post += fragment_data->fit2 * v;
    post -= fragment_data->fit2 * 2;
    post += fragment_data->pass * w;
    post -= fragment_data->pass * 2;

    status = M_LINK(parser_data, &line, late + post + 4);
    if (status) {
	status = ALLOC_ERROR;
    } else {
	const char *text;

	*call = line;

	line += late;

	text = fragment_data->text;
	while (size) {
	    int m;

	    m = *text++;
	    if (m == ';') {
		size -= 2;
		m = *text++;
		if (m == '*') {
		    memcpy(line, c, w);
		    line += w;
		} else {
		    if (m == '0') {
			memcpy(line, a, u);
			line += u;
		    } else {
			memcpy(line, b, v);
			line += v;
		    }
		}
	    } else {
		size--;
		*line++ = m;
	    }
	}

	memcpy(line, do_clause ? "); " : ") {", 4);
    }

    return status;
}


static void
copy_fix(struct parser_type *parser_data)
{
    struct c1_slip_type *slip_data;

    slip_data = parser_data->slip_data;
    while (slip_data) {
	struct c1_slip_type *slip_free;

	slip_free = slip_data;
	slip_data = slip_data->last_slip;

	{
	    void *data;

	    data = (void *) slip_free->data;
	    if (data != parser_data->variable_deck) {
		x1f4_fini_mxdeck(&data);
	    }
	}

	lose_slip(parser_data, slip_free);
    }
}


static void
copy_mae(struct c1_node_type *node_data, int (*free)(void *, void *),
	 void *data)
{
    while (node_data) {
	struct c1_node_type *slip_node;

	slip_node = node_data;
	node_data = node_data->node_data;

	{
	    void *data;

	    data = slip_node->data.data;
	    if (data) {
		x1f4_fini_expression(&data);
	    } else {
		data = (slip_node + 1)->data.data;
		if (data) {
		    if (1) {
			x1f4_fini_expression(&data);
		    }
		}
	    }
	}

	free(data, slip_node);
    }
}


static void
copy_set(struct c1_miss_type *miss_data, unsigned bits,
	 int (*free)(void *, void *), void *data)
{
    while (miss_data) {
	struct c1_miss_type *miss_free;

	miss_free = miss_data;
	miss_data = miss_data->last_miss;
	free(data, miss_free);
    }
}


static void
fare_day(struct parser_type *parser_data, struct c1_slip_type *slip_data)
{
    slip_data->count = parser_data->count;
}


static void
fare_set(struct parser_type *parser_data)
{
    unsigned class = 0;

    if (1) {
	class |= 2;
    }
    {
	const struct c1_type_type *type_data;

	type_data = parser_data->type_data;
	if (type_data) {
	    const char *name;

	    name = type_data->name;
	    if (name) {
		do {
		    if (type_data->flat) {
			class |= 2;
			if (class & 1) {
			    break;
			}
		    }
		    if (type_data->line) {
			class |= 1;
			if (class & 2) {
			    break;
			}
		    }
		    {
			type_data++;
			name = type_data->name;
		    }
		} while (name);
	    }
	}
    }

    parser_data->class = class;
}


static void
near_red(struct parser_type *parser_data)
{
    struct c1_hook_type *hook_data;
    unsigned level;

    level = parser_data->level;

    hook_data = parser_data->tail_rack;
    if (hook_data->level == level) {
	hook_data->hook_data = parser_data->hook_slip;

	hook_data = parser_data->head_rack;
	if (hook_data->level == level) {
	    parser_data->head_rack = NULL;
	    parser_data->tail_rack = NULL;
	} else {
	    struct c1_hook_type *miss_hook;

	    miss_hook = hook_data;

	    do {
		hook_data = hook_data->hook_data;
		if (hook_data->level == level) {
		    parser_data->tail_rack = miss_hook;
		    if (1) {
			break;
		    }
		} else {
		    miss_hook = hook_data;
		}
	    } while (1);
	}

	parser_data->hook_slip = hook_data;
    }
}


static void
post_fix(struct parser_type *parser_data, unsigned count)
{
    int (*free)(void *, void *);
    struct c1_miss_type *miss_data;
    void *data;

    miss_data = parser_data->miss_data;

    parser_data->count -= count;

    data = parser_data->m.data;
    free = parser_data->m.free;

    while (count) {
	struct c1_miss_type *miss_free;

	miss_free = miss_data;
	miss_data = miss_data->last_miss;
#if __DEBUG_MISS__
	dDebug_1("freeing unused variable `%s'...\n",
		 ((struct x1f4_variable_type *) miss_free->data)->name);
#endif				/* __DEBUG_MISS__ */
#if 0
	_x1f4_c1_note_miss(parser_data->program_data, miss_free);
#endif				/* 0 */
	free(data, miss_free);

	count--;
    }

    parser_data->miss_data = miss_data;
}


static void
wind_far(const char **deck)
{
    const char *e;
    unsigned c;

    e = *deck;

    while (1) {
	c = *byte(e);
	if (C_TYPE_X(c_type_4, c)) {
	    e++;
	} else {
	    if (c ^ 35) {
		break;
	    } else {
		do {
		    ++e;
		    c = *byte(e);
		} while (c ^ 10);
	    }
	}
    }

    *deck = e;
}


static void
wind_new(const char **deck)
{
    const char *e;
    unsigned c;

    e = *deck;

    c = *byte(e);
    while (C_TYPE_X(c_type_4, c)) {
	e++;
	c = *byte(e);
    }

    *deck = e;
}


static void
wind_red(const char **deck)
{
    const char *e;
    unsigned c;

    e = *deck;

    c = *byte(e);
    if (C_TYPE_X(_x1f4_c_type_1, c)) {
	e++;
	c = *byte(e);
	while (C_TYPE_X(_x1f4_c_type_0, c)) {
	    e++;
	    c = *byte(e);
	}
    }

    *deck = e;
}


int
x1f4_fini_program(struct c1_program_type **program)
{
    struct c1_program_type *program_data;

    program_data = *program;

    if (program_data->flat) {
	_x1f4_c1_flat_mine(program_data, &program_data->flat);
    }

    copy_mae(program_data->node, program_data->w.free, program_data->w.data);

    copy_set(program_data->miss, program_data->bits, program_data->w.free,
	     program_data->w.data);

    if (1) {
	void *fine;

	fine = program_data->fine;
	if (fine) {
	    x1f4_fini_e4fine(&fine);
	}
    }

    E_FREE(program_data, program_data);

    return 0;
}


int
x1f4_inIt_program(struct c1_program_type **program, const char **proGram,
		  unsigned flags, const struct c1_context_type *context_data)
{
    int status;
    struct c1_program_type *program_data;
    struct parser_type parser;

    parser.e4_flags = X1f4_E4_ESTORAGE | X1f4_E4_MAXSTACK | X1f4_E4_TRANSFER;

    if (flags & (CODELINK_LINK | RESOURCE_LINK)) {
	mean_zero(&parser, flags, context_data);
    } else {
	parser.m.free = _x1f4_e4_free_data;
	parser.m.link = _x1f4_e4_link_data;
	parser.m.mode = _x1f4_e4_mode_data;
    }

    status = parser.m.link
	(parser.m.data, (void *) &program_data,
	 sizeof(struct c1_program_type));
    if (status) {
	status = ALLOC_ERROR;
    } else {
	parser.e_type_4[0] = c_type_4[0];
	parser.e_type_4[1] = c_type_4[1];
	parser.e_type_4[2] = c_type_4[2];
	parser.e_type_4[3] = c_type_4[3];
	parser.e_type_4[4] = c_type_4[4];
	parser.e_type_4[5] = c_type_4[5];
	parser.e_type_4[6] = c_type_4[6];
	parser.e_type_4[7] = c_type_4[7];
	if (flags & CASECALL_LINK) {
	    parser.e4.casecall_set.linetext = context_data->verb_b.linetext;
	    parser.e4.casecall_set.variable_data =
		context_data->verb_b.variable_data;
	    parser.e4_flags |= X1f4_E4_CASECALL;
	}
	if (flags & COMPOSER_LINK) {
	    parser.e4.composer_set = context_data->turn_0;
	    parser.e4_flags |= X1f4_E4_COMPOSER;
	}
	if (flags & DEPREFIX_LINK) {
	    parser.e4_flags |= X1f4_E4_DEPREFIX;
	    parser.e4.deprefix_set.fine = context_data->verb_e.fine;
	}
	if (flags & DIRECT_ACCESS) {
	    parser.e4_flags |= X1f4_E4_DDACCESS | X1f4_E4_DDDEVICE;
	    parser.e4.dddevice_set.lead = _x1f4_e4_bcxx_hack;
	}
	if (flags & DISPATCH_LINK) {
	    parser.base = *context_data->near_s.base;
	} else {
	    parser.base = (void *) (sizeof(void *) * X1f4_E4_USED_SIZE);
	}
	if (flags & ERROR_STORAGE) {
	    parser.e4.bcollect_set.eerecord_data =
		&((struct c1_back_type *)
		  context_data->link_e.back)->data.e_syntax.eerecord;
	    parser.back_data = context_data->link_e.back;
	    parser.e4_flags |= X1f4_E4_BCOLLECT;
	} else {
	    parser.back_data = NULL;
	}
	if (flags & IMPLICIT_LINK) {
	    parser.e4.implicit_set.get = context_data->trap_0.get;
	    parser.e4.implicit_set.context =
		context_data->trap_0.context;
	    parser.e4_flags |= X1f4_E4_IMPLICIT;
	}
	if (flags & RELATION_LINK) {
	    parser.linetrap = context_data->link_y.linetrap_data;
	    parser.trap = context_data->link_y.bits;
	} else {
	    parser.linetrap = NULL;
	    parser.trap = 0;
	}
	if (flags & RESOURCE_LINK) {
	    program_data->n = context_data->link_m;
	} else {
	    program_data->n = parser.m;
	}
	if (flags & SKIP_SCOMMENT) {
	    parser.e4_flags |= X1f4_E4_SCOMMENT;
	    parser.e_type_4[1] |= 8;
	    parser.slip = wind_far;
	} else {
	    parser.slip = wind_new;
	}
	if (flags & SUIT_SLIPTEST) {
	    parser.track = context_data->completion;
	} else {
	    parser.track = BYTE_COMPLETION;
	}
	if (flags & TEXTFLAT_LINK) {
	    program_data->x = context_data->call_e;
	} else {
	    program_data->x.data = NULL;
	    program_data->x.line = NULL;
	    program_data->x.post = NULL;
	    program_data->x.push = NULL;
	}
	if (flags & TRANSFER_LINK) {
	    parser.e4fine = context_data->link_i.fine;
	} else {
	    parser.e4fine = NULL;
	}
	if (flags & TRAVERSE_LINK) {
	    parser.progress_data = context_data->verb_a.miss;
	} else {
	    parser.progress_data = NULL;
	}
	parser.e4.completion = X1f4_E4_BACK_COMPLETION;
	parser.e4.datalink_set.v1st = &parser.v1st;
	parser.e4.datalink_set.v2nd = &parser.v2nd;
	parser.e4.function_set.fix = context_data->last_0.fix;
	parser.e4.function_set.get = context_data->last_0.get;
	parser.e4.function_set.context = context_data->last_0.context;
	parser.e4.internal_set.s1st = &parser.s1st;
	parser.e4.internal_set.s2nd = &parser.s2nd;
	parser.e4.operator1s = context_data->operator1s;
	parser.e4.operator2s = context_data->operator2s;
	parser.e4.variable_set.get = _x1f4_c1_select_variable;
	parser.e4.variable_set.context = &parser;
	parser.context_data = context_data;
	parser.flags = flags;
	parser.program = *proGram;
	parser.program_data = program_data;
	parser.r1st = 0;
	parser.r2nd = 0;
	parser.s1st = 0;
	parser.s2nd = 0;
	parser.shift = context_data->detail;
	parser.thirteen = parser.program;
	parser.v1st = NULL;
	parser.v2nd = NULL;
	parser.variable_deck = context_data->ever_0.context;
	program_data->bits = DISPATCH_LINK;
	program_data->fine = NULL;
	program_data->flat = NULL;
	program_data->line = NULL;
	program_data->miss = NULL;
	program_data->node = NULL;
	program_data->w = parser.m;
	program_data->t.text = _x1f4_c1_last_type;
	program_data->t.text.context = program_data;
	if (flags & SIDELIST_LINK) {
	    parser.side_data = context_data->user_z.miss;
	} else {
	    parser.side_data = NULL;
	}
	if (flags & EELOOKUP_LINK) {
	    program_data->y.eelookup_data = context_data->trap_e.eelookup_data;
	} else {
	    program_data->y.eelookup_data = NULL;
	}
	if (0) {
	    parser.class = 0;
	    parser.type_data = NULL;
	} else {
	    if (flags & TYPELIST_LINK) {
		parser.type_data = context_data->user_w.miss;
		fare_set(&parser);
	    } else {
		parser.class = 1;
		parser.type_data = NULL;
	    }
	}
	status = part_data(&parser);
	if (status) {
	    x1f4_fini_program(&program_data);
	} else {
	    *program = program_data;
	    if (flags & DISPATCH_LINK) {
		*context_data->near_s.base = parser.base;
	    }
	}

	*proGram = parser.program;
    }

#if 0
    if (status) {
    } else {
	struct c1_node_type *node_data;

	node_data = program_data->node;
	while (node_data) {
	    void *data;

	    fprintf(stderr, "%p %p ", node_data, node_data->node_data);
	    data = node_data->data.data;
	    if (data) {
	    } else {
		fprintf(stderr, "%p ", (node_data + 1)->node_data);
		data = (node_data + 1)->data.data;
	    }
	    if (data) {
		x1f4_print_expression(stderr, data, 0);
	    }
	    node_data = node_data->node_data;
	    fprintf(stderr, "\n");
	}
    }
#endif				/* 0 */

    return status;
}
