/*
 * c1.w.c
 * Copyright (C) 2007-2012, 2014, 2017-2020, Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <config.h>

#include <stddef.h>

#include <c1-defs.h>
#include <c1-errors.h>
#include <c1-types.h>
#include <cardinal.v-d.h>
#include <cardinal.v-f.h>
#include <e4.h>
#include <e42.h>
#include <progress.h>

#define c1_long_type			x1f4_eelookup_type

#define dump_abcdef_error_i_ARGS_0 \
    void *, int (*)(void *, const char *, unsigned), const char *,	      \
    struct c1_back_type *, struct c1_long_type *
#define dump_abcdef_error_i_ARGS_1 \
    void *data, int (*this)(void *, const char *, unsigned),		      \
    const char *head, struct c1_back_type *back_data,			      \
    struct c1_long_type *long_data
#define dump_abcdef_error_i_ARGS_2 \
    data, this, head, back_data, long_data

#define dump_abcdef_error_l_ARGS_0 \
    void *, int (*)(void *, const char *, unsigned), const char *,	      \
    struct c1_back_type *
#define dump_abcdef_error_l_ARGS_1 \
    void *data, int (*this)(void *, const char *, unsigned),		      \
    const char *head, struct c1_back_type *back_data
#define dump_abcdef_error_l_ARGS_2 \
    data, this, head, back_data

static int dump_080707_error_0(dump_abcdef_error_i_ARGS_0);
static int dump_080707_error_1(dump_abcdef_error_l_ARGS_0);
static int dump_080707_error_2(dump_abcdef_error_l_ARGS_0);
static int dump_080707_error_3(dump_abcdef_error_l_ARGS_0);
static int dump_080707_error_4(dump_abcdef_error_l_ARGS_0);
static int dump_080707_error_5(dump_abcdef_error_l_ARGS_0);
static int dump_080707_error_6(dump_abcdef_error_l_ARGS_0);
static int dump_080707_error_7(dump_abcdef_error_l_ARGS_0);
static int dump_080707_error_8(dump_abcdef_error_l_ARGS_0);
static int dump_080707_error_9(dump_abcdef_error_l_ARGS_0);
static int dump_080707_error_a(dump_abcdef_error_l_ARGS_0);
static int dump_080707_error_b(dump_abcdef_error_l_ARGS_0);
static int dump_080707_error_c(dump_abcdef_error_l_ARGS_0);
static int dump_080707_error_d(dump_abcdef_error_l_ARGS_0);
static int dump_080707_error_e(dump_abcdef_error_i_ARGS_0);
static int dump_080707_error_f(dump_abcdef_error_l_ARGS_0);
static int dump_090828_error_0(dump_abcdef_error_l_ARGS_0);
static int dump_090828_error_1(dump_abcdef_error_l_ARGS_0);
static int dump_091025_error_0(dump_abcdef_error_i_ARGS_0);
static int dump_110725_error_0(dump_abcdef_error_i_ARGS_0);
static int dump_111221_error_0(dump_abcdef_error_l_ARGS_0);
static int dump_120115_error_0(dump_abcdef_error_l_ARGS_0);
static int dump_170722_error_0(dump_abcdef_error_l_ARGS_0);
static int dump_170722_error_1(dump_abcdef_error_l_ARGS_0);
static int dump_170722_error_2(dump_abcdef_error_l_ARGS_0);
static int dump_170722_error_3(dump_abcdef_error_l_ARGS_0);
static int dump_170722_error_4(dump_abcdef_error_l_ARGS_0);
static int dump_170722_error_5(dump_abcdef_error_l_ARGS_0);
static int dump_170722_error_6(dump_abcdef_error_i_ARGS_0);
static int dump_170722_error_7(dump_abcdef_error_i_ARGS_0);
static int dump_170731_error_0(dump_abcdef_error_i_ARGS_0);
static int dump_180102_error_0(dump_abcdef_error_l_ARGS_0);
static int dump_180601_error_0(dump_abcdef_error_l_ARGS_0);
static int dump_190511_error_0(dump_abcdef_error_l_ARGS_0);
static int dump_line(void *, int (*)(void *, const char *, unsigned),
		     const char *, unsigned, struct c1_back_type *);
static int dump_mode(void *, int (*)(void *, const char *, unsigned), unsigned,
		     int, int, int, struct c1_long_type *);
static int dump_type(void *, int (*)(void *, const char *, unsigned), int,
		     struct c1_long_type *);

static int
dump_080707_error_0(dump_abcdef_error_i_ARGS_1)
{
    int status;

    if (1) {
	status = this(data, ": expression syntax: ", 21);
	if (status) {
	} else {
	    if (long_data) {
		struct x1f4_eelookup_type eelookup;

		eelookup.type_l.context = long_data->type_l.context;
		eelookup.type_l.fix = long_data->type_l.fix;
		eelookup.type_q.e4fine = long_data->type_q.e4fine;

		status = x1f4_stat_expression
		    (data, this, &back_data->data.e_syntax.eerecord,
		     &eelookup);
	    } else {
		status = x1f4_stat_expression
		    (data, this, &back_data->data.e_syntax.eerecord, NULL);
	    }
	}
    }

    return status;
}


static int
dump_080707_error_1(dump_abcdef_error_l_ARGS_1)
{
    return this(data, ": void value not ignored as it ought to be", 42);
}


static int
dump_080707_error_2(dump_abcdef_error_l_ARGS_1)
{
    return this(data, ": syntax error", 14);
}


static int
dump_080707_error_3(dump_abcdef_error_l_ARGS_1)
{
    return this(data, ": missing prefix test statements block", 38);
}


static int
dump_080707_error_4(dump_abcdef_error_l_ARGS_1)
{
    return this(data, ": unexpected block completion syntax", 36);
}


static int
dump_080707_error_5(dump_abcdef_error_l_ARGS_1)
{
    return this(data, ": statement syntax error", 24);
}


static int
dump_080707_error_6(dump_abcdef_error_l_ARGS_1)
{
    int status;

    if (1) {
	status = this(data, ": variable `", 12);
	if (status) {
	} else {
	    status = this
		(data, back_data->line, back_data->data.variable.size);
	    if (status) {
	    } else {
		status = this(data, "' redeclared", 12);
	    }
	}
    }

    return status;
}


static int
dump_080707_error_7(dump_abcdef_error_l_ARGS_1)
{
    return this(data, ": missing block completion syntax", 33);
}


static int
dump_080707_error_8(dump_abcdef_error_l_ARGS_1)
{
    return this(data, ": missing do-while statements block", 35);
}


static int
dump_080707_error_9(dump_abcdef_error_l_ARGS_1)
{
    return this(data, ": missing block leave completion syntax", 39);
}


static int
dump_080707_error_a(dump_abcdef_error_l_ARGS_1)
{
    return this(data, ": unexpected block leave statement", 34);
}


static int
dump_080707_error_b(dump_abcdef_error_l_ARGS_1)
{
    int status;

    if (1) {
	status = this(data, ": invalid variable name (`", 26);
	if (status) {
	} else {
	    status = this
		(data, back_data->line, back_data->data.reserved.size);
	    if (status) {
	    } else {
		status = this(data, "')", 2);
	    }
	}
    }

    return status;
}


static int
dump_080707_error_c(dump_abcdef_error_l_ARGS_1)
{
    return this(data, ": missing declarations completion syntax", 40);
}


static int
dump_080707_error_d(dump_abcdef_error_l_ARGS_1)
{
    return this(data, ": do-while statement syntax error", 33);
}


static int
dump_080707_error_e(dump_abcdef_error_i_ARGS_1)
{
    int status;

    if (0) {
    } else {
	status = this
	    (data, ": mismatched type of `return' statement (expected ", 50);
	if (status) {
	} else {
	    status = dump_type
		(data, this, back_data->data.m_return.slip, long_data);
	    if (status) {
	    } else {
		status = this(data, ", found ", 8);
		if (status) {
		} else {
		    status = dump_type
			(data, this, back_data->data.m_return.type, long_data);
		    if (status) {
		    } else {
			status = this(data, ")", 1);
		    }
		}
	    }
	}
    }

    return status;
}


static int
dump_080707_error_f(dump_abcdef_error_l_ARGS_1)
{
    return this(data, "missing block completion syntax", 31);
}


static int
dump_090828_error_0(dump_abcdef_error_l_ARGS_1)
{
    return this(data, ": unexpected `return' statement", 31);
}


static int
dump_090828_error_1(dump_abcdef_error_l_ARGS_1)
{
    return this(data, ": empty `return' statement", 26);
}


static int
dump_091025_error_0(dump_abcdef_error_i_ARGS_1)
{
    int status;

    if (1) {
	status = this(data, ": used ", 7);
	if (status) {
	} else {
	    status = dump_type
		(data, this, back_data->data.w_scalar.type, long_data);
	    if (status) {
	    } else {
		status = this(data, " value where scalar is required", 31);
	    }
	}
    }

    return status;
}


static int
dump_110725_error_0(dump_abcdef_error_i_ARGS_1)
{
    int status;

    if (1) {
	status = this(data, ": non declarable ", 17);
	if (status) {
	} else {
	    status = dump_type
		(data, this, back_data->data.w_scalar.type, long_data);
	    if (status) {
	    } else {
		status = this(data, " data type", 10);
	    }
	}
    }

    return status;
}


static int
dump_111221_error_0(dump_abcdef_error_l_ARGS_1)
{
    int status;

    status = this(data, ": ", 2);
    if (status) {
    } else {
	status = x1f4_stat_e4jack
	    (data, this, &back_data->data.higherof.ejrecord);
    }

    return status;
}


static int
dump_120115_error_0(dump_abcdef_error_l_ARGS_1)
{
    int status;

    status = this(data, ": bad `", 7);
    if (status) {
    } else {
	const char *line;

	line = back_data->line;

	status = this(data, line, 1);
	if (status) {
	} else {
	    status = this(data, "' (", 3);
	    if (status) {
	    } else {
		status = x1f4_vprint_fcardinal
		    (data, this, 0, 4, 8, *(unsigned char *) line);
		if (status) {
		} else {
		    status = this
			(data, ") character where variable name expected", 40);
		}
	    }
	}
    }

    return status;
}


static int
dump_170722_error_0(dump_abcdef_error_l_ARGS_1)
{
    return this(data, ": no known traversers", 21);
}


static int
dump_170722_error_1(dump_abcdef_error_l_ARGS_1)
{
    return this(data, ": variable name expected", 24);
}


static int
dump_170722_error_2(dump_abcdef_error_l_ARGS_1)
{
    return this(data, ": second variable name expected", 31);
}


static int
dump_170722_error_3(dump_abcdef_error_l_ARGS_1)
{
    return this(data, ": `in' or `of' expected in traversal syntax", 43);
}


static int
dump_170722_error_4(dump_abcdef_error_l_ARGS_1)
{
    return this(data, ": missing traversal statements block", 36);
}


static int
dump_170722_error_5(dump_abcdef_error_l_ARGS_1)
{
    int status;

    status = this(data, ": undefined `", 13);
    if (status) {
    } else {
	status = this(data, back_data->line, back_data->data.e_lookup.size);
	if (status) {
	} else {
	    status = this(data, "' variable", 10);
	}
    }

    return status;
}


static int
dump_170722_error_6(dump_abcdef_error_i_ARGS_1)
{
    return dump_mode
	(data, this, back_data->data.t1sttype.mode,
	 back_data->data.t1sttype.arg1, X1f4_E4_VOID,
	 back_data->data.t1sttype.type, long_data);
}


static int
dump_170722_error_7(dump_abcdef_error_i_ARGS_1)
{
    return dump_mode
	(data, this, back_data->data.t2ndtype.mode,
	 back_data->data.t2ndtype.arg1, back_data->data.t2ndtype.arg2,
	 back_data->data.t2ndtype.type, long_data);
}


static int
dump_170731_error_0(dump_abcdef_error_i_ARGS_1)
{
    int status;

    status = this(data, ": undeclarable collection type ", 31);
    if (status) {
    } else {
	status = dump_type
	    (data, this, back_data->data.foldtype.type, long_data);
    }

    return status;
}


static int
dump_180102_error_0(dump_abcdef_error_l_ARGS_1)
{
    int status;

    status = this(data, ": read only `", 13);
    if (status) {
    } else {
	status = this(data, back_data->line, back_data->data.e_lookup.size);
	if (status) {
	} else {
	    status = this(data, "' variable in traversal", 23);
	}
    }

    return status;
}


static int
dump_180601_error_0(dump_abcdef_error_l_ARGS_1)
{
    int status;

    status = this(data, ": unknown `", 11);
    if (status) {
    } else {
	status = this(data, back_data->line, back_data->data.losttype.size);
	if (status) {
	} else {
	    status = this(data, "' type", 6);
	}
    }

    return status;
}


static int
dump_190511_error_0(dump_abcdef_error_l_ARGS_1)
{
    int status;

    status = this(data, ": unknown `", 11);
    if (status) {
    } else {
	status = this(data, back_data->line, back_data->data.unclause.size);
	if (status) {
	} else {
	    status = this(data, "' for clause", 12);
	}
    }

    return status;
}


static int
dump_line(void *data, int (*this)(void *, const char *, unsigned),
	  const char *head, unsigned bits, struct c1_back_type *back_data)
{
    int status;

    if (bits & HINT_MASK) {
	status = back_data->hint.rule
	    (back_data->hint.data, back_data->pick, back_data->line - head);
    } else {
	const char *dana, *line;
	unsigned byte, rank;

	line = back_data->line;

	rank = back_data->pick;

	byte = line - head;

	dana = head;
	while (byte) {
	    byte--;
	    if (*dana == 10) {
		rank++;
		head = dana;
	    }

	    dana++;
	}

	status = x1f4_vprint_dcardinal(data, this, rank);
    }

    return status;
}


static int
dump_mode(void *data, int (*this)(void *, const char *, unsigned),
	  unsigned mode, int arg1, int arg2, int type,
	  struct c1_long_type *long_data)
{
    int status;

    do {
	status = this(data, ": no", 4);
	if (status) {
	    break;
	}
	if (mode & X1f4_PROGRESS_POSITION) {
	    const char *pick;

	    switch (mode & X1f4_PROGRESS_POSITION) {
	    case X1f4_PROGRESS_GECLAUSE:
		pick = " -ge";
		break;
	    case X1f4_PROGRESS_GTCLAUSE:
		pick = " -gt";
		break;
	    case X1f4_PROGRESS_LECLAUSE:
		pick = " -le";
		break;
	    case X1f4_PROGRESS_LTCLAUSE:
		pick = " -lt";
		break;
	    default:
		pick = " -eq";
	    }
	    status = this(data, pick, 4);
	    if (status) {
		break;
	    }
	}
	if (mode & X1f4_PROGRESS_BACKWARD) {
	    status = this(data, " backward", 9);
	    if (status) {
		break;
	    }
	}
	status = this(data, " traverser for the types ", 25);
	if (status) {
	    break;
	}
	status = dump_type(data, this, arg1, long_data);
	if (status) {
	    break;
	}
	if (arg2 != X1f4_E4_VOID) {
	    status = this(data, ", ", 2);
	    if (status) {
		break;
	    }
	    status = dump_type(data, this, arg2, long_data);
	    if (status) {
		break;
	    }
	}
	status = this(data, " and ", 5);
	if (status) {
	    break;
	}
	status = dump_type(data, this, type, long_data);
    } while (0);

    return status;
}


static int
dump_type(void *data, int (*this)(void *, const char *, unsigned), int type,
	  struct c1_long_type *long_data)
{
    struct x1f4_eelookup_type eelookup;

    eelookup.type_l.context = long_data->type_l.context;
    eelookup.type_l.fix = long_data->type_l.fix;
    eelookup.type_q.e4fine = long_data->type_q.e4fine;

    return x1f4_dump_type(data, this, type, &eelookup);
}


int
x1f4_lame_program(void *data, int (*this)(void *, const char *, unsigned),
		  const char *head, unsigned bits,
		  struct c1_back_type *back_data,
		  struct x1f4_eelookup_type *eelookup_data)
{
    int status;

    switch (back_data->code) {
    case L_SCOPES:
	status = dump_080707_error_f(data, this, head, back_data);
	break;
    default:
	status = dump_line(data, this, head, bits, back_data);
	if (status) {
	} else {
	    switch (back_data->code) {
	    case BLOCKOUT:
		status = dump_080707_error_4(data, this, head, back_data);
		break;
	    case B_SYNTAX:
		status = dump_080707_error_2(data, this, head, back_data);
		break;
	    case COMPLETE:
		status = dump_080707_error_c(data, this, head, back_data);
		break;
	    case CROSSING:
		status = dump_170722_error_0(data, this, head, back_data);
		break;
	    case E_LOOKUP:
		status = dump_170722_error_5(data, this, head, back_data);
		break;
	    case E_RETURN:
		status = dump_090828_error_1(data, this, head, back_data);
		break;
	    case E_SYNTAX:
		status = dump_080707_error_0
		    (data, this, head, back_data, eelookup_data);
		break;
	    case FOLDTYPE:
		status = dump_170731_error_0
		    (data, this, head, back_data, eelookup_data);
		break;
	    case HIGHEROF:
		status = dump_111221_error_0(data, this, head, back_data);
		break;
	    case INEFFECT:
		status = dump_170722_error_3(data, this, head, back_data);
		break;
	    case LOOPEXIT:
		status = dump_080707_error_9(data, this, head, back_data);
		break;
	    case LOOSE_DB:
		status = dump_080707_error_7(data, this, head, back_data);
		break;
	    case LOSTTYPE:
		status = dump_180601_error_0(data, this, head, back_data);
		break;
	    case MISSFOLD:
		status = dump_170722_error_4(data, this, head, back_data);
		break;
	    case MISSLOOP:
		status = dump_080707_error_8(data, this, head, back_data);
		break;
	    case MISSTEST:
		status = dump_080707_error_3(data, this, head, back_data);
		break;
	    case M_RETURN:
		status = dump_080707_error_e
		    (data, this, head, back_data, eelookup_data);
		break;
	    case NAMEMISS:
		status = dump_120115_error_0(data, this, head, back_data);
		break;
	    case OUTBREAK:
		status = dump_080707_error_a(data, this, head, back_data);
		break;
	    case RESERVED:
		status = dump_080707_error_b(data, this, head, back_data);
		break;
	    case SIDEDATA:
		status = dump_110725_error_0
		    (data, this, head, back_data, eelookup_data);
		break;
	    case S_SYNTAX:
		status = dump_080707_error_d(data, this, head, back_data);
		break;
	    case T1STNAME:
		status = dump_170722_error_1(data, this, head, back_data);
		break;
	    case T1STTYPE:
		status = dump_170722_error_6
		    (data, this, head, back_data, eelookup_data);
		break;
	    case T2NDNAME:
		status = dump_170722_error_2(data, this, head, back_data);
		break;
	    case T2NDTYPE:
		status = dump_170722_error_7
		    (data, this, head, back_data, eelookup_data);
		break;
	    case T_SVALUE:
		status = dump_180102_error_0(data, this, head, back_data);
		break;
	    case T_SYNTAX:
		status = dump_080707_error_5(data, this, head, back_data);
		break;
	    case UNCLAUSE:
		status = dump_190511_error_0(data, this, head, back_data);
		break;
	    case USEDVOID:
		status = dump_080707_error_1(data, this, head, back_data);
		break;
	    case VARIABLE:
		status = dump_080707_error_6(data, this, head, back_data);
		break;
	    case W_SCALAR:
		status = dump_091025_error_0
		    (data, this, head, back_data, eelookup_data);
		break;
	    case X_RETURN:
		status = dump_090828_error_0(data, this, head, back_data);
		break;
	    default:
		/*
		 * with UNLIKELY included
		 */
		status = this(data, "no description available", 24);
	    }
	}
    }

    return status;
}

