/*
 * e4-e.i.c
 * Copyright (C) 2006-2012, 2014, 2016, 2017, Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

/*
 * TODO
 *
 * replace variables in LOCK typed atoms
 */

#include <stddef.h>

#include <e4-defs.h>
#include <e4-inter.h>
#include <e4-types.h>

#define LOGIC_BITS \
    ((1 << LAST) | (1 << LOCK) | (1 << PICK))

#define EXPRESSION(expression) \
    ((e4_expression_type *) expression)

#define E_FREE(mcontext, mdata) \
    (EXPRESSION(mcontext)->m.free(EXPRESSION(mcontext)->m.data, (mdata)))
#define E_LINK(mcontext, mdata, size) \
    (EXPRESSION(mcontext)->m.link					      \
	(EXPRESSION(mcontext)->m.data, (void *) (mdata), (size)))
#define E_MODE(mcontext, mdata, size) \
    (EXPRESSION(mcontext)->m.mode					      \
	(EXPRESSION(mcontext)->m.data, (void *) (mdata), (size)))

static int ever_hack(void *, struct e4_hack_type *, struct e4_ever_type *,
		     void *, int *);
static int ever_last(void *, struct e4_atom_type *, struct e4_ever_type *,
		     void *, int *);
static int ever_miss(void *, struct e4_atom_type *, void *);

static int
ever_hack(void *expression, struct e4_hack_type *hack_data,
	  struct e4_ever_type *ever_data, void *expressiOn, int *past)
{
    int fast = 0, status = 0;
    struct e4_atom_type *atom_data;
    unsigned i;

    atom_data = hack_data->atom_data;

    i = hack_data->count;
    for (; i; i--) {
	int type;

	type = atom_data->type;
	if (type == EVER) {
	    if (atom_data->data.ever.ever_data == ever_data) {
		status = ever_miss(expression, atom_data, expressiOn);
		if (status) {
		    break;
		} else {
		    *past = fast = 1;
		    if (1) {
			break;
		    }
		}
	    }
	} else {
	    if (1 << type & LOGIC_BITS) {
		status = ever_last
		    (expression, atom_data, ever_data, expressiOn, past);
		if (*past | status) {
		    break;
		}
	    } else {
		if (type == THIS) {
		    status = ever_hack
			(expression, atom_data->data.hack.data, ever_data,
			 expressiOn, past);
		    if (*past | status) {
			break;
		    }
		}
	    }
	}

	atom_data++;
    }

    if (fast) {
	_x1f4_e4_fast_hack(hack_data, NULL);
    }

    return status;
}


static int
ever_last(void *expression, struct e4_atom_type *atom_data,
	  struct e4_ever_type *ever_data, void *expressiOn, int *past)
{
    int status = 0;
    const int *args;
    struct e4_hack_type *hack_data;
    const struct e4_last_type *last_data;
    unsigned bits, count;

    hack_data = atom_data->data.last.base;
    last_data = hack_base(hack_data)->last_data;
    bits = last_data->bits;
    count = last_data->count;
    args = last_data->args + count;
    for (; count; count--) {
	status = ever_hack(expression, hack_data, ever_data, expressiOn, past);
	if (status) {
	    break;
	}

	if (*past) {
	    unsigned class;

	    if (bits & POST_TYPE) {
		class = *args++;
	    } else {
		class = 0;
	    }

	    _x1f4_e4_fast_miss(hack_data, class);

	    break;
	}

	hack_data++;
    }

    return status;
}


static int
ever_miss(void *expression, struct e4_atom_type *atom_data, void *expressiOn)
{
    int status;
    struct e4_hack_type *hack_data;

    status = E_LINK(expression, &hack_data, sizeof(struct e4_hack_type));
    if (status) {
    } else {
	struct e4_flat_type *flat_data;

	atom_data->load = _x1f4_e4_load_this;
	atom_data->type = THIS;
	atom_data->data.hack.data = hack_data;

	*hack_data = EXPRESSION(expressiOn)->hack;

	EXPRESSION(expressiOn)->hack.atom_data = NULL;
	EXPRESSION(expressiOn)->hack.count = 0;

	_x1f4_e4_fast_hack(hack_data, NULL);

	flat_data = EXPRESSION(expressiOn)->flat_data;
	if (flat_data) {
	    struct e4_flat_type *flat_text;

	    EXPRESSION(expressiOn)->flat_data = NULL;

	    flat_text = EXPRESSION(expression)->flat_data;

	    EXPRESSION(expression)->flat_data = flat_data;

	    if (flat_text) {
		struct e4_flat_type *flat_slip;

		flat_slip = flat_data->flat_data;
		while (flat_slip) {
		    flat_data = flat_slip;
		    flat_slip = flat_data->flat_data;
		}

		flat_data->flat_data = flat_text;
	    }
	}
    }

    return status;
}


int
x1f4_rack_expression(struct e4_expression_type *expression_data,
		     struct e4_ever_type *ever_data,
		     struct e4_expression_type *expression_text)
{
    int status;

    if (ever_data->type != expression_text->type) {
	status = 1;
    } else {
	int past = 0;
	struct x1f4_trans_type trans;

	trans = expression_text->m;

	expression_text->m = expression_data->m;

	status = ever_hack
	    (expression_data, &expression_data->hack, ever_data,
	     expression_text, &past);
	if (status) {
	} else {
	    status = _x1f4_e4_sink_expression(expression_data);
	}

	expression_text->m = trans;
    }

    return status;
}
