/*
 * e4.7.3.c
 * Copyright (C) 2006-2014, 2017, Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <config.h>

#include <e4-config.h>

#include <stddef.h>
#include <string.h>

#include <e4-defs.h>
#include <e4-inter.h>
#include <e4-parse.h>
#include <e4-types.h>

#define M_FREE(mcontext, mdata) \
    (mcontext)->m.free((mcontext)->m.data, (mdata))
#define M_LINK(mcontext, mdata, size) \
    (mcontext)->m.link((mcontext)->m.data, (void *) (mdata), (size))
#define M_MODE(mcontext, mdata, size) \
    (mcontext)->m.mode((mcontext)->m.data, (void *) (mdata), (size))

static int lend_atom(struct e4_atom_type **, unsigned *, int *,
		     const struct e4_odbx_type *const *, struct parser_type *);

/*
 * will allow not matching unary/ternary priorities at this time
 */
static int
lend_atom(struct e4_atom_type **atom, unsigned *lend, int *none,
	  const struct e4_odbx_type *const *list,
	  struct parser_type *parser_data)
{
    int call, status = 0;
    struct e4_atom_type *atom_data;
    struct e4_odb1_type *odb1_data, *odb1_miss = NULL;

    atom_data = *atom;

    call = ((struct e4_odbx_type *) atom_data->odb2._1st)->pass;

    odb1_data = atom_data->odb1;
    while (odb1_data) {
	const char *e;
	struct text_type *text_data;
	unsigned seek;

	text_data = (struct text_type *) (odb1_data + 1);

	e = text_data->data;
	seek = text_data->size;
	while (seek) {
	    int mark;
	    const struct e4_odbx_type *const *odbx, *odbx_data, *odbx_text =
		NULL;
	    unsigned toxic = 0;

	    odbx = list;

	    odbx_data = *odbx;
	    while (odbx_data) {
		unsigned shift;

		shift = odbx_data->size;
		if (toxic < shift) {
		    if (seek < shift) {
		    } else {
			const char *name;

			name = odbx_data->name;
			if (memcmp((void *) (e - shift), name, shift)) {
			} else {
			    odbx_text = odbx_data;
			    toxic = shift;
			}
		    }
		}

		odbx++;
		odbx_data = *odbx;
	    }

	    if (odbx_text) {
	    } else {
		status = _x1f4_e4_080704_error_i(parser_data, e, seek);
		break;
	    }

	    mark = odbx_text->pass;
	    if (call < mark) {
		struct e4_atom_type *atom_text;
		struct e4_hack_type *hack_data;
		struct e4_odb1_type *odb1_half = NULL;
		unsigned class, half, part;

		atom_text = atom_data;
		part = *lend;
		for (; part; part--) {
		    odbx_data = atom_text->odb2._1st;
		    if (odbx_data->pass < mark) {
			if (odbx_data->bits & LEAD_TRIP) {
			    struct trip_type *trip_data;

			    trip_data = (void *)
				((char *) odbx_data
				 - offsetof(struct trip_type, lead));
			    if (trip_data->rail.pass < mark) {
			    } else {
				status = _x1f4_e4_170621_error_1
				    (parser_data, odbx_text, trip_data);
				break;
			    }
			}

			atom_text++;
		    } else {
			break;
		    }
		}
		if (status) {
		    break;
		}

		half = text_data->size - seek;

		if (half) {
		    status = M_LINK
			(parser_data, &odb1_half,
			 sizeof(struct e4_odb1_type)
			 + sizeof(struct text_type));
		    if (status) {
			status = ALLOC_ERROR;

			seek = ~0;

			break;
		    } else {
			struct text_type *text_half;

			text_half = (struct text_type *) (odb1_half + 1);

			odb1_half->odb1_data = NULL;

			text_half->data = text_data->data;
			text_half->size = half;

			text_data->data = e;
			text_data->size = seek;
		    }
		}

		atom_text++;

		class = atom_text - atom_data;

		status = M_LINK
		    (parser_data, &hack_data, sizeof(struct e4_hack_type));
		if (status) {
		    status = ALLOC_ERROR;

		    if (odb1_half) {
			M_FREE(parser_data, odb1_half);
		    }
		} else {
		    struct e4_atom_type *atom_news;
		    unsigned link;

		    link = class * sizeof(struct e4_atom_type);

		    status = M_LINK(parser_data, &atom_news, link);
		    if (status) {
			status = ALLOC_ERROR;

			M_FREE(parser_data, hack_data);
			if (odb1_half) {
			    M_FREE(parser_data, odb1_half);
			}
		    } else {
			memcpy(atom_news, atom_data, link);

			hack_data->atom_data = atom_news;
			hack_data->bits = READ_ONLY;
			hack_data->count = class;

			atom_data->data.hack.data = hack_data;
			atom_data->type = THIS;

			if (odb1_miss) {
			    atom_data->odb1 = odb1_data;
			    if (odb1_half) {
				odb1_miss->odb1_data = odb1_half;
			    } else {
				odb1_miss->odb1_data = NULL;
			    }
			} else {
			    if (odb1_half) {
				atom_news->odb1 = odb1_half;
			    } else {
				atom_news->odb1 = NULL;
			    }
			}

			class--;

			atom_news += class;

			atom_data->odb2._1st = atom_news->odb2._1st;

			atom_data++;

			part = *lend - class;

			memmove(atom_data, atom_text,
				part * sizeof(struct e4_atom_type));

			*none = 0;
			*lend = part;
		    }
		}

		seek = ~0;

		break;
	    }

	    e -= toxic;
	    seek -= toxic;
	}

	if (seek | status) {
	    break;
	}

	odb1_miss = odb1_data;

	odb1_data = odb1_data->odb1_data;
    }

    return status;
}


int
_x1f4_e4_trip_hack(struct e4_hack_type *hack_data, void *parser)
{
    int status = 0;
    struct e4_atom_type *atom_data;
    unsigned count;

    count = hack_data->count;

    atom_data = hack_data->atom_data;
    count--;
    while (count) {
	if (atom_data->odb1) {
	    int none = 1;
	    const struct e4_odbx_type *const *list;

	    list = ((struct parser_type *) parser)->context_data->odb1_data;
	    if (!list) {
		status = _x1f4_e4_080704_error_g(parser);
		if (1) {
		    break;
		}
	    } else {
		status = lend_atom(&atom_data, &count, &none, list, parser);
		if (status) {
		    break;
		} else {
		    if (none) {
			count--;
			atom_data++;
		    }
		}
	    }
	} else {
	    count--;
	    atom_data++;
	}
    }

    count += atom_data - hack_data->atom_data + 1;
    hack_data->count = count;

    if (status) {
    } else {
	struct parser_type *parser_data;

	parser_data = parser;
	if (parser_data->bits & PACK_ESTORAGE) {
	    M_MODE(parser_data, &hack_data->atom_data,
		   count * sizeof(struct e4_atom_type));
	}
    }

    return status;
}
