/*
 * file.i.n.c
 * Copyright (C) 2013, 2014, 2016, 2017, 2019, Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <dirent.h>
#include <errno.h>
#include <fcntl.h>
#include <stddef.h>
#include <string.h>

#include <sys/types.h>

#include <file-defs.h>
#include <file-types.h>
#include <lxdata.h>

#define file(file) \
    ((struct cdfile_type *) (file))

static int fine_file(void *);
static int hold_file(void *, int *);
static int lose_file(void *);
static int pull_file(void *, void *, unsigned);

static off_t seek_file(void *, off_t, int);

const struct sffile_type _libx1f4_file_i_shfile = {
/* *INDENT-OFF* */
    fine_file,
    hold_file,
    NULL,
    lose_file,
    NULL,
    pull_file,
    NULL,
    seek_file,
    HOLD_LINK
/* *INDENT-ON* */
};

static int
fine_file(void *file)
{
    return fcntl(dirfd(file(file)->data), F_SETFD, FD_CLOEXEC);
}


static int
hold_file(void *file, int *fd)
{
    int delete;

    delete = dirfd(file(file)->data);
    if (~delete) {
	*fd = delete;
	delete = 0;
    }

    return delete;
}


static int
lose_file(void *file)
{
    return closedir(file(file)->data);
}


static int
pull_file(void *file, void *flat, unsigned size)
{
    int status;

    if (size) {
	struct dirent *line;

	errno = 0;

	line = readdir(file(file)->data);
	if (line) {
	    unsigned copy;

	    copy = strlen(line->d_name) + 1;

	    if (size < copy) {
		copy = size - 1;
		memcpy(flat, line->d_name, copy);
		*((char *) flat + copy) = 0;
		copy++;
	    } else {
		memcpy(flat, line->d_name, copy);
	    }

	    status = copy;
	} else {
	    status = errno ? -1 : 0;
	}
    } else {
	status = 0;
    }

    return status;
}


static off_t
seek_file(void *file, off_t offset, int whence)
{
    errno = ENOSYS;

    return -1;
}
