/*
 * file.o.j.c
 * Copyright (C) 2013, 2014, 2016, 2017, 2020, Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <fcntl.h>
#include <stddef.h>
#include <unistd.h>

#include <sys/types.h>

#include <file-types.h>

#if __CYGWIN__
/*
 * single write horribly slow on cygwin (6.1 3.1.2?)
 */
# define __BYTE_SIZE__			1
#else
# define __BYTE_SIZE__			0
#endif				/* __CYGWIN__ */

static int fine_file(void *);
static int hold_file(void *, int *);
static int kate_file(void *, off_t);
static int lose_file(void *);
static int push_file(void *, void *, unsigned);

static off_t seek_file(void *, off_t, int);

const struct sffile_type _libx1f4_file_o_sffile = {
/* *INDENT-OFF* */
    fine_file,
    hold_file,
    kate_file,
    lose_file,
    NULL,
    NULL,
    push_file,
    seek_file,
    0
/* *INDENT-ON* */
};

static int
fine_file(void *file)
{
    return fcntl(*(int *) file, F_SETFD, FD_CLOEXEC);
}


static int
hold_file(void *file, int *fd)
{
    *fd = *(int *) file;

    return 0;
}


static int
kate_file(void *file, off_t offset)
{
    return ftruncate(*(int *) file, offset);
}


static int
lose_file(void *file)
{
    return close(*(int *) file);
}


static int
push_file(void *file, void *flat, unsigned size)
{
#if __BYTE_SIZE__
    int fd, status;
    unsigned bend;

    fd = *(int *) file;
    bend = size;
    while (bend) {
	status = write(fd, flat, bend < 4096 ? bend : 4096);
	if (status < 1) {
	    break;
	}

	bend -= status;
	flat = (char *) flat + status;
    }

    return bend ? -1 : size;
#else
    return write(*(int *) file, flat, size);
#endif				/* __BYTE_SIZE__ */
}


static off_t
seek_file(void *file, off_t offset, int whence)
{
    return lseek(*(int *) file, offset, whence);
}
