/*
 * lxcall-a.9.c
 * Copyright (C) 2011-2014, 2016, 2017, 2019, 2020, Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <stddef.h>

#include <e4.h>
#include <fd.h>
#include <it.h>
#include <lxcall-defs.h>
#include <lxcall-inter.h>
#include <lxcast.h>
#include <track.h>
#include <trans.h>

static int miss_call(void *, int, void *, struct x1f4_dispatch_type **);
static int slip_call(void **, void **, const struct x1f4_function_type *,
		     unsigned, unsigned, struct x1f4_linetext_type *, unsigned,
		     int *);

static int
miss_call(void *text, int success, void *output,
	  struct x1f4_dispatch_type **dispatch)
{
    *dispatch = NULL;

    return success;
}


static int
slip_call(void **input, void **track,
	  const struct x1f4_function_type *function_data, unsigned offset,
	  unsigned bits, struct x1f4_linetext_type *linetext_data,
	  unsigned norm, int *lock)
{
    int status = 0;

    do {
	struct x1f4_dispatch_type *dispatch_data;
	void *output, *subtext;
	unsigned turn;

	turn = linetext_data->function.flags;

	output = input[0];

	subtext = (void *) linetext_data->context;

	if (turn & (X1f4_E4_SIDE_LIST | X1f4_E4_SLIP_LIST)) {
	    for (; norm; norm--) {
		status = _libx1f4i0_lxcall_deck_call
		    (track, function_data, offset, linetext_data, *lock);
		if (status) {
		    break;
		} else {
		    lock++;
		}
	    }
	    if (status) {
		break;
	    }
	} else {
	    ((void **) track[-1])[X1f4_E4_RENDITION] =
		&linetext_data->function;
	}

	dispatch_data = input[2];

	dispatch_data->back = NULL;
	dispatch_data->call = miss_call;

	dispatch_data->transfer.context = subtext;
	dispatch_data->transfer.function_data = &linetext_data->function;
	dispatch_data->transfer.input = track;
	dispatch_data->transfer.output = output;
    } while (0);

    return status;
}


int
_libx1f4i0_lxcall_into_call(struct x1f4_linetext_type *linetext_data,
			    const struct x1f4_function_type *function_data,
			    unsigned offset, void **input,
			    unsigned bits, struct x1f4_iturn_type *iturn_data,
			    struct x1f4_composer_type *composer_data,
			    struct x1f4_track_type *track_data,
			    const struct x1f4_eelookup_type *eelookup_data,
			    unsigned norm, int *lock,
			    struct fxdata_type *fxdata_data,
			    struct x1f4_dxcast_type *dxcast_data)
{
    int status;
    unsigned i;
    struct center_type center;
    void *meta, **track;

    track = input[1];

    meta = *(track - 1);

    track += offset;

    center.composer_data = composer_data;
    center.eelookup_data = eelookup_data;
    center.function_text = &linetext_data->function;
    center.iturn_data = iturn_data;
    center.track_data = track_data;
    center.input = &track;

    i = norm;
    while (1) {
	status = _libx1f4i0_lxcall_even_mark
	    (&center, &linetext_data->function, function_data, offset, bits,
	     *lock, fxdata_data, dxcast_data);
	if (status) {
	    break;
	} else {
	    i--;
	    if (i) {
		lock++;
		fxdata_data++;
		dxcast_data++;
	    } else {
		break;
	    }
	}
    }
    if (status) {
    } else {
	track[-1] = meta;

	lock -= norm - 1;

	status = slip_call
	    (input, track, function_data, offset, bits, linetext_data, norm,
	     lock);
    }

    return status;
}
