/*
 * lxcall-e.f.c
 * Copyright (C) 2011-2014, 2016, 2017, 2019, 2020, Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <copy.h>
#include <e4.h>
#include <exerrors.h>
#include <fd.h>
#include <it.h>
#include <lxcall-types.h>
#include <lxcast.h>

#define I_BILL(t)			(*((X1f4_E4_C_BILL *) (t)))
#define I_MODE(t)			(*((X1f4_E4_C_MODE *) (t)))
#define I_REAL(t)			(*((X1f4_E4_C_REAL *) (t)))

#define l_BILL(e, output) \
    {									      \
	X1f4_E4_C_BILL *l;						      \
									      \
	l = (output);							      \
	*l = (e);							      \
    }

#define l_MODE(e, output) \
    {									      \
	X1f4_E4_C_MODE *l;						      \
									      \
	l = (output);							      \
	*l = (e);							      \
    }

#define l_REAL(e, output) \
    {									      \
	X1f4_E4_C_REAL *l;						      \
									      \
	l = (output);							      \
	*l = (e);							      \
    }

#define SCALAR_BITS \
    ((1 << X1f4_E4_BILL) | (1 << X1f4_E4_MODE) | (1 << X1f4_E4_REAL))

static int type_name(int, int, void *, void *);

static int
type_name(int type, int name, void *lead, void *seek)
{
    switch (type) {
    case X1f4_E4_BILL:
	if (name == X1f4_E4_MODE) {
	    l_BILL(I_MODE(lead), seek);
	} else {
	    l_BILL(I_REAL(lead), seek);
	}
	break;
    case X1f4_E4_MODE:
	if (name == X1f4_E4_BILL) {
	    l_MODE(I_BILL(lead), seek);
	} else {
	    l_MODE(I_REAL(lead), seek);
	}
	break;
    default:
	if (name == X1f4_E4_BILL) {
	    l_REAL(I_BILL(lead), seek);
	} else {
	    l_REAL(I_MODE(lead), seek);
	}
    }

    return 0;
}


int
_libx1f4i0_lxcall_rule_fare(struct center_type *center_data, int name,
			    int type, void **track,
			    struct fxdata_type **missed,
			    struct x1f4_dxcast_type **dxcast)
{
    int status = 0;
    struct fxdata_type *missed_data;

    missed_data = *missed;

    do {
	if ((type | name) & ~15) {
	} else {
	    if ((1 << type) & SCALAR_BITS && (1 << name) & SCALAR_BITS) {
		type_name(type, name, *track, missed_data);
		*track = missed_data;
		missed_data++;
		break;
	    }
	}

	if (name == X1f4_E4_CASE) {
	    int fold;
	    struct x1f4_dxcast_type *dxcast_data;

	    dxcast_data = *(void **) *track;
	    fold = dxcast_data->type;
	    if (fold ^ type) {
		if ((type | fold) & ~15) {
		} else {
		    if ((1 << type) & SCALAR_BITS
			&& (1 << fold) & SCALAR_BITS) {
			type_name(type, fold, &dxcast_data->data, missed_data);
			*track = missed_data;
			missed_data++;
			break;
		    }
		}
	    } else {
		copy_miss(missed_data, type, &dxcast_data->data);
		*track = missed_data;
		missed_data++;
		break;
	    }
	} else {
	    if (type == X1f4_E4_CASE) {
		struct x1f4_dxcast_type *dxcast_slip;

		dxcast_slip = *dxcast;
		*(void **) missed_data = dxcast_slip;
		copy_miss(&dxcast_slip->data, name, *track);
		dxcast_slip++->type = name;
		*dxcast = dxcast_slip;
		*track = missed_data;
		missed_data++;
		break;
	    }
	}

	if (center_data->iturn_data->link) {
	    const struct x1f4_operator_type *operator_data;

	    status = center_data->iturn_data->link
		(center_data->iturn_data->text, type, name, &operator_data);
	    if (status) {
	    } else {
		status = operator_data->operator(missed_data, track);
		if (status) {
		    break;
		} else {
		    *track = missed_data;
		    missed_data++;
		    break;
		}
	    }
	}

	if (center_data->composer_data->link) {
	    const struct x1f4_linetext_type *linetext_data;

	    status = center_data->composer_data->link
		(center_data->composer_data->text, type, name, &linetext_data);
	    if (status) {
	    } else {
		status = linetext_data->function.function
		    ((void *) linetext_data->context, missed_data, track);
		if (status) {
		    break;
		} else {
		    *track = missed_data;
		    missed_data++;
		    break;
		}
	    }
	}

	status = X1f4_EX_CANNOT_CONTINUE;
    } while (0);

    *missed = missed_data;

    return status;
}
