/*
 * lxcall-e.g.c
 * Copyright (C) 2011-2014, 2017, 2020, Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <string.h>

#include <copy.h>
#include <e4.h>
#include <fd.h>
#include <it.h>
#include <lxcall-inter.h>
#include <lxcall-types.h>
#include <lxcast.h>
#include <track.h>

static int rule_line(struct center_type *, const struct x1f4_function_type *,
		     unsigned,
		     unsigned, const struct x1f4_function_type *, unsigned,
		     unsigned, unsigned, void **);

static int
rule_line(struct center_type *center_data,
	  const struct x1f4_function_type *function_text,
	  unsigned o, unsigned post,
	  const struct x1f4_function_type *function_data, unsigned offset,
	  unsigned h, unsigned mind, void **track)
{
    const int *data;
    int delete, status = 0;
    const int *text;
    struct fxdata_type *missed_data;
    struct x1f4_dxcast_type *dxcast_slip;

    data = function_data->args + offset;
    text = function_text->args;

    missed_data = (void *) (track + h);
    missed_data++;

    dxcast_slip = (void *) (missed_data + h);

    h += offset;

    delete = o;
    for (; delete; delete--, track++) {
	int name, type;

	name = *data;
	type = *text;
	if (type == X1f4_E4_SLIP) {
	    type = name;
	}

	if (name ^ type) {
	    status = _libx1f4i0_lxcall_rule_fare
		(center_data, name, type, track, &missed_data, &dxcast_slip);
	    if (status) {
		break;
	    }
	} else {
	    if (type == X1f4_E4_CASE) {
		*(void **) missed_data = dxcast_slip;
		*dxcast_slip++ =
		    *(struct x1f4_dxcast_type *) *(void **) *track;
		*track = missed_data;
		missed_data++;
	    } else {
		copy_miss(missed_data, name, *track);
		*track = missed_data;
		missed_data++;
	    }
	}

	data++;
	text++;
    }
    if (delete) {
    } else {
	delete = h - o - offset;
	for (; delete; delete--, track++) {
	    int type;

	    type = *data++;
	    if (type == X1f4_E4_CASE) {
		*(void **) missed_data = dxcast_slip;
		*dxcast_slip++ =
		    *(struct x1f4_dxcast_type *) *(void **) *track;
		*track = missed_data;
		missed_data++;
	    } else {
		copy_miss(missed_data, type, *track);
		*track = missed_data;
		missed_data++;
	    }
	}
    }

    return status;
}


int
_libx1f4i0_lxcall_rule_list(const struct x1f4_function_type *function_text,
			    const struct x1f4_function_type *function_data,
			    unsigned offset, unsigned record, void ***input,
			    struct x1f4_trans_type *trans_data,
			    struct x1f4_iturn_type *iturn_data,
			    struct x1f4_composer_type *composer_data,
			    struct x1f4_track_type *track_data,
			    const struct x1f4_eelookup_type *eelookup_data)
{
    int delete, status = 0;
    unsigned h, o;

    h = function_data->count;
    o = function_text->count;

    delete = h - o - offset;
    if (delete && !(0 < delete && function_text->flags & X1f4_E4_SIDE_LIST)) {
	status = _libx1f4i0_lxcall_stat_list
	    (track_data, h - offset, o, function_text);
    } else {
	struct center_type center;
	unsigned flat = 0, mind, post, x = 0;
	const int *data, *text;

	data = function_data->args + offset;
	text = function_text->args;

	mind = function_data->flags & X1f4_E4_POST_TYPE;
	post = function_text->flags & X1f4_E4_POST_TYPE;

	center.composer_data = composer_data;
	center.eelookup_data = eelookup_data;
	center.function_text = function_text;
	center.iturn_data = iturn_data;
	center.track_data = track_data;
	center.input = input;

	delete = o;
	for (; delete; delete--) {
	    int name, type;

	    name = *data;
	    type = *text;
	    status = _libx1f4i0_lxcall_even_fare
		(&center, 0, h, o, delete,
		 data, name, mind, text, type, post, 2, &flat, &x);
	    if (status) {
		break;
	    }

	    data++;
	    text++;
	}
	if (delete) {
	} else {
	    delete = h - o - offset;
	    for (; delete; delete--) {
		if (*data == X1f4_E4_CASE) {
		    x++;
		}

		data++;
	    }

	    if (1) {
		void *line;

		h -= offset;

		flat = h;

		status = trans_data->link
		    (trans_data->data, &line,
		     sizeof(void *) * (h + record)
		     + (flat + 1) * sizeof(struct fxdata_type)
		     + x * sizeof(struct x1f4_dxcast_type));
		if (status) {
		    status = _libx1f4i0_lxcall_stat_link(track_data);
		} else {
		    void **track;

		    track = line;
		    track += record;

		    memcpy(track, *input, sizeof(void *) * h);

		    *input = track;

		    status = rule_line
			(&center, function_text, o, post, function_data,
			 offset, h, mind, track);
		}
	    }
	}
    }

    return status;
}
