/*
 * lxcall-v.e.c
 * Copyright (C) 2008-2017, Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <stddef.h>

#include <e4.h>
#include <frame.h>
#include <lxcall-inter.h>
#include <lxcast.h>
#include <nl.h>
#include <nt.h>
#include <track.h>

#define I_USER(t)			(*((X1f4_E4_C_USER *) (t)))

#define l_USER(e, output) \
    {									      \
	X1f4_E4_C_USER *l;						      \
									      \
	l = (void *) (output);						      \
	*l = (e);							      \
    }

typedef struct upcast_type {
    struct x1f4_dxcast_type dxcast;
    struct x1f4_nodelink_type nodelink;
    union {
	int (*free)(void *, void *, struct x1f4_nodelink_type **,
		    const struct x1f4_nodelink_type *);
    } v;
    void *text;
} upcast_type;

static int slip_data(void *);

static int
slip_data(void *upcast)
{
    int (*free)(void *, void *, struct x1f4_nodelink_type **,
		const struct x1f4_nodelink_type *), status;
    struct upcast_type *upcast_data;

    upcast_data = upcast;

    free = upcast_data->v.free;
    if (free) {
	status = free
	    (upcast_data->text, *(void **) &upcast_data->dxcast.data, NULL,
	     &upcast_data->nodelink);
    } else {
	status = 0;
    }

    return status;
}


int
_libx1f4i0_lxcall_draw_node(struct x1f4_frame_type *frame_data,
			    struct x1f4_track_type *track_data,
			    struct x1f4_nodetype_type *nodetype_data, int type,
			    const void *data, void *call)
{
    int status;
    void *cast;
    void **mind;

    status = frame_data->pick
	(frame_data->data, &mind, slip_data, sizeof(struct upcast_type),
	 &cast);
    if (status) {
	status = _libx1f4i0_lxcall_stat_link(track_data);
    } else {
	struct upcast_type *upcast_data;
	void *user;

	upcast_data = cast;

	*mind = cast;

	status = nodetype_data->copy
	    (nodetype_data->context, NULL, &user, I_USER(data),
	     &upcast_data->nodelink);
	if (status) {
	    upcast_data->v.free = NULL;
	} else {
	    struct x1f4_dxcast_type *dxcast_data;

	    cast = &upcast_data->dxcast;

	    dxcast_data = cast;

	    l_USER(cast, call);

	    dxcast_data->call = 1;
	    dxcast_data->type = type;

	    l_USER(user, &dxcast_data->data);

	    upcast_data->v.free = nodetype_data->free;
	    upcast_data->text = nodetype_data->context;
	}
    }

    return status;
}
