/*
 * lxcase-a.e.c
 * Copyright (C) 2008-2014, 2016, 2019, Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <errno.h>
#include <string.h>

#include <er.h>
#include <exerrors.h>
#include <lxcase-types.h>

#define lxcase(lxcase) \
    ((struct lxcase_type *) (lxcase))

int
_libx1f4i0_lxcase_stat_call(void *lxcase, const char *call, const char *text)
{
    int error;

    error = errno;

    LINE_EEER(lxcase(lxcase)->link_e);

    PUSH_DATA("cannot ", 7);
    PUSH_TEXT(call);
    PUSH_DATA(" `", 2);
    PUSH_TEXT(text);
    PUSH_DATA("': ", 3);
    PUSH_TEXT(strerror(error));

    POST_EEER();

    return X1f4_EX_CAN_CONTINUE;
}


int
_libx1f4i0_lxcase_stat_case(void *lxcase, const char *call, const char *text,
			    long mode)
{
    int error;

    error = errno;

    LINE_EEER(lxcase(lxcase)->link_e);

    PUSH_DATA("cannot ", 7);
    PUSH_TEXT(call);
    PUSH_DATA(" `", 2);
    PUSH_TEXT(text);
    PUSH_DATA("' ", 2);
    PUSH_BASE(8, mode);
    PUSH_DATA(": ", 2);
    PUSH_TEXT(strerror(error));

    POST_EEER();

    return X1f4_EX_CAN_CONTINUE;
}


int
_libx1f4i0_lxcase_stat_dori(void *lxcase, int rank)
{
    LINE_EEER(lxcase(lxcase)->link_e);

    PUSH_DATA("bad ", 4);
    PUSH_MODE(rank + 1);
    PUSH_RANK(rank);
    PUSH_DATA(" argument type, integer or date expected", 40);

    POST_EEER();

    return X1f4_EX_CAN_CONTINUE;
}


int
_libx1f4i0_lxcase_stat_fall(void *lxcase, const char *call, long mode)
{
    int error;

    error = errno;

    LINE_EEER(lxcase(lxcase)->link_e);

    PUSH_DATA("cannot ", 7);
    PUSH_TEXT(call);
    PUSH_DATA(" ", 1);
    PUSH_LONG(mode);
    PUSH_DATA(": ", 2);
    PUSH_TEXT(strerror(error));

    POST_EEER();

    return X1f4_EX_CAN_CONTINUE;
}


int
_libx1f4i0_lxcase_stat_fool(void *lxcase, const char *call, const char *fold,
			    const char *news)
{
    int error;

    error = errno;

    LINE_EEER(lxcase(lxcase)->link_e);

    PUSH_DATA("cannot ", 7);
    PUSH_TEXT(call);
    PUSH_DATA(" `", 2);
    PUSH_TEXT(fold);
    PUSH_DATA("' `", 3);
    PUSH_TEXT(news);
    PUSH_DATA("': ", 3);
    PUSH_TEXT(strerror(error));

    POST_EEER();

    return X1f4_EX_CAN_CONTINUE;
}


int
_libx1f4i0_lxcase_stat_hold(void *lxcase, const char *call, long hold)
{
    int error;

    error = errno;

    LINE_EEER(lxcase(lxcase)->link_e);

    PUSH_DATA("cannot ", 7);
    PUSH_TEXT(call);
    PUSH_DATA(" ", 1);
    PUSH_LONG(hold);
    PUSH_DATA(": ", 2);
    PUSH_TEXT(strerror(error));

    POST_EEER();

    return X1f4_EX_CAN_CONTINUE;
}


int
_libx1f4i0_lxcase_stat_iran(void *lxcase, int rank)
{
    LINE_EEER(lxcase(lxcase)->link_e);

    PUSH_DATA("bad ", 4);
    PUSH_MODE(rank + 1);
    PUSH_RANK(rank);
    PUSH_DATA(" argument type, integer expected", 32);

    POST_EEER();

    return X1f4_EX_CAN_CONTINUE;
}


int
_libx1f4i0_lxcase_stat_kill(void *lxcase, const char *call, long kill,
			    long mode)
{
    int error;

    error = errno;

    LINE_EEER(lxcase(lxcase)->link_e);

    PUSH_DATA("cannot ", 7);
    PUSH_TEXT(call);
    PUSH_DATA(" ", 1);
    PUSH_LONG(kill);
    PUSH_DATA(" ", 1);
    PUSH_LONG(mode);
    PUSH_DATA(": ", 2);
    PUSH_TEXT(strerror(error));

    POST_EEER();

    return X1f4_EX_CAN_CONTINUE;
}


int
_libx1f4i0_lxcase_stat_land(void *lxcase, const char *call, const char *land,
			    long lead, long slip)
{
    int error;

    error = errno;

    LINE_EEER(lxcase(lxcase)->link_e);

    PUSH_DATA("cannot ", 7);
    PUSH_TEXT(call);
    PUSH_DATA(" `", 2);
    PUSH_TEXT(land);
    PUSH_DATA("' ", 2);
    PUSH_LONG(lead);
    PUSH_DATA(" ", 1);
    PUSH_LONG(slip);
    PUSH_DATA(": ", 2);
    PUSH_TEXT(strerror(error));

    POST_EEER();

    return X1f4_EX_CAN_CONTINUE;
}


int
_libx1f4i0_lxcase_stat_mill(void *lxcase, int rank)
{
    LINE_EEER(lxcase(lxcase)->link_e);

    PUSH_DATA("wrong ", 6);
    PUSH_MODE(rank + 1);
    PUSH_RANK(rank);
    PUSH_DATA(" argument type for field selection", 34);

    POST_EEER();

    return X1f4_EX_CAN_CONTINUE;
}


int
_libx1f4i0_lxcase_stat_park(void *lxcase, long park)
{
    LINE_EEER(lxcase(lxcase)->link_e);

    PUSH_DATA("unknown ", 8);
    PUSH_LONG(park);
    PUSH_DATA(" field", 6);

    POST_EEER();

    return X1f4_EX_CAN_CONTINUE;
}
