/*
 * lxcast-i.0.c
 * Copyright (C) 2008-2015, 2017-2019, Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <stddef.h>
#include <string.h>

#include <e4.h>
#include <lxcall.h>
#include <lxcast.h>
#include <lxcast-drive.h>
#include <lxcast-fold.h>
#include <lxcast-inter.h>
#include <lxcast-types.h>

#define LINE_C0_BITS \
    X1f4_E4_KEEP_CALL | X1f4_E4_POST_TYPE | X1f4_E4_TEXT_LINK		      \
    | X1f4_E4_XSET_LINK
#define LINE_C1_BITS \
    X1f4_E4_KEEP_CALL | X1f4_E4_POST_TYPE | X1f4_E4_SLIP_LIST		      \
    | X1f4_E4_TEXT_LINK | X1f4_E4_XSET_LINK
#define LINE_C__BITS \
    X1f4_E4_KEEP_CALL | X1f4_E4_POST_TYPE | X1f4_E4_TEXT_LINK

#define LINE_D__BITS \
    X1f4_E4_KEEP_CALL | X1f4_E4_TEXT_LINK

#define SIDE_CO_BITS \
    X1f4_E4_BACK_LINK | X1f4_E4_CALL_XSET | X1f4_E4_E2ND_LINK
#define SIDE_CE_BITS \
    X1f4_E4_E2ND_LINK
#define SIDE_CL_BITS \
    X1f4_E4_BACK_LINK | X1f4_E4_LEFT_XSET | X1f4_E4_E2ND_LINK

#define a7axx_x_c			_libx1f4i0_lxcast_y_indexed
#define c7axx_f_c			_libx1f4i0_lxcast_q_indexed
#define c7axx_f_f			_libx1f4i0_lxcast_r_indexed

#define MAKE_DOUBLE(a, b)		a b
#define MAKE_SINGLE(a, b)		a

#define MAKE_VIOLET(a, b)		MAKE_DOUBLE(MAKE_DOUBLE(a, b), "=")

#define lxcast(lxcast) \
    ((struct lxcast_type *) (lxcast))

static int a2cxx_b_c(void *, void *, void **);
static int a2cxx_m_c(void *, void *, void **);
static int a2cxx_r_c(void *, void *, void **);
static int a2exx_b_c(void *, void *, void **);
static int a2exx_m_c(void *, void *, void **);
static int a2exx_r_c(void *, void *, void **);
static int a5fxx_b_c(void *, void *, void **);
static int a5fxx_m_c(void *, void *, void **);
static int a61xx_b_c(void *, void *, void **);
static int a61xx_c_b(void *, void *, void **);
static int a61xx_c_c(void *, void *, void **);
static int a61xx_c_m(void *, void *, void **);
static int a61xx_c_r(void *, void *, void **);
static int a61xx_c_t(void *, void *, void **);
static int a61xx_m_c(void *, void *, void **);
static int a61xx_r_c(void *, void *, void **);
static int a61xx_t_c(void *, void *, void **);
static int a62xx_b_c(void *, void *, void **);
static int a62xx_c_b(void *, void *, void **);
static int a62xx_c_c(void *, void *, void **);
static int a62xx_c_m(void *, void *, void **);
static int a62xx_c_r(void *, void *, void **);
static int a62xx_m_c(void *, void *, void **);
static int a62xx_r_c(void *, void *, void **);
static int a63xx_b_c(void *, void *, void **);
static int a63xx_c_b(void *, void *, void **);
static int a63xx_c_c(void *, void *, void **);
static int a63xx_c_m(void *, void *, void **);
static int a63xx_c_r(void *, void *, void **);
static int a63xx_m_c(void *, void *, void **);
static int a63xx_r_c(void *, void *, void **);
static int a64xx_b_c(void *, void *, void **);
static int a64xx_c_b(void *, void *, void **);
static int a64xx_c_c(void *, void *, void **);
static int a64xx_c_m(void *, void *, void **);
static int a64xx_c_r(void *, void *, void **);
static int a64xx_m_c(void *, void *, void **);
static int a64xx_r_c(void *, void *, void **);
static int a65xx_b_c(void *, void *, void **);
static int a65xx_c_b(void *, void *, void **);
static int a65xx_c_c(void *, void *, void **);
static int a65xx_c_m(void *, void *, void **);
static int a65xx_c_r(void *, void *, void **);
static int a65xx_c_t(void *, void *, void **);
static int a65xx_m_c(void *, void *, void **);
static int a65xx_r_c(void *, void *, void **);
static int a65xx_t_c(void *, void *, void **);
static int a66xx_b_c(void *, void *, void **);
static int a66xx_m_c(void *, void *, void **);
static int a67xx_b_c(void *, void *, void **);
static int a67xx_c_b(void *, void *, void **);
static int a67xx_c_c(void *, void *, void **);
static int a67xx_c_m(void *, void *, void **);
static int a67xx_c_r(void *, void *, void **);
static int a67xx_c_t(void *, void *, void **);
static int a67xx_m_c(void *, void *, void **);
static int a67xx_r_c(void *, void *, void **);
static int a67xx_t_c(void *, void *, void **);
static int a68xx_b_c(void *, void *, void **);
static int a68xx_m_c(void *, void *, void **);
static int a69xx_b_c(void *, void *, void **);
static int a69xx_m_c(void *, void *, void **);
static int a6axx_b_c(void *, void *, void **);
static int a6axx_m_c(void *, void *, void **);
static int a6bxx_b_c(void *, void *, void **);
static int a6bxx_c_b(void *, void *, void **);
static int a6bxx_c_c(void *, void *, void **);
static int a6bxx_c_m(void *, void *, void **);
static int a6bxx_m_c(void *, void *, void **);
static int a6cxx_b_c(void *, void *, void **);
static int a6cxx_c_b(void *, void *, void **);
static int a6cxx_c_c(void *, void *, void **);
static int a6cxx_c_m(void *, void *, void **);
static int a6cxx_c_r(void *, void *, void **);
static int a6cxx_c_t(void *, void *, void **);
static int a6cxx_m_c(void *, void *, void **);
static int a6cxx_r_c(void *, void *, void **);
static int a6cxx_t_c(void *, void *, void **);
static int a6dxx_b_c(void *, void *, void **);
static int a6dxx_c_b(void *, void *, void **);
static int a6dxx_c_c(void *, void *, void **);
static int a6dxx_c_m(void *, void *, void **);
static int a6dxx_c_r(void *, void *, void **);
static int a6dxx_m_c(void *, void *, void **);
static int a6dxx_r_c(void *, void *, void **);
static int a70xx_b_c(void *, void *, void **);
static int a70xx_m_c(void *, void *, void **);
static int a70xx_r_c(void *, void *, void **);
static int a71xx_b_c(void *, void *, void **);
static int a71xx_m_c(void *, void *, void **);
static int a71xx_r_c(void *, void *, void **);
static int a72xx_b_c(void *, void *, void **);
static int a72xx_c_b(void *, void *, void **);
static int a72xx_c_c(void *, void *, void **);
static int a72xx_c_m(void *, void *, void **);
static int a72xx_m_c(void *, void *, void **);
static int a73xx_b_c(void *, void *, void **);
static int a73xx_c_b(void *, void *, void **);
static int a73xx_c_c(void *, void *, void **);
static int a73xx_c_m(void *, void *, void **);
static int a73xx_c_r(void *, void *, void **);
static int a73xx_m_c(void *, void *, void **);
static int a73xx_r_c(void *, void *, void **);
static int a74xx_b_c(void *, void *, void **);
static int a74xx_c_c(void *, void *, void **);
static int a74xx_c_b(void *, void *, void **);
static int a74xx_c_m(void *, void *, void **);
static int a74xx_m_c(void *, void *, void **);
static int a75xx_b_c(void *, void *, void **);
static int a75xx_m_c(void *, void *, void **);
static int a76xx_b_c(void *, void *, void **);
static int a76xx_c_c(void *, void *, void **);
static int a76xx_c_b(void *, void *, void **);
static int a76xx_c_m(void *, void *, void **);
static int a76xx_c_r(void *, void *, void **);
static int a76xx_c_t(void *, void *, void **);
static int a76xx_m_c(void *, void *, void **);
static int a76xx_r_c(void *, void *, void **);
static int a76xx_t_c(void *, void *, void **);
static int a77xx_b_c(void *, void *, void **);
static int a77xx_c_b(void *, void *, void **);
static int a77xx_c_c(void *, void *, void **);
static int a77xx_c_m(void *, void *, void **);
static int a77xx_m_c(void *, void *, void **);
static int a78xx_b_c(void *, void *, void **);
static int a78xx_c_b(void *, void *, void **);
static int a78xx_c_c(void *, void *, void **);
static int a78xx_c_m(void *, void *, void **);
static int a78xx_m_c(void *, void *, void **);
static int a79xx_b_c(void *, void *, void **);
static int a79xx_c_b(void *, void *, void **);
static int a79xx_c_c(void *, void *, void **);
static int a79xx_c_m(void *, void *, void **);
static int a79xx_m_c(void *, void *, void **);
static int a7axx_b_c(void *, void *, void **);
static int a7axx_m_c(void *, void *, void **);
static int a7axx_r_c(void *, void *, void **);
static int a7axx_t_c(void *, void *, void **);
static int a7bxx_b_c(void *, void *, void **);
static int a7bxx_c_b(void *, void *, void **);
static int a7bxx_c_c(void *, void *, void **);
static int a7bxx_c_m(void *, void *, void **);
static int a7bxx_c_r(void *, void *, void **);
static int a7bxx_c_t(void *, void *, void **);
static int a7bxx_m_c(void *, void *, void **);
static int a7bxx_r_c(void *, void *, void **);
static int a7bxx_t_c(void *, void *, void **);
static int a7cxx_b_c(void *, void *, void **);
static int a7cxx_c_b(void *, void *, void **);
static int a7cxx_c_c(void *, void *, void **);
static int a7cxx_c_m(void *, void *, void **);
static int a7cxx_c_r(void *, void *, void **);
static int a7cxx_c_t(void *, void *, void **);
static int a7cxx_m_c(void *, void *, void **);
static int a7cxx_r_c(void *, void *, void **);
static int a7cxx_t_c(void *, void *, void **);
static int d61xx_t_t(void *, void *, void *, void *);
static int e61xx_t_t(void *, void *, void *, void *);

static const int c_____b_c[] = {
/* *INDENT-OFF* */
    X1f4_E4_BILL,
    X1f4_E4_CASE,
    X1f4_E4_POST_XSET,
    0
/* *INDENT-ON* */
}, c_____c_b[] = {
/* *INDENT-OFF* */
    X1f4_E4_CASE,
    X1f4_E4_BILL
/* *INDENT-ON* */
}, c_____c_c[] = {
/* *INDENT-OFF* */
    X1f4_E4_CASE,
    X1f4_E4_CASE
/* *INDENT-ON* */
}, c_____c_m[] = {
/* *INDENT-OFF* */
    X1f4_E4_CASE,
    X1f4_E4_MODE
/* *INDENT-ON* */
}, c_____c_r[] = {
/* *INDENT-OFF* */
    X1f4_E4_CASE,
    X1f4_E4_REAL
/* *INDENT-ON* */
}, c_____c_t[] = {
/* *INDENT-OFF* */
    X1f4_E4_CASE,
    X1f4_E4_TEXT
/* *INDENT-ON* */
}, c_____f_c[] = {
/* *INDENT-OFF* */
    X1f4_E4_CALL,
    X1f4_E4_CASE,
    X1f4_E4_POST_XSET,
    0
/* *INDENT-ON* */
}, c_____f_f[] = {
/* *INDENT-OFF* */
    X1f4_E4_CALL,
    X1f4_E4_CALL,
    X1f4_E4_POST_XSET,
    0
/* *INDENT-ON* */
}, c_____m_c[] = {
/* *INDENT-OFF* */
    X1f4_E4_MODE,
    X1f4_E4_CASE,
    X1f4_E4_POST_XSET,
    0
/* *INDENT-ON* */
}, c_____r_c[] = {
/* *INDENT-OFF* */
    X1f4_E4_REAL,
    X1f4_E4_CASE,
    X1f4_E4_POST_XSET,
    0
/* *INDENT-ON* */
}, c_____t_c[] = {
/* *INDENT-OFF* */
    X1f4_E4_TEXT,
    X1f4_E4_CASE,
    X1f4_E4_POST_XSET,
    0
/* *INDENT-ON* */
};
const struct x1f4_function_type _libx1f4i0_lxcast_line_0[] = {
/* *INDENT-OFF* */
    {	"copy",			c7axx_f_f,
	X1f4_E4_CALL,		c_____f_f,	2,
	LINE_C0_BITS,				4		},
    {	"copy",			c7axx_f_c,
	X1f4_E4_CALL,		c_____f_c,	2,
	LINE_C1_BITS,				4		},
    {	"copy",			a7axx_b_c,
	X1f4_E4_BILL,		c_____b_c,	2,
	LINE_C0_BITS,				4		},
    {	"copy",			a7axx_m_c,
	X1f4_E4_MODE,		c_____m_c,	2,
	LINE_C0_BITS,				4		},
    {	"copy",			a7axx_r_c,
	X1f4_E4_REAL,		c_____r_c,	2,
	LINE_C0_BITS,				4		},
    {	"copy",			a7axx_t_c,
	X1f4_E4_TEXT,		c_____t_c,	2,
	LINE_C0_BITS,				4		},
    {	"fold",			a2cxx_b_c,
	X1f4_E4_BILL,		c_____b_c,	2,
	LINE_C__BITS,				4		},
    {	"fold",			a2cxx_m_c,
	X1f4_E4_MODE,		c_____m_c,	2,
	LINE_C__BITS,				4		},
    {	"fold",			a2cxx_r_c,
	X1f4_E4_REAL,		c_____r_c,	2,
	LINE_C__BITS,				4		},
    {	"fold",			a2exx_b_c,
	X1f4_E4_BILL,		c_____b_c,	2,
	LINE_C__BITS,				4		},
    {	"fold",			a2exx_m_c,
	X1f4_E4_MODE,		c_____m_c,	2,
	LINE_C__BITS,				4		},
    {	"fold",			a2exx_r_c,
	X1f4_E4_REAL,		c_____r_c,	2,
	LINE_C__BITS,				4		},
    {	"fold",			a5fxx_b_c,
	X1f4_E4_BILL,		c_____b_c,	2,
	LINE_C__BITS,				4		},
    {	"fold",			a5fxx_m_c,
	X1f4_E4_MODE,		c_____m_c,	2,
	LINE_C__BITS,				4		},
    {	"over",			a61xx_c_b,
	X1f4_E4_CASE,		c_____c_b,	2,
	LINE_D__BITS,				4		},
    {	"over",			a61xx_c_m,
	X1f4_E4_CASE,		c_____c_m,	2,
	LINE_D__BITS,				4		},
    {	"over",			a61xx_c_r,
	X1f4_E4_REAL,		c_____c_r,	2,
	LINE_D__BITS,				4		},
    {	"over",			a61xx_c_t,
	X1f4_E4_TEXT,		c_____c_t,	2,
	LINE_D__BITS,				4		},
    {	"over",			a61xx_b_c,
	X1f4_E4_CASE,		c_____b_c,	2,
	LINE_D__BITS,				4		},
    {	"over",			a61xx_m_c,
	X1f4_E4_CASE,		c_____m_c,	2,
	LINE_D__BITS,				4		},
    {	"over",			a61xx_r_c,
	X1f4_E4_REAL,		c_____r_c,	2,
	LINE_D__BITS,				4		},
    {	"over",			a61xx_t_c,
	X1f4_E4_TEXT,		c_____t_c,	2,
	LINE_D__BITS,				4		},
    {	"over",			a61xx_c_c,
	X1f4_E4_CASE,		c_____c_c,	2,
	LINE_D__BITS,				4		},
    {	"unit",			a62xx_c_b,
	X1f4_E4_MODE,		c_____c_b,	2,
	LINE_D__BITS,				4		},
    {	"unit",			a62xx_c_m,
	X1f4_E4_MODE,		c_____c_m,	2,
	LINE_D__BITS,				4		},
    {	"unit",			a62xx_c_r,
	X1f4_E4_MODE,		c_____c_r,	2,
	LINE_D__BITS,				4		},
    {	"unit",			a62xx_b_c,
	X1f4_E4_MODE,		c_____b_c,	2,
	LINE_D__BITS,				4		},
    {	"unit",			a62xx_m_c,
	X1f4_E4_MODE,		c_____m_c,	2,
	LINE_D__BITS,				4		},
    {	"unit",			a62xx_r_c,
	X1f4_E4_MODE,		c_____r_c,	2,
	LINE_D__BITS,				4		},
    {	"unit",			a62xx_c_c,
	X1f4_E4_MODE,		c_____c_c,	2,
	LINE_D__BITS,				4		},
    {	"unit",			a63xx_c_b,
	X1f4_E4_MODE,		c_____c_b,	2,
	LINE_D__BITS,				4		},
    {	"unit",			a63xx_c_m,
	X1f4_E4_MODE,		c_____c_m,	2,
	LINE_D__BITS,				4		},
    {	"unit",			a63xx_c_r,
	X1f4_E4_MODE,		c_____c_r,	2,
	LINE_D__BITS,				4		},
    {	"unit",			a63xx_b_c,
	X1f4_E4_MODE,		c_____b_c,	2,
	LINE_D__BITS,				4		},
    {	"unit",			a63xx_m_c,
	X1f4_E4_MODE,		c_____m_c,	2,
	LINE_D__BITS,				4		},
    {	"unit",			a63xx_r_c,
	X1f4_E4_MODE,		c_____r_c,	2,
	LINE_D__BITS,				4		},
    {	"unit",			a63xx_c_c,
	X1f4_E4_MODE,		c_____c_c,	2,
	LINE_D__BITS,				4		},
    {	"over",			a64xx_c_b,
	X1f4_E4_CASE,		c_____c_b,	2,
	LINE_D__BITS,				4		},
    {	"over",			a64xx_c_m,
	X1f4_E4_CASE,		c_____c_m,	2,
	LINE_D__BITS,				4		},
    {	"over",			a64xx_c_r,
	X1f4_E4_REAL,		c_____c_r,	2,
	LINE_D__BITS,				4		},
    {	"over",			a64xx_b_c,
	X1f4_E4_CASE,		c_____b_c,	2,
	LINE_D__BITS,				4		},
    {	"over",			a64xx_m_c,
	X1f4_E4_CASE,		c_____m_c,	2,
	LINE_D__BITS,				4		},
    {	"over",			a64xx_r_c,
	X1f4_E4_REAL,		c_____r_c,	2,
	LINE_D__BITS,				4		},
    {	"over",			a64xx_c_c,
	X1f4_E4_CASE,		c_____c_c,	2,
	LINE_D__BITS,				4		},
    {	"unit",			a65xx_c_b,
	X1f4_E4_MODE,		c_____c_b,	2,
	LINE_D__BITS,				4		},
    {	"unit",			a65xx_c_m,
	X1f4_E4_MODE,		c_____c_m,	2,
	LINE_D__BITS,				4		},
    {	"unit",			a65xx_c_r,
	X1f4_E4_MODE,		c_____c_r,	2,
	LINE_D__BITS,				4		},
    {	"unit",			a65xx_c_t,
	X1f4_E4_MODE,		c_____c_t,	2,
	LINE_D__BITS,				4		},
    {	"unit",			a65xx_b_c,
	X1f4_E4_MODE,		c_____b_c,	2,
	LINE_D__BITS,				4		},
    {	"unit",			a65xx_m_c,
	X1f4_E4_MODE,		c_____m_c,	2,
	LINE_D__BITS,				4		},
    {	"unit",			a65xx_r_c,
	X1f4_E4_MODE,		c_____r_c,	2,
	LINE_D__BITS,				4		},
    {	"unit",			a65xx_t_c,
	X1f4_E4_MODE,		c_____t_c,	2,
	LINE_D__BITS,				4		},
    {	"unit",			a65xx_c_c,
	X1f4_E4_MODE,		c_____c_c,	2,
	LINE_D__BITS,				4		},
    {	"fold",			a66xx_b_c,
	X1f4_E4_BILL,		c_____b_c,	2,
	LINE_C__BITS,				4		},
    {	"fold",			a66xx_m_c,
	X1f4_E4_MODE,		c_____m_c,	2,
	LINE_C__BITS,				4		},
     {	"unit",			a67xx_c_b,
	X1f4_E4_MODE,		c_____c_b,	2,
	LINE_D__BITS,				4		},
    {	"unit",			a67xx_c_m,
	X1f4_E4_MODE,		c_____c_m,	2,
	LINE_D__BITS,				4		},
    {	"unit",			a67xx_c_r,
	X1f4_E4_MODE,		c_____c_r,	2,
	LINE_D__BITS,				4		},
    {	"unit",			a67xx_c_t,
	X1f4_E4_MODE,		c_____c_t,	2,
	LINE_D__BITS,				4		},
    {	"unit",			a67xx_b_c,
	X1f4_E4_MODE,		c_____b_c,	2,
	LINE_D__BITS,				4		},
    {	"unit",			a67xx_m_c,
	X1f4_E4_MODE,		c_____m_c,	2,
	LINE_D__BITS,				4		},
    {	"unit",			a67xx_r_c,
	X1f4_E4_MODE,		c_____r_c,	2,
	LINE_D__BITS,				4		},
    {	"unit",			a67xx_t_c,
	X1f4_E4_MODE,		c_____t_c,	2,
	LINE_D__BITS,				4		},
    {	"unit",			a67xx_c_c,
	X1f4_E4_MODE,		c_____c_c,	2,
	LINE_D__BITS,				4		},
   {	"fold",			a68xx_b_c,
	X1f4_E4_BILL,		c_____b_c,	2,
	LINE_C__BITS,				4		},
    {	"fold",			a68xx_m_c,
	X1f4_E4_MODE,		c_____m_c,	2,
	LINE_C__BITS,				4		},
    {	"fold",			a69xx_b_c,
	X1f4_E4_BILL,		c_____b_c,	2,
	LINE_C__BITS,				4		},
    {	"fold",			a69xx_m_c,
	X1f4_E4_MODE,		c_____m_c,	2,
	LINE_C__BITS,				4		},
    {	"fold",			a6axx_b_c,
	X1f4_E4_BILL,		c_____b_c,	2,
	LINE_C__BITS,				4		},
    {	"fold",			a6axx_m_c,
	X1f4_E4_MODE,		c_____m_c,	2,
	LINE_C__BITS,				4		},
    {	"over",			a6bxx_b_c,
	X1f4_E4_BILL,		c_____b_c,	2,
	LINE_D__BITS,				4		},
    {	"over",			a6bxx_m_c,
	X1f4_E4_MODE,		c_____m_c,	2,
	LINE_D__BITS,				4		},
    {	"over",			a6bxx_c_b,
	X1f4_E4_CASE,		c_____c_b,	2,
	LINE_D__BITS,				4		},
    {	"over",			a6bxx_c_m,
	X1f4_E4_CASE,		c_____c_m,	2,
	LINE_D__BITS,				4		},
    {	"over",			a6bxx_c_c,
	X1f4_E4_CASE,		c_____c_c,	2,
	LINE_D__BITS,				4		},
    {	"unit",			a6cxx_c_b,
	X1f4_E4_MODE,		c_____c_b,	2,
	LINE_D__BITS,				4		},
    {	"unit",			a6cxx_c_m,
	X1f4_E4_MODE,		c_____c_m,	2,
	LINE_D__BITS,				4		},
    {	"unit",			a6cxx_c_r,
	X1f4_E4_MODE,		c_____c_r,	2,
	LINE_D__BITS,				4		},
    {	"unit",			a6cxx_c_t,
	X1f4_E4_MODE,		c_____c_t,	2,
	LINE_D__BITS,				4		},
    {	"unit",			a6cxx_b_c,
	X1f4_E4_MODE,		c_____b_c,	2,
	LINE_D__BITS,				4		},
    {	"unit",			a6cxx_m_c,
	X1f4_E4_MODE,		c_____m_c,	2,
	LINE_D__BITS,				4		},
    {	"unit",			a6cxx_r_c,
	X1f4_E4_MODE,		c_____r_c,	2,
	LINE_D__BITS,				4		},
    {	"unit",			a6cxx_t_c,
	X1f4_E4_MODE,		c_____t_c,	2,
	LINE_D__BITS,				4		},
    {	"unit",			a6cxx_c_c,
	X1f4_E4_MODE,		c_____c_c,	2,
	LINE_D__BITS,				4		},
    {	"over",			a6dxx_c_b,
	X1f4_E4_CASE,		c_____c_b,	2,
	LINE_D__BITS,				4		},
    {	"over",			a6dxx_c_m,
	X1f4_E4_CASE,		c_____c_m,	2,
	LINE_D__BITS,				4		},
    {	"over",			a6dxx_c_r,
	X1f4_E4_REAL,		c_____c_r,	2,
	LINE_D__BITS,				4		},
    {	"over",			a6dxx_b_c,
	X1f4_E4_CASE,		c_____b_c,	2,
	LINE_D__BITS,				4		},
    {	"over",			a6dxx_m_c,
	X1f4_E4_CASE,		c_____m_c,	2,
	LINE_D__BITS,				4		},
    {	"over",			a6dxx_r_c,
	X1f4_E4_REAL,		c_____r_c,	2,
	LINE_D__BITS,				4		},
    {	"over",			a6dxx_c_c,
	X1f4_E4_CASE,		c_____c_c,	2,
	LINE_D__BITS,				4		},
    {	"fold",			a70xx_b_c,
	X1f4_E4_BILL,		c_____b_c,	2,
	LINE_C__BITS,				4		},
    {	"fold",			a70xx_m_c,
	X1f4_E4_MODE,		c_____m_c,	2,
	LINE_C__BITS,				4		},
    {	"fold",			a70xx_r_c,
	X1f4_E4_REAL,		c_____r_c,	2,
	LINE_C__BITS,				4		},
    {	"fold",			a71xx_b_c,
	X1f4_E4_BILL,		c_____b_c,	2,
	LINE_C__BITS,				4		},
    {	"fold",			a71xx_m_c,
	X1f4_E4_MODE,		c_____m_c,	2,
	LINE_C__BITS,				4		},
    {	"fold",			a71xx_r_c,
	X1f4_E4_REAL,		c_____r_c,	2,
	LINE_C__BITS,				4		},
    {	"over",			a72xx_b_c,
	X1f4_E4_BILL,		c_____b_c,	2,
	LINE_D__BITS,				4		},
    {	"over",			a72xx_m_c,
	X1f4_E4_CASE,		c_____m_c,	2,
	LINE_D__BITS,				4		},
    {	"over",			a72xx_c_b,
	X1f4_E4_BILL,		c_____c_b,	2,
	LINE_D__BITS,				4		},
    {	"over",			a72xx_c_m,
	X1f4_E4_CASE,		c_____c_m,	2,
	LINE_D__BITS,				4		},
    {	"over",			a72xx_c_c,
	X1f4_E4_CASE,		c_____c_c,	2,
	LINE_D__BITS,				4		},
    {	"over",			a73xx_c_b,
	X1f4_E4_CASE,		c_____c_b,	2,
	LINE_D__BITS,				4		},
    {	"over",			a73xx_c_m,
	X1f4_E4_CASE,		c_____c_m,	2,
	LINE_D__BITS,				4		},
    {	"over",			a73xx_c_r,
	X1f4_E4_REAL,		c_____c_r,	2,
	LINE_D__BITS,				4		},
    {	"over",			a73xx_b_c,
	X1f4_E4_CASE,		c_____b_c,	2,
	LINE_D__BITS,				4		},
    {	"over",			a73xx_m_c,
	X1f4_E4_CASE,		c_____m_c,	2,
	LINE_D__BITS,				4		},
    {	"over",			a73xx_r_c,
	X1f4_E4_REAL,		c_____r_c,	2,
	LINE_D__BITS,				4		},
    {	"over",			a73xx_c_c,
	X1f4_E4_CASE,		c_____c_c,	2,
	LINE_D__BITS,				4		},
    {	"over",			a74xx_b_c,
	X1f4_E4_BILL,		c_____b_c,	2,
	LINE_D__BITS,				4		},
    {	"over",			a74xx_m_c,
	X1f4_E4_MODE,		c_____m_c,	2,
	LINE_D__BITS,				4		},
    {	"over",			a74xx_c_b,
	X1f4_E4_CASE,		c_____c_b,	2,
	LINE_D__BITS,				4		},
    {	"over",			a74xx_c_m,
	X1f4_E4_CASE,		c_____c_m,	2,
	LINE_D__BITS,				4		},
    {	"over",			a74xx_c_c,
	X1f4_E4_CASE,		c_____c_c,	2,
	LINE_D__BITS,				4		},
    {	"fold",			a75xx_b_c,
	X1f4_E4_BILL,		c_____b_c,	2,
	LINE_C__BITS,				4		},
    {	"fold",			a75xx_m_c,
	X1f4_E4_MODE,		c_____m_c,	2,
	LINE_C__BITS,				4		},
    {	"unit",			a76xx_c_b,
	X1f4_E4_MODE,		c_____c_b,	2,
	LINE_D__BITS,				4		},
    {	"unit",			a76xx_c_m,
	X1f4_E4_MODE,		c_____c_m,	2,
	LINE_D__BITS,				4		},
    {	"unit",			a76xx_c_r,
	X1f4_E4_MODE,		c_____c_r,	2,
	LINE_D__BITS,				4		},
    {	"unit",			a76xx_c_t,
	X1f4_E4_MODE,		c_____c_t,	2,
	LINE_D__BITS,				4		},
    {	"unit",			a76xx_b_c,
	X1f4_E4_MODE,		c_____b_c,	2,
	LINE_D__BITS,				4		},
    {	"unit",			a76xx_m_c,
	X1f4_E4_MODE,		c_____m_c,	2,
	LINE_D__BITS,				4		},
    {	"unit",			a76xx_r_c,
	X1f4_E4_MODE,		c_____r_c,	2,
	LINE_D__BITS,				4		},
    {	"unit",			a76xx_t_c,
	X1f4_E4_MODE,		c_____t_c,	2,
	LINE_D__BITS,				4		},
    {	"unit",			a76xx_c_c,
	X1f4_E4_MODE,		c_____c_c,	2,
	LINE_D__BITS,				4		},
    {	"over",			a77xx_b_c,
	X1f4_E4_BILL,		c_____b_c,	2,
	LINE_D__BITS,				4		},
    {	"over",			a77xx_m_c,
	X1f4_E4_CASE,		c_____m_c,	2,
	LINE_D__BITS,				4		},
    {	"over",			a77xx_c_b,
	X1f4_E4_BILL,		c_____c_b,	2,
	LINE_D__BITS,				4		},
    {	"over",			a77xx_c_m,
	X1f4_E4_CASE,		c_____c_m,	2,
	LINE_D__BITS,				4		},
    {	"over",			a77xx_c_c,
	X1f4_E4_CASE,		c_____c_c,	2,
	LINE_D__BITS,				4		},
    {	"over",			a78xx_b_c,
	X1f4_E4_BILL,		c_____b_c,	2,
	LINE_D__BITS,				4		},
    {	"over",			a78xx_m_c,
	X1f4_E4_CASE,		c_____m_c,	2,
	LINE_D__BITS,				4		},
    {	"over",			a78xx_c_b,
	X1f4_E4_BILL,		c_____c_b,	2,
	LINE_D__BITS,				4		},
    {	"over",			a78xx_c_m,
	X1f4_E4_CASE,		c_____c_m,	2,
	LINE_D__BITS,				4		},
    {	"over",			a78xx_c_c,
	X1f4_E4_CASE,		c_____c_c,	2,
	LINE_D__BITS,				4		},
    {	"over",			a79xx_b_c,
	X1f4_E4_BILL,		c_____b_c,	2,
	LINE_D__BITS,				4		},
    {	"over",			a79xx_m_c,
	X1f4_E4_CASE,		c_____m_c,	2,
	LINE_D__BITS,				4		},
    {	"over",			a79xx_c_b,
	X1f4_E4_BILL,		c_____c_b,	2,
	LINE_D__BITS,				4		},
    {	"over",			a79xx_c_m,
	X1f4_E4_CASE,		c_____c_m,	2,
	LINE_D__BITS,				4		},
    {	"over",			a79xx_c_c,
	X1f4_E4_CASE,		c_____c_c,	2,
	LINE_D__BITS,				4		},
    {	"unit",			a7bxx_c_b,
	X1f4_E4_MODE,		c_____c_b,	2,
	LINE_D__BITS,				4		},
    {	"unit",			a7bxx_c_m,
	X1f4_E4_MODE,		c_____c_m,	2,
	LINE_D__BITS,				4		},
    {	"unit",			a7bxx_c_r,
	X1f4_E4_MODE,		c_____c_r,	2,
	LINE_D__BITS,				4		},
    {	"unit",			a7bxx_c_t,
	X1f4_E4_MODE,		c_____c_t,	2,
	LINE_D__BITS,				4		},
    {	"unit",			a7bxx_b_c,
	X1f4_E4_MODE,		c_____b_c,	2,
	LINE_D__BITS,				4		},
    {	"unit",			a7bxx_m_c,
	X1f4_E4_MODE,		c_____m_c,	2,
	LINE_D__BITS,				4		},
    {	"unit",			a7bxx_r_c,
	X1f4_E4_MODE,		c_____r_c,	2,
	LINE_D__BITS,				4		},
    {	"unit",			a7bxx_t_c,
	X1f4_E4_MODE,		c_____t_c,	2,
	LINE_D__BITS,				4		},
    {	"unit",			a7bxx_c_c,
	X1f4_E4_MODE,		c_____c_c,	2,
	LINE_D__BITS,				4		},
    {	"unit",			a7cxx_c_b,
	X1f4_E4_MODE,		c_____c_b,	2,
	LINE_D__BITS,				4		},
    {	"unit",			a7cxx_c_m,
	X1f4_E4_MODE,		c_____c_m,	2,
	LINE_D__BITS,				4		},
    {	"unit",			a7cxx_c_r,
	X1f4_E4_MODE,		c_____c_r,	2,
	LINE_D__BITS,				4		},
    {	"unit",			a7cxx_c_t,
	X1f4_E4_MODE,		c_____c_t,	2,
	LINE_D__BITS,				4		},
    {	"unit",			a7cxx_b_c,
	X1f4_E4_MODE,		c_____b_c,	2,
	LINE_D__BITS,				4		},
    {	"unit",			a7cxx_m_c,
	X1f4_E4_MODE,		c_____m_c,	2,
	LINE_D__BITS,				4		},
    {	"unit",			a7cxx_r_c,
	X1f4_E4_MODE,		c_____r_c,	2,
	LINE_D__BITS,				4		},
    {	"unit",			a7cxx_t_c,
	X1f4_E4_MODE,		c_____t_c,	2,
	LINE_D__BITS,				4		},
    {	"unit",			a7cxx_c_c,
	X1f4_E4_MODE,		c_____c_c,	2,
	LINE_D__BITS,				4		}
/* *INDENT-ON* */
}, _libx1f4i0_lxcast_line_x[] = {
/* *INDENT-OFF* */
    {	"copy",			a7axx_x_c,
	0,			NULL,		2,
	LINE_C0_BITS,				4		}
/* *INDENT-ON* */
};
const struct x1f4_operator_type _libx1f4i0_lxcast_side_o[] = {
/* *INDENT-OFF* */
    {	MAKE_SINGLE("=", " "),	NULL,		0400,
	X1f4_E4_CALL,		c_____f_f,
	SIDE_CO_BITS,		1,
	NULL,			NULL				},
    {	MAKE_SINGLE("=", " "),	NULL,		0400,
	X1f4_E4_CALL,		c_____f_c,
	SIDE_CO_BITS,		1,
	NULL,			NULL				},
    {	MAKE_SINGLE("=", " "),	NULL,		0400,
	X1f4_E4_BILL,		c_____b_c,
	SIDE_CL_BITS,		1,
	NULL,			NULL				},
    {	MAKE_SINGLE("=", " "),	NULL,		0400,
	X1f4_E4_MODE,		c_____m_c,
	SIDE_CL_BITS,		1,
	NULL,			NULL				},
    {	MAKE_SINGLE("=", " "),	NULL,		0400,
	X1f4_E4_REAL,		c_____r_c,
	SIDE_CL_BITS,		1,
	NULL,			NULL				},
    {	MAKE_SINGLE("=", " "),	NULL,		0400,
	X1f4_E4_TEXT,		c_____t_c,
	SIDE_CL_BITS,		1,
	NULL,			NULL				},
    {	MAKE_DOUBLE("/", "="),	NULL,		0400,
	X1f4_E4_BILL,		c_____b_c,
	SIDE_CL_BITS,		2,
	NULL,			NULL				},
    {	MAKE_DOUBLE("/", "="),	NULL,		0400,
	X1f4_E4_MODE,		c_____m_c,
	SIDE_CL_BITS,		2,
	NULL,			NULL				},
    {	MAKE_DOUBLE("/", "="),	NULL,		0400,
	X1f4_E4_REAL,		c_____r_c,
	SIDE_CL_BITS,		2,
	NULL,			NULL				},
    {	MAKE_DOUBLE("*", "="),	NULL,		0400,
	X1f4_E4_BILL,		c_____b_c,
	SIDE_CL_BITS,		2,
	NULL,			NULL				},
    {	MAKE_DOUBLE("*", "="),	NULL,		0400,
	X1f4_E4_MODE,		c_____m_c,
	SIDE_CL_BITS,		2,
	NULL,			NULL				},
    {	MAKE_DOUBLE("*", "="),	NULL,		0400,
	X1f4_E4_REAL,		c_____r_c,
	SIDE_CL_BITS,		2,
	NULL,			NULL				},
    {	MAKE_DOUBLE("%", "="),	NULL,		0400,
	X1f4_E4_BILL,		c_____b_c,
	SIDE_CL_BITS,		2,
	NULL,			NULL				},
    {	MAKE_DOUBLE("%", "="),	NULL,		0400,
	X1f4_E4_MODE,		c_____m_c,
	SIDE_CL_BITS,		2,
	NULL,			NULL				},
    {	MAKE_SINGLE("+", " "),	NULL,		0230,
	X1f4_E4_CASE,		c_____c_b,
	SIDE_CE_BITS,		1,
	NULL,			NULL				},
    {	MAKE_SINGLE("+", " "),	NULL,		0230,
	X1f4_E4_CASE,		c_____c_m,
	SIDE_CE_BITS,		1,
	NULL,			NULL				},
    {	MAKE_SINGLE("+", " "),	NULL,		0230,
	X1f4_E4_REAL,		c_____c_r,
	SIDE_CE_BITS,		1,
	NULL,			NULL				},
    {	MAKE_SINGLE("+", " "),	NULL,		0230,
	X1f4_E4_TEXT,		c_____c_t,
	SIDE_CE_BITS,		1,
	NULL,			NULL				},
    {	MAKE_SINGLE("+", " "),	NULL,		0230,
	X1f4_E4_CASE,		c_____b_c,
	SIDE_CE_BITS,		1,
	NULL,			NULL				},
    {	MAKE_SINGLE("+", " "),	NULL,		0230,
	X1f4_E4_CASE,		c_____m_c,
	SIDE_CE_BITS,		1,
	NULL,			NULL				},
    {	MAKE_SINGLE("+", " "),	NULL,		0230,
	X1f4_E4_REAL,		c_____r_c,
	SIDE_CE_BITS,		1,
	NULL,			NULL				},
    {	MAKE_SINGLE("+", " "),	NULL,		0230,
	X1f4_E4_TEXT,		c_____t_c,
	SIDE_CE_BITS,		1,
	NULL,			NULL				},
    {	MAKE_SINGLE("+", " "),	NULL,		0230,
	X1f4_E4_CASE,		c_____c_c,
	SIDE_CE_BITS,		1,
	NULL,			NULL				},
    {	MAKE_DOUBLE("&", "&"),	NULL,		0320,
	X1f4_E4_MODE,		c_____c_b,
	SIDE_CE_BITS,		2,
	NULL,			NULL				},
    {	MAKE_DOUBLE("&", "&"),	NULL,		0320,
	X1f4_E4_MODE,		c_____c_m,
	SIDE_CE_BITS,		2,
	NULL,			NULL				},
    {	MAKE_DOUBLE("&", "&"),	NULL,		0320,
	X1f4_E4_MODE,		c_____c_r,
	SIDE_CE_BITS,		2,
	NULL,			NULL				},
    {	MAKE_DOUBLE("&", "&"),	NULL,		0320,
	X1f4_E4_MODE,		c_____b_c,
	SIDE_CE_BITS,		2,
	NULL,			NULL				},
    {	MAKE_DOUBLE("&", "&"),	NULL,		0320,
	X1f4_E4_MODE,		c_____m_c,
	SIDE_CE_BITS,		2,
	NULL,			NULL				},
    {	MAKE_DOUBLE("&", "&"),	NULL,		0320,
	X1f4_E4_MODE,		c_____r_c,
	SIDE_CE_BITS,		2,
	NULL,			NULL				},
    {	MAKE_DOUBLE("&", "&"),	NULL,		0320,
	X1f4_E4_MODE,		c_____c_c,
	SIDE_CE_BITS,		2,
	NULL,			NULL				},
    {	MAKE_DOUBLE("|", "|"),	NULL,		0330,
	X1f4_E4_MODE,		c_____c_b,
	SIDE_CE_BITS,		2,
	NULL,			NULL				},
    {	MAKE_DOUBLE("|", "|"),	NULL,		0330,
	X1f4_E4_MODE,		c_____c_m,
	SIDE_CE_BITS,		2,
	NULL,			NULL				},
    {	MAKE_DOUBLE("|", "|"),	NULL,		0330,
	X1f4_E4_MODE,		c_____c_r,
	SIDE_CE_BITS,		2,
	NULL,			NULL				},
    {	MAKE_DOUBLE("|", "|"),	NULL,		0330,
	X1f4_E4_MODE,		c_____b_c,
	SIDE_CE_BITS,		2,
	NULL,			NULL				},
    {	MAKE_DOUBLE("|", "|"),	NULL,		0330,
	X1f4_E4_MODE,		c_____m_c,
	SIDE_CE_BITS,		2,
	NULL,			NULL				},
    {	MAKE_DOUBLE("|", "|"),	NULL,		0330,
	X1f4_E4_MODE,		c_____r_c,
	SIDE_CE_BITS,		2,
	NULL,			NULL				},
    {	MAKE_DOUBLE("|", "|"),	NULL,		0330,
	X1f4_E4_MODE,		c_____c_c,
	SIDE_CE_BITS,		2,
	NULL,			NULL				},
    {	MAKE_SINGLE("/", " "),	NULL,		0220,
	X1f4_E4_CASE,		c_____c_b,
	SIDE_CE_BITS,		1,
	NULL,			NULL				},
    {	MAKE_SINGLE("/", " "),	NULL,		0220,
	X1f4_E4_CASE,		c_____c_m,
	SIDE_CE_BITS,		1,
	NULL,			NULL				},
    {	MAKE_SINGLE("/", " "),	NULL,		0220,
	X1f4_E4_REAL,		c_____c_r,
	SIDE_CE_BITS,		1,
	NULL,			NULL				},
    {	MAKE_SINGLE("/", " "),	NULL,		0220,
	X1f4_E4_CASE,		c_____b_c,
	SIDE_CE_BITS,		1,
	NULL,			NULL				},
    {	MAKE_SINGLE("/", " "),	NULL,		0220,
	X1f4_E4_CASE,		c_____m_c,
	SIDE_CE_BITS,		1,
	NULL,			NULL				},
    {	MAKE_SINGLE("/", " "),	NULL,		0220,
	X1f4_E4_REAL,		c_____r_c,
	SIDE_CE_BITS,		1,
	NULL,			NULL				},
    {	MAKE_SINGLE("/", " "),	NULL,		0220,
	X1f4_E4_CASE,		c_____c_c,
	SIDE_CE_BITS,		1,
	NULL,			NULL				},
    {	MAKE_DOUBLE("=", "="),	NULL,		0260,
	X1f4_E4_MODE,		c_____c_b,
	SIDE_CE_BITS,		2,
	NULL,			NULL				},
    {	MAKE_DOUBLE("=", "="),	NULL,		0260,
	X1f4_E4_MODE,		c_____c_m,
	SIDE_CE_BITS,		2,
	NULL,			NULL				},
    {	MAKE_DOUBLE("=", "="),	NULL,		0260,
	X1f4_E4_MODE,		c_____c_r,
	SIDE_CE_BITS,		2,
	NULL,			NULL				},
    {	MAKE_DOUBLE("=", "="),	NULL,		0260,
	X1f4_E4_MODE,		c_____c_t,
	SIDE_CE_BITS,		2,
	NULL,			NULL				},
    {	MAKE_DOUBLE("=", "="),	NULL,		0260,
	X1f4_E4_MODE,		c_____b_c,
	SIDE_CE_BITS,		2,
	NULL,			NULL				},
    {	MAKE_DOUBLE("=", "="),	NULL,		0260,
	X1f4_E4_MODE,		c_____m_c,
	SIDE_CE_BITS,		2,
	NULL,			NULL				},
    {	MAKE_DOUBLE("=", "="),	NULL,		0260,
	X1f4_E4_MODE,		c_____r_c,
	SIDE_CE_BITS,		2,
	NULL,			NULL				},
    {	MAKE_DOUBLE("=", "="),	NULL,		0260,
	X1f4_E4_MODE,		c_____t_c,
	SIDE_CE_BITS,		2,
	NULL,			NULL				},
    {	MAKE_DOUBLE("=", "="),	NULL,		0260,
	X1f4_E4_MODE,		c_____c_c,
	SIDE_CE_BITS,		2,
	NULL,			NULL				},
    {	MAKE_VIOLET("<", "<"),	NULL,		0400,
	X1f4_E4_BILL,		c_____b_c,
	SIDE_CL_BITS,		3,
	NULL,			NULL				},
    {	MAKE_VIOLET("<", "<"),	NULL,		0400,
	X1f4_E4_MODE,		c_____m_c,
	SIDE_CL_BITS,		3,
	NULL,			NULL				},
    {	MAKE_SINGLE(">", " "),	NULL,		0250,
	X1f4_E4_MODE,		c_____c_b,
	SIDE_CE_BITS,		1,
	NULL,			NULL				},
    {	MAKE_SINGLE(">", " "),	NULL,		0250,
	X1f4_E4_MODE,		c_____c_m,
	SIDE_CE_BITS,		1,
	NULL,			NULL				},
    {	MAKE_SINGLE(">", " "),	NULL,		0250,
	X1f4_E4_MODE,		c_____c_r,
	SIDE_CE_BITS,		1,
	NULL,			NULL				},
    {	MAKE_SINGLE(">", " "),	NULL,		0250,
	X1f4_E4_MODE,		c_____c_t,
	SIDE_CE_BITS,		1,
	NULL,			NULL				},
    {	MAKE_SINGLE(">", " "),	NULL,		0250,
	X1f4_E4_MODE,		c_____b_c,
	SIDE_CE_BITS,		1,
	NULL,			NULL				},
    {	MAKE_SINGLE(">", " "),	NULL,		0250,
	X1f4_E4_MODE,		c_____m_c,
	SIDE_CE_BITS,		1,
	NULL,			NULL				},
    {	MAKE_SINGLE(">", " "),	NULL,		0250,
	X1f4_E4_MODE,		c_____r_c,
	SIDE_CE_BITS,		1,
	NULL,			NULL				},
    {	MAKE_SINGLE(">", " "),	NULL,		0250,
	X1f4_E4_MODE,		c_____t_c,
	SIDE_CE_BITS,		1,
	NULL,			NULL				},
    {	MAKE_SINGLE(">", " "),	NULL,		0250,
	X1f4_E4_MODE,		c_____c_c,
	SIDE_CE_BITS,		1,
	NULL,			NULL				},
    {	MAKE_DOUBLE("&", "="),	NULL,		0400,
	X1f4_E4_BILL,		c_____b_c,
	SIDE_CL_BITS,		2,
	NULL,			NULL				},
    {	MAKE_DOUBLE("&", "="),	NULL,		0400,
	X1f4_E4_MODE,		c_____m_c,
	SIDE_CL_BITS,		2,
	NULL,			NULL				},
    {	MAKE_DOUBLE("^", "="),	NULL,		0400,
	X1f4_E4_BILL,		c_____b_c,
	SIDE_CL_BITS,		2,
	NULL,			NULL				},
    {	MAKE_DOUBLE("^", "="),	NULL,		0400,
	X1f4_E4_MODE,		c_____m_c,
	SIDE_CL_BITS,		2,
	NULL,			NULL				},
    {	MAKE_DOUBLE("|", "="),	NULL,		0400,
	X1f4_E4_BILL,		c_____b_c,
	SIDE_CL_BITS,		2,
	NULL,			NULL				},
    {	MAKE_DOUBLE("|", "="),	NULL,		0400,
	X1f4_E4_MODE,		c_____m_c,
	SIDE_CL_BITS,		2,
	NULL,			NULL				},
    {	MAKE_DOUBLE("<", "<"),	NULL,		0240,
	X1f4_E4_BILL,		c_____b_c,
	SIDE_CE_BITS,		2,
	NULL,			NULL				},
    {	MAKE_DOUBLE("<", "<"),	NULL,		0240,
	X1f4_E4_MODE,		c_____m_c,
	SIDE_CE_BITS,		2,
	NULL,			NULL				},
    {	MAKE_DOUBLE("<", "<"),	NULL,		0240,
	X1f4_E4_CASE,		c_____c_b,
	SIDE_CE_BITS,		2,
	NULL,			NULL				},
    {	MAKE_DOUBLE("<", "<"),	NULL,		0240,
	X1f4_E4_CASE,		c_____c_m,
	SIDE_CE_BITS,		2,
	NULL,			NULL				},
    {	MAKE_DOUBLE("<", "<"),	NULL,		0240,
	X1f4_E4_CASE,		c_____c_c,
	SIDE_CE_BITS,		2,
	NULL,			NULL				},
    {	MAKE_SINGLE("<", " "),	NULL,		0250,
	X1f4_E4_MODE,		c_____c_b,
	SIDE_CE_BITS,		1,
	NULL,			NULL				},
    {	MAKE_SINGLE("<", " "),	NULL,		0250,
	X1f4_E4_MODE,		c_____c_m,
	SIDE_CE_BITS,		1,
	NULL,			NULL				},
    {	MAKE_SINGLE("<", " "),	NULL,		0250,
	X1f4_E4_MODE,		c_____c_r,
	SIDE_CE_BITS,		1,
	NULL,			NULL				},
    {	MAKE_SINGLE("<", " "),	NULL,		0250,
	X1f4_E4_MODE,		c_____c_t,
	SIDE_CE_BITS,		1,
	NULL,			NULL				},
    {	MAKE_SINGLE("<", " "),	NULL,		0250,
	X1f4_E4_MODE,		c_____b_c,
	SIDE_CE_BITS,		1,
	NULL,			NULL				},
    {	MAKE_SINGLE("<", " "),	NULL,		0250,
	X1f4_E4_MODE,		c_____m_c,
	SIDE_CE_BITS,		1,
	NULL,			NULL				},
    {	MAKE_SINGLE("<", " "),	NULL,		0250,
	X1f4_E4_MODE,		c_____r_c,
	SIDE_CE_BITS,		1,
	NULL,			NULL				},
    {	MAKE_SINGLE("<", " "),	NULL,		0250,
	X1f4_E4_MODE,		c_____t_c,
	SIDE_CE_BITS,		1,
	NULL,			NULL				},
    {	MAKE_SINGLE("<", " "),	NULL,		0250,
	X1f4_E4_MODE,		c_____c_c,
	SIDE_CE_BITS,		1,
	NULL,			NULL				},
    {	MAKE_SINGLE("*", " "),	NULL,		0220,
	X1f4_E4_CASE,		c_____c_b,
	SIDE_CE_BITS,		1,
	NULL,			NULL				},
    {	MAKE_SINGLE("*", " "),	NULL,		0220,
	X1f4_E4_CASE,		c_____c_m,
	SIDE_CE_BITS,		1,
	NULL,			NULL				},
    {	MAKE_SINGLE("*", " "),	NULL,		0220,
	X1f4_E4_REAL,		c_____c_r,
	SIDE_CE_BITS,		1,
	NULL,			NULL				},
    {	MAKE_SINGLE("*", " "),	NULL,		0220,
	X1f4_E4_CASE,		c_____b_c,
	SIDE_CE_BITS,		1,
	NULL,			NULL				},
    {	MAKE_SINGLE("*", " "),	NULL,		0220,
	X1f4_E4_CASE,		c_____m_c,
	SIDE_CE_BITS,		1,
	NULL,			NULL				},
    {	MAKE_SINGLE("*", " "),	NULL,		0220,
	X1f4_E4_REAL,		c_____r_c,
	SIDE_CE_BITS,		1,
	NULL,			NULL				},
    {	MAKE_SINGLE("*", " "),	NULL,		0220,
	X1f4_E4_CASE,		c_____c_c,
	SIDE_CE_BITS,		1,
	NULL,			NULL				},
    {	MAKE_DOUBLE("+", "="),	NULL,		0400,
	X1f4_E4_BILL,		c_____b_c,
	SIDE_CL_BITS,		2,
	NULL,			NULL				},
    {	MAKE_DOUBLE("+", "="),	NULL,		0400,
	X1f4_E4_MODE,		c_____m_c,
	SIDE_CL_BITS,		2,
	NULL,			NULL				},
    {	MAKE_DOUBLE("+", "="),	NULL,		0400,
	X1f4_E4_REAL,		c_____r_c,
	SIDE_CL_BITS,		2,
	NULL,			NULL				},
    {	MAKE_DOUBLE("-", "="),	NULL,		0400,
	X1f4_E4_BILL,		c_____b_c,
	SIDE_CL_BITS,		2,
	NULL,			NULL				},
    {	MAKE_DOUBLE("-", "="),	NULL,		0400,
	X1f4_E4_MODE,		c_____m_c,
	SIDE_CL_BITS,		2,
	NULL,			NULL				},
    {	MAKE_DOUBLE("-", "="),	NULL,		0400,
	X1f4_E4_REAL,		c_____r_c,
	SIDE_CL_BITS,		2,
	NULL,			NULL				},
    {	MAKE_SINGLE("%", " "),	NULL,		0220,
	X1f4_E4_BILL,		c_____b_c,
	SIDE_CE_BITS,		1,
	NULL,			NULL				},
    {	MAKE_SINGLE("%", " "),	NULL,		0220,
	X1f4_E4_CASE,		c_____m_c,
	SIDE_CE_BITS,		1,
	NULL,			NULL				},
    {	MAKE_SINGLE("%", " "),	NULL,		0220,
	X1f4_E4_BILL,		c_____c_b,
	SIDE_CE_BITS,		1,
	NULL,			NULL				},
    {	MAKE_SINGLE("%", " "),	NULL,		0220,
	X1f4_E4_CASE,		c_____c_m,
	SIDE_CE_BITS,		1,
	NULL,			NULL				},
    {	MAKE_SINGLE("%", " "),	NULL,		0220,
	X1f4_E4_CASE,		c_____c_c,
	SIDE_CE_BITS,		1,
	NULL,			NULL				},
    {	MAKE_SINGLE("-", " "),	NULL,		0230,
	X1f4_E4_CASE,		c_____c_b,
	SIDE_CE_BITS,		1,
	NULL,			NULL				},
    {	MAKE_SINGLE("-", " "),	NULL,		0230,
	X1f4_E4_CASE,		c_____c_m,
	SIDE_CE_BITS,		1,
	NULL,			NULL				},
    {	MAKE_SINGLE("-", " "),	NULL,		0230,
	X1f4_E4_REAL,		c_____c_r,
	SIDE_CE_BITS,		1,
	NULL,			NULL				},
    {	MAKE_SINGLE("-", " "),	NULL,		0230,
	X1f4_E4_CASE,		c_____b_c,
	SIDE_CE_BITS,		1,
	NULL,			NULL				},
    {	MAKE_SINGLE("-", " "),	NULL,		0230,
	X1f4_E4_CASE,		c_____m_c,
	SIDE_CE_BITS,		1,
	NULL,			NULL				},
    {	MAKE_SINGLE("-", " "),	NULL,		0230,
	X1f4_E4_REAL,		c_____r_c,
	SIDE_CE_BITS,		1,
	NULL,			NULL				},
    {	MAKE_SINGLE("-", " "),	NULL,		0230,
	X1f4_E4_CASE,		c_____c_c,
	SIDE_CE_BITS,		1,
	NULL,			NULL				},
    {	MAKE_DOUBLE(">", ">"),	NULL,		0240,
	X1f4_E4_BILL,		c_____b_c,
	SIDE_CE_BITS,		2,
	NULL,			NULL				},
    {	MAKE_DOUBLE(">", ">"),	NULL,		0240,
	X1f4_E4_MODE,		c_____m_c,
	SIDE_CE_BITS,		2,
	NULL,			NULL				},
    {	MAKE_DOUBLE(">", ">"),	NULL,		0240,
	X1f4_E4_CASE,		c_____c_b,
	SIDE_CE_BITS,		2,
	NULL,			NULL				},
    {	MAKE_DOUBLE(">", ">"),	NULL,		0240,
	X1f4_E4_CASE,		c_____c_m,
	SIDE_CE_BITS,		2,
	NULL,			NULL				},
    {	MAKE_DOUBLE(">", ">"),	NULL,		0240,
	X1f4_E4_CASE,		c_____c_c,
	SIDE_CE_BITS,		2,
	NULL,			NULL				},
    {	MAKE_VIOLET(">", ">"),	NULL,		0400,
	X1f4_E4_BILL,		c_____b_c,
	SIDE_CL_BITS,		3,
	NULL,			NULL				},
    {	MAKE_VIOLET(">", ">"),	NULL,		0400,
	X1f4_E4_MODE,		c_____m_c,
	SIDE_CL_BITS,		3,
	NULL,			NULL				},
    {	MAKE_DOUBLE("!", "="),	NULL,		0260,
	X1f4_E4_MODE,		c_____c_b,
	SIDE_CE_BITS,		2,
	NULL,			NULL				},
    {	MAKE_DOUBLE("!", "="),	NULL,		0260,
	X1f4_E4_MODE,		c_____c_m,
	SIDE_CE_BITS,		2,
	NULL,			NULL				},
    {	MAKE_DOUBLE("!", "="),	NULL,		0260,
	X1f4_E4_MODE,		c_____c_r,
	SIDE_CE_BITS,		2,
	NULL,			NULL				},
    {	MAKE_DOUBLE("!", "="),	NULL,		0260,
	X1f4_E4_MODE,		c_____c_t,
	SIDE_CE_BITS,		2,
	NULL,			NULL				},
    {	MAKE_DOUBLE("!", "="),	NULL,		0260,
	X1f4_E4_MODE,		c_____b_c,
	SIDE_CE_BITS,		2,
	NULL,			NULL				},
    {	MAKE_DOUBLE("!", "="),	NULL,		0260,
	X1f4_E4_MODE,		c_____m_c,
	SIDE_CE_BITS,		2,
	NULL,			NULL				},
    {	MAKE_DOUBLE("!", "="),	NULL,		0260,
	X1f4_E4_MODE,		c_____r_c,
	SIDE_CE_BITS,		2,
	NULL,			NULL				},
    {	MAKE_DOUBLE("!", "="),	NULL,		0260,
	X1f4_E4_MODE,		c_____t_c,
	SIDE_CE_BITS,		2,
	NULL,			NULL				},
    {	MAKE_DOUBLE("!", "="),	NULL,		0260,
	X1f4_E4_MODE,		c_____c_c,
	SIDE_CE_BITS,		2,
	NULL,			NULL				},
    {	MAKE_SINGLE("&", " "),	NULL,		0270,
	X1f4_E4_BILL,		c_____b_c,
	SIDE_CE_BITS,		1,
	NULL,			NULL				},
    {	MAKE_SINGLE("&", " "),	NULL,		0270,
	X1f4_E4_CASE,		c_____m_c,
	SIDE_CE_BITS,		1,
	NULL,			NULL				},
    {	MAKE_SINGLE("&", " "),	NULL,		0270,
	X1f4_E4_BILL,		c_____c_b,
	SIDE_CE_BITS,		1,
	NULL,			NULL				},
    {	MAKE_SINGLE("&", " "),	NULL,		0270,
	X1f4_E4_CASE,		c_____c_m,
	SIDE_CE_BITS,		1,
	NULL,			NULL				},
    {	MAKE_SINGLE("&", " "),	NULL,		0270,
	X1f4_E4_CASE,		c_____c_c,
	SIDE_CE_BITS,		1,
	NULL,			NULL				},
    {	MAKE_SINGLE("^", " "),	NULL,		0300,
	X1f4_E4_BILL,		c_____b_c,
	SIDE_CE_BITS,		1,
	NULL,			NULL				},
    {	MAKE_SINGLE("^", " "),	NULL,		0300,
	X1f4_E4_CASE,		c_____m_c,
	SIDE_CE_BITS,		1,
	NULL,			NULL				},
    {	MAKE_SINGLE("^", " "),	NULL,		0300,
	X1f4_E4_BILL,		c_____c_b,
	SIDE_CE_BITS,		1,
	NULL,			NULL				},
    {	MAKE_SINGLE("^", " "),	NULL,		0300,
	X1f4_E4_CASE,		c_____c_m,
	SIDE_CE_BITS,		1,
	NULL,			NULL				},
    {	MAKE_SINGLE("^", " "),	NULL,		0300,
	X1f4_E4_CASE,		c_____c_c,
	SIDE_CE_BITS,		1,
	NULL,			NULL				},
    {	MAKE_SINGLE("|", " "),	NULL,		0310,
	X1f4_E4_BILL,		c_____b_c,
	SIDE_CE_BITS,		1,
	NULL,			NULL				},
    {	MAKE_SINGLE("|", " "),	NULL,		0310,
	X1f4_E4_CASE,		c_____m_c,
	SIDE_CE_BITS,		1,
	NULL,			NULL				},
    {	MAKE_SINGLE("|", " "),	NULL,		0310,
	X1f4_E4_BILL,		c_____c_b,
	SIDE_CE_BITS,		1,
	NULL,			NULL				},
    {	MAKE_SINGLE("|", " "),	NULL,		0310,
	X1f4_E4_CASE,		c_____c_m,
	SIDE_CE_BITS,		1,
	NULL,			NULL				},
    {	MAKE_SINGLE("|", " "),	NULL,		0310,
	X1f4_E4_CASE,		c_____c_c,
	SIDE_CE_BITS,		1,
	NULL,			NULL				},
    {	MAKE_DOUBLE("<", "="),	NULL,		0250,
	X1f4_E4_MODE,		c_____c_b,
	SIDE_CE_BITS,		2,
	NULL,			NULL				},
    {	MAKE_DOUBLE("<", "="),	NULL,		0250,
	X1f4_E4_MODE,		c_____c_m,
	SIDE_CE_BITS,		2,
	NULL,			NULL				},
    {	MAKE_DOUBLE("<", "="),	NULL,		0250,
	X1f4_E4_MODE,		c_____c_r,
	SIDE_CE_BITS,		2,
	NULL,			NULL				},
    {	MAKE_DOUBLE("<", "="),	NULL,		0250,
	X1f4_E4_MODE,		c_____c_t,
	SIDE_CE_BITS,		2,
	NULL,			NULL				},
    {	MAKE_DOUBLE("<", "="),	NULL,		0250,
	X1f4_E4_MODE,		c_____b_c,
	SIDE_CE_BITS,		2,
	NULL,			NULL				},
    {	MAKE_DOUBLE("<", "="),	NULL,		0250,
	X1f4_E4_MODE,		c_____m_c,
	SIDE_CE_BITS,		2,
	NULL,			NULL				},
    {	MAKE_DOUBLE("<", "="),	NULL,		0250,
	X1f4_E4_MODE,		c_____r_c,
	SIDE_CE_BITS,		2,
	NULL,			NULL				},
    {	MAKE_DOUBLE("<", "="),	NULL,		0250,
	X1f4_E4_MODE,		c_____t_c,
	SIDE_CE_BITS,		2,
	NULL,			NULL				},
    {	MAKE_DOUBLE("<", "="),	NULL,		0250,
	X1f4_E4_MODE,		c_____c_c,
	SIDE_CE_BITS,		2,
	NULL,			NULL				},
    {	MAKE_DOUBLE(">", "="),	NULL,		0250,
	X1f4_E4_MODE,		c_____c_b,
	SIDE_CE_BITS,		2,
	NULL,			NULL				},
    {	MAKE_DOUBLE(">", "="),	NULL,		0250,
	X1f4_E4_MODE,		c_____c_m,
	SIDE_CE_BITS,		2,
	NULL,			NULL				},
    {	MAKE_DOUBLE(">", "="),	NULL,		0250,
	X1f4_E4_MODE,		c_____c_r,
	SIDE_CE_BITS,		2,
	NULL,			NULL				},
    {	MAKE_DOUBLE(">", "="),	NULL,		0250,
	X1f4_E4_MODE,		c_____c_t,
	SIDE_CE_BITS,		2,
	NULL,			NULL				},
    {	MAKE_DOUBLE(">", "="),	NULL,		0250,
	X1f4_E4_MODE,		c_____b_c,
	SIDE_CE_BITS,		2,
	NULL,			NULL				},
    {	MAKE_DOUBLE(">", "="),	NULL,		0250,
	X1f4_E4_MODE,		c_____m_c,
	SIDE_CE_BITS,		2,
	NULL,			NULL				},
    {	MAKE_DOUBLE(">", "="),	NULL,		0250,
	X1f4_E4_MODE,		c_____r_c,
	SIDE_CE_BITS,		2,
	NULL,			NULL				},
    {	MAKE_DOUBLE(">", "="),	NULL,		0250,
	X1f4_E4_MODE,		c_____t_c,
	SIDE_CE_BITS,		2,
	NULL,			NULL				},
    {	MAKE_DOUBLE(">", "="),	NULL,		0250,
	X1f4_E4_MODE,		c_____c_c,
	SIDE_CE_BITS,		2,
	NULL,			NULL				}
/* *INDENT-ON* */
}, _libx1f4i0_lxcast_side_x[] = {
/* *INDENT-OFF* */
    {	MAKE_SINGLE("=", " "),	NULL,		0400,
	0,			NULL,
	SIDE_CL_BITS,		1,
	NULL,			NULL				}
/* *INDENT-ON* */
};
const unsigned _libx1f4i0_lxcast_size_o =
    sizeof(_libx1f4i0_lxcast_side_o) / sizeof(struct x1f4_operator_type);

static int
a2cxx_b_c(void *context, void *output, void **input)
{
    EVER_FOLD(/=, BILL);
}


static int
a2cxx_m_c(void *context, void *output, void **input)
{
    EVER_FOLD(/=, MODE);
}


static int
a2cxx_r_c(void *context, void *output, void **input)
{
    EVER_FOLD(/=, REAL);
}


static int
a2exx_b_c(void *context, void *output, void **input)
{
    EVER_FOLD(*=, BILL);
}


static int
a2exx_m_c(void *context, void *output, void **input)
{
    EVER_FOLD(*=, MODE);
}


static int
a2exx_r_c(void *context, void *output, void **input)
{
    EVER_FOLD(*=, REAL);
}


static int
a5fxx_b_c(void *context, void *output, void **input)
{
    EVER_LONG(%=, BILL);
}


static int
a5fxx_m_c(void *context, void *output, void **input)
{
    EVER_LONG(%=, MODE);
}


static int
a61xx_b_c(void *context, void *output, void **input)
{
    BILL_LAST(+);
}


static int
a61xx_c_b(void *context, void *output, void **input)
{
    BILL_POST(+);
}


static int
a61xx_c_c(void *context, void *output, void **input)
{
    VIEW_DASH(+, e61xx_t_t);
}


static int
a61xx_c_m(void *context, void *output, void **input)
{
    MODE_POST(+);
}


static int
a61xx_c_r(void *context, void *output, void **input)
{
    REAL_POST(+);
}


static int
a61xx_c_t(void *context, void *output, void **input)
{
    STAR_POST();
}


static int
a61xx_m_c(void *context, void *output, void **input)
{
    MODE_LAST(+);
}


static int
a61xx_r_c(void *context, void *output, void **input)
{
    REAL_LAST(+);
}


static int
a61xx_t_c(void *context, void *output, void **input)
{
    STAR_LAST();
}


static int
a62xx_b_c(void *context, void *output, void **input)
{
    UNIT_LAST(&&, BILL);
}


static int
a62xx_c_b(void *context, void *output, void **input)
{
    UNIT_POST(&&, BILL);
}


static int
a62xx_c_c(void *context, void *output, void **input)
{
    UNIT_FLAT(&&);
}


static int
a62xx_c_m(void *context, void *output, void **input)
{
    UNIT_POST(&&, MODE);
}


static int
a62xx_c_r(void *context, void *output, void **input)
{
    UNIT_POST(&&, REAL);
}


static int
a62xx_m_c(void *context, void *output, void **input)
{
    UNIT_LAST(&&, MODE);
}


static int
a62xx_r_c(void *context, void *output, void **input)
{
    UNIT_LAST(&&, REAL);
}


static int
a63xx_b_c(void *context, void *output, void **input)
{
    UNIT_LAST(||, BILL);
}


static int
a63xx_c_b(void *context, void *output, void **input)
{
    UNIT_POST(||, BILL);
}


static int
a63xx_c_c(void *context, void *output, void **input)
{
    UNIT_FLAT(||);
}


static int
a63xx_c_m(void *context, void *output, void **input)
{
    UNIT_POST(||, MODE);
}


static int
a63xx_c_r(void *context, void *output, void **input)
{
    UNIT_POST(||, REAL);
}


static int
a63xx_m_c(void *context, void *output, void **input)
{
    UNIT_LAST(||, MODE);
}


static int
a63xx_r_c(void *context, void *output, void **input)
{
    UNIT_LAST(||, REAL);
}


static int
a64xx_b_c(void *context, void *output, void **input)
{
    BILL_LAST_ZERO(/);
}


static int
a64xx_c_b(void *context, void *output, void **input)
{
    BILL_POST_ZERO(/);
}


static int
a64xx_c_c(void *context, void *output, void **input)
{
    TRUE_DASH_ZERO(/);
}


static int
a64xx_c_m(void *context, void *output, void **input)
{
    MODE_POST_ZERO(/);
}


static int
a64xx_c_r(void *context, void *output, void **input)
{
    REAL_POST_ZERO(/);
}


static int
a64xx_m_c(void *context, void *output, void **input)
{
    MODE_LAST_ZERO(/);
}


static int
a64xx_r_c(void *context, void *output, void **input)
{
    REAL_LAST_ZERO(/);
}


static int
a65xx_b_c(void *context, void *output, void **input)
{
    UNIT_LAST(==, BILL);
}


static int
a65xx_c_b(void *context, void *output, void **input)
{
    UNIT_POST(==, BILL);
}


static int
a65xx_c_c(void *context, void *output, void **input)
{
    UNIT_HALF(==);
}


static int
a65xx_c_m(void *context, void *output, void **input)
{
    UNIT_POST(==, MODE);
}


static int
a65xx_c_r(void *context, void *output, void **input)
{
    UNIT_POST(==, REAL);
}


static int
a65xx_c_t(void *context, void *output, void **input)
{
    NORM_POST(==, TEXT);
}


static int
a65xx_m_c(void *context, void *output, void **input)
{
    UNIT_LAST(==, MODE);
}


static int
a65xx_r_c(void *context, void *output, void **input)
{
    UNIT_LAST(==, REAL);
}


static int
a65xx_t_c(void *context, void *output, void **input)
{
    NORM_LAST(==, TEXT);
}


static int
a66xx_b_c(void *context, void *output, void **input)
{
    EVER_LONG(<<=, BILL);
}


static int
a66xx_m_c(void *context, void *output, void **input)
{
    EVER_LONG(<<=, MODE);
}


static int
a67xx_b_c(void *context, void *output, void **input)
{
    UNIT_LAST(>, BILL);
}


static int
a67xx_c_b(void *context, void *output, void **input)
{
    UNIT_POST(>, BILL);
}


static int
a67xx_c_m(void *context, void *output, void **input)
{
    UNIT_POST(>, MODE);
}


static int
a67xx_c_r(void *context, void *output, void **input)
{
    UNIT_POST(>, REAL);
}


static int
a67xx_c_t(void *context, void *output, void **input)
{
    NORM_POST(>, TEXT);
}


static int
a67xx_m_c(void *context, void *output, void **input)
{
    UNIT_LAST(>, MODE);
}


static int
a67xx_r_c(void *context, void *output, void **input)
{
    UNIT_LAST(>, REAL);
}


static int
a67xx_t_c(void *context, void *output, void **input)
{
    NORM_LAST(>, TEXT);
}


static int
a67xx_c_c(void *context, void *output, void **input)
{
    UNIT_HALF(>);
}


static int
a68xx_b_c(void *context, void *output, void **input)
{
    EVER_LONG(&=, BILL);
}


static int
a68xx_m_c(void *context, void *output, void **input)
{
    EVER_LONG(&=, MODE);
}


static int
a69xx_b_c(void *context, void *output, void **input)
{
    EVER_LONG(^=, BILL);
}


static int
a69xx_m_c(void *context, void *output, void **input)
{
    EVER_LONG(^=, MODE);
}


static int
a6axx_b_c(void *context, void *output, void **input)
{
    EVER_LONG(|=, BILL);
}


static int
a6axx_m_c(void *context, void *output, void **input)
{
    EVER_LONG(|=, MODE);
}


static int
a6bxx_b_c(void *context, void *output, void **input)
{
    FARE_LAST(<<=, BILL);
}


static int
a6bxx_c_b(void *context, void *output, void **input)
{
    LEAD_POST(<<, BILL);
}


static int
a6bxx_c_m(void *context, void *output, void **input)
{
    LEAD_POST(<<, MODE);
}


static int
a6bxx_m_c(void *context, void *output, void **input)
{
    FARE_LAST(<<=, MODE);
}


static int
a6bxx_c_c(void *context, void *output, void **input)
{
    HALF_DASH(<<);
}


static int
a6cxx_b_c(void *context, void *output, void **input)
{
    UNIT_LAST(<, BILL);
}


static int
a6cxx_c_b(void *context, void *output, void **input)
{
    UNIT_POST(<, BILL);
}


static int
a6cxx_c_m(void *context, void *output, void **input)
{
    UNIT_POST(<, MODE);
}


static int
a6cxx_c_r(void *context, void *output, void **input)
{
    UNIT_POST(<, REAL);
}


static int
a6cxx_c_t(void *context, void *output, void **input)
{
    NORM_POST(<, TEXT);
}


static int
a6cxx_m_c(void *context, void *output, void **input)
{
    UNIT_LAST(<, MODE);
}


static int
a6cxx_r_c(void *context, void *output, void **input)
{
    UNIT_LAST(<, REAL);
}


static int
a6cxx_t_c(void *context, void *output, void **input)
{
    NORM_LAST(<, TEXT);
}


static int
a6cxx_c_c(void *context, void *output, void **input)
{
    UNIT_HALF(<);
}


static int
a6dxx_b_c(void *context, void *output, void **input)
{
    BILL_LAST(*);
}


static int
a6dxx_c_b(void *context, void *output, void **input)
{
    BILL_POST(*);
}


static int
a6dxx_c_c(void *context, void *output, void **input)
{
    TRUE_DASH(*);
}


static int
a6dxx_c_m(void *context, void *output, void **input)
{
    MODE_POST(*);
}


static int
a6dxx_c_r(void *context, void *output, void **input)
{
    REAL_POST(*);
}


static int
a6dxx_m_c(void *context, void *output, void **input)
{
    MODE_LAST(*);
}


static int
a6dxx_r_c(void *context, void *output, void **input)
{
    REAL_LAST(*);
}


static int
a70xx_b_c(void *context, void *output, void **input)
{
    EVER_FOLD(+=, BILL);
}


static int
a70xx_m_c(void *context, void *output, void **input)
{
    EVER_FOLD(+=, MODE);
}


static int
a70xx_r_c(void *context, void *output, void **input)
{
    EVER_FOLD(+=, REAL);
}


static int
a71xx_b_c(void *context, void *output, void **input)
{
    EVER_FOLD(-=, BILL);
}


static int
a71xx_m_c(void *context, void *output, void **input)
{
    EVER_FOLD(-=, MODE);
}


static int
a71xx_r_c(void *context, void *output, void **input)
{
    EVER_FOLD(-=, REAL);
}


static int
a72xx_b_c(void *context, void *output, void **input)
{
    FARE_LAST_ZERO(%=, BILL);
}


static int
a72xx_c_b(void *context, void *output, void **input)
{
    FARE_POST_ZERO(%, BILL);
}


static int
a72xx_c_c(void *context, void *output, void **input)
{
    OVER_DASH_ZERO(%);
}


static int
a72xx_c_m(void *context, void *output, void **input)
{
    LEAD_POST_ZERO(%, MODE);
}


static int
a72xx_m_c(void *context, void *output, void **input)
{
    LEAD_LAST_ZERO(%, MODE);
}


static int
a73xx_b_c(void *context, void *output, void **input)
{
    BILL_LAST(-);
}


static int
a73xx_c_b(void *context, void *output, void **input)
{
    BILL_POST(-);
}


static int
a73xx_c_c(void *context, void *output, void **input)
{
    TRUE_DASH(-);
}


static int
a73xx_c_m(void *context, void *output, void **input)
{
    MODE_POST(-);
}


static int
a73xx_c_r(void *context, void *output, void **input)
{
    REAL_POST(-);
}


static int
a73xx_m_c(void *context, void *output, void **input)
{
    MODE_LAST(-);
}


static int
a73xx_r_c(void *context, void *output, void **input)
{
    REAL_LAST(-);
}


static int
a74xx_b_c(void *context, void *output, void **input)
{
    FARE_LAST(>>=, BILL);
}


static int
a74xx_c_b(void *context, void *output, void **input)
{
    LEAD_POST(>>, BILL);
}


static int
a74xx_c_m(void *context, void *output, void **input)
{
    LEAD_POST(>>, MODE);
}


static int
a74xx_m_c(void *context, void *output, void **input)
{
    FARE_LAST(>>=, MODE);
}


static int
a74xx_c_c(void *context, void *output, void **input)
{
    HALF_DASH(>>);
}


static int
a75xx_b_c(void *context, void *output, void **input)
{
    EVER_LONG(>>=, BILL);
}


static int
a75xx_m_c(void *context, void *output, void **input)
{
    EVER_LONG(>>=, MODE);
}


static int
a76xx_b_c(void *context, void *output, void **input)
{
    UNIT_LAST(!=, BILL);
}


static int
a76xx_c_b(void *context, void *output, void **input)
{
    UNIT_POST(!=, BILL);
}


static int
a76xx_c_c(void *context, void *output, void **input)
{
    UNIT_HALF(!=);
}


static int
a76xx_c_m(void *context, void *output, void **input)
{
    UNIT_POST(!=, MODE);
}


static int
a76xx_c_r(void *context, void *output, void **input)
{
    UNIT_POST(!=, REAL);
}


static int
a76xx_c_t(void *context, void *output, void **input)
{
    NORM_POST(!=, TEXT);
}


static int
a76xx_m_c(void *context, void *output, void **input)
{
    UNIT_LAST(!=, MODE);
}


static int
a76xx_r_c(void *context, void *output, void **input)
{
    UNIT_LAST(!=, REAL);
}


static int
a76xx_t_c(void *context, void *output, void **input)
{
    NORM_LAST(!=, TEXT);
}


static int
a77xx_b_c(void *context, void *output, void **input)
{
    FARE_LAST(&=, BILL);
}


static int
a77xx_c_b(void *context, void *output, void **input)
{
    FARE_POST(&, BILL);
}


static int
a77xx_c_c(void *context, void *output, void **input)
{
    OVER_DASH(&);
}


static int
a77xx_c_m(void *context, void *output, void **input)
{
    LEAD_POST(&, MODE);
}


static int
a77xx_m_c(void *context, void *output, void **input)
{
    LEAD_LAST(&, MODE);
}


static int
a78xx_b_c(void *context, void *output, void **input)
{
    FARE_LAST(^=, BILL);
}


static int
a78xx_c_b(void *context, void *output, void **input)
{
    FARE_POST(^, BILL);
}


static int
a78xx_c_c(void *context, void *output, void **input)
{
    OVER_DASH(^);
}


static int
a78xx_c_m(void *context, void *output, void **input)
{
    LEAD_POST(^, BILL);
}


static int
a78xx_m_c(void *context, void *output, void **input)
{
    LEAD_LAST(^, MODE);
}


static int
a79xx_b_c(void *context, void *output, void **input)
{
    FARE_LAST(|=, BILL);
}


static int
a79xx_c_b(void *context, void *output, void **input)
{
    FARE_POST(|, BILL);
}


static int
a79xx_c_c(void *context, void *output, void **input)
{
    OVER_DASH(|);
}


static int
a79xx_c_m(void *context, void *output, void **input)
{
    LEAD_POST(|, BILL);
}


static int
a79xx_m_c(void *context, void *output, void **input)
{
    LEAD_LAST(|, MODE);
}


static int
a7axx_b_c(void *context, void *output, void **input)
{
    EVER_COPY(BILL);
}


static int
a7axx_m_c(void *context, void *output, void **input)
{
    EVER_COPY(MODE);
}


static int
a7axx_r_c(void *context, void *output, void **input)
{
    EVER_COPY(REAL);
}


static int
a7axx_t_c(void *context, void *output, void **input)
{
    int name, status;
    struct dxcast_type *object_data;
    struct lxcast_type *lxcast_data;

    lxcast_data = context;

    object_data = I_USER(input[1]);

    name = object_data->type;

    if (name ^ X1f4_E4_TEXT) {
	do {
            struct x1f4_composer_type *composer_data;

            composer_data = &lxcast_data->link_q;
            if (composer_data->link) {
                const struct x1f4_linetext_type *linetext_data;

                status = composer_data->link
                    (composer_data->text, X1f4_E4_TEXT, name, &linetext_data);
                if (status) {
                } else {
                    void *track;

                    track = &object_data->data;
                    status = linetext_data->function.function
                        ((void *) linetext_data->context, output, &track);
                    if (status) {
                    } else {
			X1f4_E4_C_TEXT text;

			text = I_TEXT(output);

			status = _libx1f4i0_lxcall_swap_text
			    (&lxcast_data->ever_a, *input, text,
			     strlen(text) + 1);
                    }

		    break;
                }
            }

	    status = _libx1f4i0_lxcast_stat_luck
		(lxcast_data, 1, name, X1f4_E4_TEXT);
	} while (0);
    } else {
	X1f4_E4_C_TEXT text;

	status = 0;

	text = I_TEXT(&object_data->data);

	status = _libx1f4i0_lxcall_swap_text
	    (&lxcast_data->ever_a, *input, text, strlen(text) + 1);
	if (status) {
	} else {
	    l_TEXT(I_TEXT(input[0]), output);
	}
    }

    return status;
}


static int
a7bxx_b_c(void *context, void *output, void **input)
{
    UNIT_LAST(<=, BILL);
}


static int
a7bxx_c_b(void *context, void *output, void **input)
{
    UNIT_POST(<=, BILL);
}


static int
a7bxx_c_m(void *context, void *output, void **input)
{
    UNIT_POST(<=, MODE);
}


static int
a7bxx_c_r(void *context, void *output, void **input)
{
    UNIT_POST(<=, REAL);
}


static int
a7bxx_c_t(void *context, void *output, void **input)
{
    NORM_POST(<=, TEXT);
}


static int
a7bxx_m_c(void *context, void *output, void **input)
{
    UNIT_LAST(<=, MODE);
}


static int
a7bxx_r_c(void *context, void *output, void **input)
{
    UNIT_LAST(<=, REAL);
}


static int
a7bxx_t_c(void *context, void *output, void **input)
{
    NORM_LAST(<=, TEXT);
}


static int
a7bxx_c_c(void *context, void *output, void **input)
{
    UNIT_HALF(<=);
}


static int
a7cxx_b_c(void *context, void *output, void **input)
{
    UNIT_LAST(>=, BILL);
}


static int
a7cxx_c_b(void *context, void *output, void **input)
{
    UNIT_POST(>=, BILL);
}


static int
a7cxx_c_m(void *context, void *output, void **input)
{
    UNIT_POST(>=, MODE);
}


static int
a7cxx_c_r(void *context, void *output, void **input)
{
    UNIT_POST(>=, REAL);
}


static int
a7cxx_c_t(void *context, void *output, void **input)
{
    NORM_POST(>=, TEXT);
}


static int
a7cxx_m_c(void *context, void *output, void **input)
{
    UNIT_LAST(>=, MODE);
}


static int
a7cxx_r_c(void *context, void *output, void **input)
{
    UNIT_LAST(>=, REAL);
}


static int
a7cxx_t_c(void *context, void *output, void **input)
{
    NORM_LAST(>=, TEXT);
}


static int
a7cxx_c_c(void *context, void *output, void **input)
{
    UNIT_HALF(>=);
}


static int
d61xx_t_t(void *context, void *output, void *last, void *post)
{
    int status;
    struct lxcast_type *lxcast_data;
    unsigned lost, past;
    void *data;

    lxcast_data = context;

    last = I_TEXT(last);
    post = I_TEXT(post);

    lost = strlen(last);
    past = strlen(post);

    status = lxcast_data->link_v.link
	(lxcast_data->link_v.data, &data, lost + past);
    if (status) {
	status = _libx1f4i0_lxcast_stat_link(lxcast_data);
    } else {
	l_TEXT(data, output);

	memcpy(data, last, lost);
	data = (char *) data + lost;
	memcpy(data, post, past + 1);
    }

    return status;
}


static int
e61xx_t_t(void *context, void *output, void *last, void *post)
{
    int status;
    struct lxcast_type *lxcast_data;
    unsigned lost, past;
    void *data;

    lxcast_data = context;

    last = I_TEXT(last);
    post = I_TEXT(post);

    lost = strlen(last);
    past = strlen(post);

    status = lxcast_data->link_v.link
	(lxcast_data->link_v.data, &data,
	 sizeof(struct x1f4_dxcast_type) + lost + past);
    if (status) {
	status = _libx1f4i0_lxcast_stat_link(lxcast_data);
    } else {
	struct x1f4_dxcast_type *dxcast_data;

	dxcast_data = data;

	l_USER(data, output);

	data = dxcast_data + 1;

	dxcast_data->call = 0;
	dxcast_data->type = X1f4_E4_TEXT;

	l_TEXT(data, &dxcast_data->data);

	memcpy(data, last, lost);
	data = (char *) data + lost;
	memcpy(data, post, past + 1);
    }

    return status;
}
