/*
 * lxdash-c.e.1.c
 * Copyright (C) 2008-2011, 2013-2019, 2021, Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <stddef.h>

#include <copy.h>
#include <e4.h>
#include <exerrors.h>
#include <lxdash-drive.h>
#include <lxdash-inter.h>
#include <lxdash-types.h>

static int line_trap(void *, void *, void **);
static int rule_line(struct lxdash_type *, void *, void **);
static int slip_line(void *news);

static int
line_trap(void *context, void *output, void **input)
{
    int status;
    struct lxdash_type *lxdash_data;
    void **mind, *news;

    lxdash_data = context;

    status = lxdash_data->link_v.pick
	(lxdash_data->link_v.data, &mind, slip_line, sizeof(void *) << 1,
	 &news);
    if (status) {
	status = X1f4_EX_CRITICAL;
    } else {
	void *line, **seek;

	*mind = news;

	seek = news;

	*seek++ = lxdash_data;

	status = _libx1f4i0_lxdash_line_line(lxdash_data, &line);
	if (status) {
	    *seek = NULL;
	} else {
	    *seek = line;

	    l_LINE(line, output);

	    status = rule_line(lxdash_data, line, input);
	}
    }

    return status;
}


static int
rule_line(struct lxdash_type *lxdash_data, void *line, void **input)
{
    int status = 0;
    struct x1f4_function_type *function_data;
    unsigned count;

    function_data = ((void **) input[-1])[X1f4_E4_RENDITION];

    count = function_data->count;
    if (count) {
	struct x1f4_function_type function;
	void *slip[4], **strip;

	strip = input[-1];

	slip[0] = strip;
	slip[1] = &line;

	/*
	 * pretty error reporting:
	 */
	function.length = 9;
	function.name = "i_express";

	function.args = function_data->args - 1;

	strip[X1f4_E4_RENDITION] = &function;

	do {
	    slip[2] = *input++;
	    slip[3] = *input++;
	    status = _libx1f4i0_lxdash_l_derived(lxdash_data, NULL, slip + 1);
	    if (status) {
		break;
	    }

	    function.args += 2;
	} while (count -= 2);
    }

    return status;
}


static int
slip_line(void *news)
{
    int status;
    void *line, **side;

    side = news;
    line = side[1];
    if (line) {
	status = _libx1f4i0_lxdash_slip_line(*side, &line);
    } else {
	status = 0;
    }

    return status;
}


int
_libx1f4i0_lxdash_e_centric(void *context, void *output, void **input)
{
    int status;

    do {
	const int *args;
	struct lxdash_type *lxdash_data;
	struct x1f4_function_type *function_data;
	unsigned count;

	lxdash_data = context;

	function_data = ((void **) input[-1])[X1f4_E4_RENDITION];

	count = function_data->count;

	if (count & 1) {
	    status = _libx1f4i0_lxdash_stat_odds(lxdash_data);
	    if (1) {
		break;
	    }
	}

	args = function_data->args;
	for (; count; count -= 2) {
	    if (*args != X1f4_E4_MODE) {
		break;
	    } else {
		args += 2;
	    }
	}
	if (count) {
	    status = _libx1f4i0_lxdash_stat_gear
		(lxdash_data, "i_express", function_data->count - count);
	    if (1) {
		break;
	    }
	}

	status = line_trap(context, output, input);
    } while (0);

    return status;
}
