/*
 * lxdata.h
 * Copyright (C) 2008-2017, 2019-2021, Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef __LIBx1f4_lxdata_H__
#define __LIBx1f4_lxdata_H__

#include <dt.h>
#include <frame.h>
#include <lxnote.h>
#include <nl.h>
#include <track.h>
#include <trans.h>

/*
 * error definitions
 */
#define X1f4_LXDATA_FREE_ERROR		-2
#define X1f4_LXDATA_LINK_ERROR		-3
#define X1f4_LXDATA_MODE_ERROR		-4

/*
 * generation flag definitions
 */
#define X1f4_LXDATA_AUTOLINK		(1L << 0)
#define X1f4_LXDATA_CODELINK		(1L << 1)
#define X1f4_LXDATA_COMPOSER		(1L << 2)
#define X1f4_LXDATA_EELOOKUP		(1L << 3)
#define X1f4_LXDATA_FILELINK		(1L << 4)
#define X1f4_LXDATA_IMPLICIT		(1L << 5)
#define X1f4_LXDATA_RESOURCE		(1L << 6)
#define X1f4_LXDATA_TEXTFLAT		(1L << 7)

#if !defined __x1f4_function_type_defined__
typedef struct x1f4_function_type {
    const char *name;
    int (*function)(void *, void *, void **), type;
    const int *args;
    unsigned count, flags, length;
} x1f4_function_type;
#endif				/* !__x1f4_function_type_defined__ */

#if !defined __x1f4_function_type_defined__
# define __x1f4_function_type_defined__
#endif				/* !__x1f4_function_type_defined__ */

#if !defined __x1f4_linetext_type_defined__
typedef struct x1f4_linetext_type {
    struct x1f4_function_type function;
    const void *context;
} x1f4_linetext_type;
#endif				/* !__x1f4_linetext_type_defined__ */

#if !defined __x1f4_linetext_type_defined__
# define __x1f4_linetext_type_defined__
#endif				/* !__x1f4_linetext_type_defined__ */

typedef struct x1f4_dxdata_type {
    char *string;
    unsigned call, length;
} x1f4_dxdata_type;

typedef struct x1f4_lxdata_type {
    struct x1f4_track_type textflat_set;
    struct {
	struct x1f4_eelookup_type *eelookup_data;
    } eelookup_set;
    struct {
	int type;
    } filelink_set;
    struct {
	int type;
    } datatype_set;
    struct x1f4_trans_type resource_set;
    struct x1f4_frame_type autolink_set;
    struct x1f4_trans_type codelink_set;
} x1f4_lxdata_type;

#if !defined __x1f4_trans_type_defined__
typedef struct x1f4_trans_type {
    int (*free)(void *, void *), (*link)(void *, void **, unsigned),
	(*mode)(void *, void **, unsigned);
    void *data;
} x1f4_trans_type;
#endif				/* !__x1f4_trans_type_defined__ */

#if !defined __x1f4_trans_type_defined__
# define __x1f4_trans_type_defined__
#endif				/* !__x1f4_trans_type_defined__ */

typedef struct x1f4_transd_type {
    struct x1f4_trans_type transa;
    void *dxdata, *lxdata;
} x1f4_transd_type;

extern int x1f4_copy_lxdata
    (void *, void *, void **, const void *,
     const struct x1f4_nodelink_type *);
extern int x1f4_dset_lxdata(void *, const struct x1f4_datatype_type **);
extern int x1f4_even_lxdata(void *, void *, void *, const void *);
extern int x1f4_fare_lxdata
    (void *, void *, void **, const void *, const struct x1f4_nodelink_type *);
extern int x1f4_fini_lxdata(void **);
extern int x1f4_flat_lxdata(void *, void **);
extern int x1f4_free_lxdata
    (void *, void *, struct x1f4_nodelink_type **,
     const struct x1f4_nodelink_type *);
extern int x1f4_fset_lxdata(void *, const struct x1f4_linetext_type **);
extern int x1f4_high_dxdata(void *, struct x1f4_dxdata_type *);
extern int x1f4_hold_lxdata(void *, struct x1f4_trans_type *);
extern int x1f4_init_lxdata
    (void **, unsigned, const struct x1f4_lxdata_type *);
extern int x1f4_jack_dxdata
    (void *, struct x1f4_dxdata_type *, const void *, unsigned);
extern int x1f4_kate_lxdata(void *, void *, int, unsigned);
extern int x1f4_lead_lxdata(void *, void **);
extern int x1f4_line_lxdata(void *, void **);
extern int x1f4_link_lxdata(void *, void **);
extern int x1f4_mold_lxdata(void *, struct x1f4_transd_type *);
extern int x1f4_node_lxdata
    (void *, void **, void *, const struct x1f4_nodelink_type *);
extern int x1f4_note_lxdata(const struct x1f4_lxnote_type **);
extern int x1f4_pipe_lxdata(void *, const char *, unsigned);
extern int x1f4_post_lxdata
    (void *, void **, const struct x1f4_nodelink_type *);
extern int x1f4_rule_lxdata
    (void *, void *, unsigned, unsigned, void *);
extern int x1f4_slip_lxdata(void *, void **);

#endif				/* __LIBx1f4_lxdata_H__ */

