/*
 * lxdecq-u.e.f.c
 * Copyright (C) 2008-2010, 2013, 2014, 2016-2018, Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <lxdecq-config.h>

#include <e4.h>
#include <f_list.h>
#include <lxdecq-drive.h>
#include <lxdecq-inter.h>
#include <lxdecq-types.h>

#if __CLIP_EXTRACT__
# define clip(none) \
    ((struct clip_type *) (none))
#endif				/* __CLIP_EXTRACT__ */

#if __CLIP_EXTRACT__
typedef struct clip_type {
    unsigned type;
    void *output;
} clip_type;
#endif				/* __CLIP_EXTRACT__ */

#if __CLIP_EXTRACT__
static int clip_miss(void *, void *);
#endif				/* __CLIP_EXTRACT__ */

#if __CLIP_EXTRACT__
static int
clip_miss(void *clip, void *data)
{
    int status;
    unsigned type;

    type = effect_type(data);
    if (type ^ clip(clip)->type) {
	clip(clip)->type = type;

	status = 1;
    } else {
	status = 0;

	l_USER(E_USER(data), clip(clip)->output);
    }

    return status;
}
#endif				/* __CLIP_EXTRACT__ */


int
_libx1f4i0_lxdecq_efclass_u(void *context, void *output, void **input)
{
    int status;
    struct dxlist_type *dxlist_data;
    struct lxtext_type *lxtext_data;
#if __CLIP_EXTRACT__
    struct clip_type clip;
#else
    void *data;
#endif				/* __CLIP_EXTRACT__ */

    lxtext_data = context;

    dxlist_data = I_LIST(input[1]);

#if __CLIP_EXTRACT__
    clip.output = input[0];
    clip.type = lxtext_data->lxtype.code;
#endif				/* __CLIP_EXTRACT__ */

#if __CLIP_EXTRACT__
    status = _libx1f4i0_fine_dllist
	(dxlist_data->list, &clip, clip_miss);
    if (status) {
	status = _libx1f4i0_lxdecq_stat_call
	    (lxtext_data->text, 0, clip.type, lxtext_data->lxtype.code,
	     status);
    } else {
	dxlist_data->size--;
    }
#else
    status = _libx1f4i0_lead_dllist(dxlist_data->list, &data);
    if (status) {
	status =
	    _libx1f4i0_lxdecq_stat_list(lxtext_data->text, 0, status);
    } else {
	unsigned type;

	type = effect_type(data);
	if (type != lxtext_data->lxtype.code) {
	    status = _libx1f4i0_lxdecq_stat_type
		(lxtext_data->text, 0, type, lxtext_data->lxtype.code);
	} else {
	    X1f4_E4_C_USER fast;

	    fast = E_USER(data);

	    status = _libx1f4i0_mind_dllist(dxlist_data->list);
	    if (status) {
		status = _libx1f4i0_lxdecq_stat_list
		    (lxtext_data->text, 0, status);
	    } else {
		dxlist_data->size--;

		l_USER(fast, input[0]);
		l_USER(fast, output);
	    }
	}
    }
#endif				/* __CLIP_EXTRACT__ */

    return status;
}

