/*
 * lxfile-i.f.c
 * Copyright (C) 2008-2010, 2013-2016, 2018, 2020, 2022, Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <c1.h>
#include <e4.h>
#include <file.h>
#include <lxdata.h>
#include <lxfile-defs.h>
#include <lxfile-drive.h>
#include <lxfile-inter.h>
#include <lxfile-types.h>
#include <trans.h>

int
_libx1f4i0_lxfile_read_data(void *context, void *output, void **input,
			    int fill, int part)
{
    int status;

    do {
	X1f4_E4_C_MODE mode;
	int over;
	struct dxfile_type *dxfile_data;
	struct lxfile_type *lxfile_data;
	struct x1f4_dxdata_type *dxdata_data;
	unsigned keep, size;
	void *data;

	dxfile_data = I_FILE(input[0]);

	lxfile_data = context;

	if (dxfile_data->mode != READ_CLASS) {
	    status = _libx1f4i0_lxfile_stat_turn(lxfile_data, dxfile_data);
	    if (1) {
		break;
	    }
	}

	mode = I_MODE(input[2]);
	if (mode < 0) {
	    status = _libx1f4i0_lxfile_stat_rate(lxfile_data, mode);
	    if (1) {
		break;
	    }
	}

	dxdata_data = I_DATA(input[1]);

	if (part) {
	    keep = dxdata_data->length;

	    if (mode) {
		status = x1f4_kate_lxdata
		    (lxfile_data->link_l.data, dxdata_data, 1, keep + mode);
		if (status) {
		    break;
		}

		data = dxdata_data->string + keep;
	    } else {
		status = 0;

		if (fill) {
		} else {
		    l_MODE(0, output);
		}

		break;
	    }
	} else {
	    keep = 0;

	    if (mode) {
		if (dxdata_data->length != mode) {
		    /*
		     * no preserving of existing content, will not resize...
		     */
		    status = x1f4_kate_lxdata
			(lxfile_data->link_l.data, dxdata_data, 1, 0);
		    if (status) {
			break;
		    }
		    status = x1f4_kate_lxdata
			(lxfile_data->link_l.data, dxdata_data, 1, mode);
		    if (status) {
			break;
		    }
		}

		data = dxdata_data->string;
	    } else {
		if (fill) {
		} else {
		    l_MODE(0, output);
		}

		status = x1f4_kate_lxdata
		    (lxfile_data->link_l.data, dxdata_data, 1, mode);

		if (1) {
		    break;
		}
	    }
	}

	status = _libx1f4i0_read_ifile
	    (dxfile_data->file, data, &size, mode, &over);
	if (status) {
	    if (fill) {
		status = _libx1f4i0_lxfile_stat_file(lxfile_data, status);
	    } else {
		if (status == FILE_NONE_CLASS) {
		    status = 0;
		    size = 0;

		    l_MODE(0, output);
		} else {
		    if (status == FILE_OVER_CLASS) {
			status = 0;
			size = 0;

			l_MODE(-1, output);
		    } else {
			status = _libx1f4i0_lxfile_stat_file
			    (lxfile_data, status);
			if (1) {
			    break;
			}
		    }
		}
	    }
	} else {
	    if (fill) {
		if (size != mode) {
		    status = _libx1f4i0_lxfile_stat_file(lxfile_data, over);
		}
	    } else {
		l_MODE(size, output);
	    }
	}

	if (size != mode) {
	    x1f4_kate_lxdata
		(lxfile_data->link_l.data, dxdata_data, 0, keep + size);
	}
    } while (0);

    return status;
}
