/*
 * lxlist-inter.h
 * Copyright (C) 2008-2019, Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef __LIBx1f4_lxlist_inter_H__
#define __LIBx1f4_lxlist_inter_H__

#include <lxlist-config.h>

#include <cl.h>
#include <dt.h>
#include <e4.h>
#include <fd.h>
#include <lxlist-types.h>
#include <lxplus.h>
#include <lxnorm.h>
#include <lxwide.h>
#include <nt.h>
#include <progress.h>
#include <trans.h>

#define _libx1f4i0_lxlist_cfix_list	x1f4_cfix_lxlist
#define _libx1f4i0_lxlist_clip_list	x1f4_clip_lxlist
#define _libx1f4i0_lxlist_copy_list	x1f4_copy_lxlist
#define _libx1f4i0_lxlist_deck_list	x1f4_deck_lxlist
#define _libx1f4i0_lxlist_dset_list	x1f4_dset_lxlist
#define _libx1f4i0_lxlist_dump_list	x1f4_dump_lxlist
#define _libx1f4i0_lxlist_fini_list	x1f4_fini_lxlist
#define _libx1f4i0_lxlist_flat_list	x1f4_flat_lxlist
#define _libx1f4i0_lxlist_fold_list	x1f4_fold_lxlist
#define _libx1f4i0_lxlist_free_list	x1f4_free_lxlist
#define _libx1f4i0_lxlist_fset_list	x1f4_fset_lxlist
#define _libx1f4i0_lxlist_hash_list	x1f4_hash_lxlist
#define _libx1f4i0_lxlist_init_list	x1f4_init_lxlist
#define _libx1f4i0_lxlist_lead_list	x1f4_lead_lxlist
#define _libx1f4i0_lxlist_line_list	x1f4_line_lxlist
#define _libx1f4i0_lxlist_link_list	x1f4_link_lxlist
#define _libx1f4i0_lxlist_miss_list	x1f4_miss_lxlist
#define _libx1f4i0_lxlist_node_list	x1f4_node_lxlist
#define _libx1f4i0_lxlist_pass_list	x1f4_pass_lxlist
#define _libx1f4i0_lxlist_post_list	x1f4_post_lxlist
#define _libx1f4i0_lxlist_pull_list	x1f4_pull_lxlist
#define _libx1f4i0_lxlist_seek_list	x1f4_seek_lxlist
#define _libx1f4i0_lxlist_slip_list	x1f4_slip_lxlist
#define _libx1f4i0_lxlist_suit_list	x1f4_suit_lxlist
#define _libx1f4i0_lxlist_tile_list	x1f4_tile_lxlist
#define _libx1f4i0_lxlist_tune_list	x1f4_tune_lxlist
#define _libx1f4i0_lxlist_type_list	x1f4_type_lxlist
#define _libx1f4i0_lxlist_uset_list	x1f4_uset_lxlist
#define _libx1f4i0_lxlist_vset_list	x1f4_vset_lxlist

#define __types_lock(lxlist) \
    (&__types_long(lxlist)->east_1)

#define __types_long(lxlist) \
    (&((struct lxlist_type *) (lxlist))->ever_r.over_1.rule_1.nice_1)

#define _libx1f4i0_lxlist_stat_free(___l, ___1) \
    _libx1f4i0_lxlock_stat_free(__types_lock((___l)), (___1))
#define _libx1f4i0_lxlist_stat_land(___l, ___1, ___2) \
    _libx1f4i0_lxlock_stat_land(__types_lock((___l)), (___1), (___2))
#define _libx1f4i0_lxlist_stat_link(___l) \
    _libx1f4i0_lxlock_stat_link(__types_lock((___l)))
#define _libx1f4i0_lxlist_stat_mode(___l) \
    _libx1f4i0_lxlock_stat_mode(__types_lock((___l)))
#define _libx1f4i0_lxlist_stat_roll(___l, ___1, ___2) \
    _libx1f4i0_lxlock_stat_roll(__types_lock((___l)))
#define _libx1f4i0_lxlist_stat_zero(___l) \
    _libx1f4i0_lxlock_stat_zero(__types_lock((___l)))

#define _libx1f4i0_lxlist_s_forward	_libx1f4i0_lxnorm_s_forward

extern int _libx1f4i0_lxlist_a_centric(void *, void *, void **);
extern int _libx1f4i0_lxlist_a_keeping(void *, void *, void **);
extern int _libx1f4i0_lxlist_a_logique(void *, void *, void **);
extern int _libx1f4i0_lxlist_a_pending(void *, void *, void **);
extern int _libx1f4i0_lxlist_b_centric(void *, void *, void **);
extern int _libx1f4i0_lxlist_b_forward(void *, void *, void **);
extern int _libx1f4i0_lxlist_b_keeping(void *, void *, void **);
extern int _libx1f4i0_lxlist_b_pending(void *, void *, void **);
extern int _libx1f4i0_lxlist_c_centric(void *, void *, void **);
extern int _libx1f4i0_lxlist_c_keeping(void *, void *, void **);
extern int _libx1f4i0_lxlist_c_logique(void *, void *, void **);
extern int _libx1f4i0_lxlist_c_nearest(void *, void *, void **);
extern int _libx1f4i0_lxlist_c_pending(void *, void *, void **);
extern int _libx1f4i0_lxlist_cfix_list(void *, unsigned, void *);
extern int _libx1f4i0_lxlist_clip_list
    (unsigned, const void *, struct x1f4_nodetype_type **, int *);
extern int _libx1f4i0_lxlist_copy_link(void *, void *, const void *);
extern int _libx1f4i0_lxlist_copy_list
    (void *, void *, void **, const void *, const struct excase_type *);
extern int _libx1f4i0_lxlist_copy_text
    (void *, void *, void **, const void *, const struct excase_type *);
extern int _libx1f4i0_lxlist_d_forward(void *, void *, void **);
extern int _libx1f4i0_lxlist_d_keeping(void *, void *, void **);
extern int _libx1f4i0_lxlist_d_logique(void *, void *, void **);
extern int _libx1f4i0_lxlist_down_list(void *, void *, void *);
extern int _libx1f4i0_lxlist_draw_list
    (void *, void *, void **, int, const char *, unsigned);
extern int _libx1f4i0_lxlist_dset_list
    (void *, const struct x1f4_datatype_type **);
extern int _libx1f4i0_lxlist_dump_list(void *, struct excase_type *);
extern int _libx1f4i0_lxlist_e0class_b(void *, void *, void **);
extern int _libx1f4i0_lxlist_e0class_m(void *, void *, void **);
extern int _libx1f4i0_lxlist_e0class_r(void *, void *, void **);
extern int _libx1f4i0_lxlist_e0class_t(void *, void *, void **);
extern int _libx1f4i0_lxlist_e_centric(void *, void *, void **);
extern int _libx1f4i0_lxlist_e_class_b(void *, void *, void **);
extern int _libx1f4i0_lxlist_e_class_m(void *, void *, void **);
extern int _libx1f4i0_lxlist_e_class_r(void *, void *, void **);
extern int _libx1f4i0_lxlist_e_class_t(void *, void *, void **);
extern int _libx1f4i0_lxlist_e_class_u(void *, void *, void **);
extern int _libx1f4i0_lxlist_e_class_v(void *, void *, void **);
extern int _libx1f4i0_lxlist_e_forward(void *, void *, void **);
extern int _libx1f4i0_lxlist_e_keeping(void *, void *, void **);
extern int _libx1f4i0_lxlist_e_logique(void *, void *, void **);
extern int _libx1f4i0_lxlist_e_pending(void *, void *, void **);
extern int _libx1f4i0_lxlist_e_press_b(void *, void *, void **);
extern int _libx1f4i0_lxlist_e_press_m(void *, void *, void **);
extern int _libx1f4i0_lxlist_e_press_r(void *, void *, void **);
extern int _libx1f4i0_lxlist_e_press_t(void *, void *, void **);
extern int _libx1f4i0_lxlist_e_press_v(void *, void *, void **);
extern int _libx1f4i0_lxlist_ebclass_u(void *, void *, void **);
extern int _libx1f4i0_lxlist_ebclass_v(void *, void *, void **);
extern int _libx1f4i0_lxlist_efclass_u(void *, void *, void **);
extern int _libx1f4i0_lxlist_efclass_v(void *, void *, void **);
extern int _libx1f4i0_lxlist_fast_list(void *, void *);
extern int _libx1f4i0_lxlist_fine_list(void *, void *);
extern int _libx1f4i0_lxlist_fini_list(void **);
extern int _libx1f4i0_lxlist_flat_list(void *, void **);
extern int _libx1f4i0_lxlist_flat_text(void *, void **);
extern int _libx1f4i0_lxlist_fold_call
    (void *, void *, void **, struct x1f4_linetext_type *, unsigned,
     const struct x1f4_function_type *, int, int, int *, struct fxdata_type *);
extern int _libx1f4i0_lxlist_fold_list(void *, unsigned *);
extern int _libx1f4i0_lxlist_free_link(void *, void *);
extern int _libx1f4i0_lxlist_free_list
    (void *, void *, struct excase_type **, const struct excase_type *);
extern int _libx1f4i0_lxlist_free_text
    (void *, void *, struct excase_type **, const struct excase_type *);
extern int _libx1f4i0_lxlist_fset_list
    (void *, const struct x1f4_linetext_type **);
extern int _libx1f4i0_lxlist_g0class_b(void *, void *, void **);
extern int _libx1f4i0_lxlist_g0class_m(void *, void *, void **);
extern int _libx1f4i0_lxlist_g0class_r(void *, void *, void **);
extern int _libx1f4i0_lxlist_g0class_t(void *, void *, void **);
extern int _libx1f4i0_lxlist_g_class_b(void *, void *, void **);
extern int _libx1f4i0_lxlist_g_class_m(void *, void *, void **);
extern int _libx1f4i0_lxlist_g_class_r(void *, void *, void **);
extern int _libx1f4i0_lxlist_g_class_t(void *, void *, void **);
extern int _libx1f4i0_lxlist_g_class_u(void *, void *, void **);
extern int _libx1f4i0_lxlist_g_class_v(void *, void *, void **);
extern int _libx1f4i0_lxlist_g_derived(void *, void *, void **);
extern int _libx1f4i0_lxlist_gbclass_u(void *, void *, void **);
extern int _libx1f4i0_lxlist_gbclass_v(void *, void *, void **);
extern int _libx1f4i0_lxlist_gfclass_u(void *, void *, void **);
extern int _libx1f4i0_lxlist_gfclass_v(void *, void *, void **);
extern int _libx1f4i0_lxlist_hash_list
    (void *, struct excase_type **, struct excase_type *);
extern int _libx1f4i0_lxlist_heap_list(void *, struct excase_type **, void *);
extern int _libx1f4i0_lxlist_i_class_b(void *, void *, void **);
extern int _libx1f4i0_lxlist_i_class_m(void *, void *, void **);
extern int _libx1f4i0_lxlist_i_class_r(void *, void *, void **);
extern int _libx1f4i0_lxlist_i_class_t(void *, void *, void **);
extern int _libx1f4i0_lxlist_i_class_v(void *, void *, void **);
extern int _libx1f4i0_lxlist_i_keeping(void *, void *, void **);
extern int _libx1f4i0_lxlist_i_pending(void *, void *, void **);
extern int _libx1f4i0_lxlist_init_list
    (void **, unsigned, const struct screen_type *);
extern int _libx1f4i0_lxlist_into_list
    (void *, void *, void **, int, const char *, unsigned);
extern int _libx1f4i0_lxlist_j_class_b(void *, void *, void **);
extern int _libx1f4i0_lxlist_j_class_m(void *, void *, void **);
extern int _libx1f4i0_lxlist_j_class_r(void *, void *, void **);
extern int _libx1f4i0_lxlist_j_class_t(void *, void *, void **);
extern int _libx1f4i0_lxlist_j_class_u(void *, void *, void **);
extern int _libx1f4i0_lxlist_j_class_v(void *, void *, void **);
extern int _libx1f4i0_lxlist_j_derived(void *, void *, void **);
extern int _libx1f4i0_lxlist_j_keeping(void *, void *, void **);
extern int _libx1f4i0_lxlist_k_logique(void *, void *, void **);
extern int _libx1f4i0_lxlist_l_derived(void *, void *, void **);
extern int _libx1f4i0_lxlist_l_forward(void *, void *, void **);
extern int _libx1f4i0_lxlist_l_logique(void *, void *, void **);
extern int _libx1f4i0_lxlist_l_pending(void *, void *, void **);
extern int _libx1f4i0_lxlist_l_sequent(void *, void *, void **);
extern int _libx1f4i0_lxlist_late_list
    (void *, void *, void *, const void *);
extern int _libx1f4i0_lxlist_lead_list(void *, void **);
extern int _libx1f4i0_lxlist_lead_text(void *, void **);
extern int _libx1f4i0_lxlist_lift_call
    (void *, void *, void **, struct x1f4_linetext_type *, unsigned,
     const struct x1f4_function_type *, int, int, int *, struct fxdata_type *);
extern int _libx1f4i0_lxlist_line_list(void *, void **);
extern int _libx1f4i0_lxlist_link_link(void *, void *, const void *);
extern int _libx1f4i0_lxlist_link_list(void *, void **);
extern int _libx1f4i0_lxlist_long_list(void *, void **);
extern int _libx1f4i0_lxlist_m_logique(void *, void *, void **);
extern int _libx1f4i0_lxlist_miss_link(void *, void *);
extern int _libx1f4i0_lxlist_miss_list
    (void *, void *, const void *, const void *);
extern int _libx1f4i0_lxlist_n_forward(void *, void *, void **);
extern int _libx1f4i0_lxlist_n_keeping(void *, void *, void **);
extern int _libx1f4i0_lxlist_n_logique(void *, void *, void **);
extern int _libx1f4i0_lxlist_node_list
    (void *, void **, void *, const struct excase_type *);
extern int _libx1f4i0_lxlist_null_list
    (void *, struct excase_type **, void *);
extern int _libx1f4i0_lxlist_o_class_b(void *, void *, void **);
extern int _libx1f4i0_lxlist_o_class_m(void *, void *, void **);
extern int _libx1f4i0_lxlist_o_class_r(void *, void *, void **);
extern int _libx1f4i0_lxlist_o_class_t(void *, void *, void **);
extern int _libx1f4i0_lxlist_o_class_u(void *, void *, void **);
extern int _libx1f4i0_lxlist_o_class_v(void *, void *, void **);
extern int _libx1f4i0_lxlist_o_derived(void *, void *, void **);
extern int _libx1f4i0_lxlist_o_logique(void *, void *, void **);
extern int _libx1f4i0_lxlist_o_pending(void *, void *, void **);
extern int _libx1f4i0_lxlist_p0class_b(void *, void *, void **);
extern int _libx1f4i0_lxlist_p0class_m(void *, void *, void **);
extern int _libx1f4i0_lxlist_p0class_r(void *, void *, void **);
extern int _libx1f4i0_lxlist_p_class_b(void *, void *, void **);
extern int _libx1f4i0_lxlist_p_class_m(void *, void *, void **);
extern int _libx1f4i0_lxlist_p_class_r(void *, void *, void **);
extern int _libx1f4i0_lxlist_p_class_t(void *, void *, void **);
extern int _libx1f4i0_lxlist_p_class_u(void *, void *, void **);
extern int _libx1f4i0_lxlist_p_class_v(void *, void *, void **);
extern int _libx1f4i0_lxlist_p_derived(void *, void *, void **);
extern int _libx1f4i0_lxlist_p_forward(void *, void *, void **);
extern int _libx1f4i0_lxlist_p_logique(void *, void *, void **);
extern int _libx1f4i0_lxlist_p_nearest(void *, void *, void **);
extern int _libx1f4i0_lxlist_p_pending(void *, void *, void **);
extern int _libx1f4i0_lxlist_pass_list
    (void *, struct x1f4_progress_type **, char **);
extern int _libx1f4i0_lxlist_pbclass_t(void *, void *, void **);
extern int _libx1f4i0_lxlist_pbclass_u(void *, void *, void **);
extern int _libx1f4i0_lxlist_pbclass_v(void *, void *, void **);
extern int _libx1f4i0_lxlist_pfclass_t(void *, void *, void **);
extern int _libx1f4i0_lxlist_pfclass_u(void *, void *, void **);
extern int _libx1f4i0_lxlist_pfclass_v(void *, void *, void **);
extern int _libx1f4i0_lxlist_pick_text(void *, void *);
extern int _libx1f4i0_lxlist_post_list
    (void *, void **, const struct excase_type *);
extern int _libx1f4i0_lxlist_pull_list
    (void *, void *, void *, const void *, struct trans_type *,
     struct x1f4_caselink_type **);
extern int _libx1f4i0_lxlist_push_text
    (void *, void **, void *, const struct excase_type *);
extern int _libx1f4i0_lxlist_q0class_b(void *, void *, void **);
extern int _libx1f4i0_lxlist_q0class_m(void *, void *, void **);
extern int _libx1f4i0_lxlist_q0class_r(void *, void *, void **);
extern int _libx1f4i0_lxlist_q0class_t(void *, void *, void **);
extern int _libx1f4i0_lxlist_q_class_b(void *, void *, void **);
extern int _libx1f4i0_lxlist_q_class_m(void *, void *, void **);
extern int _libx1f4i0_lxlist_q_class_r(void *, void *, void **);
extern int _libx1f4i0_lxlist_q_class_t(void *, void *, void **);
extern int _libx1f4i0_lxlist_q_class_u(void *, void *, void **);
extern int _libx1f4i0_lxlist_q_class_v(void *, void *, void **);
extern int _libx1f4i0_lxlist_q_forward(void *, void *, void **);
extern int _libx1f4i0_lxlist_q_logique(void *, void *, void **);
extern int _libx1f4i0_lxlist_qbclass_u(void *, void *, void **);
extern int _libx1f4i0_lxlist_qbclass_v(void *, void *, void **);
extern int _libx1f4i0_lxlist_qfclass_u(void *, void *, void **);
extern int _libx1f4i0_lxlist_qfclass_v(void *, void *, void **);
extern int _libx1f4i0_lxlist_r_class_b(void *, void *, void **);
extern int _libx1f4i0_lxlist_r_class_m(void *, void *, void **);
extern int _libx1f4i0_lxlist_r_class_r(void *, void *, void **);
extern int _libx1f4i0_lxlist_r_class_t(void *, void *, void **);
extern int _libx1f4i0_lxlist_r_class_u(void *, void *, void **);
extern int _libx1f4i0_lxlist_r_class_v(void *, void *, void **);
extern int _libx1f4i0_lxlist_r_derived(void *, void *, void **);
extern int _libx1f4i0_lxlist_r_drive_b(void *, void *);
extern int _libx1f4i0_lxlist_r_drive_m(void *, void *);
extern int _libx1f4i0_lxlist_r_drive_r(void *, void *);
extern int _libx1f4i0_lxlist_r_forward(void *, void *, void **);
extern int _libx1f4i0_lxlist_r_logique(void *, void *, void **);
extern int _libx1f4i0_lxlist_r_pending(void *, void *, void **);
extern int _libx1f4i0_lxlist_roll_list
    (void *, void *, void **, void **, struct x1f4_linetext_type *, unsigned,
     const struct x1f4_function_type *, int, int, void *, int, int, int);
extern int _libx1f4i0_lxlist_s_keeping(void *, void *, void **);
extern int _libx1f4i0_lxlist_s_logique(void *, void *, void **);
extern int _libx1f4i0_lxlist_s_pending(void *, void *, void **);
extern int _libx1f4i0_lxlist_s_sequent(void *, void *, void **);
extern int _libx1f4i0_lxlist_seek_list(const void *);
extern int _libx1f4i0_lxlist_side_list(void *, void *);
extern int _libx1f4i0_lxlist_slip_list(void *, void **);
extern int _libx1f4i0_lxlist_slip_text(void *, void **);
extern int _libx1f4i0_lxlist_spin_list(void *, void *, void *);
extern int _libx1f4i0_lxlist_stat_call(void *, int, unsigned, unsigned, int);
extern int _libx1f4i0_lxlist_stat_copy(void *);
extern int _libx1f4i0_lxlist_stat_debt(void *, int);
extern int _libx1f4i0_lxlist_stat_deck(void *, unsigned, long);
extern int _libx1f4i0_lxlist_stat_edit(void *, unsigned, long, long);
extern int _libx1f4i0_lxlist_stat_ends(void *, int, unsigned, unsigned);
extern int _libx1f4i0_lxlist_stat_flat(void *, int);
extern int _libx1f4i0_lxlist_stat_list(void *, int, int);
extern int _libx1f4i0_lxlist_stat_null(void *);
extern int _libx1f4i0_lxlist_stat_pull(void *, int, int);
extern int _libx1f4i0_lxlist_stat_size(void *);
extern int _libx1f4i0_lxlist_stat_time(void *, unsigned, long);
extern int _libx1f4i0_lxlist_stat_type(void *, int, unsigned, unsigned);
extern int _libx1f4i0_lxlist_suit_list(void *, void *, void *);
extern int _libx1f4i0_lxlist_swap_list(void *, void *);
extern int _libx1f4i0_lxlist_t_keeping(void *, void *, void **);
extern int _libx1f4i0_lxlist_t_logique(void *, void *, void **);
extern int _libx1f4i0_lxlist_t_pending(void *, void *, void **);
extern int _libx1f4i0_lxlist_tile_list
    (void **, unsigned, struct x1f4_lxwide_type *, unsigned, const void *);
extern int _libx1f4i0_lxlist_tune_list(void *);
extern int _libx1f4i0_lxlist_type_list(struct x1f4_nodetype_type *, int);
extern int _libx1f4i0_lxlist_u_derived(void *, void *, void **);
extern int _libx1f4i0_lxlist_u_forward(void *, void *, void **);
extern int _libx1f4i0_lxlist_u_pending(void *, void *, void **);
extern int _libx1f4i0_lxlist_u_sequent(void *, void *, void **);
extern int _libx1f4i0_lxlist_uset_list
    (void *, const struct x1f4_operator_type **);
extern int _libx1f4i0_lxlist_vset_list(void *, void **);
extern int _libx1f4i0_lxlist_w_logique(void *, void *, void **);
extern int _libx1f4i0_lxlist_x0class_b(void *, void *, void **);
extern int _libx1f4i0_lxlist_x0class_m(void *, void *, void **);
extern int _libx1f4i0_lxlist_x0class_r(void *, void *, void **);
extern int _libx1f4i0_lxlist_x0class_t(void *, void *, void **);
extern int _libx1f4i0_lxlist_xbclass_u(void *, void *, void **);
extern int _libx1f4i0_lxlist_xbclass_v(void *, void *, void **);
extern int _libx1f4i0_lxlist_xfclass_u(void *, void *, void **);
extern int _libx1f4i0_lxlist_xfclass_v(void *, void *, void **);
extern int _libx1f4i0_lxlist_x_class_b(void *, void *, void **);
extern int _libx1f4i0_lxlist_x_class_m(void *, void *, void **);
extern int _libx1f4i0_lxlist_x_class_r(void *, void *, void **);
extern int _libx1f4i0_lxlist_x_class_t(void *, void *, void **);
extern int _libx1f4i0_lxlist_x_class_u(void *, void *, void **);
extern int _libx1f4i0_lxlist_x_class_v(void *, void *, void **);
extern int _libx1f4i0_lxlist_x_derived(void *, void *, void **);
extern int _libx1f4i0_lxlist_x_pending(void *, void *, void **);
extern int _libx1f4i0_lxlist_x_press_b(void *, void *, void **);
extern int _libx1f4i0_lxlist_x_press_m(void *, void *, void **);
extern int _libx1f4i0_lxlist_x_press_r(void *, void *, void **);
extern int _libx1f4i0_lxlist_x_press_t(void *, void *, void **);
extern int _libx1f4i0_lxlist_x_press_v(void *, void *, void **);
extern int _libx1f4i0_lxlist_x_sequent(void *, void *, void **);
extern int _libx1f4i0_lxlist_y_class_b(void *, void *, void **);
extern int _libx1f4i0_lxlist_y_class_m(void *, void *, void **);
extern int _libx1f4i0_lxlist_y_class_r(void *, void *, void **);
extern int _libx1f4i0_lxlist_y_class_t(void *, void *, void **);
extern int _libx1f4i0_lxlist_y_class_v(void *, void *, void **);
extern int _libx1f4i0_lxlist_y_derived(void *, void *, void **);

#endif				/* __LIBx1f4_lxlist_inter_H__ */
