/*
 * lxlist-k.a.c
 * Copyright (C) 2008-2018, Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <stddef.h>

#include <e4.h>
#include <l_list.h>
#include <lxlist-drive.h>
#include <lxlist-inter.h>
#include <lxlist-types.h>

int
_libx1f4i0_lxlist_a_keeping(void *context, void *output, void **input)
{
    int status;
    struct cxbend_type *cxbend_data;
    struct dxlist_type *dxlist_data;
    struct lxlist_type *lxlist_data;
    unsigned size;

    cxbend_data = context;

    lxlist_data = cxbend_data->list;

    dxlist_data = I_LIST(input[0]);

    size = dxlist_data->size;
    if (!size) {
	status = _libx1f4i0_lxlist_stat_null(lxlist_data);
    } else {
	void *data;

	status = cxbend_data->pick(dxlist_data->list, &data);
	if (status) {
	    status = _libx1f4i0_lxlist_stat_list(lxlist_data, 0, status);
	} else {
	    do {
		unsigned link;

		link = effect_link(data);
		if (~link) {
		    int (*free)(void *, void *, struct excase_type **,
				const struct excase_type *);
		    struct lxtext_type *lxtext_link;

		    lxtext_link =
			((struct lxtext_type *) lxlist_data->link_f.text)
			+ link;

		    link_effect(data, ~0);

		    free = lxtext_link->lxtype.free;
		    if (free) {
			status = free
			    (lxtext_link->lxtype.context, E_USER(data), NULL,
			     lock_excase(dxlist_data));
			if (status) {
			    break;
			}
		    }
		}

		size--;

#if defined __MISSING_SLIP_L_LIST__
		status = _libx1f4i0_miss_l4list
		    (dxlist_data->list, cxbend_data->bend ? size : 0);
#else
		status = cxbend_data->slip(dxlist_data->list);
#endif				/* __MISSING_SLIP_L_LIST__ */
		if (status) {
		    status = _libx1f4i0_lxlist_stat_list
			(lxlist_data, 0, status);
		} else {
		    dxlist_data->size = size;

		    l_LIST(dxlist_data, output);
		}
	    } while (0);
	}
    }

    return status;
}
