/*
 * lxlist-x.l.c
 * Copyright (C) 2008-2011, 2013, 2014, 2017, 2018, 2020, Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <stddef.h>

#include <e4.h>
#include <exerrors.h>
#include <l_list.h>
#include <lxdata.h>
#include <lxlist-drive.h>
#include <lxlist-inter.h>
#include <lxlist-types.h>
#include <trans.h>

#define dxlist(list) \
    ((struct dxlist_type *) (list))
#define lxlist(list) \
    ((struct lxlist_type *) (list))

#define warp(miss) \
    ((void **) (miss))

static int fast_deck(void *, void *, unsigned);

static int
fast_deck(void *warp, void *slip, unsigned copy)
{
    int status;
    struct dxlist_type *dxlist_data;
    struct lxtext_type *lxtext_data;
    unsigned type;
    void *data;

    lxtext_data = warp(warp)[1];

    dxlist_data = warp(warp)[0];

    type = lxtext_data->lxtype.code;

#if defined __MISSING_TAIL_L_LIST__
    status = _libx1f4i0_push_l4list
	(dxlist_data->list, dxlist_data->size, &data);
#else
    status = _libx1f4i0_tail_l4list(dxlist_data->list, &data);
#endif				/* __MISSING_TAIL_L_LIST__ */
    if (status) {
	status = _libx1f4i0_lxlist_stat_list(lxtext_data->text, 0, status);
    } else {
	void *post;

	type_effect(data, type);

	status = lxtext_data->lxtype.post
	    (lxtext_data->lxtype.context, &post, lock_excase(dxlist_data));
	if (status) {
	    link_effect(data, ~0);

	    _libx1f4i0_miss_l4list(dxlist_data->list, dxlist_data->size);
	} else {
	    struct x1f4_dxdata_type *dxdata_data;
	    unsigned link;

	    l_USER(post, effect_data(data));

	    dxlist_data->size++;

	    link = (integral_q) warp(warp)[2];

	    link_effect(data, link);

	    dxdata_data = post;

	    dxdata_data->string = slip;
	    dxdata_data->length = copy;
	}
    }

    return status;
}


int
x1f4_wave_lxlist(void *subtext, void *dxlist, void *text, void **warp,
		 int (**call)(void *, void *, unsigned))
{
    int status;

    do {
	struct lxtext_type *lxtext_data, *lxtext_slip;
	unsigned miss;

	lxtext_data = lxlist(subtext)->link_f.text;

	lxtext_slip = lxtext_data;

	miss = ((struct lxmiss_type *) lxtext_data - 1)->data.miss;
	while (miss) {
	    if (lxtext_slip->lxtype.context == text) {
		break;
	    } else {
		miss--;
		lxtext_slip++;
	    }
	}

	if (miss) {
	} else {
	    status = X1f4_EX_CRITICAL;
	    if (1) {
		break;
	    }
	}

	if (dxlist(dxlist)->size) {
	    status = _libx1f4i0_lxlist_null_list(subtext, NULL, dxlist);
	    if (status) {
		break;
	    }
	} else {
	    status = 0;
	}

	*call = fast_deck;

	warp[0] = dxlist;
	warp[1] = lxtext_slip;
	warp[2] = (void *) (integral_q) (lxtext_slip - lxtext_data);
	warp[3] = &lxlist(subtext)->link_m;
    } while (0);

    return status;
}
