/*
 * lxroll-f.l.c
 * Copyright (C) 2017, 2019, Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <netdb.h>
#include <unistd.h>

#include <arpa/inet.h>

#include <netinet/in.h>
#include <netinet/tcp.h>

#include <sys/socket.h>
#include <sys/types.h>

#include <e4.h>
#include <lxfile.h>
#include <lxroll-defs.h>
#include <lxroll-drive.h>
#include <lxroll-inter.h>
#include <lxroll-types.h>

static int case_inet(void *, int, unsigned, int);

static int
case_inet(void *lxroll, int sort, unsigned bits, int server)
{
    int status;

    do {
	struct sockaddr_in address;

	if (bits & REUSE_ADDRESS) {
	    int part = 1;

	    status = setsockopt
		(server, SOL_SOCKET, SO_REUSEADDR, &part, sizeof(int));
	    if (status) {
		status = _libx1f4i0_lxroll_stat_part(lxroll, REUSE_ADDRESS);
		if (1) {
		    break;
		}
	    }
	}

	address.sin_family = AF_INET;
	address.sin_port = htons(sort);
	address.sin_addr.s_addr = htonl(INADDR_ANY);
	status = bind(server, (struct sockaddr *) &address, sizeof(address));
	if (status) {
	    status = _libx1f4i0_lxroll_stat_call(lxroll, "bind");
	} else {
	    status = listen(server, 1);
	    if (status) {
		status = _libx1f4i0_lxroll_stat_call(lxroll, "listen");
	    }
	}
    } while (0);

    return status;
}


int
_libx1f4i0_lxroll_l_forward(void *context, void *output, void **input)
{
    int fd = -1, status;

    do {
        struct lxroll_type *lxroll_data;
        struct x1f4_dxfile_type *dxfile_data;
        void *lxfile;

        lxroll_data = context;

        lxfile = lxroll_data->link_l.file;

        dxfile_data = I_FILE(input[0]);

	fd = socket(PF_INET, SOCK_STREAM, 0);
	if (fd == -1) {
	    status = _libx1f4i0_lxroll_stat_call(lxroll_data, "socket");
	    if (1) {
		break;
	    }
	}

	status = case_inet
	    (lxroll_data, I_MODE(input[1]), I_MODE(input[2]), fd);
	if (status) {
	    break;
	}

	if (dxfile_data->file) {
	    status = x1f4_lose_dxfile(lxfile, dxfile_data);
	    if (status) {
		break;
	    }
	}

	status = x1f4_pick_dxfile(lxfile, dxfile_data, fd, 0);
	if (status) {
	    break;
	}

	l_FILE(dxfile_data, output);

	fd = -1;
    } while (0);

    if (fd ^ -1) {
	close(fd);
    }

    return status;
}
