/*
 * lxroll-i.c.c
 * Copyright (C) 2017, 2019, Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <lxfile.h>
#include <lxroll-defs.h>
#include <lxroll-inter.h>

#define lxroll(lxroll) \
    ((struct lxroll_type *) (lxroll))

int
_libx1f4i0_lxroll_land_roll(void *lxroll, unsigned bits, void *dxdata, int *ed,
			    void *dxtext, int *el)
{
    int status;

    do {
	struct x1f4_dxfile_type *dxfile_data, *dxfile_text;
	void *lxfile;

	lxfile = lxroll(lxroll)->link_l.file;

	dxfile_data = dxdata;

	if (dxfile_data->file) {
            status = x1f4_lose_dxfile(lxfile, dxfile_data);
            if (status) {
                break;
            }
        }

	dxfile_text = dxtext;

        if (dxfile_text->file) {
            status = x1f4_lose_dxfile(lxfile, dxfile_text);
            if (status) {
                break;
            }
        }

	if (bits & NONBLOCKING_INPUT) {
	    status = _libx1f4i0_lxroll_lock_link(lxroll, *ed);
	    if (status) {
		break;
	    }
	}

	status = x1f4_pick_dxfile(lxfile, dxfile_data, *ed, 0);
	if (status) {
	    break;
	}

	*ed = -1;

	if (bits & NONBLOCKING_OUTPUT) {
	    status = _libx1f4i0_lxroll_lock_link(lxroll, *el);
	    if (status) {
		break;
	    }

	    status = x1f4_posh_dxfile
		(lxfile, dxfile_text, *el, X1f4_LXFILE_POSH_BLOCK);
	    if (status) {
		break;
	    }
	} else {
	    status = x1f4_posh_dxfile(lxfile, dxfile_text, *el, 0);
	    if (status) {
		break;
	    }
	}

	*el = -1;
    } while (0);

    return 0;
}
