/*
 * lxtrap-d.u.c
 * Copyright (C) 2011-2015, 2017-2019, Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <stddef.h>
#include <stdlib.h>
#include <string.h>

#include <e4.h>
#include <lxcall.h>
#include <lxcast.h>
#include <lxtrap-drive.h>
#include <lxtrap-inter.h>
#include <lxtrap-types.h>

#define POST_BITS \
    X1f4_LXCALL_POST_ARGS | X1f4_LXCALL_POST_DATA | X1f4_LXCALL_POST_LINE

#define lxtrap(trap) \
    ((struct lxtrap_type *) (trap))

extern const void *const x1f4_a1_walk_link;

typedef struct pick_type {
    struct fxdata_type finite[2];
    struct x1f4_dxcast_type dxcast, serial[2];
    struct x1f4_function_type function;
    struct x1f4_linetext_type linetext;
} pick_type;

typedef struct slip_type {
    int (*call)(void *, int, void *, struct x1f4_dispatch_type **), *deck,
	miss;
    const int *over;
    struct fxdata_type finite[2];
    struct lxtrap_type *lxtrap_data;
    struct x1f4_dispatch_type *dispatch_data;
    struct x1f4_dxcast_type dxcast, serial[2];
    struct x1f4_function_type function;
    struct x1f4_linetext_type linetext, *linetext_data;
    unsigned class, pull;
    void *back, *clear[3], **track;
} slip_type;

static int deck_call(int *, unsigned, void **, unsigned);
static int fold_call(struct lxtrap_type *, void *, void **,
		     struct x1f4_linetext_type *, int,
		     const struct x1f4_function_type *, int, unsigned);
static int lift_call(struct lxtrap_type *, void *, void **,
		     struct x1f4_linetext_type *, int,
		     const struct x1f4_function_type *, int, unsigned);
static int part_call(void *, int, void *, struct x1f4_dispatch_type **);
static int star_call(void **, void **, unsigned, int *, const int *, unsigned,
		     unsigned, int *, unsigned);
static int star_fill(const void *, const void *);

static int
deck_call(int *deck, unsigned pull, void **input, unsigned grade)
{
    unsigned i = 0;

    for (; i < pull; i++) {
	int lock;
	unsigned j;

	lock = I_MODE(input[3 + i]);

	if (lock < (X1f4_E4_C_MODE) 0) {
	    lock += grade - pull + i + 1;
	}

	j = 0;
	for (; j < i; j++) {
	    if (deck[j] < lock) {
	    } else {
		deck[j]++;
	    }
	}

	deck[i] = lock;
    }

    return 0;
}


/*
 * function does setup for an unused output
 */
static int
fold_call(struct lxtrap_type *lxtrap_data, void *output, void **input,
	  struct x1f4_linetext_type *linetext_data, int lead,
	  const struct x1f4_function_type *function_text, int line,
	  unsigned pull)
{
    int linked, status;
    unsigned class, count, grade, parts, scale = 0;
    void *beta;

    class = function_text->count;

    count = class - lead - line - pull;

    grade = line + pull;

    parts = function_text->flags;

    if (parts & X1f4_E4_POST_TYPE) {
	if (linetext_data->function.flags
	    & (X1f4_E4_LINK_PASS | X1f4_E4_POST_TYPE)) {
	    scale = 1;
	} else {
	    parts &= ~X1f4_E4_POST_TYPE;
	}
    }

    linked = sizeof(struct pick_type);

    status = lxtrap_data->link_v.link
	(lxtrap_data->link_v.data, &beta,
	 linked + (sizeof(void *) + (sizeof(int) << scale)) * grade
	 + sizeof(void *) + (sizeof(int) * pull << 1));
    if (status) {
	status = _libx1f4i0_lxtrap_stat_link(lxtrap_data);
    } else {
	int *args, *away, *deck;
	const int *over;
	int type;
	struct pick_type *call_data;
	struct x1f4_function_type *function_call;
	unsigned bits, i;
	void **frame;

	call_data = beta;

	function_call = &call_data->function;

	function_call->count = grade;

	function_call->flags = parts & X1f4_E4_POST_TYPE;

	frame = (void *) (call_data + 1);
	frame++;

	function_call->args = args = (void *) (frame + grade);

	deck = args + (grade << scale);

	frame[-1] = input[-1];

	type = linetext_data->function.type;

	if (type != X1f4_E4_CASE) {
	    l_USER(&call_data->dxcast, output);

	    call_data->dxcast.call = 0;
	    call_data->dxcast.type = type;

	    output = &call_data->dxcast.data;
	} else {
	}

	bits = POST_BITS;

	over = function_text->args;

	deck_call(deck, pull, input, grade);

	star_call
	    (frame, input + lead + pull, parts, args, over + lead + pull, line,
	     pull, deck, lead + count);

	input += lead + pull + line;
	over += lead + pull + line;

	away = args + line + pull;

	i = pull;
	for (; i; i--) {
	    int lock;

	    lock = *deck++;
	    args[lock] = *over;
	    frame[lock] = *input++;
	    if (scale) {
		away[lock] = over[class];
	    }

	    over++;
	}

	count /= pull;

	{
	    /* the args, count, flags set should do */
	    call_data->linetext = *linetext_data;

	    call_data->linetext.function.type = X1f4_E4_CASE;

	    status = _libx1f4i0_lxcall_post_call
		(&call_data->linetext, function_call, output, 0, frame, bits,
		 &lxtrap_data->link_v, &lxtrap_data->link_p,
		 &lxtrap_data->link_q, &lxtrap_data->link_e,
		 lxtrap_data->trap_e.eelookup_data);
	    if (status) {
		count = 0;
	    } else {
		count--;
	    }
	}
	for (; count; count--) {
	    /* the args, count, flags set should do */
	    call_data->linetext = *linetext_data;

	    call_data->linetext.function.type = X1f4_E4_CASE;

	    deck -= pull;

	    i = pull;
	    for (; i; i--) {
		int lock;

		lock = *deck++;
		args[lock] = *over;
		frame[lock] = *input++;
		if (scale) {
		    away[lock] = over[class];
		}

		over++;
	    }

	    status = _libx1f4i0_lxcall_rail_call
		(&call_data->linetext, function_call, output, 0, frame, bits,
		 &lxtrap_data->link_p, &lxtrap_data->link_q,
		 &lxtrap_data->link_e, lxtrap_data->trap_e.eelookup_data, pull,
		 deck, call_data->finite, call_data->serial);
	    if (status) {
		break;
	    }
	}
    }

    return status;
}


/*
 * function does setup for an unused output
 */
static int
lift_call(struct lxtrap_type *lxtrap_data, void *output, void **input,
	  struct x1f4_linetext_type *linetext_data, int lead,
	  const struct x1f4_function_type *function_text, int line,
	  unsigned pull)
{
    int linked, status;
    unsigned class, count, grade, parts, scale = 0;
    void *beta, **track;

    class = function_text->count;

    count = class - lead - line - pull;

    grade = line + pull;

    parts = function_text->flags;

    if (parts & X1f4_E4_POST_TYPE) {
	if (linetext_data->function.flags
	    & (X1f4_E4_LINK_PASS | X1f4_E4_POST_TYPE)) {
	    scale = 1;
	} else {
	    parts &= ~X1f4_E4_POST_TYPE;
	}
    }

    linked = sizeof(struct slip_type);

    status = lxtrap_data->link_v.link
	(lxtrap_data->link_v.data, &beta,
	 linked + (sizeof(void *) + (sizeof(int) << scale)) * grade
	 + sizeof(void *) + (sizeof(int) * pull << 1));
    if (status) {
	status = _libx1f4i0_lxtrap_stat_link(lxtrap_data);
    } else {
	int *args, *away, *deck;
	const int *over;
	int type;
	struct slip_type *call_data;
	struct x1f4_function_type *function_call;
	unsigned i;
	void **clear, **frame, *series;

	call_data = beta;

	function_call = &call_data->function;

	function_call->count = grade;

	function_call->flags = parts & X1f4_E4_POST_TYPE;

	frame = (void *) ((char *) beta + linked);
	frame++;

	function_call->args = args = (void *) (frame + grade);

	deck = args + (grade << scale);

	track = input[1];

	frame[-1] = track[-1];

	type = linetext_data->function.type;

	series = input[0];

	if (type != X1f4_E4_CASE) {
	    l_USER(&call_data->dxcast, series);

	    call_data->dxcast.call = 0;
	    call_data->dxcast.type = type;

	    series = &call_data->dxcast.data;
	} else {
	}

	over = function_text->args;

	deck_call(deck, pull, track, grade);

	star_call
	    (frame, track + lead + pull, parts, args, over + lead + pull, line,
	     pull, deck, lead + count);

	clear = call_data->clear;

	clear[0] = series;
	clear[1] = frame;

	track += lead + pull + line;
	over += lead + pull + line;

	away = args + line + pull;

	i = pull;
	for (; i; i--) {
	    int lock;

	    lock = *deck++;
	    args[lock] = *over;
	    frame[lock] = *track++;
	    if (scale) {
		away[lock] = over[class];
	    }

	    over++;
	}

	call_data->linetext = *linetext_data;

	call_data->linetext.function.type = X1f4_E4_CASE;

	status = _libx1f4i0_lxcall_post_call
	    (&call_data->linetext, function_call, output, 0, clear, POST_BITS,
	     &lxtrap_data->link_v, &lxtrap_data->link_p,
	     &lxtrap_data->link_q, &lxtrap_data->link_e,
	     lxtrap_data->trap_e.eelookup_data);
	if (status) {
	} else {
	    struct x1f4_dispatch_type *dispatch_data;

	    dispatch_data = input[2] = clear[2];

	    if (dispatch_data) {
		call_data->back = dispatch_data->back;
		call_data->call = dispatch_data->call;
		call_data->miss = count / pull;
		call_data->over = over;
		call_data->pull = pull;

		call_data->deck = deck - pull;

		call_data->class = class;
		call_data->track = track;

		call_data->lxtrap_data = lxtrap_data;

		call_data->linetext_data = linetext_data;

		dispatch_data->back = call_data;
		dispatch_data->call = part_call;

		call_data->linetext.function.type = type;

		call_data->dispatch_data = dispatch_data;
	    }
	}
    }

    return status;
}


static int
part_call(void *back, int status, void *output,
	  struct x1f4_dispatch_type **dispatch)
{
    do {
	int slip;
	struct slip_type *call_data;

	if (status) {
	    break;
	}

	call_data = back;

	status = call_data->call(call_data->back, 0, output, dispatch);
	if (status) {
	    break;
	}

	slip = call_data->miss;
	slip--;
	if (slip) {
	    int *args, *away, *deck;
	    const int *over;
	    struct lxtrap_type *lxtrap_data;
	    struct x1f4_dispatch_type *dispatch_data;
	    unsigned i;
	    void **frame, **track;

	    call_data->miss = slip;

	    lxtrap_data = call_data->lxtrap_data;

	    frame = (void *) (call_data + 1);
	    frame++;

	    args = (int *) call_data->function.args;
	    deck = call_data->deck;
	    if (call_data->function.flags & X1f4_E4_POST_TYPE) {
		away = args + call_data->function.count;
	    } else {
		away = NULL;
	    }

	    over = call_data->over;
	    track = call_data->track;

	    i = call_data->pull;
	    for (; i; i--) {
		int lock;

		lock = *deck++;
		args[lock] = *over;
		frame[lock] = *track++;
		if (away) {
		    away[lock] = over[call_data->class];
		}

		over++;
	    }

	    call_data->over = over;
	    call_data->track = track;

	    /* the args, count, flags set should do */
	    call_data->linetext = *call_data->linetext_data;

	    call_data->linetext.function.type = X1f4_E4_CASE;

	    call_data->clear[2] = call_data->dispatch_data;

	    status = _libx1f4i0_lxcall_into_call
		(&call_data->linetext, &call_data->function, 0,
		 call_data->clear, POST_BITS | X1f4_LXCALL_POST_PLOT,
		 &lxtrap_data->link_p, &lxtrap_data->link_q,
		 &lxtrap_data->link_e, lxtrap_data->trap_e.eelookup_data,
		 call_data->pull, call_data->deck, call_data->finite,
		 call_data->serial);
	    if (status) {
		break;
	    }

	    dispatch_data = call_data->clear[2];

	    if (dispatch_data) {
		call_data->back = dispatch_data->back;
		call_data->call = dispatch_data->call;

		dispatch_data->back = call_data;
		dispatch_data->call = part_call;

		call_data->linetext.function.type = call_data->dxcast.type;

		*dispatch = dispatch_data;
	    }
	} else {
	    *dispatch = NULL;
	}
    } while (0);

    return status;
}


static int
star_call(void **post_data, void **line_data, unsigned parts, int *post_args,
	  const int *line_args, unsigned line, unsigned pull, int *deck,
	  unsigned lead)
{
    int *star;
    unsigned base, copy, diff, i;

    star = deck + pull;

    memcpy(star, deck, sizeof(int) * pull);
    qsort(star, pull, sizeof(int), star_fill);

    base = -1;
    i = 0;
    for (; i < pull; i++) {
	unsigned lock;

	lock = star[i];

	diff = lock - base;
	copy = diff - 1;

	memcpy(post_data, line_data, copy * sizeof(void *));

	line_data += copy;
	post_data += diff;

	memcpy(post_args, line_args, copy * sizeof(int));

	line_args += copy;
	post_args += diff;

	base = lock;
    }
    {
	copy = line + pull - base - 1;

	memcpy(post_data, line_data, copy * sizeof(void *));

	memcpy(post_args, line_args, copy * sizeof(int));
    }
    if (parts & X1f4_E4_POST_TYPE) {
	line_args += copy + lead + pull;
	post_args += copy;

	base = -1;
	i = 0;
	for (; i < pull; i++) {
	    unsigned lock;

	    lock = star[i];

	    copy = lock - base - 1;

	    memcpy(post_args, line_args, copy * sizeof(int));

	    line_args += copy;
	    post_args += copy;

	    *post_args++ = 0;

	    base = lock;
	}
	{
	    copy = line + pull - base - 1;

	    memcpy(post_args, line_args, copy * sizeof(int));
	}
    }

    return 0;
}


static int
star_fill(const void *a, const void *b)
{
    return *(int *) a - *(int *) b;
}


int
_libx1f4i0_lxtrap_u_derived(void *context, void *output, void **input)
{
    int status;

    do {
	X1f4_E4_C_MODE line, pull;
	const int *args;
	struct lxtrap_type *lxtrap_data;
	struct x1f4_dxcast_type *dxcast_data;
	const struct x1f4_function_type *function_text;
	struct x1f4_linetext_type *linetext_data;
	unsigned class, grade, i;
	void **track;

	lxtrap_data = context;

	if (output == x1f4_a1_walk_link) {
	    track = input[1];
	} else {
	    track = input;
	}

	dxcast_data = I_USER(track[0]);

	if (X1f4_E4_CALL < dxcast_data->type) {
	} else {
	    status = _libx1f4i0_lxtrap_stat_roll(lxtrap_data);
	    break;
	}

	linetext_data = I_USER(&dxcast_data->data);

	if (linetext_data->function.function) {
	} else {
	    status = _libx1f4i0_lxtrap_stat_zero(lxtrap_data);
	    break;
	}

	function_text = ((void **) track[-1])[X1f4_E4_RENDITION];

	class = function_text->count - 3;

	line = I_MODE(track[1]);
	if (line < 0 || class < line) {
	    status = _libx1f4i0_lxtrap_stat_line(lxtrap_data, line, class);
	    break;
	}

	class -= line;

	pull = I_MODE(track[2]);

	if (pull < 1) {
	    status = _libx1f4i0_lxtrap_stat_fill(lxtrap_data, pull);
	    break;
	}

	if (class < pull) {
	    status = _libx1f4i0_lxtrap_stat_hull(lxtrap_data, pull, class);
	    break;
	}

	if (class % pull) {
	    status = _libx1f4i0_lxtrap_stat_pull(lxtrap_data, pull, class);
	    break;
	}

	grade = line + 1;

	status = 0;

	args = function_text->args + 3;

	i = 0;
	for (; i < pull; i++) {
	    X1f4_E4_C_MODE lock;

	    if (*args != X1f4_E4_MODE) {
		status = _libx1f4i0_lxtrap_stat_iran(context, 3 + i);
		break;
	    }

	    lock = I_MODE(track[3 + i]);

	    if (lock < -(int) grade || !(lock < (int) grade)) {
		status = _libx1f4i0_lxtrap_stat_land(context, grade, lock);
		break;
	    }

	    args++;

	    grade++;
	}
	if (status) {
	    break;
	}

	if (pull < class) {
	    if (output == x1f4_a1_walk_link) {
		status = lift_call
		    (lxtrap_data, output, input, linetext_data, 3,
		     function_text, line, pull);
	    } else {
		status = fold_call
		    (lxtrap_data, output, input, linetext_data, 3,
		     function_text, line, pull);
	    }
	} else {
	    status = 0;

	    if (output == x1f4_a1_walk_link) {
		input[2] = NULL;
	    }
	}
    } while (0);

    return status;
}
