/*
 * lxtrap-f.s.c
 * Copyright (C) 2011-2015, 2017-2019, Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <stddef.h>

#include <e4.h>
#include <e4less.h>
#include <lxcall.h>
#include <lxcast.h>
#include <lxtrap-inter.h>
#include <lxtrap-types.h>

#define POST_BITS \
    X1f4_LXCALL_POST_ARGS | X1f4_LXCALL_POST_DATA | X1f4_LXCALL_POST_LINE

#define I_MODE(t)			(*((X1f4_E4_C_MODE *) (t)))
#define I_USER(t)			(*((X1f4_E4_C_USER *) (t)))

typedef struct pick_type {
    struct x1f4_function_type function;
} pick_type;

typedef struct slip_type {
    X1f4_E4_C_MODE lead, mode;
    int (*call)(void *, int, void *, struct x1f4_dispatch_type **), lock,
	*reach;
    struct fxdata_type finite[1];
    struct lxtrap_type *lxtrap_data;
    struct x1f4_dispatch_type *dispatch_data;
    struct x1f4_dxcast_type serial[1];
    struct x1f4_function_type function;
    struct x1f4_linetext_type linetext, *linetext_data;
    unsigned loop;
    void *back, *clear[3], **order, *rendition11;
} slip_type;

extern const void *const x1f4_a1_walk_link;

static int part_call(void *, int, void *, struct x1f4_dispatch_type **);
static int pick_call(struct x1f4_linetext_type *, void *, void **,
		     struct lxtrap_type *, unsigned, int *, void **, unsigned,
		     int, X1f4_E4_C_MODE, X1f4_E4_C_MODE, unsigned,
		     struct pick_type *);
static int slip_call(struct x1f4_linetext_type *, void *, void **,
		     struct lxtrap_type *, unsigned, int *, void **, unsigned,
		     int, X1f4_E4_C_MODE, X1f4_E4_C_MODE, unsigned,
		     struct slip_type *);

static int
part_call(void *back, int status, void *output,
	  struct x1f4_dispatch_type **dispatch)
{
    do {
	unsigned slip;
	struct slip_type *call_data;

	if (status) {
	    break;
	}

	call_data = back;

	status = call_data->call(call_data->back, 0, output, dispatch);
	if (status) {
	    break;
	}

	slip = call_data->loop;
	if (slip) {
	    struct lxtrap_type *lxtrap_data;
	    struct x1f4_dispatch_type *dispatch_data;

	    dispatch_data = call_data->dispatch_data;

	    call_data->loop = slip - 1;

	    call_data->lead += call_data->mode;

	    lxtrap_data = call_data->lxtrap_data;

	    *call_data->reach = X1f4_E4_MODE;
	    *call_data->order = &call_data->lead;

	    call_data->linetext = *call_data->linetext_data;

	    call_data->clear[2] = call_data->dispatch_data;

            status = _libx1f4i0_lxcall_into_call
                (&call_data->linetext, &call_data->function, 0,
                 call_data->clear, POST_BITS | X1f4_LXCALL_POST_PLOT,
                 &lxtrap_data->link_p, &lxtrap_data->link_q,
                 &lxtrap_data->link_e, lxtrap_data->trap_e.eelookup_data, 1,
                 &call_data->lock, call_data->finite, call_data->serial);
            if (status) {
		break;
            }

	    dispatch_data = call_data->clear[2];

	    call_data->back = dispatch_data->back;
	    call_data->call = dispatch_data->call;

	    dispatch_data->back = call_data;
	    dispatch_data->call = part_call;

	    *dispatch = dispatch_data;
	} else {
	    *dispatch = NULL;
	}
    } while (0);

    return status;
}


/*
 * likely, used[X1f4_E4_RENDITION] is set twice for the first call and
 * uselessly for error.
 */
static int
pick_call(struct x1f4_linetext_type *linetext_data, void *output, void **input,
	  struct lxtrap_type *lxtrap_data, unsigned grade, int *args,
	  void **frame, unsigned parts, int jack, X1f4_E4_C_MODE lead,
	  X1f4_E4_C_MODE mode, unsigned count, struct pick_type *call_data)
{
    int status;
    struct x1f4_function_type *function_call;
    struct x1f4_linetext_type linetext;
    void *clear[3], **order, **used;

    function_call = &call_data->function;

    function_call->args = args;
    function_call->count = grade;
    function_call->flags = parts & X1f4_E4_POST_TYPE;

    clear[0] = output;
    clear[1] = frame;

    order = frame + jack;

    *order = &lead;

    used = input[-1];

    linetext = *linetext_data;

    status = _libx1f4i0_lxcall_post_call
	(&linetext, &call_data->function, (void *) x1f4_a1_walk_link, 0,
	 clear, POST_BITS, &lxtrap_data->link_v, &lxtrap_data->link_p,
	 &lxtrap_data->link_q, &lxtrap_data->link_e,
	 lxtrap_data->trap_e.eelookup_data);
    if (status) {
    } else {
	int *reach;
	struct x1f4_dispatch_type *dispatch_data;
	struct x1f4_transfer_type *transfer_data;
	void *rendition;

	reach = args + jack;

	dispatch_data = clear[2];

	transfer_data = &dispatch_data->transfer;

	rendition = used[X1f4_E4_RENDITION];

	input = transfer_data->input;
	output = transfer_data->output;

	for (; count; count--) {
	    struct fxdata_type finite[1];
	    struct x1f4_dxcast_type serial[1];

	    linetext = *linetext_data;

	    *reach = X1f4_E4_MODE;
	    *order = &lead;

	    status = _libx1f4i0_lxcall_rail_call
		(&linetext, &call_data->function, output, 0, input, POST_BITS,
		 &lxtrap_data->link_p, &lxtrap_data->link_q,
		 &lxtrap_data->link_e, lxtrap_data->trap_e.eelookup_data, 1,
		 &jack, finite, serial);
	    if (status) {
		break;
	    } else {
	    }

	    lead += mode;
	}

	used[X1f4_E4_RENDITION] = rendition;

	if (status) {
	} else {
	    status = dispatch_data->call
		(dispatch_data->back, 0, output, &dispatch_data);
	}
    }

    return status;
}


static int
slip_call(struct x1f4_linetext_type *linetext_data, void *output, void **input,
	  struct lxtrap_type *lxtrap_data, unsigned grade, int *args,
	  void **frame, unsigned parts, int jack, X1f4_E4_C_MODE lead,
	  X1f4_E4_C_MODE mode, unsigned count, struct slip_type *call_data)
{
    int status;
    struct x1f4_function_type *function_call;
    void **order;

    function_call = &call_data->function;

    function_call->args = args;
    function_call->count = grade;
    function_call->flags = parts & X1f4_E4_POST_TYPE;

    call_data->clear[0] = *input;
    call_data->clear[1] = frame;

    call_data->lock = jack;

    order = frame + jack;

    *order = &call_data->lead;

    call_data->lead = lead;
    call_data->mode = mode;

    call_data->linetext = *linetext_data;

    status = _libx1f4i0_lxcall_post_call
	(&call_data->linetext, &call_data->function, output, 0,
	 call_data->clear, POST_BITS, &lxtrap_data->link_v,
	 &lxtrap_data->link_p, &lxtrap_data->link_q, &lxtrap_data->link_e,
	 lxtrap_data->trap_e.eelookup_data);
    if (status) {
    } else {
	struct x1f4_dispatch_type *dispatch_data;

	dispatch_data = call_data->clear[2];

	input[2] = dispatch_data;

	count--;
	if (count) {
	    call_data->back = dispatch_data->back;
	    call_data->call = dispatch_data->call;
	    call_data->loop = count;

	    call_data->reach = args + jack;
	    call_data->order = frame + jack;

	    dispatch_data->back = call_data;
	    dispatch_data->call = part_call;

	    call_data->lxtrap_data = lxtrap_data;
	    call_data->linetext_data = linetext_data;

	    call_data->dispatch_data = dispatch_data;
	}
    }

    return status;
}


int
_libx1f4i0_lxtrap_s_forward(void *context, void *output, void **input)
{
    int status;

    do {
	X1f4_E4_C_MODE fill, jack, lead, mode;
	struct lxtrap_type *lxtrap_data;
	struct x1f4_dxcast_type *dxcast_data;
	const struct x1f4_function_type *function_text;
	struct x1f4_linetext_type *linetext_data;
	unsigned class, count, grade;
	void **track, **used;

	lxtrap_data = context;

	if (output == x1f4_a1_walk_link) {
	    track = input[1];
	} else {
	    track = input;
	}

	used = track[-1];

	dxcast_data = I_USER(track[0]);

	if (X1f4_E4_CALL < dxcast_data->type) {
	} else {
	    status = _libx1f4i0_lxtrap_stat_call(lxtrap_data, 0);

	    break;
	}

	linetext_data = I_USER(&dxcast_data->data);

	if (linetext_data->function.function) {
	} else {
	    status = _libx1f4i0_lxtrap_stat_zero(lxtrap_data);

	    break;
	}

	function_text = used[X1f4_E4_RENDITION];

	class = function_text->count - 5;

	grade = class + 1;

	jack = I_MODE(track[1]);

	if (jack < -(int) grade || !(jack < (int) grade)) {
	    status = _libx1f4i0_lxtrap_stat_land(context, grade, jack);

	    break;
	}

	if (jack < (X1f4_E4_C_MODE) 0) {
	    jack += grade;
	}

	lead = I_MODE(track[2]);
	fill = I_MODE(track[3]);
	mode = I_MODE(track[4]);

	if (lead < fill) {
	    if (0 < mode) {
		count = (fill - lead + mode - 1) / mode;
	    } else {
		count = 0;
	    }
	} else {
	    if (fill < lead) {
		if (mode < 0) {
		    count = (fill - lead + mode + 1) / mode;
		} else {
		    count = 0;
		}
	    } else {
		count = 0;
	    }
	}

	if (count) {
	    int *args;
	    unsigned linked, parts, scale = 0;
	    void *beta, **frame;

	    parts = function_text->flags;

	    if (parts & X1f4_E4_POST_TYPE) {
		if (linetext_data->function.flags
		    & (X1f4_E4_LINK_PASS | X1f4_E4_POST_TYPE)) {
		    scale = 1;
		} else {
		    parts &= ~X1f4_E4_POST_TYPE;
		}
	    }

	    linked = output == x1f4_a1_walk_link
		? sizeof(struct slip_type) : sizeof(struct pick_type);

	    status = lxtrap_data->link_v.link
		(lxtrap_data->link_v.data, &beta,
		 linked + (sizeof(void *) + (sizeof(int) << scale)) * grade
		 + sizeof(void *));
	    if (status) {
		status = _libx1f4i0_lxtrap_stat_link(lxtrap_data);
		if (1) {
		    break;
		}
	    } else {
		frame = (void *) ((char *) beta + linked);
		frame++;

		args = (void *) (frame + grade);

		frame[-1] = used;

		_libx1f4i0_lxcall_hold_call
		    (frame, track, parts, args, function_text->args, class, 5,
		     jack);

		args[jack] = X1f4_E4_MODE;
	    }

	    if (output == x1f4_a1_walk_link) {
		status = slip_call
		    (linetext_data, output, input, lxtrap_data, grade, args,
		     frame, parts, jack, lead, mode, count, beta);
	    } else {
		status = pick_call
		    (linetext_data, output, input, lxtrap_data, grade, args,
		     frame, parts, jack, lead, mode, count, beta);
	    }
	} else {
	    status = 0;

	    if (output == x1f4_a1_walk_link) {
		input[2] = NULL;
	    }
	}
    } while (0);

    return status;
}

