/*
 * lxtrap-i.a.c
 * Copyright (C) 2019, Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <stddef.h>

#include <e4.h>
#include <ln.h>
#include <lxcall.h>
#include <lxlead.h>
#include <lxtrap-close.h>
#include <lxtrap-drive.h>
#include <lxtrap-types.h>

int
_libx1f4i0_lxtrap_free_args(struct dxnews_type *line_data,
			    struct x1f4_nodelink_type **nodelink)
{
    const int *args;
    int excess, status = 0;
    struct lxtrap_type *lxtrap_data;
    struct x1f4_linetext_type *linetext_data;
    struct x1f4_nodelink_type *nodelink_data, *nodelink_slip;
    unsigned fold;
    void **track;

    linetext_data = (void *) line_data;
    linetext_data--;

    nodelink_data = &((struct x1f4_leadnode_type *) linetext_data - 1)->link;

    args = linetext_data->function.args;

    fold = line_data->fold;

    line_data->fold = 0;

    track = line_data->track + fold;

    lxtrap_data = line_data->lxtrap_data;

    if (nodelink) {
	nodelink_slip = *nodelink;
    } else {
	nodelink_slip = NULL;
    }

    for (; fold; fold--) {
	int type;
	const struct lxtype_type *lxtype_data;
	unsigned miss;
	void *side;

	side = *--track;

	type = *--args;

	if (type == X1f4_E4_CASE) {
	    struct x1f4_dxcast_type *dxcast_data;

	    dxcast_data = I_USER(side);
	    type = dxcast_data->type;
	    side = &dxcast_data->data;
	}

	lxtype_data = lxtrap_data->link_t.data;

	close_frame(miss, lxtype_data, lxtrap_data);

	if (miss) {
	    int (*free)(void *, void *, struct x1f4_nodelink_type **,
			const struct x1f4_nodelink_type *);

	    if (lxtype_data->bits & X1f4_LX_PICK_ACCESS) {
		free = lxtype_data->pick;
	    } else {
		free = lxtype_data->free;
	    }
	    if (free) {
		excess = free
		    (lxtype_data->text,
		     I_USER(side), &nodelink_slip, nodelink_data);
		if (excess) {
		    status = _libx1f4i0_lxcall_land_slip(excess, status);
		}
	    }
	}
    }

    if (nodelink) {
	*nodelink = nodelink_slip;
    } else {
	if (nodelink_slip) {
	    excess = x1f4_fail_lxlead(nodelink_slip);
	    if (excess) {
		status = _libx1f4i0_lxcall_land_slip(excess, status);
	    }
	}
    }

    return status;
}
