/*
 * lxtrap-types.h
 * Copyright (C) 2008-2015, 2017, 2019, 2021, Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef __LIBx1f4_lxtrap_types_H__
#define __LIBx1f4_lxtrap_types_H__

#include <config.h>

#include <e4.h>
#include <frame.h>
#include <it.h>
#include <lxmile.h>
#include <track.h>
#include <trans.h>

typedef struct dxnews_type {
    struct lxtrap_type *lxtrap_data;
    struct x1f4_linetext_type *linetext_data;
    unsigned fold;
    unsigned char *fuse;
    void **track;
} dxnews_type;

typedef struct fxcase_type {
    X1f4_E4_C_TEXT text;
    int missed;
    struct x1f4_trans_type trans;
    unsigned size, trip;
    void *record;
} fxcase_type;

typedef struct lxtrap_type {
    struct lxmile_type ever_n;
    struct {
	struct x1f4_track_type **call;
    } link_h;
    struct {
	void *fine;
    } link_o;
    struct x1f4_iturn_type link_p;
    struct x1f4_composer_type link_q;
    struct {
	unsigned miss;
	const void *data;
    } link_t;
    struct {
        struct lxfold_type fold[1];
    } link_x;
} lxtrap_type;

typedef struct lxtype_type {
    int (*copy)(void *, void *, void **, const void *,
		const struct x1f4_nodelink_type *),
	(*free)(void *, void *, struct x1f4_nodelink_type **,
		const struct x1f4_nodelink_type *),
        (*lead)(void *, void **), (*slip)(void *, void **),
	(*node)(void *, void **, void *, const struct x1f4_nodelink_type *),
	(*pick)(void *, void *, struct x1f4_nodelink_type **,
		const struct x1f4_nodelink_type *), type;
    unsigned bits;
    void *text;
} lxtype_type;

typedef struct screen_type {
    struct x1f4_track_type lock_e;
    struct {
	const struct x1f4_eelookup_type *eelookup_data;
    } lock_g;
    struct {
	struct x1f4_track_type **call;
    } link_h;
    struct x1f4_trans_type lock_m;
    struct {
	void *fine;
    } link_o;
    struct x1f4_iturn_type link_p;
    struct x1f4_composer_type link_q;
    struct {
	int (*call)(void *, struct x1f4_nodelink_type *),
	    (*fine)(void *, struct x1f4_nodelink_type **,
		    struct x1f4_nodelink_type *),
	    (*miss)(void *, struct x1f4_nodelink_type **,
		    struct x1f4_nodelink_type *);
	void *text;
    } lock_r;
    struct {
	unsigned miss;
	const void *data;
    } link_t;
    struct x1f4_frame_type lock_v;
    struct x1f4_trans_type lock_w;
} screen_type;

#endif				/* __LIBx1f4_lxtrap_types_H__ */
