/*
 * lxvery-a.0.c
 * Copyright (C) 2008-2021, Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <stddef.h>
#include <string.h>

#include <af.h>
#include <d_fset.h>
#include <e4-m.0.h>
#include <e4.h>
#include <e4fine.h>
#include <lxcall.h>
#include <lxvery-defs.h>
#include <lxvery-inter.h>
#include <lxvery-types.h>
#include <lxnext.h>
#include <mxdeck.h>

#define MAKE_DOUBLE(a, b)		a b
#define MAKE_SINGLE(a, b)		a

#define true(e)				1

#define screen(screen) \
    ((struct screen_type *) (screen))

#define fb_size() \
    (sizeof(side_o) / sizeof(struct x1f4_operator_type))
#define fp_size() \
    (sizeof(side_u) / sizeof(struct x1f4_operator_type))

static int init_line(struct lxvery_type *, unsigned,
		     const struct screen_type *);
static int line_line(struct lxvery_type *, unsigned,
		     const struct screen_type *);
static int line_plus(struct lxvery_type *);
static int line_text(struct lxvery_type *, unsigned,
		     const struct screen_type *, void *, void **, void **);
static int link_line(struct lxvery_type *, unsigned,
		     const struct screen_type *);
static int part_text(struct lxvery_type *, unsigned,
		     const struct screen_type *, struct x1f4_linetext_type **,
		     void **, unsigned, const struct line_type *);

static void bo_lmm00x(u_case_args_____0);
static void born_lm0x(u_case_args_____0);
static void borne_lcm(u_case_args_____0);
static void borneo__l(u_case_args_____0);
static void borneo_l_(u_case_args_____0);
static void borneo_lm(u_case_args_____0);
static void line_nset(u_case_args_____0);
static void line_sset(u_case_args_____0);
static void theme_lll(u_case_args_____0);
static void themed__l(u_case_args_____0);
static void themed_ll(u_case_args_____0);
static void themed_lm(u_case_args_____0);

static const char fg_fixf[] = "m_fix", fg_putf[] = "m_put", fg_setf[] =
    "m_set", fg_sizef[] = "m_size", *const side_f[] = {
/* *INDENT-OFF* */
    fg_setf
/* *INDENT-ON* */
}, *const side_l[] = {
/* *INDENT-OFF* */
    fg_sizef
/* *INDENT-ON* */
};
static const struct line_type auto_line[] = {
/* *INDENT-OFF* */
#define r_fit_count			1
    {	{	"m_fit",
		_libx1f4i0_lxvery_f_centric,	0,
		NULL,				r_fit_count,
		X1f4_E4_KEEP_CALL | X1f4_E4_SIDE_LIST
		| X1f4_E4_TEXT_LINK,		5		},
	themed__l,							},
#define r_fit_reach			(0 + r_fit_count)
#define r_new_count			1
    {	{	"m_new",
		_libx1f4i0_lxvery_n_forward,	X1f4_E4_VOID,
		NULL,				r_new_count,
		X1f4_E4_KEEP_CALL | X1f4_E4_POST_TYPE
		| X1f4_E4_TEXT_LINK,		5		},
	line_nset,							},
#define r_new_reach			(r_fit_reach + (r_new_count << 1))
#define r_render_count			0
    {	{	"m_render",
		_libx1f4i0_lxvery_r_centric,	0,
		NULL,				r_render_count,
		X1f4_E4_KEEP_CALL | X1f4_E4_SIDE_LIST
		| X1f4_E4_TEXT_LINK,
						8		},
	borneo_l_,							},
#define r_render_reach			(r_new_reach + r_render_count)
#define r_vcall_count			3
    {	{	"m_vcall",
		_libx1f4i0_lxvery_v_pending,	X1f4_E4_VOID,
		NULL,				r_vcall_count,
		X1f4_E4_KEEP_CALL | X1f4_E4_LINK_PASS
		| X1f4_E4_SIDE_LIST | X1f4_E4_TEXT_LINK
		| X1f4_E4_WALK_LINK,
						7		},
	borne_lcm,							},
#define r_vcall_reach			(r_render_reach + r_vcall_count)
#define mint_count			0
    {	{	"mint",
		_libx1f4i0_lxvery_r_centric,	0,
		NULL,				mint_count,
		X1f4_E4_KEEP_CALL | X1f4_E4_SIDE_LIST
		| X1f4_E4_TEXT_LINK,
						4		},
	borneo_l_,							}
#define mint_reach			(r_vcall_reach + mint_count)
/* *INDENT-ON* */
#define auto_line_count			mint_reach
}, ever_line[] = {
/* *INDENT-OFF* */
#define r_set_count			2
    {	{	fg_setf,
		_libx1f4i0_lxvery_s_pending,	0,
		NULL,				r_set_count,
		X1f4_E4_KEEP_CALL | X1f4_E4_POST_TYPE
		| X1f4_E4_TEXT_LINK,		5		},
	line_sset,							}
#define r_set_reach			(0 + (r_set_count << 1))
/* *INDENT-ON* */
#define ever_line_count			r_set_reach
}, fail_line[] = {
/* *INDENT-OFF* */
#define r_fix_count			2
    {	{	fg_fixf,
		_libx1f4i0_lxvery_f_derived,	X1f4_E4_VOID,
		NULL,				r_fix_count,
		X1f4_E4_KEEP_CALL | X1f4_E4_TEXT_LINK,
						5		},
	borneo_lm,							},
#define r_fix_reach			(0 + r_fix_count)
#define r_put_count			2
    {	{	fg_putf,
		_libx1f4i0_lxvery_p_forward,	X1f4_E4_VOID,
		NULL,				r_put_count,
		X1f4_E4_KEEP_CALL | X1f4_E4_TEXT_LINK,
						5		},
	borneo_lm,							}
#define r_put_reach			(r_fix_reach + r_put_count)
/* *INDENT-ON* */
#define fail_line_count			r_put_reach
}, gray_line[] = {
/* *INDENT-OFF* */
#define r_high_count			1
    {	{	"m_high",
		_libx1f4i0_lxvery_g_keeping,	X1f4_E4_MODE,
		NULL,				r_high_count,
		X1f4_E4_KEEP_CALL | X1f4_E4_TEXT_LINK,
						6		},
	borneo__l,							},
#define r_high_reach			(0 + r_high_count)
#define r_last_count			2
    {	{	"m_last",
		_libx1f4i0_lxvery_h_keeping,	X1f4_E4_MODE,
		NULL,				r_last_count,
		X1f4_E4_KEEP_CALL | X1f4_E4_POST_TYPE
		| X1f4_E4_TEXT_LINK,		6		},
	born_lm0x,							},
#define r_last_reach			(r_high_reach + (r_last_count << 1))
#define rb_delete_count			1
    {	{	"mb_delete",
		_libx1f4i0_lxvery_a_keeping,	0,
		NULL,				rb_delete_count,
		X1f4_E4_KEEP_CALL | X1f4_E4_TEXT_LINK,
						9		},
	themed__l,							},
#define rb_delete_reach			(r_last_reach + rb_delete_count)
/* *INDENT-ON* */
#define gray_line_count			rb_delete_reach
}, slip_line[] = {
/* *INDENT-OFF* */
#define r_blend_count			3
    {	{	"m_blend",
		_libx1f4i0_lxvery_b_derived,	0,
		NULL,				r_blend_count,
		X1f4_E4_KEEP_CALL | X1f4_E4_TEXT_LINK,
						7		},
	theme_lll,							},
#define r_blend_reach			(0 + r_blend_count)
#define r_clear_count			1
    {	{	"m_clear",
		_libx1f4i0_lxvery_c_pending,	0,
		NULL,				r_clear_count,
		X1f4_E4_KEEP_CALL | X1f4_E4_TEXT_LINK,
						7		},
	themed__l,							},
#define r_clear_reach			(r_blend_reach + r_clear_count)
#define r_copy_count			2
    {	{	"m_copy",
		_libx1f4i0_lxvery_c_logique,	0,
		NULL,				r_copy_count,
		X1f4_E4_KEEP_CALL | X1f4_E4_TEXT_LINK,
						6		},
	themed_ll,							},
#define r_copy_reach			(r_clear_reach + r_copy_count)
#define r_delete_count			2
    {	{	"m_delete",
		_libx1f4i0_lxvery_d_logique,	0,
		NULL,				r_delete_count,
		X1f4_E4_KEEP_CALL | X1f4_E4_TEXT_LINK,
						8		},
	themed_lm,							},
#define r_delete_reach			(r_copy_reach + r_delete_count)
#define r_key_count			2
    {	{	"m_key",
		_libx1f4i0_lxvery_k_logique,	X1f4_E4_MODE,
		NULL,				r_key_count,
		X1f4_E4_KEEP_CALL,
						5		},
	borneo_lm,							},
#define r_key_reach			(r_delete_reach + r_key_count)
#define r_merge_count			2
    {	{	"m_merge",
		_libx1f4i0_lxvery_m_derived,	0,
		NULL,				r_merge_count,
		X1f4_E4_KEEP_CALL | X1f4_E4_TEXT_LINK,
						7		},
	themed_ll,							},
#define r_merge_reach			(r_key_reach + r_merge_count)
#define r_resign_count			2
    {	{	"m_resign",
		_libx1f4i0_lxvery_r_logique,	0,
		NULL,				r_resign_count,
		X1f4_E4_KEEP_CALL | X1f4_E4_TEXT_LINK,
						8		},
	themed_lm,							},
#define r_resign_reach			(r_merge_reach + r_resign_count)
#define r_size_count			1
    {	{	fg_sizef,
		_libx1f4i0_lxvery_r_forward,	X1f4_E4_MODE,
		NULL,				r_size_count,
		0,				6		},
	borneo__l,							},
#define r_size_reach			(r_resign_reach + r_size_count)
#define r_swap_count			2
    {	{	"m_swap",
		_libx1f4i0_lxvery_s_sequent,	0,
		NULL,				r_swap_count,
		X1f4_E4_KEEP_CALL | X1f4_E4_TEXT_LINK,
						6		},
	themed_ll,							}
#define r_swap_reach			(r_size_reach + r_swap_count)
/* *INDENT-ON* */
#define slip_line_count			r_swap_reach
}, used_line[] = {
/* *INDENT-OFF* */
#define r_first_count			2
    {	{	"m_first",
		_libx1f4i0_lxvery_h_keeping,	X1f4_E4_MODE,
		NULL,				r_first_count,
		X1f4_E4_KEEP_CALL | X1f4_E4_POST_TYPE
		| X1f4_E4_TEXT_LINK,		7		},
	born_lm0x,							},
#define r_first_reach			(0 + (r_first_count << 1))
#define r_low_count			1
    {	{	"m_low",
		_libx1f4i0_lxvery_g_keeping,	X1f4_E4_MODE,
		NULL,				r_low_count,
		X1f4_E4_KEEP_CALL | X1f4_E4_TEXT_LINK,
						5		},
	borneo__l,							},
#define r_low_reach			(r_first_reach + r_low_count)
#define rf_delete_count			1
    {	{	"mf_delete",
		_libx1f4i0_lxvery_a_keeping,	0,
		NULL,				rf_delete_count,
		X1f4_E4_KEEP_CALL | X1f4_E4_TEXT_LINK,
						9		},
	themed__l,							},
#define rf_delete_reach			(r_low_reach + rf_delete_count)
/* *INDENT-ON* */
#define used_line_count			rf_delete_reach
}, wrap_line[] = {
/* *INDENT-OFF* */
#define rsk_greater_count		3
    {	{	"msk_greater",
		_libx1f4i0_lxvery_0k_queued,	X1f4_E4_MODE,
		NULL,				rsk_greater_count,
		X1f4_E4_KEEP_CALL | X1f4_E4_POST_TYPE
		| X1f4_E4_TEXT_LINK,		11		},
	bo_lmm00x,							},
#define rsk_greater_reach		(0 + (rsk_greater_count << 1))
#define rsk_upper_count			3
    {	{	"msk_upper",
		_libx1f4i0_lxvery_0k_queued,	X1f4_E4_MODE,
		NULL,				rsk_upper_count,
		X1f4_E4_KEEP_CALL | X1f4_E4_POST_TYPE
		| X1f4_E4_TEXT_LINK,		9		},
	bo_lmm00x,							},
#define rsk_upper_reach \
    (rsk_greater_reach + (rsk_upper_count << 1))
#define rsk_lower_count		3
    {	{	"msk_lower",
		_libx1f4i0_lxvery_0k_queued,	X1f4_E4_MODE,
		NULL,				rsk_lower_count,
		X1f4_E4_KEEP_CALL | X1f4_E4_POST_TYPE
		| X1f4_E4_TEXT_LINK,		9		},
	bo_lmm00x,							},
#define rsk_lower_reach \
    (rsk_upper_reach + (rsk_lower_count << 1))
#define rsk_less_count			3
    {	{	"msk_less",
		_libx1f4i0_lxvery_0k_queued,	X1f4_E4_MODE,
		NULL,				rsk_less_count,
		X1f4_E4_KEEP_CALL | X1f4_E4_POST_TYPE
		| X1f4_E4_TEXT_LINK,		8		},
	bo_lmm00x,							}
#define rsk_less_reach \
    (rsk_lower_reach + (rsk_less_count << 1))
/* *INDENT-ON* */
#define wrap_line_count		rsk_less_reach
};
static const struct x1f4_operator_type side_o[] = {
/* *INDENT-OFF* */
    {	MAKE_SINGLE("=", " "),  NULL,		0400,
	0,			NULL,
	X1f4_E4_BACK_LINK | X1f4_E4_E2ND_LINK | X1f4_E4_LEFT_XSET,
				1,
	NULL,			NULL					}
/* *INDENT-ON* */
}, side_u[] = {
/* *INDENT-OFF* */
    {	MAKE_SINGLE("~", " "),  NULL,		0210,
	0,			NULL,
	X1f4_E4_E2ND_LINK,
				1,
	NULL,			NULL					}
/* *INDENT-ON* */
};

static int
init_line(struct lxvery_type *lxvery_data, unsigned bits,
	  const struct screen_type *screen_data)
{
    if (bits & AUTOLINK_LINK) {
	lxvery_data->link_v = screen_data->lock_v;
    } else {
	lxvery_data->link_v.data = NULL;
	lxvery_data->link_v.free = NULL;
	lxvery_data->link_v.link = NULL;
	lxvery_data->link_v.mode = NULL;
	lxvery_data->link_v.pick = NULL;
	lxvery_data->link_v.slip = NULL;
    }

    if (bits & CODELINK_LINK) {
	lxvery_data->link_w = screen_data->lock_w;
    } else {
	lxvery_data->link_w.data = NULL;
	lxvery_data->link_w.free = _x1f4_e4_free_data;
	lxvery_data->link_w.link = _x1f4_e4_link_data;
	lxvery_data->link_w.mode = _x1f4_e4_mode_data;
    }

    if (bits & RESOURCE_LINK) {
	lxvery_data->link_m = screen_data->lock_m;
    } else {
	lxvery_data->link_m.data = NULL;
	lxvery_data->link_m.free = _x1f4_e4_free_data;
	lxvery_data->link_m.link = _x1f4_e4_link_data;
	lxvery_data->link_m.mode = _x1f4_e4_mode_data;
    }

    if (bits & TEXTFLAT_LINK) {
	lxvery_data->link_e = screen_data->lock_e;
    } else {
	lxvery_data->link_e.data = NULL;
	lxvery_data->link_e.line = NULL;
	lxvery_data->link_e.post = NULL;
	lxvery_data->link_e.push = NULL;
    }

    return link_line(lxvery_data, bits, screen_data);
}


static int
line_line(struct lxvery_type *lxvery_data, unsigned bits,
	  const struct screen_type *screen_data)
{
    lxvery_data->link_t.datatype[0].context = lxvery_data;

    lxvery_data->link_t.datatype[0].flat = _libx1f4i0_lxvery_flat_line;
    lxvery_data->link_t.datatype[0].lead = _libx1f4i0_lxvery_lead_line;
    lxvery_data->link_t.datatype[0].line = _libx1f4i0_lxvery_line_line;
    lxvery_data->link_t.datatype[0].link = _libx1f4i0_lxvery_link_line;
    lxvery_data->link_t.datatype[0].name = "mint";
    lxvery_data->link_t.datatype[0].slip = _libx1f4i0_lxvery_slip_line;
    lxvery_data->link_t.datatype[0].size = 4;
    lxvery_data->link_t.datatype[0].type = screen_data->link_l.type;

    lxvery_data->link_t.datatype[1].context = NULL;

    lxvery_data->link_t.datatype[1].flat = NULL;
    lxvery_data->link_t.datatype[1].lead = NULL;
    lxvery_data->link_t.datatype[1].line = NULL;
    lxvery_data->link_t.datatype[1].link = NULL;
    lxvery_data->link_t.datatype[1].name = NULL;
    lxvery_data->link_t.datatype[1].slip = NULL;
    lxvery_data->link_t.datatype[1].size = 0;
    lxvery_data->link_t.datatype[1].type = 0;

    return 0;
}


static int
line_plus(struct lxvery_type *lxvery_data)
{
    int i;
    struct cxbend_type *cxbend_data;
    struct cxseek_type *cxseek_data;

    cxbend_data = lxvery_data->rule_f.cxbend;

    cxbend_data[0].bend = 0;
    cxbend_data[0].line = lxvery_data;
    cxbend_data[0].pick = _libx1f4i0_lead_lxfset;
    cxbend_data[0].sign = 1;
    cxbend_data[0].slip = _libx1f4i0_mind_lxfset;

    cxbend_data++;

    cxbend_data[0].bend = 1;
    cxbend_data[0].line = lxvery_data;
    cxbend_data[0].pick = _libx1f4i0_ever_lxfset;
    cxbend_data[0].sign = -1;
    cxbend_data[0].slip = _libx1f4i0_slip_lxfset;

    cxseek_data = lxvery_data->rule_f.cxseek;

    cxseek_data[0].seek = _libx1f4i0_fgxd_lxfset;
    cxseek_data[1].seek = _libx1f4i0_fxxd_lxfset;
    cxseek_data[2].seek = _libx1f4i0_lxxd_lxfset;
    cxseek_data[3].seek = _libx1f4i0_llxd_lxfset;

    i = 4;
    for (; i; i--) {
	cxseek_data->line = lxvery_data;
	cxseek_data++;
    }

    return 0;
}


static int
line_text(struct lxvery_type *lxvery_data, unsigned bits,
	  const struct screen_type *part_data, void *data, void **args,
	  void **dana)
{
#define cxbend_ever			lxvery_data->rule_f.cxbend + 1
#define cxbend_lead			lxvery_data->rule_f.cxbend + 0

    struct x1f4_linetext_type *linetext_data;

    linetext_data = data;

    if (1) {
	_libx1f4i0_lxcall_side_line
	    (cxbend_ever, bits, part_data, &linetext_data, args,
	     sizeof(gray_line) / sizeof(struct line_type), gray_line);
	_libx1f4i0_lxcall_side_line
	    (lxvery_data, bits, part_data, &linetext_data, args,
	     sizeof(slip_line) / sizeof(struct line_type), slip_line);
	_libx1f4i0_lxcall_side_line
	    (cxbend_lead, bits, part_data, &linetext_data, args,
	     sizeof(used_line) / sizeof(struct line_type), used_line);
    }
    if (1) {
        part_text
            (lxvery_data, bits, part_data, &linetext_data, args,
             sizeof(wrap_line) / sizeof(struct line_type), wrap_line);
    }
    if (bits & AUTOLINK_LINK) {
	_libx1f4i0_lxcall_side_line
	    (lxvery_data, bits, part_data, &linetext_data, args,
	     sizeof(auto_line) / sizeof(struct line_type), auto_line);
	_libx1f4i0_lxcall_side_line
	    (lxvery_data, bits, part_data, &linetext_data, args,
	     sizeof(ever_line) / sizeof(struct line_type), ever_line);
	_libx1f4i0_lxcall_side_line
	    (lxvery_data, bits, part_data, &linetext_data, args,
	     sizeof(fail_line) / sizeof(struct line_type), fail_line);
    }
    if (1) {
	_libx1f4i0_lxcall_zero_line(linetext_data);
    }

    return 0;

#undef cxbend_lead
#undef cxbend_ever
}


static int
link_line(struct lxvery_type *lxvery_data, unsigned bits,
	  const struct screen_type *screen_data)
{
    int status;
    unsigned args_count, line_count, main_count;
    void *data;

    args_count = 0;
    line_count = 1;
    main_count = 0;

    {
	args_count += gray_line_count;
	line_count += sizeof(gray_line) / sizeof(struct line_type);
	args_count += slip_line_count;
	line_count += sizeof(slip_line) / sizeof(struct line_type);
	args_count += used_line_count;
	line_count += sizeof(used_line) / sizeof(struct line_type);
    }
    if (1) {
	args_count += wrap_line_count;
	line_count += sizeof(wrap_line) / sizeof(struct line_type);
    }
    if (bits & AUTOLINK_LINK) {
	args_count += auto_line_count;
	line_count += sizeof(auto_line) / sizeof(struct line_type);
	args_count += ever_line_count;
	line_count += sizeof(ever_line) / sizeof(struct line_type);
	args_count += fail_line_count;
	line_count += sizeof(fail_line) / sizeof(struct line_type);
    }

    status = lxvery_data->link_w.link
	(lxvery_data->link_w.data, &data,
	 args_count * sizeof(int)
	 + line_count * sizeof(struct x1f4_linetext_type)
	 + (sizeof(struct ticket_type) + sizeof(struct x1f4_operator_type *))
	 * (fb_size() + fp_size()) + sizeof(struct x1f4_operator_type *) * 2
	 + sizeof(struct x1f4_raillist_type) + main_count);
    if (status) {
	status = LINK_ERROR;
    } else {
	void *args, *dana, *port, *rate, *side;

	side = (struct x1f4_linetext_type *) data + line_count;
	side = (struct ticket_type *) side + fb_size();
	rate = (struct x1f4_operator_type **) side + fb_size() + 1;
	rate = (struct ticket_type *) rate + fp_size();
	port = (struct x1f4_operator_type **) rate + fp_size() + 1;
	args = (struct x1f4_raillist_type *) port + 1;
	dana = (int *) args + args_count;

	line_plus(lxvery_data);

	line_text(lxvery_data, bits, screen_data, data, &args, &dana);
	_libx1f4i0_lxcall_line_side(rate, data, side_u, side_l, fp_size());
	_libx1f4i0_lxcall_line_vice
	    (side, data,
	     (struct x1f4_linetext_type *) data + line_count - 1, side_o,
	     side_f, fb_size());

	/*
	 * link_f abused for line_data, etc, not to be set until after.
	 */

	lxvery_data->link_f.data = data;
	lxvery_data->link_f.rate = rate;
	lxvery_data->link_f.side = side;

	line_line(lxvery_data, bits, screen_data);

	if (status) {
	    lxvery_data->link_w.free(lxvery_data->link_w.data, data);
	} else {
	}
    }

    return status;
}


static int
part_text(struct lxvery_type *lxvery_data, unsigned bits,
	  const struct screen_type *part_data,
	  struct x1f4_linetext_type **linetext, void **args, unsigned count,
	  const struct line_type *line_data)
{
    struct cxseek_type *cxseek_data;
    struct x1f4_linetext_type *linetext_data;
    unsigned i = 4;

    cxseek_data = lxvery_data->rule_f.cxseek;

    linetext_data = *linetext;

    count >>= 2;

    for (; i; i--) {
	_libx1f4i0_lxcall_side_line
	    (cxseek_data, bits, part_data, &linetext_data, args, count,
	     line_data);
	cxseek_data++;
	line_data += count;
    }

    *linetext = linetext_data;

    return 0;
}


static void
bo_lmm00x(u_case_args_____1)
{
    int *line;

    line = *args;

    linetext_data->function.args = line;

    *line++ = screen(screen)->link_l.type;
    *line++ = X1f4_E4_MODE;
    *line++ = X1f4_E4_MODE;
    *line++ = 0;
    *line++ = 0;
    *line++ = X1f4_E4_POST_XSET;

    *args = line;
}


static void
born_lm0x(u_case_args_____1)
{
    int *line;

    line = *args;

    linetext_data->function.args = line;

    *line++ = screen(screen)->link_l.type;
    *line++ = X1f4_E4_MODE;
    *line++ = 0;
    *line++ = X1f4_E4_POST_XSET;

    *args = line;
}


static void
borne_lcm(u_case_args_____1)
{
    int *line;

    line = *args;

    linetext_data->function.args = line;

    *line++ = screen(screen)->link_l.type;
    *line++ = X1f4_E4_CASE;
    *line++ = X1f4_E4_MODE;

    *args = line;
}


static void
borneo__l(u_case_args_____1)
{
    int *line;

    line = *args;

    linetext_data->function.args = line;

    *line++ = screen(screen)->link_l.type;

    *args = line;
}


static void
borneo_l_(u_case_args_____1)
{
    linetext_data->function.type = screen(screen)->link_l.type;
}


static void
borneo_lm(u_case_args_____1)
{
    int *line;

    line = *args;

    linetext_data->function.args = line;

    *line++ = screen(screen)->link_l.type;
    *line++ = X1f4_E4_MODE;

    *args = line;
}


static void
line_nset(u_case_args_____1)
{
    int *line;

    line = *args;

    linetext_data->function.args = line;

    *line++ = screen(screen)->link_l.type;
    *line++ = X1f4_E4_POST_XSET;

    *args = line;
}


static void
line_sset(u_case_args_____1)
{
    int *line;

    line = *args;

    linetext_data->function.args = line;

    *line++ = screen(screen)->link_l.type;
    *line++ = screen(screen)->link_l.type;
    *line++ = X1f4_E4_POST_FLAT | X1f4_E4_POST_XSET;
    *line++ = 0;

    *args = line;

    linetext_data->function.type = screen(screen)->link_l.type;
}


static void
theme_lll(u_case_args_____1)
{
    int *line;

    line = *args;

    linetext_data->function.args = line;

    *line++ = screen(screen)->link_l.type;
    *line++ = screen(screen)->link_l.type;
    *line++ = screen(screen)->link_l.type;

    *args = line;

    linetext_data->function.type = screen(screen)->link_l.type;
}


static void
themed__l(u_case_args_____1)
{
    int *line;

    line = *args;

    linetext_data->function.args = line;

    *line++ = screen(screen)->link_l.type;

    *args = line;

    linetext_data->function.type = screen(screen)->link_l.type;
}


static void
themed_ll(u_case_args_____1)
{
    int *line;

    line = *args;

    linetext_data->function.args = line;

    *line++ = screen(screen)->link_l.type;
    *line++ = screen(screen)->link_l.type;

    *args = line;

    linetext_data->function.type = screen(screen)->link_l.type;
}


static void
themed_lm(u_case_args_____1)
{
    int *line;

    line = *args;

    linetext_data->function.args = line;

    *line++ = screen(screen)->link_l.type;
    *line++ = X1f4_E4_MODE;

    *args = line;

    linetext_data->function.type = screen(screen)->link_l.type;
}


int
x1f4_init_lxvery(void **lxvery, unsigned bits,
		 const struct screen_type *screen_data)
{
    int (*link)(void *, void **, unsigned), status;
    void *line, *text;

    if (bits & CODELINK_LINK) {
	link = screen_data->lock_w.link;
	text = screen_data->lock_w.data;
    } else {
	link = _x1f4_e4_link_data;
	text = (void *) 0;
    }

    status = link(text, &line, sizeof(struct lxvery_type));
    if (status) {
	status = LINK_ERROR;
    } else {
	status = init_line(line, bits, screen_data);
	if (status) {
	    if (bits & CODELINK_LINK) {
		screen_data->lock_w.free(text, line);
	    } else {
		_x1f4_e4_free_data(text, line);
	    }
	} else {
	    *lxvery = line;
	}
    }

    return status;
}
