/*
 * lxvery-d.b.c
 * Copyright (C) 2008-2015, 2017, 2018, 2021, Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <lxvery-config.h>

#include <stddef.h>

#include <d_fset.h>
#include <e4.h>
#include <lxvery-drive.h>
#include <lxvery-inter.h>
#include <lxvery-types.h>

static int push_mode(void *, void *);

static int
push_mode(void *text, void *data)
{
    int status;
    struct dxvery_type *dxvery_data;
    void **line;
#if __VERY_PSET__
#else
    void *fine;
#endif				/* __VERY_PSET__ */

    line = text;

    dxvery_data = line[0];

#if __VERY_PSET__
    data = *(void **) data;
#endif				/* __VERY_PSET__ */

#if __VERY_PSET__
    status = x1f4_find_bqpset(line[1], data);
    if (status) {
	status = x1f4_post_bqpset(dxvery_data->dash, data);
	if (status) {
	    if (status == X1f4_BQFSET_EVER_MATCH) {
		status = 0;
	    } else {
	    }
	} else {
	    dxvery_data->size++;
	}
    }
#else
    status = x1f4_find_lxfset(line[1], data, &fine);
    if (status) {
	status = x1f4_post_lxfset(dxvery_data->dash, data, &fine);
	if (status) {
	    if (status == X1f4_BQFSET_EVER_MATCH) {
		status = 0;
	    } else {
	    }
	} else {
	    X1f4_E4_C_MODE *trip;

	    trip = fine;

	    *trip = *(X1f4_E4_C_MODE *) data;

	    dxvery_data->size++;
	}
    }
#endif				/* __VERY_PSET__ */

    return status;
}


int
_libx1f4i0_lxvery_b_derived(void *context, void *output, void **input)
{
    int status;
    struct dxvery_type *dxvery_data, *dxvery_slip, *dxvery_text;
    void *line[2];

    dxvery_data = I_LINE(input[0]);
    dxvery_slip = I_LINE(input[1]);
    dxvery_text = I_LINE(input[2]);

    line[0] = dxvery_data;

    if (dxvery_slip->size < dxvery_text->size) {
	line[1] = dxvery_text->dash;
	status = x1f4_lime_bqfset(dxvery_slip->dash, line, push_mode);
    } else {
	line[1] = dxvery_slip->dash;
	status = x1f4_lime_bqfset(dxvery_text->dash, line, push_mode);
    }
    if (status) {
	status = _libx1f4i0_lxvery_stat_line(context, 0, status);
    } else {
	l_LINE(dxvery_data, output);
    }

    return status;
}
