/*
 * lxvery-types.h
 * Copyright (C) 2008-2017, 2021, Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef __LIBx1f4_lxvery_types_H__
#define __LIBx1f4_lxvery_types_H__

#include <config.h>

#include <e4.h>
#include <frame.h>
#include <integral_q.h>
#include <ln.h>
#include <lxcast.h>
#include <lxmode.h>
#include <nl.h>
#include <nt.h>
#include <track.h>
#include <trans.h>

#define excase_type			x1f4_nodelink_type

#define here_NULL			((void *) 0)

#define __types_lock(lxvery) \
    (&__types_long(lxvery)->east_1)

#define __types_long(lxvery) \
    (&__types_mode(lxvery)->nice_1)

#define __types_mode(lxvery) \
    (&((struct lxvery_type *) (lxvery))->ever_m)

#define here_offsetof(type, ceil) \
    ((integral_q) (&((type *) here_NULL)->ceil))

#define post_dxvery(dxvery_text) \
    ((struct dxvery_type *) (dxvery_text) + 1)

#define __struct_dxvery_type_alloc_size__ \
    sizeof(struct dxvery_type)

typedef struct cxbend_type {
    int bend, (*pick)(void *, void **), sign, (*slip)(void *);
    void *line;
} cxbend_type;

typedef struct cxseek_type {
    int (*seek)(void *, void *, void **);
    void *line;
} cxseek_type;

typedef struct dxvery_type {
    unsigned call, size;
    void *dash;
} dxvery_type;

typedef struct lxvery_type {
    struct lxmode_type ever_m;
    struct {
	int data;
    } link_i;
    struct {
	void *data;
    } link_l;
    struct {
	struct cxbend_type cxbend[2];
	struct cxseek_type cxseek[4];
    } rule_f;
} lxvery_type;

typedef struct screen_type {
    struct x1f4_track_type lock_e;
    struct {
	int type;
    } link_l;
    struct x1f4_trans_type lock_m;
    struct x1f4_frame_type lock_v;
    struct x1f4_trans_type lock_w;
} screen_type;

#endif				/* __LIBx1f4_lxvery_types_H__ */
