/*
 * qsce.f.c
 * Copyright (C) 2008-2012, 2014, 2016, Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <qscc-config.h>

#include <stddef.h>

#include <mask.0.h>
#include <qscc-defs.h>
#include <qscc-inter.h>
#include <qscc-names.h>
#include <qscc-types.h>
#include <trans.h>

#define qsrate(qsrate)			((struct qsrate_type *) (qsrate))

static int fast_list(struct qsrate_type *, unsigned, unsigned, unsigned);
static int rule_less(struct fpnode_type *, void **, unsigned, unsigned,
		     unsigned, unsigned, unsigned, unsigned, unsigned);
static int rule_list(struct qsrate_type *, unsigned, unsigned, unsigned,
		     unsigned, unsigned, unsigned, unsigned);
static int rule_rate(struct qsrate_type *, unsigned, unsigned, unsigned,
		     unsigned);

static int
fast_list(struct qsrate_type *qsrate_data, unsigned mall, unsigned line,
	  unsigned star)
{
    int status;
    void *data;

#if __MICRO_RATES__
    while (1) {
	if (3 * line < star << 2) {
	    break;
	} else {
	    if (line & 2) {
		break;
	    } else {
		line >>= 1;
	    }
	}
    }
#endif				/* __MICRO_RATES__ */

    data = qsrate_data->link_a.fplist.fpnode;

    status = qsrate_data->link_m.mode
	(qsrate_data->link_m.data, &data, mall * 3 * line >> 1);
    if (status) {
	status = LINK_ERROR;
    } else {
	qsrate_data->link_a.fplist.size = star;

	qsrate_data->link_a.fplist.fpnode = data;
    }

    return status;
}


static int
rule_less(struct fpnode_type *fpnode_data, void **seek, unsigned less,
	  unsigned node, unsigned line, unsigned ever, unsigned lead,
	  unsigned last, unsigned fade)
{
    unsigned clip, dash, mind, rule;

    mind = node;
    for (; mind; mind--) {
	fpnode_data->call = 0;
	fpnode_data->node = 0;
	fpnode_data++;
    }

    fpnode_data -= node;

    dash = less;

    dash |= dash >> 001;
    dash |= dash >> 002;
    dash |= dash >> 004;
    dash |= dash >> 010;
    dash |= dash >> 020;

    dash >>= 1;

    clip = less & dash;
    dash = less ^ clip;
    dash = node / dash;

    clip <<= 1;

    mind = less;
    rule = 0;
    for (; mind; mind--) {
	unsigned back, fast, this, trap = 1;

	if (2 < mind) {
	    fast = line * ever;
	    this = line;
	} else {
	    if (1 < mind) {
		if (lead) {
		    fast = lead * ever;
		    this = lead;
		} else {
		    fast = line * ever;
		    this = line;
		}
	    } else {
		fast = fade;
		if (last) {
		    this = last;
		} else {
		    this = line;
		}
	    }
	}

	back = rule + 1;
	while (back < node) {
	    if (back & trap) {
		fpnode_data[back].call += fast;
		back += trap;
	    }

	    trap <<= 1;
	}

	fpnode_data[rule].node = this;
	fpnode_data[rule].star = *seek++;

	if (clip) {
	    clip--;
	    rule += dash >> 1;
	} else {
	    rule += dash;
	}
    }

    return 0;
}


static int
rule_list(struct qsrate_type *qsrate_data, unsigned mall, unsigned line,
	  unsigned lock, unsigned lift, unsigned spin, unsigned trip,
	  unsigned turn)
{
    int (*link) (void *, void **, unsigned), status;
    unsigned node, star;
    void *data, *file;

    star = lift;

    node = 1 << lock;

    data = qsrate_data->link_m.data;
    link = qsrate_data->link_m.link;

    status =
	link(data, &file, sizeof(void *) * spin + node * sizeof(unsigned));
    if (status) {
	status = LINK_ERROR;
    } else {
	int (*free) (void *, void *), excess;
	unsigned pass, rate = 0, size, slip;
	void **seek, *zero;

	seek = file;

	size = mall * line;

	slip = turn;
	for (; slip; slip--) {
	    status = link(data, seek, size);
	    if (status) {
		status = LINK_ERROR;
		if (1) {
		    break;
		}
	    } else {
		seek++;
	    }
	}
	if (status) {
	} else {
	    size = sizeof(struct fpnode_type) << lock;

	    slip = spin - turn;
	    for (; slip; slip--) {
		status = link(data, seek, size);
		if (status) {
		    status = LINK_ERROR;
		    if (1) {
			break;
		    }
		} else {
		    seek++;
		}
	    }
	}

	free = qsrate_data->link_m.free;

	if (status) {
	    slip = seek - (void **) file;
	    for (; slip; slip--) {
		seek--;
		free(data, *seek);
	    }
	} else {
	    unsigned ever, part, *rule;
	    void **deck;

	    rule = (void *) seek;

	    part = node - 1;

	    seek = file;
	    deck = seek + turn;

	    trip = turn;

	    pass = node >> 1;
	    size = line;
	    slip = 2;
	    for (; pass; pass >>= 1, slip <<= 1) {
		unsigned *pick;

		spin = pass;
		pick = rule;
		pick += slip >> 1;
		for (; spin; spin--) {
		    *pick = size;
		    pick += slip;
		}

		size <<= 1;
	    }

	    ever = 1;

	    while (trip & ~1) {
		unsigned land, last, lead, near, rail, sort;

		pass = trip;
		trip += part;
		trip >>= lock;

		if (rate) {
		    lead = star & part;
		    if (lead) {
			last = node >> 1;
			if (lead < last) {
			    lead += last;
			} else {
			    last = lead;
			    lead = 0;
			}
		    } else {
			last = 0;
		    }
		} else {
		    lead = star % line;
		    if (lead) {
			last = line >> 1;
			if (lead < last) {
			    lead += last;
			} else {
			    last = lead;
			    lead = 0;
			}
		    } else {
			last = 0;
		    }
		}

		near = pass & part;
		if (near) {
		    if (pass < node) {
			rail = near;
			near = 0;
		    } else {
			rail = node >> 1;
			if (near < rail) {
			    near += rail;
			} else {
			    rail = near;
			    near = 0;
			}
		    }
		} else {
		    if (last) {
			near = 0;
			rail = node;
		    } else {
			rail = 0;
		    }
		}

		rate++;

		land = trip;
		if (rail) {
		    land--;
		    if (near) {
			land--;
		    }
		}

		sort = line * ever;

		for (; land; land--) {
		    struct fpnode_type *fpnode_data;
		    unsigned mind, *pick;

		    fpnode_data = *deck++;
		    pick = rule;
		    mind = node;
		    for (; mind; mind--) {
			fpnode_data->call = *pick++;
			fpnode_data->node = line;
			fpnode_data->star = *seek++;
			fpnode_data++;
		    }
		}

		if (rail) {
		    unsigned fade;

		    fade = lift - (star - (last ? last : line)) * ever;
		    if (near) {
			rule_less
			    (*deck++, seek, near, node, line, ever, 0, 0,
			     sort);
			seek += near;
			rule_less
			    (*deck++, seek, rail, node, line, ever, lead, last,
			     fade);
		    } else {
			rule_less
			    (*deck++, seek, rail, node, line, ever, lead, last,
			     fade);
		    }

		    seek += rail;
		}

		star = pass;

		ever *= line;

		line = node;

		{
		    unsigned mind, *pick;

		    pick = rule;
		    mind = node;
		    for (; mind; mind--) {
			*pick++ <<= lock;
		    }
		}
	    }

	    zero = *(deck - 1);
	}

	excess = free(data, file);
	if (excess) {
	    if (status) {
	    } else {
		status = FREE_ERROR;

		_libx1f4l2_gate_qsrate(qsrate_data, zero, rate - 1, node);
		free(data, zero);
	    }
	}

	if (status) {
	} else {
	    qsrate_data->link_a.fplist.node = pass;
	    qsrate_data->link_a.fplist.rate = rate;

	    qsrate_data->link_a.fplist.size = lift;

	    qsrate_data->link_a.fplist.fpnode = zero;
	}
    }

    return status;
}


static int
rule_rate(struct qsrate_type *qsrate_data, unsigned mall, unsigned line,
	  unsigned lock, unsigned lift)
{
    int status;
    unsigned node, part, rate = 0, spin = 1, trip, turn;
    void *zero;

    trip = (lift + line - 1) / line;

    turn = trip;

    node = 1 << lock;
    part = node - 1;

    while (trip & ~1) {
	rate++;
	spin += trip;
	trip += part;
	trip >>= lock;
    }

    zero = qsrate_data->link_a.fplist.fpnode;

    status = _libx1f4l2_mall_qsrate(qsrate_data, rate);
    if (status) {
    } else {
#if __MICRO_RATES__
	do {
	    status = qsrate_data->link_m.mode
		(qsrate_data->link_m.data, &zero,
		 qsrate_data->link_a.fplist.line * 3
		 * qsrate_data->link_a.fplist.mall >> 1);
	    if (status) {
		status = MODE_ERROR;
	    }
#endif				/* __MICRO_RATES__ */

	    qsrate_data->link_e.fplink_text = zero;

	    status = rule_list
		(qsrate_data, mall, line, lock, lift, spin, trip, turn);

	    if (status) {
		_libx1f4l2_fade_qsrate(qsrate_data);
		if (1) {
		    qsrate_data->link_e.fplink_data = NULL;
		}
	    }
#if __MICRO_RATES__
        } while (0);
#endif				/* __MICRO_RATES__ */
    }

    return status;
}


int
x1f4_star_qsrate(void *qsrate, unsigned star)
{
    int status;

    if (qsrate(qsrate)->link_a.fplist.size) {
	status = SIZE_ERROR;
    } else {
	unsigned line, lock, mall;

	line = qsrate(qsrate)->link_a.fplist.line;
	lock = qsrate(qsrate)->link_a.fplist.lock;
	mall = qsrate(qsrate)->link_a.fplist.mall;

	if (3 * line < star << 1) {
	    status = rule_rate(qsrate, mall, line, lock, star);
	} else {
	    status = fast_list(qsrate, mall, line, star);
	}
    }

    return status;
}
