/*
 * sfar.d.c
 * Copyright (C) 2009-2013, 2018, Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <stddef.h>
#include <string.h>

#include <sfas-defs.h>
#include <sfas-inter.h>
#include <sfas-names.h>
#include <sfas-patch.h>
#include <sfas-types.h>

#define back(miss)			((const char *) (miss))
#define fast(miss)			((const byte *) (miss))

#define sfnote(sfnote)			((struct sfnote_type *) (sfnote))

static int blue_note(void *, struct fpnode_type *, struct fpnode_type *,
		     void *, int (*) (void *, void *));
static int fail_note(void *, struct fpnode_type *, void *,
		     int (*) (void *, void *));

static int
blue_note(void *sfnote, struct fpnode_type *fpnode_slip,
	  struct fpnode_type *fpnode_data, void *back,
	  int (*call) (void *, void *))
{
    int error, status = 0;

    error = _libx1f4l2_hale_sfnote(sfnote, fpnode_data, back, call, &status);
    if (error == CALL_ERROR) {
    } else {
	int excess;

	excess = _libx1f4l2_yank_sfnote(sfnote, fpnode_slip, fpnode_data);
	if (excess) {
	    if (status) {
	    } else {
		status = excess;
	    }
	}
    }

    return status;
}


static int
fail_note(void *sfnote, struct fpnode_type *fpnode_data, void *back,
	  int (*call) (void *, void *))
{
    int error, status = 0;

    error = _libx1f4l2_hale_sfnote(sfnote, fpnode_data, back, call, &status);
    if (error == CALL_ERROR) {
    } else {
	int excess;

	excess = sfnote(sfnote)->link_m.free
	    (sfnote(sfnote)->link_m.data, fpnode_data);
	if (excess) {
	    if (status) {
	    } else {
		status = FREE_ERROR;
	    }
	}

	sfnote(sfnote)->link_a.fpdeck.fpnode_data = NULL;
    }

    return status;
}


int
x1f4_drop_sfnote(void *sfnote, const char *name, void *back,
		 int (*call) (void *, void *))
{
    int status;
    struct fpnode_type *fpnode_data, *fpnode_slip = NULL;

    fpnode_data = sfnote(sfnote)->link_a.fpdeck.fpnode_data;
    if (fpnode_data) {
	const char *lead;

	lead = name;

	do {
	    byte *slip;
	    int e;
	    unsigned length;

	    e = prefix(fpnode_data);
	    if (e) {
		if (e ^ *fast(name)) {
		    status = MISS_CLASS;
		    if (1) {
			break;
		    }
		} else {
		    name++;
		    abstract_node(fpnode_data, slip, length);
		}
	    } else {
		length = 0;
		slip = NULL;
	    }

	    while (1) {
		int c;

		c = *fast(name);
		if (c) {
		    name++;
		    if (length) {
			if (c == *slip) {
			    length--;
			    slip++;
			} else {
			    status = MISS_CLASS;
			    if (1) {
				break;
			    }
			}
		    } else {
			unsigned bits;

			bits = fpnode_data->bits;
			if (Extant(bits)) {
			    fpnode_slip = fpnode_data;
			    dispatch_shut(fpnode_data, bits, c);
			} else {
			    status = MISS_CLASS;
			    if (1) {
				break;
			    }
			}
		    }
		} else {
		    if (fpnode_slip) {
			status = blue_note
			    (sfnote, fpnode_slip, fpnode_data, back, call);
		    } else {
			status = fail_note(sfnote, fpnode_data, back, call);
		    }

		    break;
		}
	    }
	} while (0);
    } else {
	status = MISS_CLASS;
    }

    return status;
}
